﻿using System;
using System.Collections.Generic;
using System.Text;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;

namespace Ice.Net.Ecommerce.BusinessObjects.Common
{
	public class AppDefaults
	{
		//// static bool initDone = new Func<bool>(() => { Ice.Net.Ecommerce.PaymentGateways.PGLoader.Initialize(); return true; }).Invoke();

		public Database.VISTACART.AppGlobalSetting AppGlobalSettingsItem
		{
			get
			{

				CommonUtility.UpdateStatuses(false);

				AppGlobalSetting sett = CacheProvider.ApplicationStore.Retrieve<AppGlobalSetting>();

				if (sett == null)
				{
					// load from database
					sett = BusinessObjects.Database.VISTACART.AppGlobalSetting.GetDefault();
					if (sett != null)
					{
						CacheProvider.ApplicationStore.Save(sett);
					}
					else
					{
						throw new Exception("No active AppSetting found!");
					}
				}
				return sett;
			}
		}

		public int AuthenticationTimeoutMinutes
		{
			get { return this.AppGlobalSettingsItem.AuthenticationTimeoutMinutes; }
		}

		const string mCurrencyName = "CurrencyName";
		/// <summary>
		/// Get default currency. Keep it's reference in local variable.
		/// </summary>
		public Database.VISTACART.Currency Currency
		{
			get
			{
				Currency currency = CacheProvider.ApplicationStore.RetrieveByName<Currency>(mCurrencyName);
				if (currency == null)
				{
					currency = SmartObjectLoader.LoadSmartObject<Database.VISTACART.Currency>(this.AppGlobalSettingsItem.CurrencyID.GetValueOrDefault());
					CacheProvider.ApplicationStore.Save(currency, mCurrencyName);
				}
				return currency;
			}
		}

		const string mWeightName = "WeightName";

		/// <summary>
		/// Get default weight unit. Keep it's reference in local variable.
		/// </summary>
		public Database.VISTACART.WeightUnit WeightUnit
		{
			get
			{
				WeightUnit wu = CacheProvider.ApplicationStore.RetrieveByName<WeightUnit>(mWeightName);
				if(wu == null)
				{
					wu = SmartObjectLoader.LoadSmartObject<Database.VISTACART.WeightUnit>(this.AppGlobalSettingsItem.WeightUnitID.GetValueOrDefault());
					CacheProvider.ApplicationStore.Save(wu, mWeightName);
				}
				return wu;
			}
		}

		const string mPagerSizeName = "PagerSizeName";
		/// <summary>
		/// Get default PagerSize-+ unit. Keep it's reference in local variable.
		/// </summary>
		public Database.VISTACART.PagerSize PagerSize
		{
			get
			{
				PagerSize psize = CacheProvider.ApplicationStore.RetrieveByName<PagerSize>(mPagerSizeName);
				if (psize == null)
				{
					psize = SmartObjectLoader.LoadSmartObject<Database.VISTACART.PagerSize>(this.AppGlobalSettingsItem.PagerSizeID.GetValueOrDefault());
					CacheProvider.ApplicationStore.Save(psize, mPagerSizeName);
				}
				return psize;
			}
		}

		const string mCurrentTaxName = "CurrentTaxName";

		//private Database.VISTACART.Tax mTax = null;
		/// <summary>
		/// Get default PagerSize-+ unit. Keep it's reference in local variable.
		/// </summary>
		public Database.VISTACART.Tax Tax
		{
			get
			{
				Tax tax = CacheProvider.RequestStore.RetrieveByName<Tax>(mCurrentTaxName);
				if (tax != null)
				{
					return tax;
				}

				if (VISTACARTHelper.CurrentUser.UserTicket.IsAuthenticated)
				{
					Address address = VISTACARTHelper.CurrentUser.User.GetAddress();

					// load by State
					if (address.StateID > 0) 
					{
						// load tax by state
						State state = address.GetState();
						tax = state.GetTax();
						if (tax.TaxID > 0 && tax.IsActive)
						{
							// use this tax
						}
						else
						{
							tax = null;
						}
					}

					if (tax == null && address.CountryID > 0)
					{
						// load tax by country
						Country country = address.GetCountry();
						tax = country.GetTax();
						if (tax.TaxID > 0 && tax.IsActive)
						{
							// use this tax
						}
						else
						{
							tax = null;
						}
					}
					// else load default tax
				}

				if (tax == null)
				{
					tax = SmartObjectLoader.LoadSmartObject<Database.VISTACART.Tax>(this.AppGlobalSettingsItem.TaxID.GetValueOrDefault());
				}

				CacheProvider.RequestStore.Save(tax, mCurrentTaxName);

				return tax;
			}
		}


		////const string mCurrenPaymentGatewayName = "CurrenPaymentGatewayName";

		////internal PaymentGateways.PGLoader PGLoader
		////{
		////    get
		////    {
		////        ////PaymentGateways.PGLoader pgLoader = CacheProvider.ApplicationStore.RetrieveByName<PaymentGateways.PGLoader>(mCurrenPaymentGatewayName);
		////        ////if(pgLoader == null)
		////        ////{
		////        ////    pgLoader = new Ice.Net.Ecommerce.PaymentGateways.PGLoader();
		////        ////    CacheProvider.ApplicationStore.Save(pgLoader, mCurrenPaymentGatewayName);
		////        ////}
		////        PaymentGateways.PGLoader pgLoader = new Ice.Net.Ecommerce.PaymentGateways.PGLoader();
		////        return pgLoader;
		////    }
		////}


	}
}
