﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using System.Text.RegularExpressions;

namespace Ice.Net.Ecommerce.BusinessObjects.Common
{
	/// <summary>
	/// Web site utility methods.
	/// </summary>
	public class ApplicationUtility
	{
		/// <summary>
		/// Dynamic skins handler prefix (g-)
		/// </summary>
		public static string HandlerPrefix
		{
			get { return "g-"; }
		}

		/// <summary>
		/// <para>Returns Http site root with '/' at the end.</para>
		/// <para>example: http://www.iceasp.net/</para>
		/// </summary>
		public static string ApplicationHttpRoot
		{
			get { return GetHttpRoot(true); }
		}

		/// <summary>
		/// <para>use real app root with full path, don't use url saved into database</para>
		/// </summary>
		public static string ApplicationHttpRootCurrent
		{
			get { return GetHttpRoot(false); }
		}

		private static string GetHttpRoot(bool usePreprogrammedWebsite)
		{
			string site = "";
			string webSite = VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.WebSite;
			if (usePreprogrammedWebsite && !string.IsNullOrEmpty(webSite)) // use url stored in database
			{
				site = webSite;
				if (!site.StartsWith("http"))
				{
					site = "http://" + site;
				}
				if (!site.EndsWith("/"))
				{
					site += "/";
				}
			}
			else // use current web site
			{
				site = "http://" + HttpContext.Current.Request.ServerVariables["HTTP_HOST"] + HttpContext.Current.Request.ApplicationPath;
				if (!site.EndsWith("/"))
				{
					site += "/";
				}
			}

			if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SSLMode == SSLModeUtility.SSLModeEnum.CreateSSLLinks ||
				VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SSLMode == SSLModeUtility.SSLModeEnum.ForceAlways)
			{
				site = site.Replace("http://", "https://");
			}

			return site;
		}

		/// <summary>
		/// <para>Returns Http relative path, based on ApplicationHttpRootCurrent, from current request without '/' at start</para>
		/// </summary>
		public static string RelativeHttpPath
		{
			get
			{
				string relativeHttpPath = HttpContext.Current.Request.CurrentExecutionFilePath.Remove(HttpContext.Current.Request.CurrentExecutionFilePath.LastIndexOf("/"));
				relativeHttpPath = relativeHttpPath.Substring(HttpContext.Current.Request.ApplicationPath.Length);
				if (relativeHttpPath.StartsWith("/"))
				{
					relativeHttpPath = relativeHttpPath.Substring(1);
				}
				if (relativeHttpPath.EndsWith("/") == false)
				{
					relativeHttpPath += "/";
				}
				return relativeHttpPath;
			}
		}

		public static string mApplicationPhysicalRoot = null;
		/// <summary>
		/// <para>Returns web application physical root with '\' at the end.</para>
		/// <para>example: C:\inetpub\wwwroot\mysite\</para>
		/// </summary>
		public static string ApplicationPhysicalRoot
		{
			get
			{
				if (mApplicationPhysicalRoot == null)
				{
					mApplicationPhysicalRoot = HttpContext.Current.Request.PhysicalApplicationPath;
				}
				return mApplicationPhysicalRoot;
			}
		}

		/// <summary>
		/// Returns absolute http link based on relativeLink and ApplicationHttpRoot value
		/// </summary>
		/// <param name="relativeLink"></param>
		/// <returns></returns>
		public static string GetAbsoluteLink(string relativeLink)
		{
			if (relativeLink.StartsWith("/"))
			{
				relativeLink = relativeLink.TrimStart('/');
			}
			return ApplicationHttpRoot + relativeLink;
		}

		private static string mSkinDirectory = null;
		/// <summary>
		/// Skin directory, format: ~/skins/base/
		/// </summary>
		public static string SkinDirectory
		{
			get
			{
				if (mSkinDirectory == null)
				{
					mSkinDirectory = "~/" + System.Configuration.ConfigurationManager.AppSettings["SkinsRoot"].Trim('/') + "/";
				}
				if (OnlineDemoSkinHelper.IsOnlineDemo)
				{
					return mSkinDirectory + OnlineDemoSkinHelper.GetCurrentSkinDirectory() + "/";
				}
				else
				{
					return mSkinDirectory + VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SkinDirectory + "/";
				}
			}
		}

		/// <summary>
		/// Returns relative: skins/base/
		/// </summary>
		public static string SkinDirectoryRelative
		{
			get{return SkinDirectory.TrimStart('~', '/');}
		}

	}
}
