﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace Ice.Net.Ecommerce.BusinessObjects.Common
{
	using BusinessObjects.Database.VISTACART;

	public class CurrencyUtility
	{
		/// <summary>
		/// Convert default price into currently selected currency and add ISO code in front ( USD 32.55 )  or currency symbol is enabled in App Settings
		/// </summary>
		/// <param name="price"></param>
		/// <returns></returns>
		public static string GetFormatedPriceInCurrency(decimal price)
		{
			return GetFormatedPriceInCurrency(price, 2);
		}

		public static string GetFormatedPriceInCurrency(decimal price, int decimalPlaces)
		{
			string strprice = CurrencyUtility.ConvertPrice(price).ToString("n" + decimalPlaces);

			if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.IsCurrencySymbolUsed && !string.IsNullOrEmpty(VISTACARTHelper.CurrentUser.Currency.Symbol))
			{
				if (VISTACARTHelper.CurrentUser.Currency.IsSymbolBeforeAmount)
				{
					strprice = VISTACARTHelper.CurrentUser.Currency.Symbol + strprice;
				}
				else
				{
					strprice = strprice + VISTACARTHelper.CurrentUser.Currency.Symbol;
				}
			}
			else
			{
				strprice = VISTACARTHelper.CurrentUser.Currency.ISOCode + " " + strprice;
			}

			return strprice;
		}

		/// <summary>
		/// Convert price from default currency into current user selected currency.
		/// </summary>
		/// <param name="price">price in default currency</param>
		/// <returns></returns>
		public static decimal ConvertPrice(decimal price)
		{
			return ConvertPrice(price, VISTACARTHelper.CurrentUser.Currency, VISTACARTHelper.AppDefaults.Currency);
		}

		/// <summary>
		/// Convert price from default currency into toCurrency.
		/// </summary>
		/// <param name="price">price in default currency</param>
		/// <param name="toCurrency">new currency</param>
		/// <returns></returns>
		public static decimal ConvertPrice(decimal price, Currency toCurrency)
		{
			return ConvertPrice(price, toCurrency, VISTACARTHelper.AppDefaults.Currency);
		}

		/// <summary>
		/// Convert price fromCurrency into toCurrency.
		/// </summary>
		/// <param name="price">price in 'fromCurrency' Currency</param>
		/// <param name="toCurrency">convert to Currency</param>
		/// <param name="fromCurrency">input Currency</param>
		/// <returns></returns>
		public static decimal ConvertPrice(decimal price, Currency toCurrency, Currency fromCurrency)
		{
			if (fromCurrency.CurrencyID == toCurrency.CurrencyID)
			{
				return Math.Round(price, 2);
			}
			else
			{
				decimal convertedPrice = price * toCurrency.ConversionRate / fromCurrency.ConversionRate ;
				return Math.Round(convertedPrice, 2);
			}
		}

	}
}
