﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ice.Net.Ecommerce.BusinessObjects.Common
{
	using Database.VISTACART;

	public class CurrentUser
	{
		/// <summary>
		/// Load on-demand from session or cookie and keep it cached until the end of request.
		/// </summary>
		public UserInfo UserInfo
		{
			get
			{
				// load from request store first
				UserInfo userInfo = CacheProvider.RequestStore.Retrieve<UserInfo>();
				if (userInfo == null)
				{
					// not found, get from cookie
					UserInfo.UserInfoData userInfoData = CacheProvider.CookieStore.Retrieve<UserInfo.UserInfoData>();
					userInfo = new UserInfo(userInfoData);
					if (userInfoData == null)
					{
						// new user
						// save cookie just to enable user tracking
						userInfo.Save();
					}
					CacheProvider.RequestStore.Save(userInfo);
				}
				return userInfo;
			}
		}

		/// <summary>
		/// Load on-demand from session or cookie and keep it cached until the end of request.
		/// </summary>
		public UserTicket AdminTicket
		{
			get
			{
				UserTicket.UserTicketData userTicketData = CacheProvider.CookieStore.Retrieve<UserTicket.AdminTicketData>();
				if (userTicketData == null)
				{
					userTicketData = new UserTicket.AdminTicketData();
				}
				return new UserTicket(userTicketData);
			}
		}

		/// <summary>
		/// Load on-demand from session or cookie and keep it cached until the end of request.
		/// </summary>
		public UserTicket UserTicket
		{
			get
			{
				UserTicket.UserTicketData userTicketData = CacheProvider.CookieStore.Retrieve<UserTicket.UserTicketData>();
				if (userTicketData == null)
				{
					userTicketData = new UserTicket.UserTicketData();
				}
				return new UserTicket(userTicketData);
			}
		}

		private Database.VISTACART.User mUser = null;
		/// <summary>
		/// <para>Load UserItem based on UserTicket.UserID</para>
		/// <para>If something went wrong, returs null.</para>
		/// </summary>
		public Database.VISTACART.User User
		{
			get
			{
				if (mUser == null && this.UserTicket.IsAuthenticated && this.UserTicket.UserID > 0)
				{
					mUser = new BusinessObjects.Database.VISTACART.User();
					if (mUser.LoadFromDB(this.UserTicket.UserID))
					{
						// loaded
					}
					else
					{
						// can't load this user
						Logging.WriteError("Can't load UserItem, UserID: " + this.UserTicket.UserID);
						// remove UserTicket

						//this.SignOut();

						mUser = null;
					}
				}
				return mUser;
			}
		}

		private Database.VISTACART.Currency mCurrency = null;
		/// <summary>
		/// <para>Load current currency or default currency if current not found or not active.</para>
		/// <para>Keep it cached in local variable.</para>
		/// </summary>
		public Database.VISTACART.Currency Currency
		{
			get
			{
				mCurrency = SmartObjectLoader.LoadSmartObject<Database.VISTACART.Currency>(this.UserInfo.CurrencyID);
				if (mCurrency == null || mCurrency.IsActive == false)
				{
					Logging.WriteError("Can't load user requested currency, CurrencyID: " + this.UserInfo.CurrencyID);

					mCurrency = VISTACARTHelper.AppDefaults.Currency;
					this.UserInfo.Update(mCurrency.CurrencyID, 0, 0);
				}
				return mCurrency;
			}
		}

		private Database.VISTACART.WeightUnit mWeightUnit = null;
		/// <summary>
		/// <para>Load current user weightUnits or default if current not found.</para>
		/// <para>Keep it cached in local variable.</para>
		/// </summary>
		public Database.VISTACART.WeightUnit WeightUnit
		{
			get
			{
				mWeightUnit = SmartObjectLoader.LoadSmartObject<Database.VISTACART.WeightUnit>(this.UserInfo.WeightUnitID);
				if (mWeightUnit == null || mWeightUnit.IsActive == false)
				{
					Logging.WriteError("Can't load user requested weight unit, WeightUnitID: " + this.UserInfo.WeightUnitID);

					mWeightUnit = VISTACARTHelper.AppDefaults.WeightUnit;
					this.UserInfo.Update(0, 0, mWeightUnit.WeightUnitID);
				}

				return mWeightUnit;
			}
		}

		private Database.VISTACART.ShoppingCart mShoppingCart = null;
		/// <summary>
		/// Load ShoppingCart from database if user is authenticated or from cookie if not authenticated
		/// </summary>
		/// <returns></returns>
		public Database.VISTACART.ShoppingCart ShoppingCart
		{
			get
			{
				if (mShoppingCart == null)
				{
					mShoppingCart = Database.VISTACART.ShoppingCart.GetCart();
				}
				return mShoppingCart;
			}
		}


	}
}