﻿using System;
using System.Data;
using System.Configuration;
using System.Web;

namespace Ice.Net.Ecommerce.BusinessObjects.Common
{
	public class UserInfo
	{
		public class UserInfoData
		{
			public string Username = "";
			public int UserID;
			public int CurrencyID;
			public int LanguageID;
			public int WeightUnitID;
			public DateTime LastUpdate;
			public DateTime LastLogin;

			/// <summary>
			/// Load defaults on deserialization.
			/// </summary>
			public UserInfoData()
			{

			}
		}

		private UserInfoData mUi = null;

		public string Username
		{
			get { return mUi.Username; }
		}
		public int UserID
		{
			get { return mUi.UserID; }
		}
		public int CurrencyID
		{
			get { return mUi.CurrencyID; }
		}
		public int LanguageID
		{
			get { return mUi.LanguageID; }
		}
		public int WeightUnitID
		{
			get { return mUi.WeightUnitID; }
		}
		/// <summary>
		/// UTC DateTime
		/// </summary>
		public DateTime LastUpdate
		{
			get { return mUi.LastUpdate; }
		}
		/// <summary>
		/// UTC DateTime
		/// </summary>
		public DateTime LastLogin
		{
			get { return mUi.LastLogin; }
		}

		public UserInfo(UserInfoData uiData)
		{
			if (uiData != null)
			{
				mUi = uiData;
			}
			else
			{
				mUi = new UserInfoData();
			}
		}

		/// <summary>
		/// Save UserInfo to Cookie
		/// </summary>
		internal void Save()
		{
			//save to persistent cookie
			CacheProvider.CookieStore.Save(this.mUi, DateTime.Now.AddYears(100));
		}

		public void Update(int currencyID, int languageID, int weightUnitID)
		{
			mUi.LastUpdate = DateTime.UtcNow;

			if (mUi.CurrencyID < 1 || mUi.LanguageID < 1 || mUi.WeightUnitID < 1)
			{
				BusinessObjects.Database.VISTACART.AppGlobalSetting globalDefaultSetting = VISTACARTHelper.AppDefaults.AppGlobalSettingsItem;
				if (globalDefaultSetting != null)
				{
					mUi.CurrencyID = globalDefaultSetting.CurrencyID.GetValueOrDefault();
					mUi.WeightUnitID = globalDefaultSetting.WeightUnitID.GetValueOrDefault();
				}
			}

			if (currencyID > 0)
			{
				mUi.CurrencyID = currencyID;
			}
			if (languageID > 0)
			{
				mUi.LanguageID = languageID;
			}
			if (weightUnitID > 0)
			{
				mUi.WeightUnitID = weightUnitID;
			}
			Save();
		}

		internal void SetLoginInfo(string username, int userID)
		{
			mUi.LastLogin = DateTime.UtcNow;

			mUi.Username = username;
			mUi.UserID = userID;
			Save();
		}


	}
}
