﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ice.Net.Ecommerce.BusinessObjects.Common
{
	public class UserTicket
	{
		public class UserTicketData
		{
			public bool IsPersistentCookie;
			public DateTime AuthenticatedUntil;
			public string Username = "";
			public int UserID;
		}

		public class AdminTicketData: UserTicketData
		{
		}


		private UserTicketData mUt = null;

		public string Username
		{
			get { return mUt.Username; }
		}
		public int UserID
		{
			get { return mUt.UserID; }
		}
		public bool IsAuthenticated
		{
			get {return mUt.AuthenticatedUntil.ToUniversalTime() > DateTime.UtcNow; }
		}

		public UserTicket(UserTicketData utData)
		{

			//////if (utData != null)
			//////{
			//////    mUt = utData;
			//////}
			//////else
			//////{
			//////    mUt = new UserTicketData();
			//////}

			mUt = utData;
		}

		/// <summary>
		/// Set UserTicked to valid state. User is Authenticated.
		/// </summary>
		/// <param name="username"></param>
		/// <param name="userID"></param>
		/// <param name="rememberUser">
		/// <para>true: persistent cookie will be created. User is allways logged-in until he/she sign-out.</para>
		/// <para>false: create session cookie, authentication expires when user closes browser or after AuthenticationTimeoutMinutes minutes.</para></param>
		internal void SetTicket(string username, int userID, bool rememberUser)
		{
			mUt.Username = username;
			mUt.UserID = userID;

			mUt.IsPersistentCookie = rememberUser;
			if (rememberUser)
			{
				mUt.AuthenticatedUntil = DateTime.UtcNow.AddYears(100);
			}
			else
			{
				mUt.AuthenticatedUntil = DateTime.UtcNow.AddMinutes(VISTACARTHelper.AppDefaults.AuthenticationTimeoutMinutes);
			}
			Save();
		}

		/// <summary>
		/// Destroy UserTicket. User is no longer authenticated.
		/// </summary>
		internal void DestroyTicket()
		{
			// update current object cached to session
			mUt.IsPersistentCookie = false;
			mUt.AuthenticatedUntil = DateTime.MinValue;
			// delete cookie
		    CacheProvider.CookieStore.Delete(mUt.GetType());
		}

		/// <summary>
		/// Save UserInfo to Cookie
		/// </summary>
		private void Save()
		{
			if (mUt.IsPersistentCookie)
			{
				// persistent cookie
				CacheProvider.CookieStore.Save(mUt, DateTime.UtcNow.AddYears(100));
			}
			else
			{
				// session cookie
				CacheProvider.CookieStore.Save(mUt);
			}
		}

		public void SignIn(BusinessObjects.Database.VISTACART.User user, bool rememberUser)
		{
			if (user.UserID > 0 && user.IsActive && !string.IsNullOrEmpty(user.Username))
			{
				this.SetTicket(user.Username, user.UserID, rememberUser);
				VISTACARTHelper.CurrentUser.UserInfo.SetLoginInfo(user.Username, user.UserID);
				// move cookie cart to database
				BusinessObjects.Database.VISTACART.ShoppingCart.MoveCookieCartToDatabase(user.UserID);
			}
			else
			{
				// error!
				Logging.WriteError("Can't login user. UserID=" + user.UserID + ", IsActive=" + user.IsActive + ", Username=" + user.Username);
			}
		}

		/// <summary>
		/// Delete UserTicket
		/// </summary>
		public void SignOut()
		{
			this.DestroyTicket();
		}
	}
}
