﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using Ice.Orm.Core.Entity; using Ice.Orm.Core.Criteria;
using System.Data;

namespace Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART
{
	public partial class Product
	{
		#region Public properties

		private List<ProductImage> mProductImageCollection = null;
		public List<ProductImage> GetProductImageCollection()
		{
			if (mProductImageCollection == null)
			{
				mProductImageCollection = ProductImageCollection.LoadCollectionByProductID(this.ProductID);
			}
			return mProductImageCollection;
		}

		private string GetImageAtIndex(int index)
		{
			if (GetProductImageCollection().Count > index)
			{
				return GetProductImageCollection()[index].ImageLocation;
			}
			else
			{
				return "";
			}
		}

		private void SetImageAtIndex(int index, string imageLocation)
		{
			if (GetProductImageCollection().Count > index)
			{
				GetProductImageCollection()[index].ImageLocation = imageLocation;
			}
			else
			{
				// add new element
				GetProductImageCollection().Add(new ProductImage());
				SetImageAtIndex(index, imageLocation);
			}
		}

		public System.String SmallImage
		{
			get{	return GetImageAtIndex(0);}
			//set { SetImageAtIndex(0, value);}
		}

		public System.String LargeImage
		{
			get { return GetImageAtIndex(1); }
			//set { SetImageAtIndex(1, value); }
		}

		private List<AttributeGroup> mAttributeGroupCollection = null;

		public List<AttributeGroup> GetAttributeGroupCollection()
		{
			if (mAttributeGroupCollection == null)
			{
				mAttributeGroupCollection = AttributeGroupCollection.LoadByProductID(this.ProductID);
			}
			return mAttributeGroupCollection;
		}

		/// <summary>
		/// Adjust product stock quantity by 'quantity' value, + or -. Product is saved to database automatically - don't have to call SaveToDB() method!.
		/// </summary>
		/// <param name="quantity"></param>
		/// <returns></returns>
		public int AdjustStockQuantity(int quantity)
		{
			if (this.ProductID > 0 && quantity != 0)
			{
				// reload ProductStatistic
				this.StockQuantity += quantity;
				this.LastStockRefillDate = DateTime.Now;
				base.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

				return this.StockQuantity;
			}
			else
			{
				return 0;
			}
		}

		/////// <summary>
		/////// Increment ProductStatistic.ClicksCount
		/////// <para>Saves updated value to database also.</para>
		/////// </summary>
		/////// <returns></returns>
		////public int IncrementClickCount()
		////{
		////    if(this.ProductID > 0)
		////    {
		////        this.ClicksCount++;
		////        this.LastClickDate = DateTime.Now;
		////        this.SaveToDB();
		////    }
		////    return this.ClicksCount;
		////}

		private Manufacturer mManufacturerItem = null;

		public Manufacturer GetManufacturer()
		{
			if (mManufacturerItem == null)
			{
				mManufacturerItem = new Manufacturer();
				mManufacturerItem.LoadFromDB(this.ManufacturerID);
			}
			return mManufacturerItem;
		}

		private List<ProductInCategory> mProductInCategoryItems = null;

		private List<ProductInCategory> GetProductInCategoryCollection()
		{
			if (mProductInCategoryItems == null && this.IsLoadedFromDB())
			{
				// load from db
				mProductInCategoryItems = ProductInCategoryCollection.LoadByProductID(this.ProductID);
			}
			return mProductInCategoryItems;
		}

		private List<Category>mOwnerCategoryItems = null;
		/// <summary>
		/// Categories which owns this product. Loads on-demand and keep in local variable.
		/// </summary>
		public List<Category> GetCategoryCollection()
		{
			if (mOwnerCategoryItems == null && this.IsLoadedFromDB())
			{
				// load from db
				mOwnerCategoryItems = CategoryCollection.LoadByProductID(this.ProductID);
			}
			return mOwnerCategoryItems;
		}

		private List<Campaign> mCampaignCollection = null;
		/// <summary>
		/// Load on-demand campaigns where this product is included
		/// </summary>
		/// <returns></returns>
		public List<Campaign> GetCampaignCollection()
		{
			if (mCampaignCollection == null)
			{
				mCampaignCollection = CampaignCollection.LoadByProductID(this.ProductID);
			}
			return mCampaignCollection;
		}

		private CampaignHelper mCampaignHelper = null;
		/// <summary>
		/// <para>Currently active ProductCampaignData for this product</para>
		/// <para>Returns NULL if no active campaign found</para>
		/// </summary>
		public CampaignHelper GetCampaignHelper()
		{
			if (mCampaignHelper == null)
			{
				// load it
				mCampaignHelper = new CampaignHelper(this);
			}
			return mCampaignHelper;
		}

		/// <summary>
		/// Get SalePrice depending on currently active campaign sale price or discount percent
		/// </summary>
		/// <returns></returns>
		public decimal GetCalculatedSalePrice()
		{
			return GetCampaignHelper().GetSalePrice();
		}

		/// <summary>
		/// Get TaxAmount depending on currently active campaign sale price
		/// </summary>
		/// <returns></returns>
		public decimal GetCalculatedTaxAmount()
		{
			return GetCampaignHelper().GetTaxAmount();
		}

		/// <summary>
		/// Get SalePrice with TaxAmount based in current currency, depending on currently active campaign sale price
		/// </summary>
		/// <returns></returns>
		public decimal GetCalculatedSalePriceWithTaxAmount()
		{
			return GetCalculatedSalePrice() + GetCalculatedTaxAmount();
		}

		#endregion

		private int mAddToStock = 0;

		public int AddToStock
		{
			get { return mAddToStock; }
			set { mAddToStock = value; }
		}

		#region Database methods overrides
		// Database methods

		/// <summary>
		/// Save this and it's ProductInLanguageItem
		/// </summary>
		/// <returns></returns>
		public override Product SaveToDB(WriteModeEnum mode)
		{
			if (this.ProductID > 0)
			{
				this.AdjustStockQuantity(this.AddToStock);
				this.AddToStock = 0;
			}

			base.SaveToDB(mode);
			if (mProductImageCollection != null)
			{
				foreach (ProductImage pimg in mProductImageCollection)
				{
					pimg.ProductID = this.ProductID;
				}
				mProductImageCollection.ForEach(o => o.SaveToDB());
				mProductImageCollection = null; // force reloadin
			}

			return this;
		}

		/// <summary>
		/// Delete this and it's ProductInLanguageItems
		/// </summary>
		/// <returns></returns>
		public override bool DeleteFromDB()
		{
			GetDataStore().UseTransaction(DeleteCallback);
			return true;
		}

		private void DeleteCallback()
		{
			// test if the product is already ordered
			if (new OrderDetailCollection().CountByProductID(this.ProductID) > 0)
			{
				throw new Exception("Product was already ordered, can't be deleted! Set it's property IsActive = false instead.");
			}

			// delete images
			GetProductImageCollection().ForEach(o => o.DeleteFromDB());
			mProductImageCollection = null;

			// delte attributes
			foreach (AttributeGroup atgroup in GetAttributeGroupCollection())
			{
				atgroup.DeleteFromDB(); // delte AttributeGroupInLanguage too
			}
			mAttributeGroupCollection = null;

			// delete ProductInCategoryItems
			ProductInCategoryCollection prodInCategories = new ProductInCategoryCollection();
			prodInCategories.DeleteByProductID(this.ProductID);
			mProductInCategoryItems = null;

			// delete ProductInCampaignItems
			ProductInCampaignCollection prodInCampaign = new ProductInCampaignCollection();
			prodInCampaign.DeleteByProductID(this.ProductID);
			mCampaignHelper = null;

			// delete this
			base.DeleteFromDB();
		}

		#endregion

		#region Public methods

		/// <summary>
		/// Add product into category.
		/// </summary>
		/// <param name="categoryItem"></param>
		/// <returns></returns>
		public bool AddCategory(Category categoryItem)
		{
			if (this.IsLoadedFromDB() && this.IsInCategory(categoryItem) == false)
			{
				// force reloading
				mOwnerCategoryItems = null;
				mProductInCategoryItems = null;
				// add new 
				ProductInCategory productInCategory = new ProductInCategory();
				productInCategory.CategoryID = categoryItem.CategoryID;
				productInCategory.ProductID = this.ProductID;
				productInCategory.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				return true;
			}
			return false;
		}

		/// <summary>
		/// Remove product from category.
		/// </summary>
		/// <param name="categoryItem"></param>
		/// <returns></returns>
		public bool RemoveCategory(Category categoryItem)
		{
			mProductInCategoryItems = null;
			return ProductInCategory.DeleteByProductIDAndCagegoryID(this.ProductID, categoryItem.CategoryID);
		}

		public bool IsInCategory(Category categoryItem)
		{
			return ProductInCategory.LoadByProductIDAndCategoryID(this.ProductID, categoryItem.CategoryID) != null;
		}

		public bool AddCampaign(Campaign campaign)
		{
			mCampaignCollection = null;

			return campaign.AddProduct(this);
		}

		public bool RemoveCampaign(Campaign campaign)
		{
			mCampaignCollection = null;

			return campaign.RemoveProduct(this);
		}

		public bool IsInCampaign(Campaign campaign)
		{
			return ProductInCampaign.LoadByCampaignIDAndProductID(campaign.CampaignID, this.ProductID) != null;
		}

	



		#endregion

	}

}
