﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using Ice.Orm.Core.Entity; 
using Ice.Orm.Core.Criteria;

namespace Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART
{
	public partial class User
	{
		#region Public properties

		private UserInRoleCollection mUserInRoleCollection = null;

		private UserInRoleCollection GetUserInRoleCollection()
		{
			if (mUserInRoleCollection == null)
			{
				mUserInRoleCollection = new UserInRoleCollection();
				mUserInRoleCollection.AddRange(UserInRoleCollection.LoadByUserID(this.UserID));
			}
			return mUserInRoleCollection;
		}

		private RoleCollection mRoleCollection = null;
		/// <summary>
		/// Load on-demand and keep in local variable.
		/// </summary>
		public RoleCollection GetRoleCollection()
		{
			if (mRoleCollection == null)
			{
				mRoleCollection = new RoleCollection();
				mRoleCollection.AddRange(RoleCollection.LoadByUserID(this.UserID));
			}
			return mRoleCollection;
		}

		private Address mAddress = null;
		/// <summary>
		/// Load on-demand and keep in local variable
		/// </summary>
		public Address GetAddress()
		{
			if (mAddress == null)
			{
				mAddress = new Address();
				mAddress.LoadFromDB(this.AddressID);
			}
			return mAddress;
		}

		private NewsletterGroupCollection mNewsletterGroupCollection = null;
		/// <summary>
		/// Load on-demand and keep in local variable
		/// </summary>
		/// <returns></returns>
		public NewsletterGroupCollection GetNewsletterGroupCollection()
		{
			if (mNewsletterGroupCollection == null)
			{
				mNewsletterGroupCollection = new NewsletterGroupCollection();
				mNewsletterGroupCollection.AddRange(NewsletterGroupCollection.LoadByUserID(this.UserID));
			}
			return mNewsletterGroupCollection;
		}

		#endregion

		#region Database methods overrides
		// Database methods


		/// <summary>
		/// Delete this and it's UserInRoleItems too.
		/// </summary>
		/// <returns></returns>
		public override bool DeleteFromDB()
		{
			try
			{
				GetDataStore().UseTransaction(this.DeleteCallback);
				return true;
			}
			catch
			{
				throw new Exception("User account can't be deleted, products were already ordered!");
			}
		}

		public void DeleteCallback()
		{
			UserInNewsletterGroupCollection userInNewsletterGroupCollection = new UserInNewsletterGroupCollection();
			userInNewsletterGroupCollection.DeleteByUserID(this.UserID);

			UserInRoleCollection.DeleteByUserID(this.UserID);
			mRoleCollection = null;
			mUserInRoleCollection = null;

			// delete Address
			mAddress = null;
			VISTACART.Address address = new Address();
			address.AddressID = this.AddressID.GetValueOrDefault();

			List<LicenseInfo>licCollection = LicenseInfoCollection.LoadByUserID(this.UserID);

			foreach (LicenseInfo licInfo in licCollection)
			{
				licInfo.DeleteFromDB();
			}

			// delete User first, than Address
			base.DeleteFromDB();
			address.DeleteFromDB();
		}

		public override User SaveToDB(WriteModeEnum mode)
		{
			//////// save address first or create new address
			//////if (this.AddressID < 1)
			//////{
			//////    // create new address object
			//////    mAddress = new Address();
			//////}
			
			if (this.mAddress != null)
			{
				mAddress.Email = this.Email;
				mAddress.SaveToDB(mode);
				this.AddressID = mAddress.AddressID;
			}
			mAddress = null; // force Address reloading
			return base.SaveToDB(mode);
		}
		#endregion

		#region Public methods

		/// <summary>
		/// Add UserInRole and update database. Returns true if successful.
		/// </summary>
		/// <param name="role"></param>
		/// <returns></returns>
		public bool AddRole(Role role)
		{
			if (this.IsLoadedFromDB() && this.IsInRole(role) == false)
			{
				UserInRole userInRole = new UserInRole();
				userInRole.UserID = this.UserID;
				userInRole.RoleID = role.RoleID;
				userInRole.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					// force reloading from database
					mUserInRoleCollection = null;
					mRoleCollection = null;
					return true;
				}
			}
			return false;
		}

		/// <summary>
		/// Remove UserInRoleItem and updates database. returns true if successful.
		/// </summary>
		/// <param name="role"></param>
		/// <returns></returns>
		public bool RemoveRole(Role role)
		{
			if (this.IsLoadedFromDB())
			{
				foreach (UserInRole userInRole in this.GetUserInRoleCollection())
				{
					if (userInRole.RoleID == role.RoleID)
					{
						// delete from db
						if (userInRole.DeleteFromDB())
						{
							// set null to reload it all from database:
							mUserInRoleCollection = null;
							mRoleCollection = null;
							return true;
						}
					}
				}
			}
			return false;
		}

		/// <summary>
		/// Returns true if user is in role.
		/// </summary>
		/// <param name="role"></param>
		/// <returns></returns>
		public bool IsInRole(Role role)
		{
			foreach(UserInRole userInRole in this.GetUserInRoleCollection())
			{
				if(userInRole.RoleID == role.RoleID)
				{
					return true;
				}
			}
			return false;
		}

		/// <summary>
		/// Returns true if user is in role, else false.
		/// </summary>
		/// <param name="roleName"></param>
		/// <returns></returns>
		public bool IsInRole(string roleName)
		{
			foreach(Role role in this.GetRoleCollection())
			{
				if (role.Name == roleName)
				{
					return true;
				}
			}
			return false;
		}

		public bool IsInRole(Object roleName)
		{
			return IsInRole(roleName.ToString());
		}


		public bool IsInNewsletterGroup(int newsletterGroupID)
		{
			foreach (NewsletterGroup ng in this.GetNewsletterGroupCollection())
			{
				if (ng.NewsletterGroupID == newsletterGroupID)
				{
					return true;
				}
			}
			return false;
		}

		/// <summary>
		/// Add user to NewsletterGroup and update database. Returns true if user added, else false.
		/// </summary>
		/// <param name="role"></param>
		/// <returns></returns>
		public bool AddNewsletterGroup(NewsletterGroup newsletterGroup)
		{
			if (this.IsLoadedFromDB() && this.IsInNewsletterGroup(newsletterGroup.NewsletterGroupID) == false)
			{
				UserInNewsletterGroup userInNg = new UserInNewsletterGroup();
				userInNg.UserID = this.UserID;
				userInNg.NewsletterGroupID = newsletterGroup.NewsletterGroupID;
				userInNg.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					mNewsletterGroupCollection = null;
					return true;
				}
			}
			return false;
		}

		/// <summary>
		/// Remove user from NewsletterGroup and updates database. returns true if removed, else false.
		/// </summary>
		/// <param name="role"></param>
		/// <returns></returns>
		public bool RemoveNewsletterGroup(Role role)
		{
			if (this.IsLoadedFromDB())
			{
				UserInNewsletterGroupCollection uingcoll = new UserInNewsletterGroupCollection();
				int affected = uingcoll.DeleteByUserID(this.UserID);
				mNewsletterGroupCollection = null;
				return affected > 0;
			}
			return false;
		}

		#endregion

		#region Database methods

		public static User LoadByUsername(string username)
		{
			return new Query<User>().Where(o => o.Username == username).FirstOrDefault();
		}

		public static User LoadByEmail(string email)
		{
			return new Query<User>().Where(o => o.Email == email).FirstOrDefault();
		}

		#endregion

	}

}
