﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using Ice.Net.Ecommerce.Security.Cryptography;
using System.Collections.Generic;
using Ice.Serialization.Json.Core.Serializer;

namespace Ice.Net.Ecommerce.CacheProvider
{
	internal class CookieStore
	{
		//////////////////////////////////////////////////////////////
		static bool mUseEncryption = true;
		//////////////////////////////////////////////////////////////
		static DESCryptoHelper mDesCrypto = new DESCryptoHelper(Configuration.EncryptionSection.CookieEncryptionKey);

		/// <summary>
		/// Remove all cookies - set expiration in the past time.
		/// </summary>
		public static void RemoveAll()
		{
			// response
			HttpContext.Current.Response.Cookies.Clear();
			foreach (HttpCookie cookie in HttpContext.Current.Request.Cookies)
			{
				cookie.Value = "";
				cookie.Expires = DateTime.UtcNow.AddYears(-10);
				HttpContext.Current.Response.Cookies.Set(cookie);
			}

			//request
			HttpContext.Current.Request.Cookies.Clear();
		}

		private static DESCryptoHelper GetEncoder()
		{
			return mDesCrypto;
		}

		/// <summary>
		/// <para>Save any object to cookie. Cookie expires with session.</para>
		/// <para>Serialize obj to JSON, encrypt than save object to cookie</para>
		/// <para>Returns true if successful</para>
		/// </summary>
		/// <param name="obj"></param>
		public static bool Save(Object obj)
		{
			return Save(obj, DateTime.MaxValue, obj.GetType().Name, false);
		}

		/// <summary>
		/// <para>Save any object to persistent cookie</para>
		/// <para>Serialize obj to JSON, encrypt than save object to cookie</para>
		/// <para>Returns true if successful</para>
		/// </summary>
		/// <param name="obj"></param>
		/// <param name="expiration"></param>
		public static bool Save(Object obj, DateTime expiration)
		{
			return Save(obj, expiration, obj.GetType().Name, true);
		}

		public static bool Save(Object obj, DateTime expiration, string cookieName, bool persistentCookie)
		{
			HttpCookie cookie = new HttpCookie(cookieName);
			cookie.Value = Ice.Serialization.Json.IceJson.Serialize(obj);

			cookie.Value = HttpUtility.UrlEncode(cookie.Value);

			if (mUseEncryption)
			{
				DESCryptoHelper cc = GetEncoder();
				cookie.Value = Convert.ToBase64String(cc.Encrypt(cookie.Value));
			}

			if (persistentCookie)
			{
				cookie.Expires = expiration; // expiration not set, cookie expires with session
			}
			HttpContext.Current.Response.Cookies.Remove(cookie.Name);
			HttpContext.Current.Request.Cookies.Remove(cookie.Name);

			HttpContext.Current.Response.Cookies.Set(cookie);
			HttpContext.Current.Request.Cookies.Set(cookie);

			return true;
		}

		/// <summary>
		/// <para>Type.Name is used as cookie name.</para>
		/// </summary>
		/// <param name="type"></param>
		/// <returns></returns>
		public static bool Delete(Type type)
		{
			return Delete(type.Name);
		}

		/// <summary>
		/// Delete cookie with object
		/// </summary>
		public static bool Delete(string cookieName)
		{
			HttpCookie cookie = new HttpCookie(cookieName, "");
			cookie.Expires = DateTime.UtcNow.AddYears(-10);
			HttpContext.Current.Response.Cookies.Set(cookie);
			HttpContext.Current.Request.Cookies.Remove(cookie.Name);
			return true;
		}


		/// <summary>
		/// <para>Retrieve object from cookie.</para>
		/// <para>Type.Name is used as cookie name.</para>
		/// <para>Retruns null if not found</para>
		/// </summary>
		/// <returns></returns>
		public static T Retrieve<T>()
		{
			return RetrieveByName<T>(typeof(T).Name);
		}

		/////// <summary>
		/////// <para>Retrieve collection from cookie.</para>
		/////// <para>C: collection type, T: element type</para>
		/////// <para>Type.Name is used as cookie name.</para>
		/////// <para>Retruns null if not found</para>
		/////// </summary>
		/////// <returns></returns>
		////public static C Retrieve<C, T>() where C: List<T>
		////{
		////	List<Object> list = RetrieveByName<List<Object>>(typeof(C).Name);
		////	C collection = System.Activator.CreateInstance<C>();
		////	if (list != null)
		////	{
		////		JavaScriptSerializer ser = new JavaScriptSerializer();
		////		foreach (Object obj in list)
		////		{
		////			T element = ser.ConvertToType<T>(obj);
		////			collection.Add(element);
		////		}
		////	}
		////	return collection;
		////}

		/// <summary>
		/// <para>Retrieve object from cookie</para>
		/// <para>Retruns null if not found</para>
		/// </summary>
		/// <returns></returns>
		public static T RetrieveByName<T>(string cookieName)
		{
			HttpCookie cookie = HttpContext.Current.Request.Cookies.Get(cookieName);
			if (cookie != null)
			{
				string val = cookie.Value;

				if (mUseEncryption)
				{
					DESCryptoHelper cc = GetEncoder();

					try
					{
						val = cc.DecryptToString(Convert.FromBase64String(val));
					}
					catch
					{
					}
				}

				val = HttpUtility.UrlDecode(val); // val may be null

				try
				{
					T obj = Ice.Serialization.Json.IceJson.Deserialize<T>(val);
					return obj;
				}
				catch
				{
				}
			}

			return default(T);

		}
	}
}
