﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Threading;

namespace Ice.Net.Ecommerce.EmailUtility
{
	internal enum WorkerThreadMethodResult
	{
		/// <summary>
		/// Will sleep thread for given amount of time (50ms).
		/// </summary>
		NothingToDo,
		/// <summary>
		/// Will exit MainThreadLoop method and thread will stop.
		/// </summary>
		StopThread,
		/// <summary>
		/// Task successful and will continue to call WorkerMethod immediately.
		/// </summary>
		Successful
	}

	internal delegate WorkerThreadMethodResult WorkerMethodDelegate();

	internal class WorkerThread
	{
		Thread mThread = null;
		bool mStopRequested = false;
		WorkerMethodDelegate mWorkerMethod;

		public WorkerThread(WorkerMethodDelegate workerMethod)
		{
			mWorkerMethod = workerMethod;
		}

		/// <summary>
		/// Start thread only if IsAlive == false
		/// </summary>
		public void Start()
		{
			if ((mThread != null && mThread.IsAlive == false))
			{
				mThread.Abort();
				mThread = null;
			}
			if (mThread == null)
			{
				mThread = new Thread(new ThreadStart(this.MainThreadLoop));
				mStopRequested = false;
				mThread.Start();
			}
		}

		/// <summary>
		/// Abort thread than start it
		/// </summary>
		public void ReStart()
		{
			if (mThread != null)
			{
				mThread.Abort();
			}
			mThread = new Thread(new ThreadStart(this.MainThreadLoop));
			mStopRequested = false;
			mThread.Start();
		}

		public void Abort()
		{
			if (mThread != null)
			{
				mThread.Abort();
				mThread = null;
			}
		}
		public void Stop()
		{
			mStopRequested = true;
		}

		/// <summary>
		/// Request thread to stop and wait x time. If not stopped in x time, abort the tread.
		/// </summary>
		public void StopSync(int waitMiliSeconds)
		{
			this.Stop();
			int circles = waitMiliSeconds / 100;
			while(true)
			{
				Thread.Sleep(100);
				if (this.IsThreadAlive == false)
				{
					return;
				}
				if ((--circles) < 0) // thread didn't stop after given number of miliseconds
				{
					break;
				}
			}
			// abort thread if
			this.Abort();
		}

		public bool IsThreadAlive
		{
			get
			{
				if (mThread != null)
				{
					return mThread.IsAlive;
				}
				else
				{
					return false;
				}
			}
		}

		private void MainThreadLoop()
		{
			while (true)
			{
				if (mStopRequested)
				{
					// exit the while loop and stop thread
					break;
				}

#if DEBUG
				WorkerThreadMethodResult result = mWorkerMethod.Invoke();
#else // release
				WorkerThreadMethodResult result = WorkerThreadMethodResult.NothingToDo;
				try
				{
					result = mWorkerMethod.Invoke();
				}
				catch (Exception ex)
				{
					string errMsg = this.mWorkerMethod.Method.ReflectedType.Name + "." + this.mWorkerMethod.Method.Name + "(): " + ex.Message;
					//GLogWritter.WriteErrorLog("UNHANDLED EXCEPTION: " + errMsg);
				}
#endif

				if (result == WorkerThreadMethodResult.Successful)
				{
					continue;
				}
				else if (result == WorkerThreadMethodResult.StopThread)
				{
					// exit the while loop and stop thread
					break;
				}
				else if (result == WorkerThreadMethodResult.NothingToDo)
				{
					Thread.Sleep(50);
				}
			}
		}




	}
}