﻿using System;
using System.Net;
using System.Net.Mail;
using System.Net.Mime;
using System.Threading;
using System.ComponentModel;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;

namespace Ice.Net.Ecommerce.EmailUtility
{
	internal class WorkerThreadsController 
	{
		List<WorkerThread> mWorkerThreadsCollection = new List<WorkerThread>();

		public WorkerThreadsController()
		{
		}

		public WorkerThreadsController(int numberOfWorkerThreads, WorkerMethodDelegate workerMethod)
		{
			// create threads
			while (mWorkerThreadsCollection.Count < numberOfWorkerThreads)
			{
				mWorkerThreadsCollection.Add(new WorkerThread(workerMethod));
			}
		}
		
		/// <summary>
		/// Add
		/// </summary>
		/// <param name="workThread"></param>
		public void AddWorkerThread(WorkerThread workThread)
		{
			this.mWorkerThreadsCollection.Add(workThread);
		}

		/// <summary>
		/// Start only dead threads
		/// </summary>
		public void StartThreads()
		{
			foreach (WorkerThread wthread in mWorkerThreadsCollection)
			{
				wthread.Start();
			}
		}

		/// <summary>
		/// Start threads.
		/// </summary>
		public void ReStartThreads()
		{
			foreach (WorkerThread wthread in mWorkerThreadsCollection)
			{
				wthread.ReStart();
			}
		}

		/// <summary>
		/// Abort threads immediately.
		/// </summary>
		public void AbortThreads()
		{
			foreach (WorkerThread wthread in mWorkerThreadsCollection)
			{
				wthread.Abort();
			}
		}

		/// <summary>
		/// Send stop to threads, but let them to finish current task.
		/// </summary>
		public void StopThreads()
		{
			foreach (WorkerThread wthread in mWorkerThreadsCollection)
			{
				wthread.Stop();
			}
		}

		/// <summary>
		/// Send Stop command to all threads and wait until threads are stopped.
		/// If threads are not stoped in waitMiliSeconds, Abort them.
		/// </summary>
		public void StopThreadsSync(int waitMiliSeconds)
		{
			int circles = waitMiliSeconds / 100;
			StopThreads();
			while (AliveThreadsCount > 0)
			{
				Thread.Sleep(100);
				if ((--circles) < 0)
				{
					break;
				}
			}
			// abort them if not stopped
			AbortThreads();
		}

		/// <summary>
		/// Returns number of alive threads.
		/// </summary>
		public int AliveThreadsCount
		{
			get
			{
				int count = 0;
				foreach (WorkerThread wthr in this.mWorkerThreadsCollection)
				{
					if (wthr.IsThreadAlive)
					{
						count++;
					}
				}
				return count;
			}
		}
	}

}