﻿using System;
using System.Collections.Generic;
using System.Text;
using Ice.Orm.Core.Entity;
using Ice.Orm.Core.Criteria;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using Ice.Net.Ecommerce.BusinessObjects.Common;

namespace Ice.Net.Ecommerce.OrderContentForEmail
{
	internal class OrderBuilder
	{
		StringBuilder mSb = new StringBuilder();

		public static string GetHtml(Order order)
		{
			OrderBuilder ob = new OrderBuilder();

			ob.mSb.Append(@"
<table cellpadding='5px' cellspacing='2px' style='width:100%'>
");
			ob.CreateHeader();
			ob.CreateItemsList(order);
			ob.CreateShipping(ob.GetPrice(order.RedundantShippingPrice), order.GetShippingType().Name);

			string taxValue = BusinessObjects.Common.CurrencyUtility.GetFormatedPriceInCurrency(order.RedundantTotalTaxPrice);

			if (!VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ShowPriceWithTax.GetValueOrDefault())
			{
				ob.CreateTax(taxValue);
			}

			ob.CreateTotals(ob.GetPrice(order.RedundantTotalToPay, false)); // tax is already included

			if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ShowPriceWithTax.GetValueOrDefault())
			{
				ob.CreateTax(taxValue);
			}

			ob.mSb.Append(@"
</table>");

			return ob.mSb.ToString();
		}

		private string GetPrice(decimal price)
		{
			return GetPrice(price, true);
		}

		private string GetPrice(decimal price, bool addTax)
		{
			if (addTax)
			{
				price = IncludeTax(price);
			}
			return BusinessObjects.Common.CurrencyUtility.GetFormatedPriceInCurrency(price);
		}

		private decimal IncludeTax(decimal price)
		{
			if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ShowPriceWithTax.GetValueOrDefault())
			{
				return price * (1 + BusinessObjects.Common.VISTACARTHelper.AppDefaults.Tax.RatePercent / 100);
			}
			else
			{
				return price;
			}
		}

		private void CreateHeader()
		{
			mSb.Append(@"
	<tr style='font-weight:bold; font-size:12px; background:#E8E8EB; height:25px;'>
		<td>ID</td>
		<td>Name</td>
		<td>Attributes</td>
		<td>Price</td>
		<td>Quantity</td>
		<td>Sum</td>
	</tr>
");
		}

		private void CreateShipping(string priceInCurrency, string shippingName)
		{
			mSb.Append(@"
<tr><td colspan='6' style='height:10px; background-color:#E8E8EB;'></td></tr>");

			mSb.Append(String.Format(@"
<tr style='font-size:12px'>
	<td colspan='1' align='left'></td>
	<td colspan='4' align='left'><b>shipping</b> ({1})</td>
	<td colspan='1' align='left'>{0}</td>
</tr>", priceInCurrency, shippingName));

		}

		private void CreateTax(string priceInCurrency)
		{
			////            mSb.Append(@"
			////<tr><td colspan='6' style='height:5px; background-color:#E8E8EB;'></td></tr>");

			mSb.Append(String.Format(@"
<tr style='font-size:12px'>
	<td colspan='1' align='left'></td>
	<td colspan='4' align='left'><b>tax</b></td>
	<td colspan='1' align='left'>{0}</td>
</tr>", priceInCurrency));

		}

		private void CreateTotals(string priceInCurrency)
		{
			mSb.Append(@"
<tr><td colspan='6' style='height:10px; background-color:#E8E8EB; '></td></tr>");

			mSb.Append(String.Format(@"
<tr style='font-weight:bold; font-size:16px;'>
	<td colspan='1' align='left'></td>
	<td colspan='4' align='left'>total</td>
	<td colspan='1' align='left'>{0}</td>
</tr>", priceInCurrency));

		}

		private void CreateItemsList(Order order)
		{
			foreach (OrderDetail detail in order.GetOrderDetailCollection())
			{
				this.mSb.Append(String.Format(@"
<tr>
	<td>{0}</td>
	<td>{1}</td>
	<td>{5}</td>
	<td style='white-space:nowrap'>{2}</td>
	<td style='white-space:nowrap'>{3}</td>
	<td style='white-space:nowrap'>{4}</td>
</tr>",
				detail.ProductID,
				GetLink(detail.GetProduct()),
				GetPrice(detail.RedundantSalePrice),
				detail.Quantity,
				GetPrice(detail.RedundantSalePrice * detail.Quantity),
				detail.Attributes.Replace(", ", "<br/>")));
			}
		}

		private string GetLink(Product product)
		{
			FrontParameters fp = new FrontParameters();
			fp.ProductID = product.ProductID;

			return String.Format("<a href='{0}' target='_blank'>{1}</a>", fp.AbsoluteLink, GetName(product));
		}

		private string GetName(Product product)
		{
			return System.Web.HttpUtility.HtmlEncode(product.Name.Replace(">", "").Replace("<", ""));
		}
	}
}
