﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using Ice.Net.Ecommerce.Web;

namespace Ice.Net.Ecommerce.PaymentGateways.PG
{
	internal class PGAuthorizeNetAIM : BasePG
	{
		public override bool AIMMethod
		{
			get { return true; }
		}

		public override PGNameEnum PGNameEnum
		{
			get { return PGNameEnum._AuthorizeNetAIM; }
		}

		public override PGResult GetFormWithParameters(Order order)
		{
			return GetFormWithParametersAIM();
		}

		public override PGStatus GetReceivedStatus(HttpContext context)
		{
			return null;
		}

		#region Control Panel HTML

		public override string GetControlPanelHtml()
		{
			return string.Format(@"
<!--AUTHORIZE.NET CONTROL PANEL-->
<table cellpadding='0px' cellspacing='5px' >
<tr><td colspan='2' align='center'><b style='color:Red'>* SSL IS REQUIRED *</b></td></tr>
<tr><td >Name</td><td><input type='text' name='Name' style='width:255px;'/></td></tr>
<tr><td >Active</td><td><input type='checkbox' name='IsActive' /></td></tr>
<tr><td >Demo</td><td><input type='checkbox' name='IsDemo' /></td></tr>
<tr><td >Diagnostic</td><td><input type='checkbox' name='IsDiagnostic' /></td></tr>
<tr><td >API Login ID</td><td><input name='MerchantId' type='text' style='width:256px;' /></td></tr>
<tr><td >Transaction Key</td><td><input name='MerchantKey' type='text' style='width:255px;' /></td></tr>
<tr><td >Cart Description</td><td><input name='CartDescription' type='text' style='width:255px;' /></td></tr>
<tr><td >Description<br/></td><td><textarea id='authorizeNetDescription' name='Description' rows='200' cols='2000' style='height:50px;width:255px;'></textarea></td></tr>
<tr><td >Long Description<br/></td><td><textarea id='authorizeNetLongDescription' name='LongDescription' rows='200' cols='2000' style='height:50px;width:255px;'></textarea></td></tr>

<tr><td colspan='2' style='text-align:center;height:60px;background-image:url({0}admin/pglogo/authorizenet.gif); background-repeat:no-repeat; background-position:center center'>&nbsp;</td></tr>
<tr><td colspan='2' style='text-align:center; height:20px'><a href='http://www.Authorize.Net' target='_blank'><b> http://www.Authorize.Net </b></a></td></tr>
<tr><td></td><td><div style='display:none'>
<!--quick help content-->
<b>IMPORTANT: Authorize.net accepts USD only! Default shop currency should be USD, otherwise price is converted into USD using shop currency rate before sending to Authorize.net. Shop currency exchange rates has to be updated regularly.</b><br /><br />
</div><div align='right'><a href='#' onclick='QoDesk.PaymentTypeEditor.ShowQuickHelp(this);'><b>Quick Help</b></a></div>
</td></tr>
</table>

",
 mSiteRoot);

		}

		#endregion

		#region Response parser class

#if !TRIAL

		private class AuthorizeNetAimResponse
		{
			public bool CCAuthorized = false;
			public string Description = "";
			public string TransactionId = "";
			public string Amount = "";

			public AuthorizeNetAimResponse(string responseString)
			{
				// here is one example or returned string:
				// "3^1^6^(TESTMODE) The credit card number is invalid.^000000^P^0^1^cart d^75.62^CC^auth_capture^^rrdsa^has^^fads g^jadsgf^NB^12345^US^^^^^^^^^^^^^^^^^D95D0979A32724E624993D5CE8B7757B^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"
				
				string[] respArr = responseString.Split(new char[] { '^' }); // don't remove empty entries!!! StringSplitOptions.RemoveEmptyEntries);

				this.CCAuthorized = respArr[0] == "1";
				this.Description = respArr[3];
				this.TransactionId = respArr[6];
				this.Amount = respArr[9];
			}
		}
#endif

		#endregion

		// USER DEPENDING PARAMTERS
		// The transaction results will be posted to the screen and will list all name/value pairs your script is submitting. You can then verify that all field names and values in your transaction request are correct.
		// dump https://developer.authorize.net/param_dump.asp
		private static string mTransactionUrl = "https://secure.authorize.net/gateway/transact.dll"; // LIVE ACCOUNT
		private static string mDemoTransactionUrl = "https://test.authorize.net/gateway/transact.dll"; // test account
		private static string mSiteRoot = BusinessObjects.Common.ApplicationUtility.ApplicationHttpRoot;

		private string GetTransactionURL()
		{
			if (base.PaymentType.IsDemo)
			{
				return mDemoTransactionUrl;
			}
			else
			{
				return mTransactionUrl;
			}
		}

		private static Currency mUSDCurrency = null;
		/// <summary>
		/// If price is not in USD, convert it into USD
		/// </summary>
		/// <param name="amount"></param>
		/// <returns></returns>
		private decimal ConvertToUSDCurrency(decimal amount)
		{
			if (mUSDCurrency == null && BusinessObjects.Common.VISTACARTHelper.AppDefaults.Currency.ISOCode != "USD")
			{
				if ((mUSDCurrency = Currency.LoadByISOCode("USD")) != null && mUSDCurrency.IsActive)
				{
					// ok, convert prices from shop default currency into USD because Authorize.net accepts value in USD only
				}
				else
				{
					mUSDCurrency = null;
					throw new Exception("Can't load USD currency, conversion is not possible!");
				}
			}

			if (mUSDCurrency != null)
			{
				// convert from shop default currency into USD
				amount = BusinessObjects.Common.CurrencyUtility.ConvertPrice(amount, mUSDCurrency, BusinessObjects.Common.VISTACARTHelper.AppDefaults.Currency);
			}
			
			return amount;
		}

		// separate dollars from cents and place dot between them
		// PayPal requires format: dollars.cents, some languages places "," between !!!!
		private string GetAmountToPay(decimal amount)
		{
			amount = ConvertToUSDCurrency(amount);

			string strtotal = amount.ToString("n2");
			string dollars = strtotal.Substring(0, strtotal.Length - 3);
			string cents = strtotal.Substring(strtotal.Length - 2);
			dollars = dollars.Replace(",", ""); // remove , if any left
			dollars = dollars.Replace(".", ""); // remove . if any
			strtotal = dollars + "." + cents;
			return strtotal;
		}

		// AIM METHOD (SSL REQUIRED)
		private PGResult GetFormWithParametersAIM()
		{
			string form = String.Format(@"
<form method='get' action='{0}'>
<input type='hidden' name='p' value='{1}'/>
</form>",
		BusinessObjects.Common.ApplicationUtility.ApplicationHttpRoot,
		Category.Shortcut.checkoutpayment + "");
		
////            string js = String.Format(@"
////var form = ""{0}"";
////var div = document.getElementById('divPGDataContainer');
////div.innerHTML = """";
////div.innerHTML = form;
////", CreditCardInfo.GetFormHtml((int)PaymentType.NameEnum.AuthorizeNetAIM));
			return new PGResult(null, form, null);
		}

		public override PGResult ChargeAIMMethodWorker(CreditCardInfo ccInfo, Order order)
		{
#if !TRIAL
			// build post parameters
			string param = BuildPostParameters(ccInfo, order);
			GHttpResponseStatus resp = GHttpRequest.MakePostRequest(GetTransactionURL(), param);

			if (base.PaymentType.IsDiagnostic)
			{
				UtilityHelper.SaveDiagnostic(PGNameEnum._AuthorizeNetAIM.ToString(), "OrderID: " + order.OrderID + ", ResponseText: " + resp.ResponseText);
			}

			if (resp.StatusCode == System.Net.HttpStatusCode.OK)
			{
				// if CC vas charged, show Checkout4.ascx:
				AuthorizeNetAimResponse aimResp = new AuthorizeNetAimResponse(resp.ResponseText);
				if (aimResp.CCAuthorized)
				{
					// cretate form to redirect user to
					return new PGResult(aimResp.Description, null, null, true, aimResp.TransactionId);
				}
				else
				{
					return new PGResult("FAILED! " + aimResp.Description, null, null);
				}
			}
			else
			{
				return new PGResult("FAILED! " + resp.ResponseText, null, null);
			}
#else
			throw new Exception("*********************************\r\nDISABLED IN TRIAL VERSION!\r\n*********************************");
#endif
		}

#if !TRIAL
		private string BuildPostParameters(CreditCardInfo ccInfo, Order order)
		{
			NameValuePairCollection nv = new NameValuePairCollection();

			// Currency setting. Check the guide for other supported currencies
			nv.Add("x_currency_code", "USD");// Authorize.net accepts USD only!!!

			// Merchant Informations
			nv.Add("x_login", base.PaymentType.MerchantId);
			nv.Add("x_tran_key", base.PaymentType.MerchantKey);
			////nv.Add("x_password", this.StringOne); // not used

			// Transaction Information
			nv.Add("x_version", "3.1");
			nv.Add("x_type", "AUTH_CAPTURE");
			nv.Add("x_method", "CC");
			nv.Add("x_amount", GetAmountToPay(order.RedundantTotalToPay));
			nv.Add("x_card_num", ccInfo.CCNumber);
			nv.Add("x_exp_date", ccInfo.ExpMonth + ccInfo.ExpYear); // MMYY, MM/YY, MM-YY, MMYYYY, MM/YYYY, MM-YYYY 
			nv.Add("x_card_code", ccInfo.CVVCode); // Card Code Verification (CCV) security feature. Must be a valid CVV2, CVC2 or CID value.
			nv.Add("x_test_request", base.PaymentType.IsDemo ? "TRUE" : "FALSE"); // Switch this to False once you go live

			// Order Information
			nv.Add("x_invoice_num", order.OrderID + ""); // The merchant assigned invoice number for the transaction, up to 20 chars
			nv.Add("x_description", base.PaymentType.CartDescription);

			// Itemized Order Information

			// Customer Information (Billing Informations)
			Address billingAddress = new Address();
			if (billingAddress.LoadFromDB(order.BillingAddressID))
			{
				nv.Add("x_first_name", billingAddress.FirstName);
				nv.Add("x_last_name", billingAddress.LastName);
				nv.Add("x_address", billingAddress.Address1 + " " + billingAddress.Address2);
				nv.Add("x_city", billingAddress.City);
				nv.Add("x_state", billingAddress.GetState().Symbol);
				nv.Add("x_zip", billingAddress.Zip);
				nv.Add("x_country", billingAddress.GetCountry().SymbolShort); // 2 letters country code required
			}

			// Other
			nv.Add("x_delim_data", "TRUE");
			nv.Add("x_delim_char", "^");
			////nv.Add("x_encap_char", "");
			nv.Add("x_relay_response", "FALSE");

			return nv.ToFormGETString();
		}
#endif

		#region SIM Method specific

		#endregion

	}
}



