﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;

namespace Ice.Net.Ecommerce.PaymentGateways.PG
{
#if !TRIAL

	internal class PGRBA_CertHelper
	{
		private SHA1CryptoServiceProvider sha1 = new SHA1CryptoServiceProvider();
		private RSACryptoServiceProvider mRsaCryptoMerchant = null;
		private RSACryptoServiceProvider mRsaCryptoRBA = null;

		private string mRBACertFile; // file path: D:\\temp4\\CERTIFIKATI_RADNI\\rbaCert.crt
		private string mMerchantCertFile; // file path: D:\\temp4\\CERTIFIKATI_RADNI\\merchantCert.p12
		private string mMerchantPassword; // password for .p12 certificate to read private key

		public PGRBA_CertHelper(string RBACertFile, string merchantCertFile, string merchantPassword)
		{
			this.mRBACertFile = RBACertFile;
			this.mMerchantCertFile = merchantCertFile;
			this.mMerchantPassword = merchantPassword;

			mRsaCryptoMerchant = this.LoadMerchantPrivateServiceProvider();
			mRsaCryptoRBA = this.LoadRBAPublicServiceProvider();
		}

		private RSACryptoServiceProvider LoadMerchantPrivateServiceProvider()
		{
			try
			{
				X509Certificate2 cc = new X509Certificate2(this.mMerchantCertFile, this.mMerchantPassword, X509KeyStorageFlags.Exportable);
				RSACryptoServiceProvider rsaPrivate = (RSACryptoServiceProvider)cc.PrivateKey; // << merchant's private key
				return rsaPrivate;
			}
			catch
			{
				throw new Exception("Can't load certificates!");
				return null;
			}
		}

		private RSACryptoServiceProvider LoadRBAPublicServiceProvider()
		{
			try
			{
				X509Certificate2 cc = new X509Certificate2(this.mRBACertFile);
				RSACryptoServiceProvider rsaPublic = (RSACryptoServiceProvider)cc.PublicKey.Key; // << RBA javni kljuc
				return rsaPublic;
			}
			catch
			{
				throw new Exception("Can't load certificates!");
				return null;
			}
		}

		/// <summary>
		/// Returns signature.
		/// </summary>
		/// <param name="ourDataString">string to be signed</param>
		/// <returns></returns>
		public string CreateSignature(string ourDataString)
		{
			try
			{
				// data: string to bin 
				byte[] binData = System.Text.Encoding.ASCII.GetBytes(ourDataString);

				// signature: compute, bin to string(Base64)
				byte[] binSignature = mRsaCryptoMerchant.SignData(binData, this.sha1);
				return Convert.ToBase64String(binSignature);
			}
			catch
			{
			}
			// if error, return error
			return "error";
		}

		/// <summary>
		/// Verify signature.
		/// </summary>
		/// <param name="receivedDataString">string with data</param>
		/// <param name="receivedSignature">base64 encoded signature</param>
		/// <returns></returns>
		public bool IsSignatureValid(string receivedDataString, string receivedSignature)
		{
			try
			{
				// data: string to binary
				byte[] data = System.Text.Encoding.ASCII.GetBytes(receivedDataString);

				// signature: string to bin (Base64)
				byte[] signature = Convert.FromBase64String(receivedSignature);

				// verify signature
				return mRsaCryptoRBA.VerifyData(data, this.sha1, signature);
			}
			catch
			{

			}
			// if error, return false
			return false;
		}

		public string BuildOurDataString(string merchantID, string terminalID, string purchaseTime, string orderID, string currency, string totalAmount, string sd)
		{
			return ( merchantID + ";" + terminalID + ";" + purchaseTime + ";" + orderID + ";" + currency + ";" + totalAmount + ";" + sd + ";" );
		}

		public string BuildReceivedDataString(string merchantID, string terminalID, string purchaseTime, string orderID, string xid, string currency, string totalAmount, string sd, string tranCode, string approvalCode)
		{
			return (merchantID + ";" + terminalID + ";" + purchaseTime + ";" + orderID + ";" + xid + ";" + currency + ";" + totalAmount + ";" + sd + ";" + tranCode + ";" + approvalCode + ";");
		}
	}
#endif
}
