﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ice.Net.Ecommerce.Security.Cryptography
{
	internal class RSAParameters2
	{
		public byte[] D;
		public byte[] DP;
		public byte[] DQ;
		public byte[] Exponent;
		public byte[] InverseQ;
		public byte[] Modulus;
		public byte[] P;
		public byte[] Q;

		#region private methods

		private static byte[] GetValue(BigInteger bi)
		{
			if (Object.ReferenceEquals(bi, null) == false)
			{
				return bi.getBytes();
			}
			else
			{
				return null;
			}
		}

		////private static void Clone(Object source, Object dest)
		////{
		////    System.Reflection.FieldInfo[] fields = source.GetType().GetFields(System.Reflection.BindingFlags.Public | System.Reflection.BindingFlags.Instance);
		////    foreach (System.Reflection.FieldInfo fi in fields)
		////    {
		////        System.Reflection.FieldInfo fiDest = dest.GetType().GetField(fi.Name, System.Reflection.BindingFlags.Public | System.Reflection.BindingFlags.Instance);
		////        if (fiDest != null)
		////        {
		////            fiDest.SetValue(dest, fi.GetValue(source));
		////        }
		////    }
		////}

		#endregion

		#region public methods

		public void FromRSASecretKey(RSASecretKey skey)
		{
			this.D = GetValue(skey.d);
			this.Exponent = GetValue(skey.e);
			this.Modulus = GetValue(skey.n);
			this.Q = GetValue(skey.q);
			this.P = GetValue(skey.p);
		}

		public RSASecretKey ToRSASecretKey()
		{
			return new RSASecretKey(this);
		}

		public void FromRSAParameters(System.Security.Cryptography.RSAParameters rsaParameters)
		{
			this.D = rsaParameters.D;
			this.DP = rsaParameters.DP;
			this.DQ = rsaParameters.DQ;
			this.Exponent = rsaParameters.Exponent;
			this.InverseQ = rsaParameters.InverseQ;
			this.Modulus = rsaParameters.Modulus;
			this.P = rsaParameters.P;
			this.Q = rsaParameters.Q;
		}

		public System.Security.Cryptography.RSAParameters ToRSAParameters()
		{
			System.Security.Cryptography.RSAParameters rsa = new System.Security.Cryptography.RSAParameters();
			rsa.D = this.D;
			rsa.DP = this.DP;
			rsa.DQ = this.DQ;
			rsa.Exponent = this.Exponent;
			rsa.InverseQ = this.InverseQ;
			rsa.Modulus = this.Modulus;
			rsa.P = this.P;
			rsa.Q = this.Q;
			return rsa;
		}

		////public string ToXmlString()
		////{
		////    System.Text.StringBuilder sb = new StringBuilder();
		////    sb.Append("<RSAKeyValue>");
		////    System.Reflection.FieldInfo[] fields = this.GetType().GetFields(System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.Public);
		////    foreach (System.Reflection.FieldInfo fi in fields)
		////    {
		////        byte[] data = (byte[])fi.GetValue(this);
		////        if (data != null)
		////        {
		////            sb.Append("<" + fi.Name + ">");
		////            sb.Append(Convert.ToBase64String(data));
		////            sb.Append("</" + fi.Name + ">");
		////        }
		////    }
		////    sb.Append("</RSAKeyValue>");
		////    return sb.ToString();
		////}

		private void AddXMLNode(StringBuilder sb, byte[] data, string nodeName)
		{
			if(data != null)
			{
				sb.Append("<" + nodeName + ">" + Convert.ToBase64String(data) + "</" + nodeName + ">");
			}
		}

		public string ToXmlString()
		{
			System.Text.StringBuilder sb = new StringBuilder();
			sb.Append("<RSAKeyValue>");

			AddXMLNode(sb, this.D, "D");
			AddXMLNode(sb, this.DP, "DP");
			AddXMLNode(sb, this.DQ, "DQ");
			AddXMLNode(sb, this.Exponent, "Exponent");
			AddXMLNode(sb, this.InverseQ, "InverseQ");
			AddXMLNode(sb, this.Modulus, "Modulus");
			AddXMLNode(sb, this.P, "P");
			AddXMLNode(sb, this.Q, "Q");

			sb.Append("</RSAKeyValue>");
			return sb.ToString();
		}

		////public void FromXmlString(string xmlString)
		////{
		////    System.Xml.XmlDocument doc = new System.Xml.XmlDocument();
		////    doc.LoadXml(xmlString);

		////    System.Reflection.FieldInfo[] fields = this.GetType().GetFields(System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.Public);
		////    foreach (System.Reflection.FieldInfo fi in fields)
		////    {
		////        System.Xml.XmlNode node = doc.SelectSingleNode("//RSAKeyValue/" + fi.Name);
		////        if (node != null)
		////        {
		////            try
		////            {
		////                fi.SetValue(this, Convert.FromBase64String(node.InnerText));
		////            }
		////            catch { }
		////        }
		////    }

		private byte[] GetData(System.Xml.XmlDocument doc, string nodeName)
		{
			System.Xml.XmlNode node = doc.SelectSingleNode("//RSAKeyValue/" + nodeName);
			if (node != null)
			{
				try
				{
					return Convert.FromBase64String(node.InnerText);
				}
				catch
				{
				}
			}
			return null;
		}

		public void FromXmlString(string xmlString)
		{
			System.Xml.XmlDocument doc = new System.Xml.XmlDocument();
			doc.LoadXml(xmlString);

			this.D = GetData(doc, "D");
			this.DP = GetData(doc, "DP");
			this.DQ = GetData(doc, "DQ");
			this.Exponent = GetData(doc, "Exponent");
			this.InverseQ = GetData(doc, "InverseQ");
			this.Modulus = GetData(doc, "Modulus");
			this.P = GetData(doc, "P");
			this.Q = GetData(doc, "Q");
		}


		#endregion

	}
}
