﻿using System;
using System.Collections;
using System.Data;
using System.Web;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Security.Permissions;
using System.Web.UI;
using System.Web.Compilation;
using Ice.Net.Ecommerce.Web;
using Ice.Web;

namespace Ice.Net.Ecommerce.Web.HttpHandler
{
	public class GPageHandler : IHttpHandler, System.Web.SessionState.IRequiresSessionState
	{
		static int mCount = 0;

		string mVirtualPath;
		Modules.Common.RequestHelper mRequestHelper;

		private static int mPageCacheSeconds = -1;

		public static int PageCacheSeconds
		{
			get 
			{
				if (mPageCacheSeconds < 0)
				{
					string value = System.Configuration.ConfigurationManager.AppSettings["PageCacheSeconds"];
					int.TryParse(value, out mPageCacheSeconds);
				}

				return mPageCacheSeconds; 
			}
		}

		public GPageHandler()
		{
		}

		public GPageHandler(string virtualPath)
		{
			mVirtualPath = virtualPath;
		}

		private bool IsCachable(HttpContext context)
		{
			return context.Request.QueryString.Count == 0 || context.Request.QueryString["p"] == "newproducts" || context.Request.QueryString["p"] == "specials";
		}

		public bool IsReusable 
		{ 
			get { return false;} 
		}

		public void ProcessRequest(HttpContext context)
		{

#if TRIAL

			mCount++;
			if (mCount % 4 == 0)
			{
				// show trial message
				context.Response.Cache.SetNoStore();
				context.Response.Write(@"
VISTA CART TRIAL EDITION<br/><br/>
Buy full version at <a href=""http://www.iceasp.net/"" target=""_blank"">www.iceasp.net</a><br/><br/>
<a href=""javascript:window.location=window.location"">CONTINUE TRIAL</a> ");
				return;
			}
#endif

			// invoke for clicks count
			 mRequestHelper = Modules.Common.RequestHelper.Current;

			//ProcessNormal(context);
			ProcessWithCaching(context);
		}

		private void ProcessNormal(HttpContext context)
		{
			IHttpHandler h = BuildManager.CreateInstanceFromVirtualPath(mVirtualPath, typeof(Page)) as Page;
			h.ProcessRequest(context);
		}

		private void ProcessWithCaching(HttpContext context)
		{
			string key = GetCacheKey(context);
			string html = context.Cache[key] as string;

			if (html == null)
			{
				// load it and put into cache
				html = GetHtml(context);
				if (IsCachable(context) && PageCacheSeconds > 0)
				{
#if ! DEBUG
					context.Cache.Insert(key, html, null, DateTime.UtcNow.AddSeconds(PageCacheSeconds), System.Web.Caching.Cache.NoSlidingExpiration);
#endif
				}
			}

			// can't cache on client side because client may change language or currency
			////if (isPageCachable)
			////{
			////    context.Response.Cache.SetCacheability(HttpCacheability.Private);
			////    context.Response.Cache.SetExpires(DateTime.UtcNow.AddHours(1));
			////    context.Response.Cache.SetValidUntilExpires(true);
			////}

			//**************************************************************************
			IceHttpCompressionFilter.AddCompressionFilter(HttpContext.Current);
			//**************************************************************************

			context.Response.Cache.SetNoStore();
			context.Response.Write(html);
		}

		private string GetHtml(HttpContext context)
		{
			System.IO.TextWriter wr = new System.IO.StringWriter();
			HttpResponse resp = new HttpResponse(wr);
			HttpContext cc = new HttpContext(context.Request, resp);

			IHttpHandler h = BuildManager.CreateInstanceFromVirtualPath(mVirtualPath, typeof(Page)) as Page;
			h.ProcessRequest(cc);

			if (cc.Response.IsRequestBeingRedirected)
			{
				context.Response.Redirect(cc.Response.RedirectLocation, true);
			}

			return wr.ToString();
		}

		public static string GetCacheKey(HttpContext context)
		{
			return	context.Request.RawUrl +
						"_curr_" + VISTACARTHelper.CurrentUser.Currency.CurrencyID;
		}
	}



}
