﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Text;

namespace Ice.Net.Ecommerce.Web.Modules.Common
{
	public class NavigationMap
	{
		public static string GetNavigationMap(FrontParameters fp, int? rootCategoryID, string separator, bool excludeRootCategory)
		{
			FrontParameters fpTmp = new FrontParameters();

			StringBuilder sb = new StringBuilder();
			string cls = "";

			Category cat = null;
			if (fp.Kind == FrontParameters.KindEnum.CategoryShortcut && (cat = Category.LoadByShortcutName(fp.CategoryShortcutName)) != null)
			{
				fp.CategoryID = cat.CategoryID;
				fp.LinkName = cat.Name;
			}

			if (!string.IsNullOrEmpty(fp.SearchExpression))
			{
				if ( ! excludeRootCategory)
				{
					return "<span class='current'>" + Message.GetContent(Message.NameEnum.SearchResult) + "</span>";
				}
				else
				{
					return separator + " " + "<span class='current'>" + Message.GetContent(Message.NameEnum.SearchResult) + "</span>";
				}
			}

			if(fp.ProductID == 0 && excludeRootCategory == false && RequestHelper.Current.CurrentCategory.CategoryID == rootCategoryID)
			{
				return "<span class='current'>" + RequestHelper.Current.CurrentCategory.MetaTitle +"</span>";
			}

			// create categories map
			CategoryCollection chain = new CategoryCollection();
			chain.LoadCategoriesChain(fp.CategoryID, rootCategoryID, true);

			foreach (Category currentCategory in chain)
			{
				if (excludeRootCategory && currentCategory.CategoryID == rootCategoryID)
				{
					continue;
				}
				fpTmp.CategoryID = currentCategory.CategoryID;

				if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.IsNameInURL.GetValueOrDefault())
				{
					fpTmp.LinkName = currentCategory.Name;
				}
				else
				{
					fpTmp.LinkName = "";
				}

				if (currentCategory.CategoryID != rootCategoryID)
				{
					sb.Append(separator);
				}

				if (fp.WebPageID == 0 && fp.ProductID == 0 && currentCategory.CategoryID == fp.CategoryID)
				{
					cls = " class=\"current\"";
					sb.Append(String.Format("<span {2}>{1}</span>", "", currentCategory.Name, cls));
				}
				else
				{
					cls = "";
					string link = fpTmp.RelativeLink;
					if (currentCategory.CategoryID == rootCategoryID)
					{
						link = "./";
					}
					sb.Append(String.Format("<a {2} href=\"{0}\">{1}</a>", link, currentCategory.Name, cls));
				}
			}

			// create web page name
			if (fp.WebPageID > 0)
			{
				// show specific web page only
				WebPage webPage = new WebPage();
				if (webPage.LoadFromDB(fp.WebPageID) && webPage.IsActive)
				{
					cls = " class=\"current\"";

					fpTmp = new FrontParameters();
					fpTmp.WebPageID = webPage.WebPageID;
					fpTmp.LinkName = webPage.MetaTitle;
					if (sb.Length > 0)
					{
						sb.Append(separator);
					}
					sb.Append(String.Format("<span {2}>{1}</span>", "", webPage.MetaTitle, cls));
				}
			}
			else if (fp.ProductID > 0)
			{
				// create product name
				Product product = new Product();
				if (product.LoadFromDB(fp.ProductID) && product.IsActive)
				{
					cls = " class=\"current\"";

					fpTmp = new FrontParameters();
					fpTmp.ProductID = product.ProductID;
					if (!string.IsNullOrEmpty(product.Name))
					{
						fpTmp.LinkName = product.Name;
					}
					else
					{
						fpTmp.LinkName = "Product No. " + product.ProductID;
					}
					if (sb.Length > 0)
					{
						sb.Append(separator);
					}

					sb.Append(String.Format("<span {2}>{1}</span>", "", fpTmp.LinkName, cls));
				}
			}
			return sb.ToString();
		}

	}
}
