﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Text;

namespace Ice.Net.Ecommerce.Web.Modules.Common
{
	public class PageMetaInfo
	{
		string mTitle = "";
		string mKeywords = "";
		string mDescription = "";

		public string Title { get { return mTitle; } }
		public string Keywords { get { return mKeywords; } }
		public string Description { get { return mDescription; } }


		private static Object mObjLock = new Object();

		public PageMetaInfo(FrontParameters fp, int? rootCategoryID)
		{
			CreateMetaInfo(fp, rootCategoryID);

			this.mTitle = CleanString(this.Title);
			this.mKeywords = CleanString(this.Keywords);
			this.mDescription = CleanString(this.Description);
		}

		private string CleanString(string str)
		{
			if (string.IsNullOrEmpty(str))
			{
				return "";
			}
			return str.Replace("\"", "'").Replace("\r", "").Replace("\n", " ");
		}

		private void CreateMetaInfo(FrontParameters fp, int? rootCategoryID)
		{
			// create Meta info
			if (fp.WebPageID > 0)
			{
				WebPage webPage = new WebPage();
				if (webPage.LoadFromDB(fp.WebPageID))
				{
					// meta info
					if (!string.IsNullOrEmpty(webPage.MetaTitle))
					{
						this.mTitle = webPage.MetaTitle;
					}
					this.mKeywords = webPage.MetaKeywords;
					this.mDescription = webPage.MetaDescription;
				}
			}
			else if (fp.ProductID > 0)
			{
				Product prod = new Product();
				if (prod.LoadFromDB(fp.ProductID))
				{
					// meta info
					if (!string.IsNullOrEmpty(prod.MetaTitle))
					{
						this.mTitle = prod.MetaTitle;
					}
					else
					{
						this.mTitle = prod.Name;
					}
					this.mKeywords = prod.MetaKeywords;
					this.mDescription = prod.MetaDescription;
				}
			}
			else
			{
				int? categoryID = fp.CategoryID;
				if (categoryID.GetValueOrDefault() < 1)
				{
					categoryID = rootCategoryID; // Root Category is used to keep MetaInfo for start page!
				}
				Category cat = new Category();
				if (cat.LoadFromDB(categoryID))
				{
					// meta info
					if (!string.IsNullOrEmpty(cat.MetaTitle))
					{
						this.mTitle = cat.MetaTitle;
					}
					else
					{
						this.mTitle = cat.Name;
					}
					this.mKeywords = cat.MetaKeywords;
					this.mDescription = cat.MetaDescription;
				}
			}
		}

	}
}
