﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Text;
using System.Collections.Specialized;
using System.Collections.Generic;

namespace Ice.Net.Ecommerce.Web.Modules.Common
{
	public class ProductData
	{
		private static string mNoProductPicture = "Gallery/System/nopic.gif";
		private static string mOutOfStockMsg = null;

		public string OutOfStockMsg
		{
			get
			{
				if (mOutOfStockMsg == null)
				{
					mOutOfStockMsg = Message.GetContent(Message.NameEnum.OutOfStock);
				}
				return mOutOfStockMsg;
			}
		}

		public Product Product;
	////	public string CurrencyUnit;
	////	public string CurrencySign;
		public string Name;
		public string DetailsLink;
		public string SmallImageLink;
		public string LargeImageLink;
		public string OriginalPriceStr;
		public string CurrentPriceStr;
		public string TaxName;
		public string LongDescription;
		public string ShortDescription;
	////	public bool IsInCampaign;

		public bool IsOutOfStock = false;

		private String mProductAttributes = null;
		public String ProductAttributes
		{
			get
			{
				if (mProductAttributes == null)
				{
					mProductAttributes = Modules.HtmlElements.AttributesSelectorsBuilder.GetProductAttributesSelectors(this.Product);
				}
				return mProductAttributes;
			}
		}

		public ProductData(Product product, FrontParameters fp)
		{
			decimal currentPrice = product.GetCalculatedSalePrice();
			decimal originalPrice = product.SalePrice;

			currentPrice = CommonUtility.GetPriceWithTax(currentPrice);
			originalPrice = CommonUtility.GetPriceWithTax(originalPrice);

			string currentPriceStr = CurrencyUtility.GetFormatedPriceInCurrency(currentPrice);
			string originalPriceStr = "";
			
			CampaignHelper campaignHelper = product.GetCampaignHelper();
			if (campaignHelper.IsCampaignValid && campaignHelper.ApplyToCurrentProduct)
			{
				originalPriceStr = CurrencyUtility.GetFormatedPriceInCurrency(originalPrice);
			}

			string taxName = "";
			if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ShowPriceWithTax.GetValueOrDefault())
			{
				// price already includes tax, we don't have to show tax note
				taxName = "(" + Message.GetContent(Message.NameEnum.TaxIncluded).ToLower() + ")";
			}
			else
			{
				taxName = "+ " + VISTACARTHelper.AppDefaults.Tax.Name;
			}

		////	this.CurrencyUnit = RequestHelper.Current.CurrentUser.Currency.ISOCode;
		////	this.CurrencySign = RequestHelper.Current.CurrentUser.Currency.Symbol;
			this.ShortDescription = GetShortDescription(product);
			this.LongDescription = GetLongDescription(product);
			this.IsOutOfStock = product.StockQuantity < 1;
			this.Product = product;
			this.Name = GetProductName(product) + "&nbsp;";
			this.DetailsLink = GetDetailsLink(product, fp);
			this.SmallImageLink = GetSmallImageLink(product);
			this.LargeImageLink = GetLargeImageLink(product);
			this.OriginalPriceStr = originalPriceStr;
			this.CurrentPriceStr = currentPriceStr;
			this.TaxName = taxName;
		}

		private string GetShortDescription(Product product)
		{
			string text = string.IsNullOrEmpty(product.ShortDescription) ? product.LongDescription : product.ShortDescription;
			text = CommonUtility.GetTextInLength(text + "", 60);
			return text;
		}

		private string GetLongDescription(Product product)
		{
			string text = string.IsNullOrEmpty(product.LongDescription) ? product.ShortDescription : product.LongDescription;
			return text + "";
		}

		private string GetProductName(Product product)
		{
			if (string.IsNullOrEmpty(product.Name))
			{
				return "Product No. " + product.ProductID;
			}
			else
			{
				return product.Name;
			}
		}

		public string GetDetailsLink(Product product, FrontParameters fp)
		{
			// remember previous ProductID
			int productID = fp.ProductID;

			fp.ProductID = product.ProductID;

			if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.IsNameInURL.GetValueOrDefault())
			{
				fp.LinkName = GetProductName(product);
			}
			else
			{
				fp.LinkName = "";
			}

			string link = fp.RelativeLink;

			// restore ProductID
			fp.ProductID = productID;

			return link;
		}

		private string GetLargeImageLink(Product product)
		{
			string image = null;
			if (!string.IsNullOrEmpty(product.LargeImage))
			{
				image = product.LargeImage;
			}
			else if (!string.IsNullOrEmpty(product.SmallImage))
			{
				image = product.SmallImage;
			}
			else
			{
				image = mNoProductPicture;
			}
			return image;
		}

		private string GetSmallImageLink(Product product)
		{
			string image = null;
			if (!string.IsNullOrEmpty(product.SmallImage))
			{
				image = product.SmallImage;
			}
			else if (!string.IsNullOrEmpty(product.LargeImage))
			{
				image = product.LargeImage;
			}
			else
			{
				image = mNoProductPicture;
			}

			return image;
		}

	}

	public class ProductDataCollection : List<ProductData>
	{
		public ProductDataCollection(List<Product> products, FrontParameters fp)
		{
			foreach (Product product in products)
			{
				this.Add(new ProductData(product, fp));
			}
		}
	}
}