﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Text;
using System.Collections.Specialized;
using System.Collections.Generic;
using Ice.Net.Ecommerce.BusinessObjects.Database;

namespace Ice.Net.Ecommerce.Web.Modules.Common
{
	using BusinessObjects.Database.VISTACART;

	public class ProductsPager
	{
		private int mNextPageIndex = 0;
		private int mPreviousPageIndex = 0;
		private int mCurrentPageIndex = 0;
		private FrontParameters mFP = null;

		public int TotalPages = 0;
		public int TotalProducts = 0;
		public int CurrentPage = 0;
		public int NextPage = 0;
		public int PreviousPage = 0;
		public string NextPageLink = "";
		public string PreviousPageLink = "";

		public List<Product> ProductCollection = null;

		private ProductDataCollection mProductDataCollection = null;
		/// <summary>
		///  Get on demand and keep cached
		/// </summary>
		public ProductDataCollection ProductDataCollection
		{
			get
			{
				if (mProductDataCollection == null)
				{
					mProductDataCollection = new ProductDataCollection(this.ProductCollection, mFP);
				}
				return mProductDataCollection;
			}
		}


		public ProductsPager(int currentCategoryID, FrontParameters fp, bool specials, bool newProducts)
		{
			mFP = fp;
			int totalProductsCount = LoadProducts(currentCategoryID, fp, specials, newProducts); // return total products count by given category or search expression
			BuildPagerData(totalProductsCount, fp);
		}

		private int LoadProducts(int currentCategoryID, FrontParameters fp, bool specials, bool newProducts)
		{
			int productsCount = 0;

			if (specials)
			{
				// load by current campaign
				Campaign currentCampaign = Campaign.LoadCurrentCampaign();
				if (currentCampaign != null)
				{
					Category specialsCategory = Category.LoadByShortcutName(Category.Shortcut.specials.ToString());
					fp.CategoryID = specialsCategory.CategoryID;
					fp.LinkName = currentCampaign.Name;

					productsCount = BusinessObjects.Database.VISTACART.ProductCollection.CountByCampaignID(currentCampaign.CampaignID, true);
					productsCount = VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ProductsPerPage; // don't use paging, show first page only
					this.ProductCollection = BusinessObjects.Database.VISTACART.ProductCollection.LoadPagedByCampaignID(currentCampaign.CampaignID, VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ProductsPerPage, fp.PagerPageIndex, db.Product.Ordinal, false, true);
				}
			}
			else if (newProducts)
			{
				// load by product UpdateDate
				productsCount = VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ProductsPerPage;
				this.ProductCollection = BusinessObjects.Database.VISTACART.ProductCollection.LoadNewProducts(productsCount, false, true);
			}
			else if (!string.IsNullOrEmpty(fp.SearchExpression))
			{
				// count total products
				productsCount = BusinessObjects.Database.VISTACART.ProductCollection.CountBySearchExpression(fp.SearchExpression, true);

				// load products by search expression
				this.ProductCollection = BusinessObjects.Database.VISTACART.ProductCollection.LoadPagedBySearchExpression(fp.SearchExpression, VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ProductsPerPage, fp.PagerPageIndex, db.Product.Ordinal, false, true);
			}
			else
			{
				// count total products
				productsCount = BusinessObjects.Database.VISTACART.ProductCollection.CountByCategoryID(currentCategoryID, true);

				// load products by category
				this.ProductCollection = BusinessObjects.Database.VISTACART.ProductCollection.LoadPagedByCategoryID(currentCategoryID, VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ProductsPerPage, fp.PagerPageIndex, db.Product.Ordinal, false, true);
			}
			return productsCount;
		}

		private void BuildPagerData(int productsCount, FrontParameters fp)
		{
			// pager
			this.TotalProducts = productsCount;
			mCurrentPageIndex = fp.PagerPageIndex;
			mPreviousPageIndex = mCurrentPageIndex - 1;
			mNextPageIndex = mCurrentPageIndex + 1;
			this.TotalPages = productsCount / VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ProductsPerPage;

			if (this.TotalProducts % VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ProductsPerPage != 0)
			{
				this.TotalPages++;
			}

			if (mNextPageIndex >= this.TotalPages)
			{
				mNextPageIndex = this.TotalPages - 1;
			}

			if (mPreviousPageIndex < 0)
			{
				mPreviousPageIndex = 0;
			}

			// save pager index 
			int currentPagerIndex = fp.PagerPageIndex;

			fp.PagerPageIndex = mNextPageIndex;
			this.NextPageLink = fp.RelativeLinkWithSearch;

			fp.PagerPageIndex = mPreviousPageIndex;
			this.PreviousPageLink = fp.RelativeLinkWithSearch;

			//restore pager index
			fp.PagerPageIndex = currentPagerIndex;

			// set visual variables
			this.CurrentPage = mCurrentPageIndex + 1;
			this.NextPage = mNextPageIndex + 1;
			this.PreviousPage = mPreviousPageIndex + 1;
			// end of pager

		}
	}
}