﻿using System;
using System.Collections.Generic;
using System.Text;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;

namespace Ice.Net.Ecommerce.Web.Modules.HtmlElements
{
	public class OrdersListHtmlBuilder
	{
		public static string GetHtml(User user)
		{
			OrdersListHtmlBuilder builder = new OrdersListHtmlBuilder();
			return builder.GetOrdersListHtml(user);
		}

		private string GetOrdersListHtml(User user)
		{
			// load orders list for user
			List<Order> orderCollection = OrderCollection.LoadByUserID(user.UserID, 500);

			StringBuilder sb = new StringBuilder();

			// order list header
			sb.Append(String.Format(@"
<table class='cartlist' border='0'>
<tr>
<th>{0}</th>
<th class='right'>{1}</th>
<th>{2}</th>
<th>{3}</th>
<th>{4}</th>
<th>{5}</th>
<th>{6}</th>
</tr>
",

Message.GetContent(Message.NameEnum.OrderNumber),
Message.GetContent(Message.NameEnum.Total) + " (" + VISTACARTHelper.CurrentUser.Currency.ISOCode + ")",
Message.GetContent(Message.NameEnum.Date),
Message.GetContent(Message.NameEnum.Shipping),
Message.GetContent(Message.NameEnum.Payment),
Message.GetContent(Message.NameEnum.Status),
"" // details link
));


			// order items
			int rows = 0;
			foreach (Order order in orderCollection)
			{
				if (order.OrderStatus == Order.OrderStatusEnum.ShoppingCart || order.IsActive == false)
				{
					// don't show shopping carts
					continue;
				}

				rows++;

				sb.Append(@"
<tr>");
				sb.Append(String.Format("<td class=\"center\">{0}</td>", order.OrderID));
				sb.Append(String.Format("<td class='right'>{0}</td>", BusinessObjects.Common.CurrencyUtility.GetFormatedPriceInCurrency(order.RedundantTotalToPay)));
				sb.Append(String.Format("<td class=\"center\">{0}</td>", order.CheckoutDate));
				sb.Append(String.Format("<td class=\"center\">{0}</td>", order.GetShippingType().Name));
				sb.Append(String.Format("<td class=\"center\">{0}</td>", order.PaymentStatus));
				sb.Append(String.Format("<td class=\"center\">{0}</td>", order.OrderStatus));
				sb.Append(String.Format("<td class='center'><a href='#' onclick='Ice.GCart.GetOrderDetail({0}); return false;'>details</a></td>", order.OrderID));
				sb.Append(@"</tr>
");
			}

			if (rows == 0)
			{
				// no previous orders
				sb.Append("<tr class='focus'><td colspan='7' class='center'>" + Message.GetContent(Message.NameEnum.NoPreviousOrders) + "</td></tr>");
			}


			sb.Append("<tr class='spacer'><td colspan='7'></td></tr>");
			sb.Append("</table>");

			return sb.ToString();
		}
	}
}