﻿using System;
using System.Collections.Generic;
using System.Text;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;

namespace Ice.Net.Ecommerce.Web.Modules.HtmlElements
{
	public class ShoppingCartHtmlBuilder
	{
		public static string GetHtml(Order order, bool enableEditing)
		{
			ShoppingCartHtmlBuilder builder = new ShoppingCartHtmlBuilder();
			return builder.GetShoppingCartHtml(order, enableEditing);
		}

		private string GetShoppingCartHtml(Order order, bool enableEditing)
		{
			StringBuilder sb = new StringBuilder();

			// cart header
			sb.Append(String.Format(@"
<table class='cartlist' border='0'>
<tr>
<th style='width:35px;'>{0}</th>
<th class='first'>{1}</th>
<th class='second'>{2}</th>
<th class='center'>{3}</th>
<th class='fourth'>{4}</th>
<th class='fifth'>{5}</th>
" + (enableEditing ? "<th class='sixth'>{6}</th>" : "<th style='width:2px'></th>") + @"
</tr>
",
 "",
Message.GetContent(Message.NameEnum.Product),
Message.GetContent(Message.NameEnum.Size),
Message.GetContent(Message.NameEnum.Quantity),
Message.GetContent(Message.NameEnum.Price),
Message.GetContent(Message.NameEnum.Subtotal),
Message.GetContent(Message.NameEnum.Remove)
 ));

			// cart is empty message
			if (order.GetOrderDetailCollection().Count == 0)
			{
				sb.Append("<tr class='focus'><td colspan='7' class='center'>" + Message.GetContent(Message.NameEnum.CartIsEmpty) + "</td></tr>");
				sb.Append("<tr class='spacer'><td colspan='7'></td></tr>");
			}
			else
			{
				string remove = "";
				if (enableEditing)
				{
					remove = "<div class=\"removearrow\"><a href=\"#\" onclick='Ice.GCart.RemoveFromCart({0});return false;'>&nbsp;</a></div>";
				}

				// items in cart list
				int detailIndex = 0;
				foreach (OrderDetail detail in order.GetOrderDetailCollection())
				{
					string quantity = "";
					if (enableEditing && !VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SingleCartItem.GetValueOrDefault())
					{
						quantity = "<input maxlength='6' type=\"text\" value='" + detail.Quantity + "' style='text-align:center' class=\"items inpQuantity\" onfocus=\"Ice.GCart.QuantityChanged(this);\"/>";
					}
					else
					{
						quantity = "<b>" + detail.Quantity + "</b>";
					}

					sb.Append(@"
<tr>");
					sb.Append(String.Format("<td class=\"left\"><div class='loadingMask' style=\"height:30px; width:30px;overflow:hidden;\"><img class='imageLoader' src='{0}'/></div></td>", detail.GetProduct().SmallImage));
					sb.Append(String.Format("<td><a href='{0}'>{1}</a></td>", GetDetailsLink(detail.GetProduct()), detail.GetProduct().Name));
					sb.Append(String.Format("<td class=\"left\">{0}</td>", detail.Attributes.Replace(", ", "<br/>")));
					sb.Append(String.Format("<td class=\"center\">{0}</td>", quantity));
					sb.Append(String.Format("<td class=\"left noWrap\">{0}</td>", CurrencyUtility.GetFormatedPriceInCurrency(IncludeTax(detail.RedundantSalePrice))));
					sb.Append(String.Format("<td class=\"left noWrap\">{0}</td>", CurrencyUtility.GetFormatedPriceInCurrency(IncludeTax(detail.RedundantSalePrice) * detail.Quantity)));
					sb.Append(String.Format("<td class=\"center\">" + remove + "</td>",
						detailIndex++));
					sb.Append(@"</tr>
");
				}

				sb.Append("<tr class='spacer'><td colspan='7'></td></tr>");

				// shipping
				sb.Append("<tr class=\"focusShipping\">");
				sb.Append(String.Format("<td class=\"left\" colspan=\"1\">{0}</td>", Message.GetContent(Message.NameEnum.Shipping)));
				sb.Append(String.Format("<td class=\"left\" colspan='1'>{0}</td>", GetShippingTypesSelect(order.ShippingTypeID, enableEditing)));
				sb.Append(String.Format("<td class=\"left\" colspan='3'><div class='shippingDescription'>{0}</div></td>", GetShippingTypeDesc(order.ShippingTypeID)));
				sb.Append(String.Format("<td class=\"left\" colspan=\"2\">{0}</td>", CurrencyUtility.GetFormatedPriceInCurrency(IncludeTax(order.RedundantShippingPrice))));
				sb.Append(@"</tr>
			");

				if (!VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ShowPriceWithTax.GetValueOrDefault())
				{
					// tax
					sb.Append("<tr class=\"focusTax\">");
					sb.Append(String.Format("<td class=\"left\" colspan=\"2\">{0}</td>", Message.GetContent(Message.NameEnum.Tax)));
					sb.Append(String.Format("<td class=\"left\" colspan=\"3\">{0}</td>", GetTaxDescription(VISTACARTHelper.AppDefaults.Tax)));
					sb.Append(String.Format("<td class=\"left\" colspan=\"2\">{0}</td>", CurrencyUtility.GetFormatedPriceInCurrency(order.RedundantTotalTaxPrice)));
					sb.Append(@"</tr>
");
				}

				sb.Append("<tr class='spacer'><td colspan='7'></td></tr>");

				// total to pay
				sb.Append("<tr class=\"focus\">");
				sb.Append(String.Format("<td class=\"left\" colspan=\"5\">{0}</td>", Message.GetContent(Message.NameEnum.Subtotal)));
				sb.Append(String.Format("<td class=\"left\" colspan=\"2\">{0}</td>", CurrencyUtility.GetFormatedPriceInCurrency(order.RedundantTotalToPay)));
				sb.Append(@"</tr>
");

				sb.Append("<tr class='spacer'><td colspan='7'></td></tr>");

				if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ShowPriceWithTax.GetValueOrDefault())
				{
					// tax
					sb.Append("<tr class=\"focusTax\">");
					sb.Append(String.Format("<td class=\"left\" colspan=\"2\">{0}</td>", Message.GetContent(Message.NameEnum.Tax)));
					sb.Append(String.Format("<td class=\"left\" colspan=\"3\">{0}</td>", GetTaxDescription(VISTACARTHelper.AppDefaults.Tax)));
					sb.Append(String.Format("<td class=\"left\" colspan=\"2\">{0}</td>", CurrencyUtility.GetFormatedPriceInCurrency(order.RedundantTotalTaxPrice)));
					sb.Append(@"</tr>
");
				}

				// recalculate button
				if (enableEditing)
				{
					sb.Append(String.Format("<tr><td colspan='7'><div align='right'><input class='buttonRecalculateCart' type='button' value='{0}' onclick='Ice.GCart.UpdateCartItemQuantity();return false;'/></div></td></tr>",
						Message.GetContent(Message.NameEnum.Recalculate)));
				}

				// payment type selector
				sb.Append("<tr>");
				sb.Append(String.Format("<td colspan='1'>{0}</td>", Message.GetContent(Message.NameEnum.PaymentType)));
				sb.Append(String.Format("<td colspan='1'>{0}</td>", GetPaymentTypesSelect(order.PaymentTypeID, enableEditing)));
				sb.Append(String.Format("<td colspan='3'>{0}</td>", GetPaymentTypeDescription(order.PaymentTypeID)));
				sb.Append(String.Format("<td colspan='2'>{0}</td>", ""));
				sb.Append("</tr>");
			}

			sb.Append("</table>");

			return sb.ToString();
		}

		private decimal IncludeTax(decimal price)
		{
			if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ShowPriceWithTax.GetValueOrDefault())
			{
				return price * (1 + BusinessObjects.Common.VISTACARTHelper.AppDefaults.Tax.RatePercent / 100);
			}
			else
			{
				return price;
			}
		}

		private string GetDetailsLink(Product product)
		{
			FrontParameters fp = new FrontParameters();
			fp.ProductID = product.ProductID;
			return fp.RelativeLink; // fp.AbsoluteLink;
		}

		private string GetShippingTypeDesc(int? selectedShippingTypeID)
		{
			ShippingType shType = new ShippingType();
			if (shType.LoadFromDB(selectedShippingTypeID))
			{
				return shType.Description;
			}
			else
			{
				return "";
			}
		}

		private string GetTaxDescription(Tax tax)
		{
			return tax.Description;
		}

		private string GetShippingTypesSelect(int? selectedShippingTypeID, bool enableEditing)
		{
			List<ShippingType> shippingTypes = SmartObjectLoader.LoadCollection<ShippingType>(100, true, db.ShippingType.Ordinal, false);
			StringBuilder sb = new StringBuilder();

			if (enableEditing == false)
			{
				ShippingType shType = new ShippingType();
				if (shType.LoadFromDB(selectedShippingTypeID))
				{
					sb.Append("<b>" + shType.Name + "</b>");
				}
			}
			else
			{
				sb.Append(@"<select id='shippingTypeSelect' name='ShippingTypeID' onchange='Ice.GCart.UpdateShippingType(this, true)'>");
				//////            sb.Append(@"
				//////<option value='0'> - " + DynamicContent.GetContentInLanguage(DynamicContent.NameEnum.Select).ToLower() + " - </option>");

				foreach (ShippingType shType in shippingTypes)
				{
					string selected = "";
					if (shType.ShippingTypeID == selectedShippingTypeID)
					{
						selected = " selected='selected'";
					}

					sb.Append(@"
<option value='" + shType.ShippingTypeID + "'" + selected + " title='" + shType.Description + "'>" + shType.Name + "</option>");
				}
				sb.Append(@"
</select>");
			}

			return sb.ToString();
		}

		private string GetPaymentTypesSelect(int? selectedPaymentTypeID, bool enableEditing)
		{
			StringBuilder sb = new StringBuilder();
			List<PaymentType> paymentTypes = SmartObjectLoader.LoadCollection<PaymentType>(100, true, db.PaymentType.Ordinal, false);
			if (enableEditing == false)
			{
				PaymentType shType = new PaymentType();
				if (shType.LoadFromDB(selectedPaymentTypeID))
				{
					sb.Append("<b>" + shType.Name + "</b>");
				}
			}
			else
			{
				sb.Append(@"<select id='paymentTypeSelect' name='PaymentTypeID' onchange='Ice.GCart.UpdatePaymentType(this, true)'>");
				//////            sb.Append(@"
				//////<option value='0'> - " + DynamicContent.GetContentInLanguage(DynamicContent.NameEnum.Select).ToLower() + " - </option>");

				foreach (PaymentType payType in paymentTypes)
				{
					string selected = "";
					if (payType.PaymentTypeID == selectedPaymentTypeID)
					{
						selected = " selected='selected'";
					}

					sb.Append(@"
<option value='" + payType.PaymentTypeID + "'" + selected + " title='" + payType.Description + "'>" + payType.Name + "</option>");
				}
				sb.Append(@"
</select>");
			}
			return sb.ToString();
		}

		private string GetPaymentTypeDescription(int? selectedPaymentTypeID)
		{
			PaymentType pt = new PaymentType();
			pt.LoadFromDB(selectedPaymentTypeID);
			return pt.Description;
		}


	}
}
