﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Collections.Generic;

namespace Ice.Net.Ecommerce.Web.WebService
{
	using Ice.Orm.Core.Entity;
	using Ice.Net.Ecommerce.EmailUtility;
	using Ice.Orm.Core.Criteria;

	/// <summary>
	/// Summary description for GService
	/// </summary>
	[WebService(Namespace = "http://iceasp.net")]
	[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
	[ToolboxItem(false)]
	//[System.Web.Script.Services.ScriptService]
	public sealed partial class GUserService : System.Web.Services.WebService
	{
		[IceWebMethod]
		public bool SetUserLanguage(int languageID)
		{
			VISTACARTHelper.CurrentUser.UserInfo.Update(0, languageID, 0);
			return true;
		}

		[IceWebMethod]
		public bool SetUserCurrency(int currencyID)
		{
			VISTACARTHelper.CurrentUser.UserInfo.Update(currencyID, 0, 0);
			return true;
		}

		[IceWebMethod]
		public GUserResult UserLogin(string username, string pass)
		{
			User user = new User();
			if (!string.IsNullOrEmpty(username)
				&& !string.IsNullOrEmpty(pass)
				&& (user = Ecommerce.BusinessObjects.Database.VISTACART.User.LoadByUsername(username)) != null
				&& user.Password == pass)
			{
				if (user.IsActive)
				{
					user.LastLoginDate = DateTime.Now;
					user.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
					// ok
					BusinessObjects.Common.VISTACARTHelper.CurrentUser.UserTicket.SignIn(user, false);
					return new GUserResult(true, "");
				}
				else
				{
					return new GUserResult(false, Message.GetContent(Message.NameEnum.AccountLocked));
				}
			}
			System.Threading.Thread.Sleep(3000); // slow down invalid login attempts
			return new GUserResult(false, Message.GetContent(Message.NameEnum.InvalidLogin));
		}

		[IceWebMethod]
		public bool UserLogout()
		{
			VISTACARTHelper.CurrentUser.UserTicket.SignOut();
			return true;
		}

		[IceWebMethod]
		public GUserResult UserSendPassword(string email)
		{
			// validate email
			if (!ValidationUtility.IsEmailValidFormat(email))
			{
				return new GUserResult(false, Message.GetContent(Message.NameEnum.InvalidEmail));
			}
			User user = Ecommerce.BusinessObjects.Database.VISTACART.User.LoadByEmail(email);
			if(user != null)
			{
				// ok
				string message = @"

" + Message.GetContent(Message.NameEnum.Username) + ": " + user.Username + @"
" + Message.GetContent(Message.NameEnum.Password) + ": " + user.Password + @"

" + VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.WebSite;


				string errorMessage = "";
				if (BusinessObjects.Common.EmailSender.SendEmail(Message.GetContent(Message.NameEnum.YourPassword).ToUpper(), message, user.Email, ref errorMessage))
				{
					return new GUserResult(true, Message.GetContent(Message.NameEnum.UsernameAndPasswordSentTo) + email);
				}
				else
				{
					return new GUserResult(false, errorMessage);
				}
			}
			else
			{
				return new GUserResult(false, Message.GetContent(Message.NameEnum.EmailNotFound));
			}

		}

		[IceWebMethod]
		public GUserResult UserNewsletterSignIn(string email)
		{
			// validate email
			if (!ValidationUtility.IsEmailValidFormat(email))
			{
				return new GUserResult(false, Message.GetContent(Message.NameEnum.InvalidEmail));
			}
			User user = Ecommerce.BusinessObjects.Database.VISTACART.User.LoadByEmail(email);
			if (user != null)
			{
				// already exist
				return new GUserResult(false, Message.GetContent(Message.NameEnum.EmailAlreadyExist));
			}
			else
			{
				user.Email = email;
				user.Newsletter = true;
				user.IsActive = true;
				user.CreationDate = DateTime.Now;
				user.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					return new GUserResult(true, Message.GetContent(Message.NameEnum.EmailAdded));
				}
			}

		}

		[IceWebMethod]
		public GUserResult UserAccountCreateOrUpdate(User user, Address address, bool autoLogin)
		{
			// validate email
			if (!ValidationUtility.IsEmailValidFormat(user.Email))
			{
				return new GUserResult(false, Message.GetContent(Message.NameEnum.InvalidEmail));
			}

			if (VISTACARTHelper.CurrentUser.UserTicket.IsAuthenticated)
			{
				// we're updating an logged in user
				if (user.UserID < 1)
				{
					// trying to create new account while there is already user logged in
					return new GUserResult(false, Message.GetContent(Message.NameEnum.UserAlreadySingedIn));
				}
				if (VISTACARTHelper.CurrentUser.UserTicket.UserID != user.UserID)
				{
					// logged-in userID is different from the one we're trying to update
					return new GUserResult(false, Message.GetContent(Message.NameEnum.WrongUser));
				}
			}
			else
			{
				// create new user account
				// validate username
				if (user.Username.Length < VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.MinUsernameLength)
				{
					return new GUserResult(false, Message.GetContent(Message.NameEnum.MinUsernameLength) + ": " + VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.MinUsernameLength);
				}
			}

			// validate password
			if (user.Password.Length < VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.MinPasswordLength)
			{
				return new GUserResult(false, Message.GetContent(Message.NameEnum.MinPasswordLength) + ": " + VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.MinPasswordLength);
			}

			User existingUser = new User();

			if (VISTACARTHelper.CurrentUser.UserTicket.IsAuthenticated)
			{
				// update an existing user account

				// test if email is already taken by another user account
				User anotherUser = Ecommerce.BusinessObjects.Database.VISTACART.User.LoadByEmail(user.Email);
				if (anotherUser != null && anotherUser.UserID != user.UserID)
				{
					return new GUserResult(false, Message.GetContent(Message.NameEnum.EmailBelongToAnotherAccount));
				}

				if (existingUser.LoadFromDB(user.UserID))
				{
					if (existingUser.AddressID > 0)
					{
						address.AddressID = existingUser.AddressID.GetValueOrDefault();
					}

					////address.Email = user.Email; // allow different email for Address and for User objects
					address.CurrencyID = VISTACARTHelper.CurrentUser.Currency.CurrencyID;
					address.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
					{
						UpdateAccountAddress(address);

						existingUser.AddressID = address.AddressID;
						existingUser.Newsletter = user.Newsletter;
						existingUser.Password = user.Password;
						existingUser.Email = user.Email;
						existingUser.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
						{
							return new GUserResult(true, Message.GetContent(Message.NameEnum.SavedSucessfully));
						}
					}
					return new GUserResult(false, Message.GetContent(Message.NameEnum.ServerInternalError));
				}
				else
				{
					// invalid addressID
					return new GUserResult(false, Message.GetContent(Message.NameEnum.WrongUser));
				}
			}
			else
			{
				// CREATE NEW ACCOUNT

				existingUser = Ecommerce.BusinessObjects.Database.VISTACART.User.LoadByEmail(user.Email);
				if (existingUser != null) // verify if account with the same email already exist
				{
					// email already exist, but account is not created yet (email is signed in for newsletter only)
					// create new account with existing email
					if (string.IsNullOrEmpty(existingUser.Username))
					{
						// new user, he was signed for newsletter only
						existingUser.Username = user.Username;
						existingUser.Password = user.Password;
						existingUser.Newsletter = user.Newsletter;
						user.CreationDate = DateTime.Now;
						existingUser.IsActive = true;
						existingUser.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
					}
					else
					{
						// account already exist
						return new GUserResult(false, Message.GetContent(Message.NameEnum.EmailBelongToAnotherAccount));
					}
				}
				else //// if (existingUser.LoadByUsername(user.Username)) // verify if username already exist
				{
					existingUser = Ecommerce.BusinessObjects.Database.VISTACART.User.LoadByUsername(user.Username);
					if (existingUser != null)
					{
						return new GUserResult(false, Message.GetContent(Message.NameEnum.UsernameTaken));
					}
				}

				// make sure we create new objects (must set identity = 0)
				address.AddressID = existingUser.AddressID.GetValueOrDefault();
				user.UserID = existingUser.UserID;
				user.IsActive = true;

				if (string.IsNullOrEmpty(address.Email))
				{
					address.Email = user.Email;
				}
				address.CurrencyID = VISTACARTHelper.CurrentUser.Currency.CurrencyID;

				address.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					UpdateAccountAddress(address);

					user.AddressID = address.AddressID;
					user.CreationDate = DateTime.Now;
					if (autoLogin)
					{
						user.LastLoginDate = DateTime.Now;
					}
					user.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
					{
						if (autoLogin)
						{
							VISTACARTHelper.CurrentUser.UserTicket.SignIn(user, false);
						}
						return new GUserResult(true, Message.GetContent(Message.NameEnum.AccountCreated));

					}
				}
			}

			return new GUserResult(false, Message.GetContent(Message.NameEnum.ServerInternalError));
		}

		[IceWebMethod]
		public CartData AddProductToCart(int productID, int quantity, string attributes)
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			cart.AddProduct(productID, quantity, attributes);
			cart.SaveCartWithRecalculate();

			return new CartData(cart.Order.RedundantTotalProductsCount, CurrencyUtility.ConvertPrice(cart.Order.RedundantTotalToPay).ToString("n2"));
		}

		[IceWebMethod]
		public CartData GetCartData()
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			return new CartData(cart.Order.RedundantTotalProductsCount, CurrencyUtility.ConvertPrice(cart.Order.RedundantTotalToPay).ToString("n2"));
		}

		[IceWebMethod]
		public CartData RemoveFromCart(int detailIndex)
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			cart.RemoveAtDetailIndex(detailIndex);
			cart.SaveCartWithRecalculate();

			return new CartData(cart.Order.RedundantTotalProductsCount, CurrencyUtility.ConvertPrice(cart.Order.RedundantTotalToPay).ToString("n2"));
		}

		[IceWebMethod]
		public CartData SetCartItemQuantity(int detailIndex, int quantity)
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			cart.SetQuantity(detailIndex, quantity);
			cart.SaveCartWithRecalculate();

			return new CartData(cart.Order.RedundantTotalProductsCount, CurrencyUtility.ConvertPrice(cart.Order.RedundantTotalToPay).ToString("n2"));
		}

		[IceWebMethod]
		public CartData SetShippingType(int shippingTypeID)
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			cart.Order.ShippingTypeID = shippingTypeID;
			cart.SaveCartWithRecalculate();

			return new CartData(cart.Order.RedundantTotalProductsCount, CurrencyUtility.ConvertPrice(cart.Order.RedundantTotalToPay).ToString("n2"));
		}

		[IceWebMethod]
		public CartData SetPaymentType(int paymentTypeID)
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			cart.Order.PaymentTypeID = paymentTypeID;
			cart.SaveCartWithRecalculate();

			return new CartData(cart.Order.RedundantTotalProductsCount, CurrencyUtility.ConvertPrice(cart.Order.RedundantTotalToPay).ToString("n2"));
		}

		[IceWebMethod]
		public GUserResult IsCheckoutPossible()
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			if (cart.Order.RedundantTotalProductsCount == 0)
			{
				return new GUserResult(false, Message.GetContent(Message.NameEnum.CartIsEmpty));
			}

			PaymentType pt = new PaymentType();
			ShippingType st = new ShippingType();
			if (pt.LoadFromDB(cart.Order.PaymentTypeID) && pt.IsActive)
			{
			}
			else
			{
				cart.Order.PaymentTypeID = VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.PaymentTypeID;
				cart.SaveCartWithRecalculate();
				//return new GUserResult(false, Message.GetContent(Message.NameEnum.SelectPaymentType));
			}

			if (st.LoadFromDB(cart.Order.ShippingTypeID) && st.IsActive)
			{
			}
			else
			{
				cart.Order.ShippingTypeID = VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.ShippingTypeID;
				cart.SaveCartWithRecalculate();
				//return new GUserResult(false, Message.GetContent(Message.NameEnum.SelectShippingType));
			}

			if (VISTACARTHelper.CurrentUser.UserTicket.IsAuthenticated)
			{
				return new GUserResult(true, "");
			}
			else if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.IsAllowGuestCheckout)
			{
				if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.IsGuestCheckoutWarning)
				{
					GUserResult result = new GUserResult(true, Message.GetContent(Message.NameEnum.GuestCheckoutWarning));
					result.Warning = true;
					return result;
				}
				else
				{
					return new GUserResult(true, "");
				}
			}
			else
			{
				return new GUserResult(false, Message.GetContent(Message.NameEnum.LoginFirst), true);
			}

		}

		[IceWebMethod]
		public User GetCurrentUser()
		{
			if (VISTACARTHelper.CurrentUser.UserTicket.IsAuthenticated)
			{
				return VISTACARTHelper.CurrentUser.User;
			}
			else
			{
				return new User();
			}
		}

		[IceWebMethod]
		public Address GetCurrentUserAddress()
		{
			Address addr = new Address();
			if (VISTACARTHelper.CurrentUser.UserTicket.IsAuthenticated && VISTACARTHelper.CurrentUser.User.AddressID > 0)
			{
				addr.LoadFromDB(VISTACARTHelper.CurrentUser.User.AddressID);
			}
			return addr;
		}

		////[IceWebMethod]
		////public string GetAddressForm()
		////{
		////    System.Net.WebClient wc = new System.Net.WebClient();
		////    byte[] bb = wc.DownloadData(BusinessObjects.Common.ApplicationUtility.GetAbsoluteLink("Forms/AddressForm.aspx"));
		////    return System.Text.Encoding.UTF8.GetString(bb);
		////}

		////[IceWebMethod]
		////public string GetUserForm()
		////{
		////    System.Net.WebClient wc = new System.Net.WebClient();
		////    byte[] bb = wc.DownloadData(BusinessObjects.Common.ApplicationUtility.GetAbsoluteLink("Forms/UserForm.aspx"));
		////    return System.Text.Encoding.UTF8.GetString(bb);
		////}

		[IceWebMethod]
		public UserAccountData GetUserAccountFormWithData()
		{
			System.Net.WebClient wc = new System.Net.WebClient();
			wc.Headers.Add("Cookie", HttpContext.Current.Request.Headers["Cookie"] + ""); // to enable user authentication - we need it to disable Username field in UserData.aspx
			byte[] bb = wc.DownloadData(BusinessObjects.Common.ApplicationUtility.GetAbsoluteLink("Forms/UserAccountForm.aspx"));

			UserAccountData account = GetUserAccountData();
			account.Form = System.Text.Encoding.UTF8.GetString(bb);
			return account;
		}

		[IceWebMethod]
		public UserAccountData GetUserAccountData()
		{
			UserAccountData account = new UserAccountData();
			account.User = GetCurrentUser();
			account.User.Email += ""; // remove null
			account.User.Username += ""; // remove null
			account.User.Password = ""; // don't send password
			account.Address = GetCurrentUserAddress();
			return account;
		}

		[IceWebMethod]
		public CheckoutAddresses GetCheckoutAddressesFormWithData()
		{
			System.Net.WebClient wc = new System.Net.WebClient();
			byte[] bb = wc.DownloadData(BusinessObjects.Common.ApplicationUtility.GetAbsoluteLink("Forms/CheckoutAddressesForm.aspx"));
			CheckoutAddresses addresses = GetCheckoutAddresses();
			addresses.Form = System.Text.Encoding.UTF8.GetString(bb);
			return addresses;
		}

		[IceWebMethod]
		public CheckoutAddresses GetCheckoutAddresses()
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			return new CheckoutAddresses(cart.Order);
		}

		/// <summary>
		/// Update CountryID and StateID for main address which are used for tax calculation
		/// </summary>
		/// <param name="billing"></param>
		private void UpdateAccountAddress(Address billing)
		{
			if (billing == null)
			{
				return;
			}

			if (VISTACARTHelper.CurrentUser.UserTicket.IsAuthenticated)
			{
				Address address = VISTACARTHelper.CurrentUser.User.GetAddress();
				if (address.AddressID > 0)
				{
					if (billing.CountryID != address.CountryID || billing.StateID != address.StateID)
					{
						address.StateID = billing.StateID;
						address.CountryID = billing.CountryID;
						address.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

						ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
						cart.SaveCartWithRecalculate();

					}
				}
			}
		}

		[IceWebMethod]
		public GUserResult UpdateCheckoutAddresses(Address billing, Address shipping)
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			if (cart.Order.BillingAddressID > 0 && cart.Order.BillingAddressID == billing.AddressID)
			{
				UpdateAccountAddress(billing);

				billing.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				if (cart.Order.ShippingAddressID > 0 && cart.Order.ShippingAddressID == shipping.AddressID)
				{
					shipping.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

					return new GUserResult(true, "");
				}
			}
			return new GUserResult(false, "Can't update database.");
		}

		[IceWebMethod]
		public GUserResult UpdateCheckoutAddressesV2(Address billing, Address shipping, User user, bool userDataProvided)
		{
			if (VISTACARTHelper.CurrentUser.UserTicket.IsAuthenticated)
			{
				// user is null!!!
				user = VISTACARTHelper.CurrentUser.User;
			}
			else
			{
				if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.IsAllowGuestCheckout == false)
				{
					return new GUserResult(false, "Guest checkout forbidden!");
				}

				string email = "";

				// user is not null !!!
				if (userDataProvided)
				{
					// will create account with given username, password and email
					email = user.Email;
				}
				else
				{
					// try to load user object by billing.Email
					email = billing.Email;

					// create random username and password
					user.Username = (DateTime.Now.Ticks / 100) + "";
					user.Password = (DateTime.Now.Ticks / 254) + "";
					// use email from billing address
					user.Email = billing.Email;
				}

				User existingUser = Ecommerce.BusinessObjects.Database.VISTACART.User.LoadByEmail(email);
				if (existingUser != null)
				{
					// account with that email exist, use existingUser account
					// do not create account
					user = existingUser;
				}
				else
				{
					GUserResult result = UserAccountCreateOrUpdate(user, billing, false);
					if (result.Success == false)
					{
						return result;
					}
				}

				VISTACARTHelper.CurrentUser.UserTicket.SignIn(user, false);
			}

			if (user.UserID < 1)
			{
				// can't create user account
				return new GUserResult(false, Message.GetContent(Message.NameEnum.ServerInternalError));
			}

			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			cart.Order.UserID = user.UserID;

			if (cart.Order.BillingAddressID > 0)
			{
				billing.AddressID = cart.Order.BillingAddressID.GetValueOrDefault();
			}
			if (cart.Order.ShippingAddressID > 0)
			{
				shipping.AddressID = cart.Order.ShippingAddressID.GetValueOrDefault();
			}

			billing.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			shipping.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

			{
				cart.Order.BillingAddressID = billing.AddressID;
				cart.Order.ShippingAddressID = shipping.AddressID;
				cart.Order.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					return new GUserResult(true, "");
				}
			}

			// Can't update database.
			return new GUserResult(false, Message.GetContent(Message.NameEnum.ServerInternalError));

		}

		[IceWebMethod]
		public string GetOrderDetailHtml(int orderID)
		{
			// load order, if belongs to current user, show it
			Order order = new Order();
			if (order.LoadFromDB(orderID))// && order.UserID == VISTACARTHelper.CurrentUser.UserTicket.UserID)
			{
				return "<div class='orderNumber'>" +
					Message.GetContent(Message.NameEnum.OrderNumber) + ": " + orderID + "</div>" +
					Modules.HtmlElements.ShoppingCartHtmlBuilder.GetHtml(order, false);
			}
			else
			{
				Logging.WriteError("Order doesn't belong to current user or doesn't exist. Requested OrderID=" + orderID + ", UserID=" + VISTACARTHelper.CurrentUser.UserTicket.UserID);
				return "Order doesn't belong to current user or doesn't exist.";
			}
		}

		[IceWebMethod]
		public PaymentGateways.PGResult GetPGHandleCode()
		{
#if !TRIAL
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			return PaymentGateways.PGHelper.GetPGRedirectionFormWithParameters(cart);
#else
			throw new Exception("*********************************\r\nTRIAL VERSION, CHECKOUT DISABLED!\r\n*********************************");
#endif
		}

		[IceWebMethod]
		public GUserResult GetCVVExplanation()
		{
			return new GUserResult(true, WebPage.GetContent(WebPage.NameEnum.CVVExplanation));
		}

		[IceWebMethod]
		public PaymentGateways.PGResult ChargeAIMMethodCC(PaymentGateways.CreditCardInfo ccInfo)
		{
			ShoppingCart cart = VISTACARTHelper.CurrentUser.ShoppingCart;
			return PaymentGateways.PGHelper.ChargeAIMMethod(ccInfo, cart);
		}

		[IceWebMethod]
		public WebPage GetWebPageByID(int webPageID)
		{
			WebPage wp = new WebPage();
			if (wp.LoadFromDB(webPageID) && wp.IsActive)
			{
				return wp;
			}
			else
			{
				throw new Exception("Forbidden!");
			}
		}

		[IceWebMethod]
		public WebPage GetWebPageByName(string name)
		{
			WebPage wp = WebPage.LoadFromDBByName(name);
			if (wp != null && wp.IsActive)
			{
				return wp;
			}
			else
			{
				throw new Exception("Forbidden!");
			}
		}

		[IceWebMethod]
		public GUserResult ContactUsSendEmail(EmailObject emailObject)
		{
			if (ValidationUtility.IsEmailValidFormat(emailObject.Email) == false)
			{
				return new GUserResult(false, "Invalid email!");
			}
			string strbody = @"Web question from
name: " + emailObject.Name + @"
email: " + emailObject.Email + @"

" + emailObject.Message;

			System.Collections.Generic.List<String> failedEmails = EmailSenderImpl.SendEmails(emailObject.Subject,
				strbody,
				false,
				null,
				string.IsNullOrEmpty(VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.WebContactFormFromEmail) ? 
						emailObject.Email : 
						VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.WebContactFormFromEmail,
				emailObject.Name,
				new String[] { emailObject.Email, BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.WebContactFormToEmail });

			if (failedEmails.Count == 0)
			{
				return new GUserResult(true, "Message sent. We will contact you shortly.");
			}
			else
			{
				return new GUserResult(false, "Message can't be sent right now. Is your email correct?");
			}
		}

		[IceWebMethod]
		public bool SetOnlineDemoSkin(string skinDirectory)
		{
			return OnlineDemoSkinHelper.SetSkinDirectory(skinDirectory);
		}

	}
}
