﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Web.Services;
using System.Collections.Generic;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Gallery;
using System.IO;
using System.Text.RegularExpressions;
using Ice.Orm.Core.Entity;

namespace Ice.Net.Ecommerce.Web.Admin.VISTACART
{
	public partial class FileUpload : System.Web.UI.Page
	{
		protected void Page_Load(object sender, EventArgs e)
		{
			// if user not authenticated, redirect to login.aspx
			User user = new User();
			if (VISTACARTHelper.CurrentUser.AdminTicket.IsAuthenticated
				&& user.LoadFromDB(VISTACARTHelper.CurrentUser.AdminTicket.UserID)
				&& user.IsActive
				&& user.IsInRole(Role.NameEnum.Administrators))
			{
				// ok
			}
			else
			{
				Response.Write("_not_authenticated_");
				Response.End();
			}


			// FILE UPLOAD
			if (Request.RequestType.ToUpper() == "POST" && Request.Files.Count > 0)
			{
				Response.ContentType = "text/html";

				// get file
				string destinationDir = Request.Form["destinationDir"] + "";
				string fileName = Request.Files[0].FileName;
				//get fileName only without path
				fileName = fileName.Substring(fileName.LastIndexOf("\\") + 1);

				if (fileName.ToLower().EndsWith(".csv"))
				{
					// csv import, read csv content
					StreamReader sr = new StreamReader(Request.Files[0].InputStream);
					string csv = sr.ReadToEnd();
					Response.Write(ImportCSVProducts(csv));
					return;
				}
				else if (GalleryImage.IsSupported(fileName))
				{
					try
					{
						Request.Files[0].SaveAs(destinationDir + fileName);
						Response.Write("ok");
						return;
					}
					catch (Exception ex)
					{
						Response.Write("error" + "<br/>_message:" + ex.Message + "<br/>_stackTrace:" + ex.StackTrace);
						return;
					}
				}
				else
				{
					Response.Write("error" + "<br/> _message: File type not supported.");
				}

				return;
			}
		}

		/// <summary>
		/// Returns message
		/// </summary>
		/// <param name="csv"></param>
		/// <returns></returns>
		private string ImportCSVProducts(string csv)
		{
			// must return "ok"... if ok

			CSVDocument csvDoc = new CSVDocument(csv);

			DataTable schemaTable = EntitySetting.Get(typeof(Product)).SchemaTableHelper.SchemaTable; // database schema table
			int count = 0;
			int imgcount = 0;
			foreach (DataRow csvDr in csvDoc.Rows)
			{
				// create product object
				Product prod = new Product();
				List<string> images = new List<string>();

				foreach (DataColumn col in csvDoc.Columns)
				{
					if (col.ColumnName.StartsWith("ImageLocation"))
					{
						if (!string.IsNullOrEmpty(csvDr[col.ColumnName] + ""))
						{
							images.Add(csvDr[col.ColumnName] + "");
						}
					}
					else
					{
						DataColumn schCol = schemaTable.Columns[col.ColumnName];

						try
						{
							// convert data type
							if (schCol.DataType == typeof(DateTime))
							{
								if (!string.IsNullOrEmpty(csvDr[col.ColumnName] + ""))
								{
									// parse date time
									prod[schCol.ColumnName] = DateTime.Parse(csvDr[col.ColumnName] + "");
								}
							}
							else
							{
								prod[schCol.ColumnName] = Convert.ChangeType(csvDr[col.ColumnName], schCol.DataType);
							}
						}
						catch
						{
						}
					}
				}

				// save product
				prod.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				if (images.Count > 0)
				{
					count++;

					// save images locations
					foreach (string imgLoc in images)
					{
						ProductImage pimg = new ProductImage();
						pimg.ProductID = prod.ProductID;
						pimg.ImageLocation = imgLoc;
						pimg.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

						imgcount++;
					}
				}
			}

			return @"<html><body>csv import complete 

Products imported: " + count + @"
Images imported: " + imgcount + "</body></html>";
		}
	}

	public class CSVDocument : DataTable
	{
		public CSVDocument(string csvContent)
		{
			LoadCSV(csvContent);
		}

		private void LoadCSV(string csvContent)
		{
			string[] rows = csvContent.Split(new Char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);

			if (rows.Length > 0)
			{
				CreateColumns(rows[0]);
			}

			for (int n = 1; n < rows.Length; n++)
			{
				CreateRow(rows[n]);
			}
		}

		private string[] GetValues(string rowTxt)
		{
			List<string> values = new List<string>();

			rowTxt = "," + rowTxt.Replace("\"\"", "~~"); // temporary, to enable regex to work correctly

			Regex re = new Regex("((\\,\\\"(?<s1>[^\\\"]+)\\\")|(\\,(?<s1>[^\\,]*)))");

			MatchCollection mc = re.Matches(rowTxt);

			foreach (Match m in mc)
			{
				if (m.Groups["s1"].Success)
				{
					values.Add(m.Groups["s1"].Value.Replace("~~", "\""));
				}
			}
			return values.ToArray();
		}

		private void CreateColumns(string titleRow)
		{
			string[] columns = GetValues(titleRow);
			foreach (string col in columns)
			{
				this.Columns.Add(new DataColumn(col));
			}
		}

		private void CreateRow(string row)
		{
			string[] values = GetValues(row);
			this.Rows.Add(values);
		}


	}
}