﻿//******** NO EXTERNAL DEPENDENCIES ********

if (typeof (Ice) == "undefined") { Ice = {}; }

Ice.FormSerialization = {};

Ice.FormSerialization.SerializeHtmlControls = function (htmlContainer, requiredClass, nameValueDataObject)
{
	///<summary>Create object with Name-Value members</summary>
	///<param name="htmlContainer" type="Object|String">Element or elementId holding elements to be serialized</param>
	///<param name="requiredClass" type="String|Null">Elements marked with this class are required - field can't be empty. If requiredCalss == null, it won't test for required fields.</param>
	///<param name="nameValueDataObject" type="Object|Null" optional="true">Object with data to be updated. If null, new object is created.</param>	
	///<returns type="Object|Null">Name:value collection or null if requiredClass is defined but all required fields are not set</returns>

	if (htmlContainer == null || typeof (htmlContainer) != "object")
	{
		htmlContainer = document.getElementById(htmlContainer);
		if (htmlContainer == null || typeof (htmlContainer) != "object")
		{
			alert(htmlContainer + " element doesn't exist!");
			return;
		}
	}

	function IsValidIfRequired(el, value)
	{
		if (requiredClass == null || requiredClass.length == 0)
		{
			return true;
		}

		if ((value == null || (value + "").length == 0) && el.className.indexOf(requiredClass) >= 0)
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	function GetValue(element)
	{
		var name = element.name;
		var finalValue = null; // output
		var tagName = element.tagName.toUpperCase();
		var type = element.type.toLowerCase();

		switch (tagName)
		{
			case "INPUT":
				switch (type)
				{
					case "text":
					case "password":
					case "hidden":
						finalValue = element.value;
						break;
					case "checkbox":
						finalValue = element.checked;
						break;
					case "radio":
						if (element.checked)
						{
							finalValue = element.value;
						}
						else
						{
							return false;
						}
						break;
				}
				break;

			case "TEXTAREA":
				finalValue = element.value;
				break;

			case "SELECT":
				switch (type)
				{
					case "select":
					case "select-one":
						if (element.selectedIndex >= 0)
						{
							finalValue = element.options[element.selectedIndex].value;
						}
						break;

					case "select-multiple":
						var selectedValues = [];
						var optionCount = element.options.length;
						for (var j = 0; j < optionCount; j++)
						{
							var option = element.options[j];
							if (option.selected)
							{
								selectedValues.push(option.value);
							}
						}
						if (selectedValues.length == 1)
						{
							finalValue = selectedValues[0];
						}
						else if (selectedValues.length > 0)
						{
							finalValue = selectedValues;
						}
						break;
				}
				break;
		}
		return { "ok": true, "value": finalValue };
	}

	function SerializeInputObjects(objCollection)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="objCollection">array of inputs, textareas</param>
		/// <returns type=""></returns>
		if (objCollection != null && objCollection.length > 0)
		{
			for (var n = 0; n < objCollection.length; n++)
			{
				var element = objCollection[n];
				var name = element.name;
				if (name == null || name.length == 0)
				{
					continue;
				}
				var retObj = GetValue(element); // returns false or object

				if (IsValidIfRequired(element, retObj.value) == false)
				{
					return false;
				}
				else if (retObj)
				{
					finalObject[name] = retObj.value;
				}
			}
			return true;
		}
		else
		{
			return true;
		}
	}

	var inps = htmlContainer.getElementsByTagName("input");
	var textareas = htmlContainer.getElementsByTagName("textarea");
	var selects = htmlContainer.getElementsByTagName("select");

	var finalObject = null;
	if (nameValueDataObject != null)
	{
		finalObject = nameValueDataObject; // update existing object
	}
	else if (htmlContainer.nameValueDataObject != null)
	{
		finalObject = htmlContainer.nameValueDataObject; // update existing object, previously used to populate controls with data
	}
	else
	{
		finalObject = {};
	}

	if (SerializeInputObjects(inps) && SerializeInputObjects(textareas) && SerializeInputObjects(selects))
	{
		return finalObject;
	}
	else
	{
		return null;
	}
};

Ice.FormSerialization.PopulateHtmlControls = function (htmlContainer, nameValueDataObject, keepNameValueDataObject)
{
	///<summary>Populate html controls with data in nameValueDataObject</summary>
	///<param name="htmlContainer" type="Object|String">Element or elementId holding elements to be populated</param>
	///<param name="nameValueDataObject" type="Object">object with name:value data</param>
	///<param name="keepNameValueDataObject" type="Bool">if true, nameValueDataObject will be bound to htmlContainer as property, it may be used later for posting back to server</param>

	if (htmlContainer == null || typeof (htmlContainer) != "object")
	{
		htmlContainer = document.getElementById(htmlContainer);
		if (htmlContainer == null || typeof (htmlContainer) != "object")
		{
			alert("Holder element doesn't exist!");
			return;
		}
	}

	if (keepNameValueDataObject)
	{
		htmlContainer.nameValueDataObject = nameValueDataObject;
	}

	function GetElement(tagName, name)
	{
		var elements = htmlContainer.getElementsByTagName(tagName);
		var namedElements = [];

		if (elements.length > 0)
		{
			for (var n = 0; n < elements.length; n++)
			{
				if (elements[n].name == name)
				{
					namedElements.push(elements[n]); // multiple elements may have the same name (radio buttons group!!!)
				}
			}
		}
		return namedElements;
	}

	function SelectValue(selectEl, value)
	{
		for (var n = 0; n < selectEl.options.length; n++)
		{
			if (selectEl.options[n].value == value)
			{
				selectEl.options[n].selected = true;
			}
		}
	}

	function Populate(element, value)
	{
		var tagName = element.tagName.toUpperCase();
		var type = element.type.toLowerCase();

		switch (tagName)
		{
			case "INPUT":
				switch (type)
				{
					case "text":
					case "password":
					case "hidden":
						element.value = value == null ? "" : value; // remove 'null'
						break;
					case "checkbox":
						element.checked = value;
						break;
					case "radio":
						if (element.value == value)
						{
							element.checked = true;
						}
						else
						{
							element.checked = false;
						}
						break;
				}
				break;

			case "TEXTAREA":
				element.value = value == null ? "" : value; // remove 'null'
				break;

			case "SELECT":
				switch (type)
				{
					case "select":
					case "select-one":
						//case "select-multiple": // correct way, but can't have JS object with multiple properties with the same property name
						// set value
						SelectValue(element, value);
						break;

					case "select-multiple":
						// deselect all first
						for (var n = 0; n < element.options.length; n++)
						{
							element.options[n].selected = false;
						}
						// select specified only
						if (value != null && value.shift != null) // is it array?
						{
							for (var n = 0; n < value.length; n++)
							{
								// set multiple values
								SelectValue(element, value[n]);
							}
						}
						else
						{
							// set value
							SelectValue(element, value);
						}

						break;
				}
				break;

		}
	}

	for (var prop in nameValueDataObject)
	{
		var elements = GetElement("input", prop); // returns array of elements with name==prop (radio buttons group!!!)

		if (elements.length == 0)
		{
			elements = GetElement("textarea", prop);
			if (elements.length == 0)
			{
				elements = GetElement("select", prop);
			}
		}

		if (elements.length > 0)
		{
			for (var index = 0; index < elements.length; index++)
			{
				Populate(elements[index], nameValueDataObject[prop]);
			}
		}
		else
		{
			////alert("Element with name='" + prop + "' not found.");
		}
	}
};


