﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;

namespace Ice.Orm.Core.Criteria
{

	public class QParameter
	{
		public QParameter()
		{
		}

		public QParameter(object value) 
		{ 
			this.Value = value; 
		}

		public QParameter(object value, string columnName)
		{
			SetColumn(columnName);
			this.Value = value;
		}

		/// <summary>
		/// Parameter name, usually @px
		/// </summary>
		public string Name { get; set; }

		/// <summary>
		/// Parameter value
		/// </summary>
		public object Value { get; set; }

		private string _column;

		/// <summary>
		/// Database column name
		/// </summary>
		public string Column { get { return _column; } set { SetColumn(value); } }

		public string Table { get; set; }

		/// <summary>
		/// accepts [dbo]..[Table1].[Column1] and create Column and Table names
		/// </summary>
		/// <param name="fullSqlName"></param>
		/// <returns></returns>
		private QParameter SetColumn(string fullSqlName)
		{
			this._column = null;
			if (!string.IsNullOrEmpty(fullSqlName))
			{
				string[] arr = SqlName.RemoveQuotes(fullSqlName).Split('.');
				this._column = arr[arr.Length - 1];
				if (arr.Length > 1)
				{
					this.Table = arr[arr.Length - 2];
				}
			}
			return this;
		}

		public string FullSqlColumn
		{
			get
			{
				string str = new []{this.Table, this.Column}.Where(o=>!string.IsNullOrEmpty(o)).Select(o=>"[" + o + "]").ToStringJoined(".");
				return string.IsNullOrEmpty(str) ? null : str;
			}
		}
	}
}
