﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Linq.Expressions;
using System.Linq;
using System.Reflection;
using Ice.Orm.Core.DataStore;
using System.IO;
using Ice.Orm.Core.LINQ;
using Ice.Orm.Core.Criteria.SqlBlocks;
using System.Data;

namespace Ice.Orm.Core.Criteria
{
	[System.Diagnostics.DebuggerDisplay("{_query.QueryWithValues}")]
	public class Query : ICompilableQuery
	{
		internal QueryBase _query = new QueryBase();

		public Query Skip(int skip)
		{
			_query.SkipCount = skip;
			return this;
		}

		public Query Take(int? count)
		{
			_query.TakeCount = count;
			return this;
		}

		public Query Select(params SqlName[] names)
		{
			_query.GetSqlBlock<SqlSelectBlock>().AddNames(names);
			return this;
		}

		public Query From(SqlName fromTable)
		{
			_query.WriteToFromBlock(fromTable);
			return this;
		}

		public Query Delete(SqlName fromTable)
		{
			_query.WriteToDeleteBlock(fromTable);
			return this;
		}

		public Query Count(SqlName fromTable)
		{
			_query.WriteToCountBlock(fromTable);
			return this;
		}
		
		public Query CreateQuery(string sql, params object[] args)
		{
			SqlCommandBlock cmd = _query.GetSqlBlock<SqlCommandBlock>();
			cmd.AddTextFormated(sql, args);
			return this;
		}

		public Query CreateQuery2(params object[] args)
		{
			_query.GetSqlBlock<SqlCommandBlock>().AddArguments(args);
			return this;
		}

		public Query NewLine(int lines = 1)
		{
			for (int n = 0; n < lines; n++)
			{
				_query.GetSqlBlock<SqlCommandBlock>().Add("\r\n");
			}
			return this;
		}

		public Query OrderBy(SqlName columnName, bool descending = false)
		{
			_query.GetSqlBlock<SqlOrderByBlock>().AddColumn(columnName, descending);
			return this;
		}

		public Query Where2(params object[] args)
		{
			SqlCommandBlock cmd = new SqlCommandBlock();
			cmd.Add("(");
			cmd.AddArguments(args); // obavezno kreirati zagrade okolo jer je ovo zasebni Where(...). ovo je bio bug kad nije bilo zagrada
			cmd.Add(")");
			_query.GetSqlBlock<SqlWhereBlock>().AddSegments(cmd.SelectSqlSegments(), false);
			return this;
		}

		public Query Update2(SqlName tableName, params QParameter[] args)
		{
			args.ForEach(o => o.Name = null); // force new name to be created
			_query.GetSqlBlock<SqlUpdateBlock>().Add(tableName, args);
			return this;
		}

		public Query LeftJoin(SqlName name, SqlName onA, SqlName equalsB)
		{
			_query.WriteToJoinBlock(name, onA, equalsB, JoinKindEnum.LEFT);
			return this;
		}

		public Query RightJoin(SqlName name, SqlName onA, SqlName equalsB)
		{
			_query.WriteToJoinBlock(name, onA, equalsB, JoinKindEnum.RIGHT);
			return this;
		}

		public Query InnerJoin(SqlName name, SqlName onA, SqlName equalsB)
		{
			_query.WriteToJoinBlock(name, onA, equalsB, JoinKindEnum.INNER);
			return this;
		}

		public Query Where(SqlName columnName, object value)
		{
			return this.Where(columnName, value, DBWhereKeyOperator.Equals);
		}

		public Query Where(SqlName columnName, object value, DBWhereKeyOperator comparisonOperator)
		{
			return this.Where(columnName, value, comparisonOperator, DBWhereKeyLogic.And);
		}

		public Query Where(SqlName columnName, object value, DBWhereKeyOperator comparisonOperator, DBWhereKeyLogic logicOperator)
		{
			return this.Where(columnName, value, comparisonOperator, logicOperator, DBWhereBrackets.NotUsed);
		}

		public Query Where(SqlName columnName, object value, DBWhereKeyOperator comparisonOperator, DBWhereKeyLogic logicOperator, DBWhereBrackets brackets)
		{
			return this.Where(columnName, value, comparisonOperator, logicOperator, brackets, false);
		}

		public Query SetDataStore(DataStore.DataStoreBase dataStore)
		{
			_query.DataStore = dataStore;
			return this;
		}

		public Query SetDataStore(string dataStoreName)
		{
			SetDataStore(DataStoreCollection.GetByDatabaseName(dataStoreName));
			return this;
		}

		/// <summary>
		/// Create where statement
		/// </summary>
		/// <param name="columnName">database column</param>
		/// <param name="value">value</param>
		/// <param name="comparisonOperator"></param>
		/// <param name="logicOperator"></param>
		/// <param name="brackets"></param>
		/// <param name="valueFirst">if true, create reversed statement: [value] = [columnName]</param>
		/// <returns></returns>
		private Query Where(SqlName columnName, 
			object value, 
			DBWhereKeyOperator comparisonOperator = DBWhereKeyOperator.Equals, 
			DBWhereKeyLogic logicOperator = DBWhereKeyLogic.And, 
			DBWhereBrackets brackets = DBWhereBrackets.NotUsed, 
			bool valueFirst = false,
			string customWhereText = null)
		{
			SqlCommandBlock cmd = new SqlCommandBlock();

			cmd.Add("("); // obavezno kreirati zagrade oko statementa, ovo je bio bug kad nije bilo zadradi jer je Where(...) zasebni statement

			cmd.Add(DBWhereKey.GetBrackets(brackets, true));

			object o = value is QParameter ? ((QParameter)value).Value : value;

			bool noParameter = false;
			string compStr = DBWhereKey.GetComparisonOperator(comparisonOperator, DBWhereKey.IsValueNull(value), ref noParameter);

			if (customWhereText != null)
			{
				cmd.Add(customWhereText);
			}
			else
			{
				if (valueFirst && !noParameter && o != null) // if value is nul, don't reverse, always create sql "name is null"
				{
					CreateWhereParameters(cmd, comparisonOperator, o);
					cmd.Add(compStr);
					cmd.Add(columnName);
				}
				else
				{
					cmd.Add(columnName);
					cmd.Add(compStr);
					if (!noParameter)
					{
						CreateWhereParameters(cmd, comparisonOperator, o);
					}
				}
			}

			cmd.Add(DBWhereKey.GetBrackets(brackets, false));

			cmd.Add(")");

			// save current and restore previous
			SqlWhereBlock where = _query.GetSqlBlock<SqlWhereBlock>();
			where.AddSegments(cmd.SelectSqlSegments(), logicOperator == DBWhereKeyLogic.Or);
			
			return this;
		}

		private void CreateWhereParameters(SqlCommandBlock cmd, DBWhereKeyOperator comparisonOperator, object value)
		{
			if (comparisonOperator == DBWhereKeyOperator.In || comparisonOperator == DBWhereKeyOperator.NotIn)
			{
				cmd.Add("(");

				int inCount = 0;
				foreach (object v in (value as Array))
				{
					if (inCount++ > 0)
					{
						cmd.Add(",");
					}
					cmd.AddParameter(v);
				}
				cmd.Add(")");
			}
			else
			{
				if (value is SqlName) // column name
				{
					cmd.Add((SqlName)value);
				}
				else
				{
					cmd.AddParameter(DBWhereKey.FixValueForLike(comparisonOperator, value));
				}
			}
		}

		public Query Case(bool condition, Func<Query, Query> ifTrue = null, Func<Query, Query> ifFalse = null)
		{
			if (condition == true && ifTrue != null)
			{
				return ifTrue.Invoke(this);
			}
			else if (condition == false && ifFalse != null)
			{
				return ifFalse.Invoke(this);
			}
			return this;
		}

		/// <summary>
		/// TResult is custom object, not EntityBase
		/// </summary>
		/// <typeparam name="TSource"></typeparam>
		/// <typeparam name="TResult"></typeparam>
		/// <returns></returns>
		public List<TResult> ToList<TSource, TResult>() where TSource : Entity.EntityBase
		{
			return _query.ToList<TSource, TResult>();
		}

		public List<TResult> ToList<TResult>() where TResult : Entity.EntityBase
		{
			return _query.ToList<TResult, TResult>();
		}

		public int ExecuteNonQuery<TSource>() where TSource : Entity.EntityBase
		{
			return _query.ExecuteNonQuery<TSource>();
		}

		public TResult ExecuteScalar<TSource, TResult>() where TSource : Entity.EntityBase
		{
			return _query.ExecuteScalar<TSource, TResult>();
		}

		public List<SimpleDataTable> ToSimpleDataSet<TSource>() where TSource : Entity.EntityBase
		{
			return _query.ToSimpleDataSet<TSource>();
		}

		public class MultipleResults<TResult0, TResult1>
		{
			public List<TResult0> Result0 { get; set; }
			public List<TResult1> Result1 { get; set; }
		}

		public List<List<object>> ToMultipleResults<TSource>(params Type[] resultTypes)
		{
			var ds = _query.ToSimpleDataSet<TSource>();
			return ds.Select((o, i) => ObjectMaterializer.Translate(resultTypes[i], ds[i], _query.DataStore)).ToList();
		}

		public MultipleResults<TResult0, TResult1> ToMultipleResults<TSource, TResult0, TResult1>()
		{
			var ds = _query.ToSimpleDataSet<TSource>();
			return new MultipleResults<TResult0, TResult1>()
			{
				Result0 = ObjectMaterializer.Translate<TResult0>(ds[0], _query.DataStore),
				Result1 = ObjectMaterializer.Translate<TResult1>(ds[1], _query.DataStore)
			};
		}

		public Query LogQuery(string fileLocation, bool writeTime, bool simulation)
		{
			_query.WriteLog += new Action<string>((string obj) =>
			{
				File.AppendAllText(fileLocation, obj);
			});

			_query.SimulationMode = simulation;
			_query.WriteTime = writeTime;
			return this;
		}

		public Query AppendQuery(Query query)
		{
			_query.AppendQuery(query._query);
			return this;
		}


		#region ICompilableQuery Members

		IEnumerable<SqlSegment> ICompilableQuery.SelectSqlSegments()
		{
			return _query.SelectSqlSegments();
		}

		CompiledQuery ICompilableQuery.CompileQuery()
		{
			return _query.CompileQuery();
		}

		#endregion

	}

}
