﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;

namespace Ice.Orm.Core.Criteria.SqlBlocks
{
	internal class SqlCommandBlock : ISqlBlock
	{
		public BlockKindEnum Kind { get { return BlockKindEnum.Command; } }
		public IEnumerable<SqlSegment> SelectSqlSegments() { return this._sqlSegments.AsEnumerable(); }
		public bool IsEmpty { get { return !_sqlSegments.Any(); } }
		public void Clear() { this._sqlSegments.Clear(); }

		private SqlSegmentCollection _sqlSegments = new SqlSegmentCollection();

		private string LastColumnName;

		#region add segments

		public virtual void Add(string txt)
		{
			this._sqlSegments.Add(txt);
		}

		public void Add(SqlName name)
		{
			this.LastColumnName = name.ToFullName();
			Add(name.ToFullSqlName());
		}

		public void Add(QParameter parameter)
		{
			this._sqlSegments.Add(parameter);
		}

		public void AddParameter(object p)
		{
			QParameter par;

			// p can be null, it means that p is value
			if (p is QParameter)
			{
				par = p as QParameter;
				if (string.IsNullOrEmpty(par.Column))
				{
					par.Column = this.LastColumnName;
				}
			}
			else
			{
				// p is value
				par = new QParameter(p, this.LastColumnName);
			}

			this.LastColumnName = null; // column name used, don't use it again

			this._sqlSegments.Add(par);
		}

		private void AddSingleArgument(object arg)
		{
			if (arg is QParameter)
			{
				this._sqlSegments.Add((QParameter) arg);
			}
			else if (arg is SqlName)
			{
				// column name
				this.LastColumnName = ((SqlName)arg).ToFullName();
				this._sqlSegments.Add((SqlName)arg);
			}
			else if (arg is ICompilableQuery)
			{
				((ICompilableQuery)arg).SelectSqlSegments().ForEach(o => this._sqlSegments.Add(o));
			}
			else if (arg is IEnumerable && !(arg is string) && !(arg is byte[]))
			{
				// create in (...) statement, every parameter inside (...) has the same column name
				//  save column name as columnName before adding parameter because this.LastColumnName is set to null after parameter is added
				string columnName = this.LastColumnName;
				bool firstAdded = false;
				foreach (object v in (IEnumerable)arg)
				{
					this.LastColumnName = columnName;
					if (firstAdded)
					{
						this._sqlSegments.Add(",");
					}
					AddParameter(v);
					firstAdded = true;
				}
			}
			////else if (arg is string) // string is considered to be value, add as QParameter
			////{
			////    this._sqlSegments.Add((string)arg);
			////}
			else // any other value, create parameter from it
			{
				AddParameter(arg);
			}
		}

		public void AddArguments(params object[] args)
		{
			foreach (object a in args)
			{
				if (a is String)
				{
					string sss = ((string)a);
					if (string.IsNullOrEmpty(sss))
					{
						continue;
					}
					else if (sss == "!=")
					{
						sss = "<>";
					}
					else if (sss == "==")
					{
						sss = "=";
					}
					else if (sss == "&&")
					{
						sss = "and";
					}
					else if (sss == "||")
					{
						sss = "or";
					}

					Add(sss);
				}
				else
				{
					// add as parameter or SqlName
					AddSingleArgument(a);
				}
			}
		}

		/// <summary>
		/// Same as String.Format
		/// </summary>
		/// <param name="sql">text or sql</param>
		/// <param name="args">params, value or QParameter, column name has to be typeof SqlName or it is read as value</param>
		/// <returns></returns>
		public void AddTextFormated(string sql, params object[] args)
		{
			if (args == null || args.Length == 0)
			{
				Add(sql);
			}
			else
			{
				new Regex("({[\\d]+})").Split(sql).ForEach(o =>
				{
					if (o.StartsWith("{") && o.EndsWith("}"))
					{
						AddSingleArgument(args[Convert.ToInt32(o.Trim('{', '}'))]);
					}
					else
					{
						Add(o);
					}
				});
			}
		}

		#endregion


	}
}
