﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace Ice.Orm.Core.Criteria.SqlBlocks
{
	internal class SqlFromBlock : ISqlBlock
	{
		/// <summary>
		/// includes tables in SqlFromBlock and SqlJoinBlock
		/// </summary>
		private Dictionary<string, string> _fromTables = new Dictionary<string, string>();
		private Dictionary<string, string> _joinTables = new Dictionary<string, string>();

		private bool IsTableIncluded(string table)
		{
			return this._fromTables.ContainsKey(table) || this._joinTables.ContainsKey(table);
		}

		/// <summary>
		/// with internal distinct
		/// </summary>
		/// <param name="tableName"></param>
		public void AddTable(SqlName tableName)
		{
			string key = tableName.ToShortName().ToLower();
			if (!IsTableIncluded(key))
			{
				_fromTables[key] = tableName.ToFullSqlName();
			}
		}

		/// <summary>
		/// With internal distinct
		/// </summary>
		/// <param name="joinKind"></param>
		/// <param name="tableName"></param>
		/// <param name="onSql"></param>
		public void AddJoinedTable(JoinKindEnum joinKind, SqlName tableName, string onSql)
		{
			string key = tableName.ToShortName().ToLower();
			if (!IsTableIncluded(key))
			{
				// multiple join blocks can be added, one JOIN in one line, so we have to join them with \r\n between
				_joinTables[key] = joinKind + " JOIN " + tableName.ToFullSqlName() + " ON " + onSql;
			}
		}

		public BlockKindEnum Kind { get { return BlockKindEnum.From; } }
		public bool IsEmpty { get { return !_fromTables.Any();}}

		public IEnumerable<SqlSegment> SelectSqlSegments()
		{
			if (!this.IsEmpty)
			{
				yield return new SqlSegment("\r\n");
				yield return new SqlSegment("FROM " + ("(".Repeat(_joinTables.Count)) + _fromTables.Values.ToStringJoined(", "));

				foreach (string singleJoin in _joinTables.Values)
				{
					yield return new SqlSegment("\r\n");
					yield return new SqlSegment(singleJoin);
					yield return new SqlSegment(")");
				}
			}
		}

		public void Clear() { _fromTables.Clear(); }


	}

}
