﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace Ice.Orm.Core.Criteria.SqlBlocks
{
	internal class SqlWhereBlock : ISqlBlock
	{
		private SqlSegmentCollection _sqlSegments = new SqlSegmentCollection();
		
		public BlockKindEnum Kind { get { return BlockKindEnum.Where; } }
		
		public void Add(string whereSql, bool isORbefore)
		{
			if (string.IsNullOrEmpty(whereSql))
			{
				return;
			}
			if (_sqlSegments.Any())
			{
				_sqlSegments.Add(isORbefore ? "OR" : "AND");
			}
			whereSql = whereSql.TrimStart();
			if (whereSql.StartsWith("where", StringComparison.OrdinalIgnoreCase))
			{
				whereSql = whereSql.Substring(5);
			}
			_sqlSegments.Add("(");
			_sqlSegments.Add(whereSql);
			_sqlSegments.Add(")");
		}

		public void AddSegments(IEnumerable<SqlSegment> segments, bool isORbefore)
		{
			if (_sqlSegments.Any())
			{
				_sqlSegments.Add(isORbefore ? "OR" : "AND");
			}
			segments.AsEnumerable().ForEach(o => _sqlSegments.Add(o));
		}

		public bool IsEmpty { get { return !_sqlSegments.Any(); } }

		public void Clear() { _sqlSegments.Clear(); }

		public IEnumerable<SqlSegment> SelectSqlSegments()
		{
			if (!this.IsEmpty)
			{
				yield return new SqlSegment("\r\n");
				yield return new SqlSegment("WHERE (");

				foreach (SqlSegment obj in _sqlSegments.AsEnumerable())
				{
					yield return obj;
				}

				yield return new SqlSegment(")");
			}
		}


	}
}

