﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Linq;

namespace Ice.Orm.Core.Criteria
{
	public class SqlName
	{
		#region static methods

		/// <summary>
		/// accepts  [GSMS]..[Ice].[Country].[Name] and returns GSMS..Ice.Country.Name
		/// </summary>
		/// <param name="fullSqlName"></param>
		/// <returns></returns>
		public static string RemoveQuotes(string fullSqlName)
		{
			// don't use this code
			if (string.IsNullOrEmpty(fullSqlName))
			{
				return fullSqlName;
			}
			return fullSqlName.Replace("[", "").Replace("]", "");
		}

		/// <summary>
		/// Use full column or table name: "GSMS..Ice.Country.Name", and return [GSMS]..[Ice].[Country].[Name]
		/// //keep two dots: BASE..Table.Column: [BASE]..[Table].[Column]
		/// </summary>
		/// <param name="columnName"></param>
		/// <returns></returns>
		public static string CreateFullSqlName(string fullName)
		{
			if (string.IsNullOrEmpty(fullName))
			{
				return fullName;
			}

			fullName = RemoveQuotes(fullName);

			if (fullName.Contains("."))
			{
				fullName = ("[" + fullName.Replace(".", "].[").Replace(".[].", "..") + "]").Replace("[*]", "*");
			}
			else if (fullName.Contains("*"))
			{
				// do nothing
			}
			else
			{
				fullName = ("[" + fullName + "]");
			}
			return fullName;
		}

		/// <summary>
		/// returns [column1]
		/// </summary>
		/// <param name="fullName"></param>
		/// <returns></returns>
		public static string CreateShortSqlName(string fullName)
		{
			return CreateFullSqlName(CreateShortName(fullName));
		}

		/// <summary>
		/// accepts MyTable.Column1 or [MyTable].[Column1] and returns Column1
		/// </summary>
		/// <param name="fullName"></param>
		/// <returns></returns>
		public static string CreateShortName(string fullName)
		{
			int index = fullName.LastIndexOf('.');
			if (index >= 0)
			{
				fullName = fullName.Substring(index + 1);
			}
			return RemoveQuotes(fullName);
		}

		#endregion

		private SqlName()
		{
		}

		public SqlName(string name)
		{
			if (string.IsNullOrEmpty(name))
			{
				throw new ArgumentNullException("name");
			}
			this.mOriginalName = name; // GSMS..InboxMsg.Column1
		}

		private string mOriginalName; // original

		private string mFullName; // original with removed quotes GSMS..Ice.Country.Column1
		private string mFullSqlName; // [GSMS]..[Ice].[Country].[Column1]
		private string mShortSqlName; // [Column1]
		private string mShortName; // Column1

		/// <summary>
		/// returns string like [GSMS]..[Ice].[Country].[Column1]
		/// </summary>
		public string ToFullSqlName()
		{
			return mFullSqlName ?? (mFullSqlName = SqlName.CreateFullSqlName(this.mOriginalName));
		}

		/// <summary>
		/// returns string like [Column1]
		/// </summary>
		/// <returns></returns>
		public string ToShortSqlName()
		{
			return mShortSqlName ?? (mShortSqlName = SqlName.CreateShortSqlName(this.mOriginalName));
		}

		/// <summary>
		/// returns string like Column1
		/// </summary>
		public string ToShortName()
		{
			return mShortName ?? (mShortName = SqlName.CreateShortName(this.mOriginalName));
		}

		/// <summary>
		/// returns original with removed quotes GSMS..Ice.Country.Column1
		/// </summary>
		/// <returns></returns>
		public string ToFullName()
		{
			return mFullName ?? (mFullName = SqlName.RemoveQuotes(this.mOriginalName)); 
		}

		/// <summary>
		/// returns original with removed quotes GSMS..Ice.Country.Column1. This method must be defined because we use conversion to string in code, eg. db.MadAd.Name + ".*" where db.MadAd.Name is SqlName
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			return this.ToFullName();
		}

		/////// <summary>
		/////// convert from string, use = to assign name: SqlName n = "Column1";
		/////// </summary>
		/////// <param name="name"></param>
		/////// <returns></returns>
		////public static implicit operator SqlName(string name)
		////{
		////    if (string.IsNullOrEmpty(name))
		////    {
		////        return null;
		////    }
		////    else
		////    {
		////        return (new SqlName(name));
		////    }
		////}

		/////// <summary>
		/////// convert into string, usage: string n = c, where c is SqlName
		/////// </summary>
		/////// <param name="c"></param>
		/////// <returns></returns>
		////public static implicit operator String(SqlName c)
		////{
		////    return c== null ? null : c.mSqlNameValue;
		////}



	}

}
