﻿using System;
using System.Collections;
using System.Data;
using System.Data.SqlClient;
using System.Collections.Generic;

namespace Ice.Orm.Core.DataStore.MSSQL
{
	using Criteria;
	using Entity;
	using Schema;
	using System.Collections.ObjectModel;

	public class MSSQLDataStore : DataStoreBase
	{
		/* PAGING EXAMPLE
			 
	select * from (
		select ROW_NUMBER() over(order by Messageid) as _n, * from [message] 
	) as m
	where _n between 100000 and 101000
	*/

		public MSSQLDataStore(DataStoreSetting dbSetting) : base(dbSetting)
		{
		}

		protected override IDbConnection NewDbConnection()
		{
			return new SqlConnection();
		}

		protected override IDbCommand NewDbCommand()
		{
			return new SqlCommand();
		}

		protected override IDbDataAdapter NewDbDataAdapter()
		{
			return new SqlDataAdapter();
		}

		/// <summary>
		/// If true, use named parameters, else use ? instead
		/// </summary>
		/// <returns></returns>
		public override bool UseNamedParameters
		{
			get { return true; }
		}

		/// <summary>
		/// if false, create TOP n, if true create TOP (n)
		/// </summary>
		public override bool ParentherizeTop
		{
			get { return false; } // sql radi i bez zagrada, a u selftestovima ionak nema zagrada
		}

		////protected override bool CanReuseParameters
		////{
		////    get { return true; }
		////}

		protected override IDbDataParameter CreateIDbDataParameter(string parameterName, object value, EntityFieldSetting schemaColumn)
		{
			SqlParameter parameter = new SqlParameter(parameterName, value);
			
			FixDbParameterValueAndType(parameter, schemaColumn);

			if (schemaColumn != null)
			{
				// SQL hack - if we assing DbType = String, it set automatically SqlDbType = NVarChar which is limited to 4000 chars
				if (schemaColumn.ColumnSize > 4000)
				{
					parameter.SqlDbType = SqlDbType.NText;
				}

				////if (schemaColumn.SqlDbType >= 0) // schemaColumn can be null if join is used!!!!
				////{
				////    // this is usefull for insert and update only!
				////    // NEW!!! ProviderType is read from database in SchemaTableHelper: mReaderSchema = schemaTable.ExtendedProperties["ReaderSchemaTable"] as DataTable;
				////    parameter.SqlDbType = schemaColumn.SqlDbType;
				////}

				if (schemaColumn.ProviderType != null)
				{
					parameter.SqlDbType = (SqlDbType)schemaColumn.ProviderType;
				}
			}

			////if (value is DateTime && (DateTime)value < Utility.SqlDateTime.MinValue)
			////{
			////    value = Utility.SqlDateTime.MinValue;
			////}

			return parameter;
		}

		protected override string PostInsertSQL
		{
			get { return "SELECT SCOPE_IDENTITY()"; }
		}

		protected override bool SupportCompoundQuery
		{
			get { return true; }
		}



	}
}
