﻿using System.Data.Odbc;
using System.Data;

namespace Ice.Orm.Core.DataStore.ODBC
{
	using Schema;
	using Ice.Orm.Core.Entity;

	/// <summary>
	/// Generic ODBC data store
	/// </summary>
	public class ODBCDataStore : DataStoreBase
	{
		/// <summary>
		/// If true, use named parameters, else use ? instead
		/// </summary>
		/// <returns></returns>
		public override bool UseNamedParameters
		{
			get { return false; }
		}

		public ODBCDataStore(DataStoreSetting dbSetting) : base(dbSetting)
		{
		}

		protected override IDbDataParameter CreateIDbDataParameter(string parameterName, object value, EntityFieldSetting schemaColumn)
		{
			if (schemaColumn != null && schemaColumn.ColumnSize > 65535) // odbc only
			{
				schemaColumn.ColumnSize = 65535;
			}

			OdbcParameter parameter = new OdbcParameter(parameterName, value);

			FixDbParameterValueAndType(parameter, schemaColumn);

			if(schemaColumn != null)
			{
				////if (schemaColumn.SqlDbType >= 0)
				////{
				////    parameter.OdbcType = Utility.TypeConverters.GetOdbcType(schemaColumn.SqlDbType);
				////}

				// enters here only if schemaColumn != null
				if (schemaColumn.ColumnSize > 255)
				{
					parameter.OdbcType = OdbcType.NText;
				}

				if (schemaColumn.ProviderType != null)
				{
					parameter.OdbcType = (OdbcType)schemaColumn.ProviderType;
				}
			}

			return parameter;
		}

	}


}
