﻿/********************************************************************
 *  System.Data.SQLite.DLL requires Full Trust
********************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Common;
using System.Data.Odbc;
using System.Text.RegularExpressions;
using System.Reflection;
using System.Data;
using Ice.Orm.Core.Utility;
using Ice.Orm.Core.Entity;
using Ice.Orm.Core.Criteria;
using Ice.IO;
using System.IO;

namespace Ice.Orm.Core.DataStore.SQLITE
{
	internal class SQLiteDataStore : DataStoreBase
	{
		public SQLiteDataStore(DataStoreSetting dbSetting)
			: base(dbSetting)
		{
		}

		static Type readerType;
		static Type connectionType;
		static Type dataAdapterType;

		/// <summary>
		/// connection which will be used to create new command only
		/// </summary>
		static IDbConnection mConnection;
		/// <summary>
		/// command which will be used to create new parameter only
		/// </summary>
		static IDbCommand mCommand;

		private static bool initDone = new Func<bool>(() =>
		{
			// System.Data.SQLite.dll ne moze biti refrenciran jer baca exception ukoliko niije Full Trust (asp.net)

			try
			{
				string[] files = null;

				try
				{
					// web application
					files = System.IO.Directory.GetFiles(System.Web.HttpRuntime.BinDirectory, "*.dll", System.IO.SearchOption.AllDirectories);
				}
				catch
				{
					// win application
					files = System.IO.Directory.GetFiles(Path.GetFullPath(Assembly.GetExecutingAssembly().Location), "*.dll", System.IO.SearchOption.AllDirectories);
				}

				string file = files.First(o => o.EndsWith("System.Data.SQLite.dll", StringComparison.OrdinalIgnoreCase));

				Assembly a = Assembly.LoadFrom(file);

				connectionType = a.GetType("System.Data.SQLite.SQLiteConnection");
				readerType = a.GetType("System.Data.SQLite.SQLiteDataReader");
				dataAdapterType = a.GetType("System.Data.SQLite.SQLiteDataAdapter");

				mConnection = (IDbConnection) Activator.CreateInstance(connectionType);
				mCommand = mConnection.CreateCommand();

				return true;
			}
			catch
			{
				throw new Exception("System.Data.SQLite.dll can't be loaded!"); 
			}

		}).Invoke();

		protected override IDbConnection NewDbConnection()
		{
			return (IDbConnection) Activator.CreateInstance(connectionType);
		}

		protected override IDbCommand NewDbCommand()
		{
			return mConnection.CreateCommand();
		}

		protected override IDbDataAdapter NewDbDataAdapter()
		{
			return (IDbDataAdapter)Activator.CreateInstance(dataAdapterType);
		}

		protected override string GetSelectSQLForSchema(string dbTableName)
		{
			return "SELECT * FROM " + SqlName.CreateFullSqlName(dbTableName) +" WHERE 1<>1"; // sqlite doesn't support TOP n, use LIMIT
		}

		/// <summary>
		/// If true, use named parameters, else use ? instead
		/// </summary>
		/// <returns></returns>
		public override bool UseNamedParameters
		{
			get { return true; }
		}

		public override bool UseOffsetAndLimit
		{
			get { return true; }
		}

		protected override IDbDataParameter CreateIDbDataParameter(string parameterName, object value, EntityFieldSetting schemaColumn)
		{
			IDbDataParameter parameter = mCommand.CreateParameter();
			parameter.ParameterName = parameterName;
			parameter.Value = value;

			FixDbParameterValueAndType(parameter, schemaColumn);

			if (schemaColumn != null)
			{
				if (schemaColumn.ProviderType != null)
				{
					parameter.DbType = (System.Data.DbType) schemaColumn.ProviderType;
				}
			}

			return parameter;
		}

		protected override string PostInsertSQL
		{
			get { return "select last_insert_rowid()"; }

		}

		protected override bool SupportCompoundQuery
		{
			get { return true; }
		}

	}
}

