﻿using System;

namespace Ice.Orm.Core.Entity
{
	using Schema;
	using DataStore;
	using System.Collections.Generic;

	[AttributeUsage(AttributeTargets.Class, AllowMultiple = false, Inherited = true)]
	public class EntitySetting : Attribute
	{
		private SchemaTableHelper mSchemaTableHelper;

		/// <summary>
		/// settings created from database schema
		/// </summary>
		public SchemaTableHelper SchemaTableHelper
		{
			get
			{
				if (mSchemaTableHelper == null)
				{
					mSchemaTableHelper = SchemaTableHelper.GetSchemaTableHelper(this.DatabaseName, this.TableName);
				}
				return mSchemaTableHelper;
			}
		}

		private DataStoreBase mDataStore;

		public DataStoreBase DataStore
		{
			get
			{
				if (mDataStore == null)
				{
					// load on demand
					mDataStore = DataStoreCollection.GetByDatabaseName(this.DatabaseName);
				}
				return mDataStore;
			}
		}

		public string DatabaseName { get; private set; }

		public string TableName { get; private set; }

		public EntitySetting(string dbTableName, string dbName)
		{
			this.DatabaseName = dbName;
			this.TableName = dbTableName;
		}

		#region static methods

		public static EntitySetting Get<T>() where T:EntityBase
		{
			return Get(typeof(T));
		}

		/// <summary>
		/// Get existing from static collection, else load new from XML and add to static collection
		/// </summary>
		/// <param name="databaseName"></param>
		/// <returns></returns>
		public static EntitySetting Get(Type entityType)
		{
			return AttributedTableHelper.GetEntitySetting(entityType);
		}

		#endregion

	}
}
