﻿#if DEBUG

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ice.Orm.Core.Entity;
using System.Data;
using Ice.Orm.Core.Criteria;
using Ice.Orm.Core.LINQ.SelfTest.TestClasses;

namespace Ice.Orm.Core.LINQ.SelfTest
{
	public class QueryLinqSelfTests
	{
		/// <summary>
		/// Set to true to skip self test
		/// </summary>
		public static bool SkipSelfTest { get; set; }

		private static void CompareResult(int no, ICompilableQuery query, string expectedSqlWithVales)
		{
			CompiledQuery cc = query.CompileQuery();
			if (cc.QueryWithValues != expectedSqlWithVales)
			{
				try
				{
					System.IO.File.AppendAllText("d:\\QueryLinqSelfTests.cs.result.txt", no + ". FAILED"
					+ "\r\n===========expected with values:\r\n" + expectedSqlWithVales
					+ "\r\n==========current result with values:\r\n" + cc.QueryWithValues+ "\r\n\r\n");
				} 
				catch
				{
				}

				throw new Exception("CallistoORM.QueryLinq self-test failed in test no " + no);
			}

		}

		public static bool RunSelfTest()
		{
			if (SkipSelfTest)
			{
				return false;
			}

			SkipSelfTest = true; // once test is started, don't call it recursively

			CompareResult(1, new Query<ApplicationObject>().Skip(51).Where(o => o.Active).Select(o => new { AppointmentID = o.Id, o.EndDate, o.Location }),
	@"SELECT [ApObj].[AppointmentID], [ApObj].[EndDate], [ApObj].[Location]
FROM [ApObj]
WHERE ([ApObj].[Active] <> 0)");

			CompareResult(2, new Query<ApplicationObject>().Take(5).Where(o => 5 == 6).Select(o => o.ReservationDate),
	@"SELECT TOP 5 [ApObj].[ReservationDate]
FROM [ApObj]
WHERE (0 <> 0)");

			DateTime? mydate = null;

			CompareResult(3, new Query<ApplicationObject>().CreateCount(o=>o.Active).Where(o => o.Published.GetValueOrDefault() && o.EndDate.Value == mydate && o.Published == null && o.Active && (o.Subject.Contains("fsadf"))).Where(o => o.StartDate == mydate && o.Published != true && (o.Subject.Contains("fsadf") || o.Note.Length == 5)),
				@"SELECT COUNT(1)
FROM [ApObj]
WHERE ([ApObj].[Active] <> 0 AND ((((COALESCE([ApObj].[Published], 0) <> 0 AND ([ApObj].[EndDate] IS NULL)) AND ([ApObj].[Published] IS NULL)) AND [ApObj].[Active] <> 0) AND [ApObj].[Subject] LIKE '%' + 'fsadf' + '%') AND ((([ApObj].[StartDate] IS NULL) AND ([ApObj].[Published] <> 1)) AND ([ApObj].[Subject] LIKE '%' + 'fsadf' + '%' OR (LEN([ApObj].[Note]) = 5))))");

			CompareResult(4, new Query<ApplicationObject>().Take(10).Skip(25).Where(o => o.StartDate == o.EndDate).Select(o=>o),
			@"SELECT TOP 35 [ApObj].*
FROM [ApObj]
WHERE (([ApObj].[StartDate] = [ApObj].[EndDate]))");

CompareResult(5, new Query<ApplicationObject>().CreateDelete(o => o.Note.StartsWith("whi")),
	@"DELETE FROM [ApObj]
WHERE ([ApObj].[Note] LIKE 'whi' + '%')");

CompareResult(6, new Query<ApplicationObject>().CreateDelete(o => o.Active && o.Published.HasValue && o.Published.Value != true && (o.Note.Length > 10 || o.Location.Count() > 5 || o.Subject != "fsd")),
	@"DELETE FROM [ApObj]
WHERE (((([ApObj].[Active] <> 0 AND ([ApObj].[Published] IS NOT NULL)) AND ([ApObj].[Published] <> 1)) AND (((LEN([ApObj].[Note]) > 10) OR (LEN([ApObj].[Location]) > 5)) OR ([ApObj].[Subject] <> 'fsd'))))");

CompareResult(7, new Query<ApplicationObject>().CreateUpdate(m => { m.Id = 22; m.Subject = "red"; }, o => o.Published.GetValueOrDefault()),
	@"UPDATE [ApObj]
SET [AppointmentID] = 22, [Subject] = 'red'
WHERE (COALESCE([ApObj].[Published], 0) <> 0)");

CompareResult(8, new Query<ApplicationObject>().CreateInsert(m => { m.Published = true; m.Subject = "green"; }),
	@"INSERT INTO [ApObj]
([Published], [Subject])
VALUES (1, 'green')");

CompareResult(9, new Query<ApplicationObject>().CreateDelete(o => o.Active && o.Published != true && (o.Subject.Length > 10 || o.Note.Count() > 5 || o.Subject == "g")),
	@"DELETE FROM [ApObj]
WHERE ((([ApObj].[Active] <> 0 AND ([ApObj].[Published] <> 1)) AND (((LEN([ApObj].[Subject]) > 10) OR (LEN([ApObj].[Note]) > 5)) OR ([ApObj].[Subject] = 'g'))))");

			var aaa = new int?[] { 43, 63, 5 };
			var bbb = new List<int?> { 21, 63, 5 };

			CompareResult(10, new Query<ApplicationObject>().Where(o => aaa.Contains(o.ChatAnimatorID)).Where(o => bbb.Contains(o.ChatAnimatorID)).Where(o => new List<int?> { 1654, 543, 5 }.Contains(o.ChatAnimatorID)).Where(o => new int?[] { 111, 3, 5 }.Contains(o.ChatAnimatorID)).Where(o => o.StartDate != mydate).Where(o => o.Note.Contains("fsd")).Where(o => o.Published.HasValue || o.EndDate.HasValue != null).Where(o => o.Published.GetValueOrDefault() && o.EndDate.Value == mydate && o.Published == null && o.Active && (o.Subject.Contains("fsadf"))),
				@"WHERE ([ApObj].[ChatAnimatorID] IN (43, 63, 5) AND [ApObj].[ChatAnimatorID] IN (21, 63, 5) AND [ApObj].[ChatAnimatorID] IN (1654, 543, 5) AND [ApObj].[ChatAnimatorID] IN (111, 3, 5) AND ([ApObj].[StartDate] IS NOT NULL) AND [ApObj].[Note] LIKE '%' + 'fsd' + '%' AND (([ApObj].[Published] IS NOT NULL) OR (([ApObj].[EndDate] IS NOT NULL) IS NOT NULL)) AND ((((COALESCE([ApObj].[Published], 0) <> 0 AND ([ApObj].[EndDate] IS NULL)) AND ([ApObj].[Published] IS NULL)) AND [ApObj].[Active] <> 0) AND [ApObj].[Subject] LIKE '%' + 'fsadf' + '%'))");

			CompareResult(11, new Query<ApplicationObject>().Where(o => object.Equals(o.Subject, o.Note) || o.Subject.Equals(o.Note) && string.IsNullOrEmpty(o.Note) && "ggg".Count() > 0 && "fsdag".Contains(o.Subject) && o.Note.Count() > 10 && o.Note.Contains(o.Subject)),
				@"WHERE ((([ApObj].[Subject] = [ApObj].[Note]) OR (((((([ApObj].[Subject] = [ApObj].[Note]) AND ([ApObj].[Note] IS NULL OR [ApObj].[Note] = '')) AND (3 > 0)) AND 'fsdag' LIKE '%' + [ApObj].[Subject] + '%') AND (LEN([ApObj].[Note]) > 10)) AND [ApObj].[Note] LIKE '%' + [ApObj].[Subject] + '%')))");

			CompareResult(12, new Query<ApplicationObject>().Where(o => "ggg".StartsWith("gg") && "fsdag".EndsWith(o.Subject) && o.Note.StartsWith("dvds") && o.Note.Contains(o.Subject)),
				@"WHERE ((((1 <> 0 AND 'fsdag' LIKE '%' + [ApObj].[Subject] + '%') AND [ApObj].[Note] LIKE 'dvds' + '%') AND [ApObj].[Note] LIKE '%' + [ApObj].[Subject] + '%'))");

			CompareResult(13, new Query<ApplicationObject>().Where(o => "ggg".Contains("gg") && "fsdag".Contains(o.Subject) && o.Subject.Contains("dvds") && o.Note.Contains(o.Subject)),
				@"WHERE ((((1 <> 0 AND 'fsdag' LIKE '%' + [ApObj].[Subject] + '%') AND [ApObj].[Subject] LIKE '%' + 'dvds' + '%') AND [ApObj].[Note] LIKE '%' + [ApObj].[Subject] + '%'))");

			CompareResult(14, new Query<ApplicationObject>().Where(o => o.Published.GetValueOrDefault() && o.EndDate.Value == mydate && o.Published == null && o.Active && (o.Subject.Contains("fsadf"))).Where(o => o.StartDate == mydate && o.Published != true && (o.Subject.Contains("fsadf") || o.Note.Length == 5)),
	@"WHERE (((((COALESCE([ApObj].[Published], 0) <> 0 AND ([ApObj].[EndDate] IS NULL)) AND ([ApObj].[Published] IS NULL)) AND [ApObj].[Active] <> 0) AND [ApObj].[Subject] LIKE '%' + 'fsadf' + '%') AND ((([ApObj].[StartDate] IS NULL) AND ([ApObj].[Published] <> 1)) AND ([ApObj].[Subject] LIKE '%' + 'fsadf' + '%' OR (LEN([ApObj].[Note]) = 5))))");

			CompareResult(16, new Query<ApplicationObject>().OrderBy(o => o.Id, true).Select(o => new { o.Active, FirstNameCustom = string.IsNullOrEmpty(o.Location) ? true : false, SecondName = o.Location.Length > 10 ? o.Note : o.Location + "empty", o.EndDate }),
				@"SELECT [ApObj].[Active], (CASE WHEN (([ApObj].[Location] IS NULL OR [ApObj].[Location] = '')) THEN (1) ELSE (0) END) AS [FirstNameCustom], (CASE WHEN ((LEN([ApObj].[Location]) > 10)) THEN ([ApObj].[Note]) ELSE (([ApObj].[Location] + 'empty')) END) AS [SecondName], [ApObj].[EndDate]
FROM [ApObj]
ORDER BY [ApObj].[AppointmentID] DESC");

			CompareResult(17, new Query<HitOglas>().Where(o=>o.AutoMarka.UrlName.Count() > 5).Select(o => new { bb = o.HitKorisnik.HitKorisnikID + 10, dd = o.HitKorisnik, cc = o }),
				@"SELECT ([HitKorisnik].[HitKorisnikID] + 10) AS [bb], [HitKorisnik].*, [HitOglas].*
FROM (([HitOglas]
INNER JOIN [AutoMarka] ON [HitOglas].[AutoMarkaID] = [AutoMarka].[AutoMarkaID])
INNER JOIN [HitKorisnik] ON [HitOglas].[HitKorisnikID] = [HitKorisnik].[HitKorisnikID])
WHERE ((LEN([AutoMarka].[UrlName]) > 5))");

			CompareResult(18, new Query<HitOglas>().Where(o=>o.Country.Currency.IsActive).Select(o => o.HitKorisnik.HitKorisnikID + 10),
				@"SELECT ([HitKorisnik].[HitKorisnikID] + 10)
FROM ((([HitOglas]
INNER JOIN [Currency] ON [Country].[CurrencyID] = [Currency].[CurrencyID])
INNER JOIN [Country] ON [HitOglas].[CountryID] = [Country].[CountryID])
INNER JOIN [HitKorisnik] ON [HitOglas].[HitKorisnikID] = [HitKorisnik].[HitKorisnikID])
WHERE ([Currency].[IsActive] <> 0)");

			CompareResult(19, new Query<HitOglas>().Where(o => o.HitOglasAuto.AutoModelID > 10).Select(o => o.HitKorisnik.Country),
				@"SELECT [Country].*
FROM ((([HitOglas]
INNER JOIN [HitOglasAuto] ON [HitOglas].[HitOglasID] = [HitOglasAuto].[HitOglasID])
INNER JOIN [Country] ON [HitKorisnik].[CountryID] = [Country].[CountryID])
INNER JOIN [HitKorisnik] ON [HitOglas].[HitKorisnikID] = [HitKorisnik].[HitKorisnikID])
WHERE (([HitOglasAuto].[AutoModelID] > 10))");

			HitOglas aaaa1 = new HitOglas(){HitKorisnikID = 23};

			CompareResult(20, new Query<HitOglasAuto>().Where(o => o.HitOglasID == aaaa1.HitKorisnikID.GetValueOrDefault()).Select(o=>new {o.AutoMarka, o.AutoModel, ddd = o.AutoModel.AutoMarka}),
				@"SELECT [AutoMarka].*, [AutoModel].*, [AutoMarka].*
FROM (([HitOglasAuto]
INNER JOIN [AutoMarka] ON [HitOglasAuto].[AutoMarkaID] = [AutoMarka].[AutoMarkaID])
INNER JOIN [AutoModel] ON [HitOglasAuto].[AutoModelID] = [AutoModel].[AutoModelID])
WHERE (([HitOglasAuto].[HitOglasID] = 23))");

			string nn11 = "gsdfds";
			CompareResult(21, new Query<HitOglas>().Where(o => !string.IsNullOrEmpty(nn11) && o.HitKorisnikID > 10),
				@"WHERE ((NOT 0 <> 0 AND ([HitOglas].[HitKorisnikID] > 10)))");

			CompareResult(22, new Query<HitOglas>().Where(o => !string.IsNullOrEmpty("hgsadf") && o.HitKorisnikID > 10),
				@"WHERE ((NOT 0 <> 0 AND ([HitOglas].[HitKorisnikID] > 10)))");

			CompareResult(23, new Query<HitOglas>().Where(o => o.CountryID.GetValueOrDefault(10) > 123),
						"WHERE ((COALESCE([HitOglas].[CountryID], 10) > 123))");

			CompareResult(24, new Query().CreateQuery2("SELECT 1 WHERE EXISTS(", new Query<HitOglas>().Where(o=>o.HitKorisnikID > 5).Select(o=>new {o.CategoryID, o.CloseDate}) ,")"),
				@"SELECT 1 WHERE EXISTS(SELECT [HitOglas].[CategoryID], [HitOglas].[CloseDate]
FROM [HitOglas]
WHERE (([HitOglas].[HitKorisnikID] > 5)))");


			return true;

		}




	}
}


#endif
