﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using Ice.Orm.Core.Criteria;
using System.Collections;
using System.Collections.ObjectModel;
using Ice.Orm.Core.Entity;
using Ice.Orm.Core.Schema;
using Ice.Orm.Core.Criteria.SqlBlocks;

namespace Ice.Orm.Core.LINQ
{
	//*************************************//
	//       Based on IQToolkit2
	//*************************************//

	internal abstract class SqlFormater : ExpressionVisitor
	{
		protected QueryBase query;

		protected string LastColumnName;
		protected QParameter LastParameter;

		Stack<SqlSegmentCollection> _stack = new Stack<SqlSegmentCollection>();

		internal SqlSegmentCollection SqlSegments = new SqlSegmentCollection();
		
		//StringBuilder sb = new StringBuilder();
		//public string Sql { get { return sb.ToString(); } }

		protected void PushWriter()
		{
			_stack.Push(this.SqlSegments);
			SqlSegments = new SqlSegmentCollection();
		}

		internal SqlSegmentCollection PopWriter()
		{
			SqlSegmentCollection curr = this.SqlSegments;
			this.SqlSegments = _stack.Pop();
			return curr;
		}

		protected override Expression Visit(Expression exp)
		{
			return base.Visit(exp);
		}

		protected override Expression VisitUnary(UnaryExpression u)
		{
			string op;

			switch (u.NodeType)
			{
				case ExpressionType.Not:
					op = this.GetOperator(u);
					if (IsBoolean(u.Operand.Type))
					{
						SqlSegments.Add(op);
						SqlSegments.Add(" ");
						return this.VisitPredicate(u.Operand);
					}
					else
					{
						SqlSegments.Add(op);
						return this.VisitValue(u.Operand);
					}

				case ExpressionType.Negate:
				case ExpressionType.NegateChecked:
					op = this.GetOperator(u);
					SqlSegments.Add(op);
					return this.VisitValue(u.Operand);
					
				case ExpressionType.UnaryPlus:
					return this.VisitValue(u.Operand);
					
				case ExpressionType.Convert:
					if(u.Operand.NodeType == ExpressionType.Constant)
					{
						object value = ((ConstantExpression)u.Operand).Value;
						return this.Visit(Expression.Constant(value, u.Type));
					}
					else
					{
						return this.Visit(u.Operand); // we must return converted value
					}
			}

			throw new NotSupportedException(string.Format("The unary operator '{0}' is not supported", u.NodeType));

		}

		protected override Expression VisitBinary(BinaryExpression b)
		{
			string op = this.GetOperator(b);
			Expression left = b.Left;
			Expression right = b.Right;

			if (b.NodeType != ExpressionType.Coalesce)
			{
				SqlSegments.Add("(");
			}

			switch (b.NodeType)
			{
				case ExpressionType.And:
				case ExpressionType.AndAlso:
				case ExpressionType.Or:
				case ExpressionType.OrElse:
				case ExpressionType.Coalesce :
					if (IsBoolean(left.Type))
					{
						this.VisitPredicate(left);
						SqlSegments.Add(" ");
						SqlSegments.Add(op);
						SqlSegments.Add(" ");
						this.VisitPredicate(right);
					}
					else
					{
						this.VisitValue(left);
						SqlSegments.Add(" ");
						SqlSegments.Add(op);
						SqlSegments.Add(" ");
						this.VisitValue(right);
					}

					if (b.NodeType == ExpressionType.Coalesce && !SqlSegments.AsEnumerable().Where(o=>o.Text == "COALESCE(").Any())
					{
						SqlSegments.Insert(0, "COALESCE(");
						SqlSegments.Add(")");
					}
					break;

				case ExpressionType.Equal:
				case ExpressionType.NotEqual:
				case ExpressionType.LessThan:
				case ExpressionType.LessThanOrEqual:
				case ExpressionType.GreaterThan:
				case ExpressionType.GreaterThanOrEqual:
					// check for special x.CompareTo(y) && type.Compare(x,y)
				case ExpressionType.Add:
				case ExpressionType.AddChecked:
				case ExpressionType.Subtract:
				case ExpressionType.SubtractChecked:
				case ExpressionType.Multiply:
				case ExpressionType.MultiplyChecked:
				case ExpressionType.Divide:
				case ExpressionType.Modulo:
				case ExpressionType.ExclusiveOr:

					if (left.NodeType == ExpressionType.MemberAccess &&  AttributedTableHelper.GetAssociation(((MemberExpression)left).Member) != null)
					{
						throw new NotSupportedException("association can't be used in expression: " + b);
					}
					else if (right.NodeType == ExpressionType.MemberAccess && AttributedTableHelper.GetAssociation(((MemberExpression)right).Member) != null)
					{
						throw new NotSupportedException("association can't be used in expression: " + b);
					}


					PushWriter();
					left = this.Visit(left);
					SqlSegmentCollection sbInLeft = PopWriter();
					// depending on left, right and operator
					PushWriter();
					right = this.Visit(right);
					SqlSegmentCollection sbInRight = PopWriter();

					if (left.NodeType == ExpressionType.Constant && ((ConstantExpression)left).Value == null)
					{
						// change left and right if null = o.Age for example to be able to create Age IS NULL
						Expression tmp = right;
						right = left;
						left = tmp;

						SqlSegmentCollection tmpsb = sbInRight;
						sbInRight = sbInLeft;
						sbInLeft = tmpsb;
					}

					if (right.NodeType == ExpressionType.Constant && ((ConstantExpression)right).Value == null)
					{
						this.LastColumnName = null; // parameter is not added, remove column name
						this.SqlSegments.AddRange(sbInLeft);
						this.SqlSegments.Add(b.NodeType == ExpressionType.Equal ? " IS NULL " : " IS NOT NULL ");
					}
					else
					{
						this.SqlSegments.AddRange(sbInLeft);
						SqlSegments.Add(" " + op + " ");
						this.SqlSegments.AddRange(sbInRight);
					}

					// if column name and value are revesed: 10 < UserID, add column name
					if (!string.IsNullOrEmpty(this.LastColumnName) && this.LastParameter != null)
					{
						if (string.IsNullOrEmpty(this.LastParameter.Column))
						{
							this.LastParameter.Column = this.LastColumnName;
							this.LastColumnName = null; // once tmpColumnName is used, remove it
						}
					}

					break;

				default:
					throw new NotSupportedException(string.Format("The binary operator '{0}' is not supported", b.NodeType));
			}

			if (b.NodeType != ExpressionType.Coalesce)
			{
				SqlSegments.Add(")");
			}

			return b;
		}

		private string GetOperator(string methodName)
		{
			switch (methodName)
			{
				case "Add": return "+";
				case "Subtract": return "-";
				case "Multiply": return "*";
				case "Divide": return "/";
				case "Negate": return "-";
				case "Remainder": return "%";
				default: return null;
			}
		}

		private string GetOperator(UnaryExpression u)
		{
			switch (u.NodeType)
			{
				case ExpressionType.Negate:
				case ExpressionType.NegateChecked:
					return "-";
				case ExpressionType.UnaryPlus:
					return "+";
				case ExpressionType.Not:
					return IsBoolean(u.Operand.Type) ? "NOT" : "~";
				default:
					return "";
			}
		}
	
		private string GetOperator(BinaryExpression b)
		{
			switch (b.NodeType)
			{
				case ExpressionType.Coalesce:
					return ",";
				case ExpressionType.And:
				case ExpressionType.AndAlso:
					return (IsBoolean(b.Left.Type)) ? "AND" : "&";
				case ExpressionType.Or:
				case ExpressionType.OrElse:
					return (IsBoolean(b.Left.Type) ? "OR" : "|");
				case ExpressionType.Equal:
					return "=";
				case ExpressionType.NotEqual:
					return "<>";
				case ExpressionType.LessThan:
					return "<";
				case ExpressionType.LessThanOrEqual:
					return "<=";
				case ExpressionType.GreaterThan:
					return ">";
				case ExpressionType.GreaterThanOrEqual:
					return ">=";
				case ExpressionType.Add:
				case ExpressionType.AddChecked:
					return "+";
				case ExpressionType.Subtract:
				case ExpressionType.SubtractChecked:
					return "-";
				case ExpressionType.Multiply:
				case ExpressionType.MultiplyChecked:
					return "*";
				case ExpressionType.Divide:
					return "/";
				case ExpressionType.Modulo:
					return "%";
				case ExpressionType.ExclusiveOr:
					return "^";
				default:
					return "";
			}
		}

		private bool IsBoolean(Type type)
		{
			return type == typeof(bool) || type == typeof(bool?);
		}

		protected bool IsPredicate(Expression expr)
		{
			switch (expr.NodeType)
			{
				case ExpressionType.And:
				case ExpressionType.AndAlso:
				case ExpressionType.Or:
				case ExpressionType.OrElse:
					return IsBoolean(((BinaryExpression)expr).Type);
				case ExpressionType.Not:
					return IsBoolean(((UnaryExpression)expr).Type);
				case ExpressionType.Equal:
				case ExpressionType.NotEqual:
				case ExpressionType.LessThan:
				case ExpressionType.LessThanOrEqual:
				case ExpressionType.GreaterThan:
				case ExpressionType.GreaterThanOrEqual:
					return true;
				case ExpressionType.Call:
					MethodCallExpression mc = (MethodCallExpression)expr;
					if (IsBoolean(mc.Type))
					{
						if (IsMethodName(expr, "GetValueOrDefault"))
						{
							return false;
						}
						else if(IsAllNotNullAndConstantExpression(mc)) // .Where(o=> "aaa".Contains("aa")) translates to 1 <> 0
						{
							return false;
						}
						return true;
					}
					else
					{
						return false;
					}
				case ExpressionType.MemberAccess:
					return IsBoolean(((MemberExpression)expr).Type) && IsPropertyName(expr, "HasValue");
				default:
					return false;
			}
		}

		protected bool IsAllNotNullAndConstantExpression(MethodCallExpression mc)
		{
			if (mc == null)
			{
				return false;
			}

			Expression[] items;
			if(mc.Object != null)
			{
				items = new List<Expression>(){mc.Object}.Concat(mc.Arguments).ToArray();
			}
			else
			{
				items = mc.Arguments.ToArray();
			}

			return items.All(o => o != null && (o.NodeType == ExpressionType.Constant || !HaveParameterExpression(o)));
		}

		protected bool IsPropertyName(Expression expr, params string[] names)
		{
			// handles nullable test o.DateStart.HasValue and creates [DateStart] IS NOT NULL
			MemberExpression ma = expr as MemberExpression;
			return ma != null && names.Where(o => o == ma.Member.Name).Take(1).Count() > 0;
		}

		protected bool IsMethodName(Expression expr, params string [] names)
		{
			// handles GetValueOrDefault() call on bool? type to create <> 0 after if there is no other condition
			MethodCallExpression mc = expr as MethodCallExpression;
			return mc != null && names.Where(o => o == mc.Method.Name).Take(1).Count() > 0;
		}

		protected virtual Expression VisitPredicate(Expression expr)
		{
			this.Visit(expr);
			if (!IsPredicate(expr))
			{
				SqlSegments.Add(" <> 0"); // or = 1
			}
			return expr;
		}

		protected virtual Expression VisitExecutable(Expression expr)
		{
			return this.Visit(Expression.Constant(Expression.Lambda(expr).Compile().DynamicInvoke()));
		}

		protected virtual Expression VisitValue(Expression expr)
		{
			if (IsPredicate(expr))
			{
				SqlSegments.Add("CASE WHEN (");
				this.Visit(expr);
				SqlSegments.Add(") THEN 1 ELSE 0 END");
			}
			else
			{
				this.Visit(expr);
			}
			return expr;
		}

		protected bool IsParameter(Expression expr)
		{
			MemberExpression m = expr as MemberExpression;
			return m!= null && m.Expression != null && m.Expression.NodeType == ExpressionType.Parameter;
		}

		protected bool IsEntityBaseMember(Expression expr)
		{
			MemberExpression m = expr as MemberExpression;
			if (m == null)
			{
				return false;
			}
			return AttributedTableHelper.GetEntityFieldSetting(m.Member) != null;
		}

		protected bool IsEntityBaseItself(Expression expr)
		{
			return AttributedTableHelper.GetEntitySetting(expr.Type) != null;
		}

		protected override Expression VisitMethodCall(MethodCallExpression m)
		{
			// for static methods, m.Object is null, it has arguments to method only
			// for instance methods, m.Object is instance, arguments are arguments to method
			if (!HaveParameterExpression(m.Object) && m.Arguments.All(o => !HaveParameterExpression(o)))
			{
				// nither object or arguments have parameter, it is constant
				return this.VisitExecutable(m);
			}
			else
			{
				// static methods have mc.Object == null
				// chained methods calls are supported:
				//  o.HitKorisnik.AdresaNapomena.Substring(10).Substring(5).Contains("fdsf"))

				// compound methods are supported:
				// .Where(o=>o.Name.Substring(5).Contains("adfs")).Select(o => o.HitKorisnik.AdresaNapomena.Substring(10).Substring(5));

				//// Where(o => new List<int?> { 1, 3, 5 }.Contains(o.ChatAnimatorID)); // parameter is Arguments[1], object is Arguments[0]
				//// Where(o=>o.Name.Contains("fsd')); // Argument[0] is object, Argument[1] is "fds"
				//// general rule: obj is m.Object ?? m.Arguments.First(), argument to method is always m.Arguments.Last()
				//// if it is extension method like .Count(), we have argument[0]
				//// example o=> o.StartDate.GetValueOrDefault()
				//// o=>o.Name.Count()
				//// o=>o.Name.FirstOrDefault()

				return MemberMethodCallResolver(m); // if not resolved in VisitMemberMethodCall(), call this.Visit(child);
			}
		}

		protected Expression MemberMethodCallResolver(MethodCallExpression m)
		{
			//// imporatant: test method name only first, than specific DeclaringType, Count() and Equals() are declared in Enumerable or object but are used on strings too

			//// Contains() used on strings, collection, array, enumerable..., sql is not the same, sql is different for string, all other cases has the same sql, test if it is string first
			//// Count() used on strings and Enumerable, sql is the same in both cases 

			if (m.Method.Name == "GetValueOrDefault" && m.Method.DeclaringType.FullName.StartsWith("System.Nullable"))
			{
				// value or default value or specified value as parameter to GetValueOrDefault(value)
				SqlSegments.Add("COALESCE("); // can't use sql parameter, values has to be created in sql text
				this.Visit(m.Object);
				SqlSegments.Add(", ");
				this.Visit(m.Arguments.Count > 0 ? m.Arguments[0] : Expression.Constant(Activator.CreateInstance(m.Method.ReturnType)));
				SqlSegments.Add(")");
				return m;
			}
			else if (m.Method.Name == "Equals")
			{
				// m.Method.DeclaringType string and object
				SqlSegments.Add("(");
				this.Visit(m.Object ?? m.Arguments.First());
				SqlSegments.Add(" = ");
				this.Visit(m.Arguments.Last());
				SqlSegments.Add(")");
				return m;
			}
			else if (m.Method.Name == "Count")
			{
				//// m.Method.DeclaringType is Enumerable
				//// .Where(o=>o.Name.Count() > 10) // arg is null
				SqlSegments.Add("LEN(");
				this.Visit(m.Object ?? m.Arguments.First()); // Enumerable.Count() is static, so m.Object is null
				SqlSegments.Add(")");
				return m;
			}
			else if (m.Method.Name == "ToString")
			{
				if (m.Object.Type == typeof(string))
				{
					this.Visit(m.Object);  // no op
				}
				else
				{
					SqlSegments.Add("CONVERT(VARCHAR, ");
					this.Visit(m.Object);
					SqlSegments.Add(")");
				}
				return m;
			}
			else if (m.Method.DeclaringType == typeof(string))
			{
				switch(m.Method.Name)
				{
					case "Contains":
					case "StartsWith":
					case "EndsWith":
						this.Visit(m.Object);
						SqlSegments.Add(" LIKE ");
						if(IsEntityBaseMember(m.Object ?? m.Arguments[0]))
						{
							//// .Where(o=> "fadsgsd".Contains(o.Name)) // m.Argument[0]
							//// .Where(o=> o.Name.Contains(o.Subject) // m.Object
							if ("EndsWith, Contains".Contains(m.Method.Name))
							{
								SqlSegments.Add("'%' + ");
							}
							this.Visit(m.Arguments[0]);
							if ("StartsWith, Contains".Contains(m.Method.Name))
							{
								SqlSegments.Add("+ '%'");
							}
							return m;
						}
						else
						{
							//// .Where(o=> o.Name.Contains("fsd");
							SqlSegments.Add("'%' + ");
							this.Visit(m.Arguments[0]);
							SqlSegments.Add("+ '%'");
							return m;
						}
						
					case "IsNullOrEmpty":
						SqlSegments.Add("(");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(" IS NULL OR ");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(" = '')");
						return m;

					case "ToUpper":
						SqlSegments.Add("UPPER(");
						this.Visit(m.Object);
						SqlSegments.Add(")");
						return m;

					case "ToLower":
						SqlSegments.Add("LOWER(");
						this.Visit(m.Object);
						SqlSegments.Add(")");
						return m;

					case "Replace":
						SqlSegments.Add("REPLACE(");
						this.Visit(m.Object);
						SqlSegments.Add(", ");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(", ");
						this.Visit(m.Arguments[1]);
						SqlSegments.Add(")");
						return m;

					case "Substring":
						SqlSegments.Add("SUBSTRING(");
						this.Visit(m.Object);
						SqlSegments.Add(", ");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(" + 1, "); // sql index starts from 1, not 0 like .net
						if (m.Arguments.Count == 2)
						{
							this.Visit(m.Arguments[1]);
						}
						else
						{
							SqlSegments.Add(new QParameter(int.MaxValue));//"8000");
						}
						SqlSegments.Add(")");
						return m;

					case "Remove":
						SqlSegments.Add("STUFF(");
						this.Visit(m.Object);
						SqlSegments.Add(", ");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(" + 1, ");
						if (m.Arguments.Count == 2)
						{
							this.Visit(m.Arguments[1]);
						}
						else
						{
							SqlSegments.Add(new QParameter(int.MaxValue));//"8000");
						}
						SqlSegments.Add(", '')");
						return m;

					case "IndexOf":
						SqlSegments.Add("(CHARINDEX(");
						this.Visit(m.Object);
						SqlSegments.Add(", ");
						this.Visit(m.Arguments[0]);
						if (m.Arguments.Count == 2 && m.Arguments[1].Type == typeof(int))
						{
							SqlSegments.Add(", ");
							this.Visit(m.Arguments[1]);
						}
						SqlSegments.Add(") - 1)");
						return m;

					case "Trim":
						SqlSegments.Add("RTRIM(LTRIM(");
						this.Visit(m.Object);
						SqlSegments.Add("))");
						return m;
				}

				throw new NotImplementedException("Method " + m.Method.Name + "() is not implemented");
			}
			else if (m.Method.DeclaringType == typeof(DateTime))
			{
				switch (m.Method.Name)
				{
					case "op_Subtract":
						if (m.Arguments[1].Type == typeof(DateTime))
						{
							SqlSegments.Add("DATEDIFF(");
							this.Visit(m.Arguments[0]);
							SqlSegments.Add(", ");
							this.Visit(m.Arguments[1]);
							SqlSegments.Add(")");
							return m;
						}
						break;
				}
				throw new NotImplementedException("Method " + m.Method.Name + "() is not implemented");
			}
			else if (m.Method.DeclaringType == typeof(Decimal))
			{
				switch (m.Method.Name)
				{
					case "Add":
					case "Subtract":
					case "Multiply":
					case "Divide":
					case "Remainder":
						SqlSegments.Add("(");
						this.VisitValue(m.Arguments[0]);
						SqlSegments.Add(" ");
						SqlSegments.Add(GetOperator(m.Method.Name));
						SqlSegments.Add(" ");
						this.VisitValue(m.Arguments[1]);
						SqlSegments.Add(")");
						return m;
					case "Negate":
						SqlSegments.Add("-");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add("");
						return m;
					case "Ceiling":
					case "Floor":
						SqlSegments.Add(m.Method.Name.ToUpper());
						SqlSegments.Add("(");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(")");
						return m;
					case "Round":
						if (m.Arguments.Count == 1)
						{
							SqlSegments.Add("ROUND(");
							this.Visit(m.Arguments[0]);
							SqlSegments.Add(", 0)");
							return m;
						}
						else if (m.Arguments.Count == 2 && m.Arguments[1].Type == typeof(int))
						{
							SqlSegments.Add("ROUND(");
							this.Visit(m.Arguments[0]);
							SqlSegments.Add(", ");
							this.Visit(m.Arguments[1]);
							SqlSegments.Add(")");
							return m;
						}
						break;
					case "Truncate":
						SqlSegments.Add("ROUND(");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(", 0, 1)");
						return m;
				}
				throw new NotImplementedException("Method " + m.Method.Name + "() is not implemented");
			}
			else if (m.Method.DeclaringType == typeof(Math))
			{
				switch (m.Method.Name)
				{
					case "Abs":
					case "Acos":
					case "Asin":
					case "Atan":
					case "Cos":
					case "Exp":
					case "Log10":
					case "Sin":
					case "Tan":
					case "Sqrt":
					case "Sign":
					case "Ceiling":
					case "Floor":
						SqlSegments.Add(m.Method.Name.ToUpper());
						SqlSegments.Add("(");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(")");
						return m;
					case "Atan2":
						SqlSegments.Add("ATN2(");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(", ");
						this.Visit(m.Arguments[1]);
						SqlSegments.Add(")");
						return m;
					case "Log":
						if (m.Arguments.Count == 1)
						{
							goto case "Log10";
						}
						break;
					case "Pow":
						SqlSegments.Add("POWER(");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(", ");
						this.Visit(m.Arguments[1]);
						SqlSegments.Add(")");
						return m;
					case "Round":
						if (m.Arguments.Count == 1)
						{
							SqlSegments.Add("ROUND(");
							this.Visit(m.Arguments[0]);
							SqlSegments.Add(", 0)");
							return m;
						}
						else if (m.Arguments.Count == 2 && m.Arguments[1].Type == typeof(int))
						{
							SqlSegments.Add("ROUND(");
							this.Visit(m.Arguments[0]);
							SqlSegments.Add(", ");
							this.Visit(m.Arguments[1]);
							SqlSegments.Add(")");
							return m;
						}
						break;
					case "Truncate":
						SqlSegments.Add("ROUND(");
						this.Visit(m.Arguments[0]);
						SqlSegments.Add(", 0, 1)");
						return m;
				}
				throw new NotImplementedException("Method " + m.Method.Name + "() is not implemented");
			}
			else if (m.Method.Name == "Contains")
			{
				//// m.Method.DeclaringType is Enumerable
				this.Visit(m.Arguments.Last()); // arg is MemberAccess// .Where(o=> new List<int>(){1,2,3}.Contains(o.CategoryID))
				SqlSegments.Add(" IN ");
				SqlSegments.Add("(");
				this.Visit(m.Object ?? m.Arguments.First());
				SqlSegments.Add(")");
				return m;
			}

			throw new NotSupportedException(string.Format("The method '{0}()' is not supported", m.Method.Name));
		}

		protected override Expression VisitParameter(ParameterExpression p)
		{
			EntitySetting es = AttributedTableHelper.GetEntitySetting(p.Type);
			if(es != null)
			{
				SqlName tableName = new SqlName(es.SchemaTableHelper.TableName);
				// add to from block
				query.WriteToFromBlock(tableName);
			}
			else
			{
				throw new NotSupportedException("\"" + p + "\" can't be translated into T-SQL, there is no EntitySetting attribute defined");
			}
			return p;
		}

		protected virtual Expression VisitAssocitation(Expression m)
		{
			// association chain is ended with VisitParameter
			MemberExpression me = (MemberExpression)m;
			AssociationAttribute attr = AttributedTableHelper.GetAssociation(me.Member);
			if(attr != null)
			{
				SqlName tableName = new SqlName(attr.OtherTableName);
				// add to from block
				this.query.WriteToJoinBlock(tableName,
					new SqlName(attr.ThisTableName + "." + attr.ThisKey),
					new SqlName(attr.OtherTableName + "." + attr.OtherKey),
					JoinKindEnum.INNER);

				this.Visit(me.Expression);
			}
			return m;
		}

		protected bool IsAssociation(Expression m)
		{
			if (m == null)
			{
				return false;
			}
			MemberExpression me = m as MemberExpression;
			if (me == null)
			{
				return false;
			}
			return AttributedTableHelper.GetAssociation(me.Member) != null;
		}

		private bool HaveParameterExpression(Expression exp)
		{
			while (exp != null)
			{
				if (exp is UnaryExpression && exp.NodeType == ExpressionType.Convert)
				{
					exp = ((UnaryExpression)exp).Operand;
				}
				else if (exp is ParameterExpression)
				{
					return true;
				}
				else if (exp is MemberExpression)
				{
					exp = ((MemberExpression)exp).Expression;
				}
				else if (exp is MethodCallExpression)
				{
					// static methods have mc.Object == null
					// chained methods calls are supported:
					//  o.HitKorisnik.AdresaNapomena.Substring(10).Substring(5).Contains("fdsf"))
					MethodCallExpression mc = (MethodCallExpression)exp;
					exp = mc.Object ?? mc.Arguments.FirstOrDefault();
				}
				else
				{
					return false;
				}
			}
			return false;
		}

		protected override Expression VisitMemberAccess(MemberExpression m)
		{
			if (!HaveParameterExpression(m))
			{
				// required to translate var gds = new Query<HitOglasAuto>().Where(o => o.HitOglasID == this.HitOglasID);
				// this.HitOglasID doesn't have paremeter, so it is evaluated as constant, else it would translate to "HitOglasAuto.HitOglasID"
				// it is constant
				return this.VisitExecutable(m);
			}
			else if (IsEntityBaseMember(m))
			{
				EntitySetting es;
				EntityFieldSetting efs;
				if((efs = AttributedTableHelper.GetEntityFieldSetting(m.Member)) != null && (es = AttributedTableHelper.GetEntitySetting(m.Member.DeclaringType)) != null)
				{
					this.LastColumnName = es.SchemaTableHelper.TableName + "." + efs.ColumnName;
					SqlSegments.Add(SqlName.CreateFullSqlName(this.LastColumnName)); // if property name has different value than database column name
				}
				else
				{
					throw new NotSupportedException("\"" + m + "\" can't be translated into T-SQL, there is no EntityFieldSetting attribute defined");
				}

				////this.Visit(m.Expression); // creates From clause always, this is not ok for delete and update

				if (IsAssociation(m.Expression))
				{
					// visit only if it is association
					this.VisitAssocitation(m.Expression);
				}

				return m;
			}
			else if (IsAssociation(m))
			{
				return VisitAssocitation(m);
			}
			else
			{
				return MemberAccessResolver(m); // if method not resolved, throw exception or discard method and call this.Visit(m.Expression)
			}
		}

		private Expression MemberAccessResolver(MemberExpression m)
		{
			if (m.Member.DeclaringType.FullName.StartsWith("System.Nullable"))
			{
				if (m.Member.Name == "Value")
				{
					//// o.Published.Value // ignore .Value
					return this.Visit(m.Expression);
				}
				else if (m.Member.Name == "HasValue")
				{
					//// o.Published.HasValue // test if o.Published is not null
					SqlSegments.Add("(");
					this.Visit(m.Expression);
					SqlSegments.Add(" IS NOT NULL");
					SqlSegments.Add(")");
					return m;
				}
			}
			else if (m.Member.DeclaringType == typeof(string))
			{
				switch (m.Member.Name)
				{
					case "Length":
						SqlSegments.Add("LEN(");
						this.Visit(m.Expression);
						SqlSegments.Add(")");
						return m;
				}

			}
			else if (m.Member.DeclaringType == typeof(DateTime) || m.Member.DeclaringType == typeof(DateTimeOffset))
			{
				if (m.Member.Name == "Now")
				{
					return this.VisitExecutable(m);
				}
				switch (m.Member.Name)
				{
					case "Day":
						SqlSegments.Add("DAY(");
						this.Visit(m.Expression);
						SqlSegments.Add(")");
						return m;
					case "Month":
						SqlSegments.Add("MONTH(");
						this.Visit(m.Expression);
						SqlSegments.Add(")");
						return m;
					case "Year":
						SqlSegments.Add("YEAR(");
						this.Visit(m.Expression);
						SqlSegments.Add(")");
						return m;
					case "Hour":
						SqlSegments.Add("DATEPART(hour, ");
						this.Visit(m.Expression);
						SqlSegments.Add(")");
						return m;
					case "Minute":
						SqlSegments.Add("DATEPART(minute, ");
						this.Visit(m.Expression);
						SqlSegments.Add(")");
						return m;
					case "Second":
						SqlSegments.Add("DATEPART(second, ");
						this.Visit(m.Expression);
						SqlSegments.Add(")");
						return m;
					case "Millisecond":
						SqlSegments.Add("DATEPART(millisecond, ");
						this.Visit(m.Expression);
						SqlSegments.Add(")");
						return m;
					case "DayOfWeek":
						SqlSegments.Add("(DATEPART(weekday, ");
						this.Visit(m.Expression);
						SqlSegments.Add(") - 1)");
						return m;
					case "DayOfYear":
						SqlSegments.Add("(DATEPART(dayofyear, ");
						this.Visit(m.Expression);
						SqlSegments.Add(") - 1)");
						return m;
				}
			}

			throw new NotSupportedException(string.Format("The property '{0}' is not supported", m.Member.Name));
		}

		protected override Expression VisitConstant(ConstantExpression c)
		{
			object value = c.Value;

			if (value != null && value is IEnumerable && !(value is string) && !(value is byte[]))
			{
				this.WriteEnumerableValues(value as IEnumerable);
			}
			else
			{
				this.WriteValue(c.Value);
			}
			return c;
		}

		protected virtual void WriteLiteral(string value)
		{
			this.SqlSegments.Add(value);
		}

		protected virtual void WriteValue(object value)
		{
			SqlSegments.Add(GetSqlValueAsString(value));
		}

		protected virtual void WriteParameter(QParameter parameter)
		{
			SqlSegments.Add(parameter);
		}

		private string GetSqlValueAsString(object value)
		{
			StringBuilder tmp = new StringBuilder();

			if (value == null)
			{
				tmp.Append("NULL");
			}
			else
			{
				switch (Type.GetTypeCode(value.GetType()))
				{
					case TypeCode.Boolean:
						tmp.Append(((bool)value) ? 1 : 0);
						break;
					case TypeCode.DateTime:
					case TypeCode.String:
						tmp.Append("'");
						tmp.Append(value);
						tmp.Append("'");
						break;
					case TypeCode.Object:
						if (value.GetType().IsEnum)
						{
							tmp.Append(Convert.ChangeType(value, typeof(int)));
						}
						else
						{
							throw new NotSupportedException(string.Format("The constant for '{0}' is not supported", value));
						}
						break;
					default:
						tmp.Append(value);
						break;
				}
			}
			return tmp.ToString();
		}

		protected virtual void WriteEnumerableValues(IEnumerable enumerable)
		{
			int count = 0;
			foreach (object h in enumerable)
			{
				if (count++ > 0)
				{
					this.WriteLiteral(", ");
				}

				if (h == null)
				{
					this.WriteLiteral("NULL");
				}
				else
				{
					this.WriteValue(h);
				}
			}
		}

		protected override ReadOnlyCollection<Expression> VisitExpressionList(ReadOnlyCollection<Expression> original)
		{
			int count = 0;

			List<Expression> list = null;
			for (int i = 0, n = original.Count; i < n; i++)
			{
				if (count ++ > 0)
				{
					this.WriteLiteral(", "); // for select statement and array inline
				}

				Expression p = this.Visit(original[i]);

				if (list != null)
				{
					list.Add(p);
				}
				else if (p != original[i])
				{
					list = new List<Expression>(n);
					for (int j = 0; j < i; j++)
					{
						list.Add(original[j]);
					}
					list.Add(p);
				}
			}
			if (list != null)
			{
				return list.AsReadOnly();
			}
			return original;
		}

		protected override IEnumerable<ElementInit> VisitElementInitializerList(ReadOnlyCollection<ElementInit> original)
		{
			int count = 0;

			List<ElementInit> list = null;
			for (int i = 0, n = original.Count; i < n; i++)
			{
				if (count++ > 0)
				{
					this.WriteLiteral(", "); // for initialization of list inline: new List<int?>(){1,2,3,4}.Contains(o.CategoryID)
				}

				ElementInit init = this.VisitElementInitializer(original[i]);

				if (list != null)
				{
					list.Add(init);
				}
				else if (init != original[i])
				{
					list = new List<ElementInit>(n);
					for (int j = 0; j < i; j++)
					{
						list.Add(original[j]);
					}
					list.Add(init);
				}
			}
			if (list != null)
				return list;
			return original;
		}

		protected override Expression VisitConditional(ConditionalExpression c)
		{
			/*
CASE
     WHEN Boolean_expression THEN result_expression [ ...n ] 
     [ ELSE else_result_expression ] 
END
			 */
			// this is used by WHERE, SELECT and by ORDER BY statements
			this.WriteLiteral("(CASE WHEN (");
			Expression test = this.VisitPredicate(c.Test);
			this.WriteLiteral(") THEN (");
			Expression ifTrue = this.Visit(c.IfTrue);
			this.WriteLiteral(") ELSE (");
			Expression ifFalse = this.Visit(c.IfFalse);
			this.WriteLiteral(") END)");
			if (test != c.Test || ifTrue != c.IfTrue || ifFalse != c.IfFalse)
			{
				return Expression.Condition(test, ifTrue, ifFalse);
			}
			return c;
		}

	}
}
