﻿#if !SILVERLIGHT

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.IO;

namespace Ice.Net.Sockets.Abstract
{
	public abstract class IceSocketBase<T>
	{
		public delegate void DataReceivedDelegate(T socket, EndPoint senderEP, byte[] data);

		public delegate void DataSentDelegate(T socket, EndPoint toEP);

		public int ReceiveBufferSize = 1024;
		protected Socket _socket;
		protected IPEndPoint _remoteEP;

		public event DataReceivedDelegate DataReceived;

		public event DataSentDelegate DataSendComplete;

		public T OnDataReceived(DataReceivedDelegate callback)
		{
			this.DataReceived += callback;
			return (T)(object) this;
		}

		public IceSocketBase(Socket socket, IPEndPoint remoteEP)
		{
			/*
			 * server: receive from any IP, IPAddress.Any
			 * client: receive from server IP only, IPEndPoint = server IPEndPoint, use IPAddress.Parse(...) to parse from eg. "192.168.0.155". Server IP has to be entered, there is no way for client to know where server is.
			 * port has to be the same on both sides
			 *
			 * server settings (UDP):
			 *	_socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
			 *	_bindToEndPoint =  new IPEndPoint(IPAddress.Any, 1000); //Assign the any IP of the machine and listen on port number 1000
			 *	_remoteEndPoint = new IPEndPoint(IPAddress.Any, 1000); // receive from everyone on port 1000
			 *
			 * client settings:
			 * _socket = ... the same as server's socket
			 * _bindToEndPoint = null; // not used;
			 * _remoteEndPoint = new IPEndPoint(IPAddress.Parse(server's ip), 1000); //Assign server's IP and the server's port, receive from server's IP only
			 */

			this._socket = socket;
			this._remoteEP = remoteEP; // server receives from Any IP, client receives from server only
		}

		protected void BeginReceiveData()
		{
			// begin receive
			// server receives from anyone, client receives from server only
			//The epSender identifies the incoming clients
			EndPoint senderEP = this._remoteEP; // server receives from anyone, client receives from server only
			byte[] buffer = new byte[this.ReceiveBufferSize];

			_socket.BeginReceiveFrom(buffer, 0, buffer.Length, SocketFlags.None, ref senderEP, delegate(IAsyncResult ar)
			{
				EndPoint fromEP = new IPEndPoint(IPAddress.Any, 0);
				// end receive and get sender end point into fromEP
				int receivedLength = _socket.EndReceiveFrom(ar, ref fromEP); // if closing port, this throws exception, fromEP is filled with sender IPEndPoint
				// callback method on data received
				if (DataReceived != null)
				{
					DataReceived.Invoke((T)(object)this, fromEP, buffer.Take(receivedLength).ToArray());
				}
				BeginReceiveData();
			},
			null);
		}

		protected T SendTo(EndPoint toEP, byte[] data)
		{
			_socket.BeginSendTo(data, 0, data.Length, SocketFlags.None, toEP, delegate(IAsyncResult ar)
			{
				int res = _socket.EndSendTo(ar);
				if (this.DataSendComplete != null)
				{
					this.DataSendComplete.Invoke((T)(object)this, toEP);
				}
			}, null);

			return (T)(object) this;
		}




	}
}

#endif
