﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Ice.Serialization.Json.Core.Converters
{
	using Serializer;
	using System.Text;
	using System.Text.RegularExpressions;

	/// <summary>
	/// <para>the main idea is to show the same date and time as it was on server, but if client browser is in different time zone, time zone will not be correct.</para>
	/// <para>on server use local time and get local ticks, send local ticks to client</para>
	/// <para>on client create Date() JS object and adjust time zone so the time and date will be exatctly the same as on server, only time zone on client may be different</para>
	/// <para>, this is not absolutely correct, but this way we have exactly the same time and date on client and server, but we have to keep in mind that client's time zone may be different</para>
	/// </summary>
	internal class IceDateTimeConverter : JavaScriptConverter
	{
		private bool CanConvert(object obj, Type toType)
		{
			if (obj is string)
			{
				string jsonStr = (string)obj;
				return !string.IsNullOrEmpty(jsonStr) && jsonStr.StartsWith("new Date(");
			}
			else
			{
				return false;
			}
		}

		//static Regex mDateRegex = new Regex("^/IceDate\\((?<ticks>-?[0-9]+)(?:[a-zA-Z]|(?:\\+|-)[0-9]{4})?\\)/$");
		static Regex mDateRegex = new Regex("^new Date\\((?<ticks>-?[0-9]+)\\)$");

		private object Convert(object obj, JavaScriptSerializer serializer)
		{
			long num;
			Match match = mDateRegex.Match((string)obj);

            if (long.TryParse(match.Groups["ticks"].Value, out num))
            {
				// must send and receive Unspecified time, eg. data loaded/saved to db is Unspecified
				DateTime dt = new DateTime((num * 10000) + JavaScriptSerializer.DatetimeMinTimeTicks, DateTimeKind.Local);
				return dt;
            }
			throw new NotSupportedException();
		}

		public override bool TryConvert(object obj, Type toType, JavaScriptSerializer serializer, out object result)
		{
			if (this.CanConvert(obj, toType))
			{
				result = Convert(obj, serializer);
				return true;
			}
			else
			{
				result = null;
				return false;
			}
		}

		private bool CanSerialize(Type type)
		{
			return type != null && type == typeof(DateTime);
		}

		private void Serialize(object obj, StringBuilder sb, JavaScriptSerializer serializer, JavaScriptSerializer.SerializationFormat serializationFormat)
		{
			// when datetime loaded from database, it is unspecified. 
			DateTime datetime = (DateTime)obj;

			if (datetime.Kind == DateTimeKind.Utc)
			{
				// if Kind = Unspecified, it is Local time, no time zone adjustment is made
				datetime = datetime.ToLocalTime();
			}

			// 1 Javascript tick == 1 mili second
			// 1 .net tick = 1ms * 10000

			////if (serializationFormat == JavaScriptSerializer.SerializationFormat.JSON)
			////{
			////	sb.Append("\"\\/IceDate(");
			////	sb.Append((long)((datetime.Ticks - JavaScriptSerializer.DatetimeMinTimeTicks) / 10000)); // 0x2710 = 10000: .NET tick count is the number of 100 nanosecond (ns) intervals since the zero time value. 
			////	sb.Append(")\\/\"");
			////}
			////else
			{
				// server ticks is local time ticks
				sb.Append("new Date(");
				sb.Append((long)((datetime.Ticks - JavaScriptSerializer.DatetimeMinTimeTicks) / 10000));
				sb.Append(")");
			}
		}

		public override bool TrySerialize(object obj, StringBuilder sb, JavaScriptSerializer serializer, JavaScriptSerializer.SerializationFormat serializationFormat)
		{
			if (obj != null && this.CanSerialize(obj.GetType()))
			{
				this.Serialize(obj, sb, serializer, serializationFormat);
				return true;
			}
			else
			{
				return false;
			}
		}

	}

}
