﻿#if SILVERLIGHT

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Text.RegularExpressions;
using Ice.Extensions;
using Ice.Utility;
using System.Windows.Browser;
using System.Net;

namespace Ice.Silverlight.Extensions
{
	public static class ControlExtensions
	{
		public enum HtmlNodeType
		{
			UNKNOWN = 0,
			ELEMENT_NODE = 1,
			ATTRIBUTE_NODE = 2,
			TEXT_NODE = 3,
			CDATA_SECTION_NODE = 4,
			ENTITY_REFERENCE_NODE = 5,
			ENTITY_NODE = 6,
			PROCESSING_INSTRUCTION_NODE = 7,
			COMMENT_NODE = 8,
			DOCUMENT_NODE = 9,
			DOCUMENT_TYPE_NODE = 10,
			DOCUMENT_FRAGMENT_NODE = 11,
			NOTATION_NODE = 12
		};

		#region Utility extensions

		public static T Set<T>(this T control, Action<T> action) where T : ScriptObject
		{
			action.Invoke(control);
			return control;
		}

		////public static T AppendJsEvent<T>(this T control, JavascriptEventEnum eventName, string js) where T : HtmlElement
		////{
		////	string name = "on" + eventName.GetStringValue();
		////	string curr = control.GetAttribute(name) ?? "";
		////	if (!string.IsNullOrEmpty(curr) && !curr.EndsWith(";"))
		////	{
		////		curr += ";";
		////	}
		////	curr += js;
		////	if (!curr.EndsWith(";"))
		////	{
		////		curr += ";";
		////	}
		////	control.SetAttribute(name, curr);
		////	return control;
		////}

		////public static string ToHtmlRendered(this Control c)
		////{
		////	System.IO.StringWriter sr = new System.IO.StringWriter();
		////	System.Web.UI.HtmlTextWriter h = new HtmlTextWriter(sr);
		////	c.RenderControl(h);
		////	return sr.ToString();
		////}

		////public static TextBox TrimText(this TextBox control)
		////{
		////	control.Text = control.Text.Trim();
		////	return control;
		////}

		////public static IEnumerable<TextBox> TrimText(this IEnumerable<TextBox> source)
		////{
		////	return source.ForEach(o => o.Text = o.Text.Trim());
		////}

		#endregion

		#region Display alert extensions

		////public static string JSEncodeString(string str)
		////{
		////	string s = str.Replace("\r", "\\r").Replace("\n", "\\n")
		////		.Replace("'", "\\'")
		////		.Replace("\"", "\\\"");

		////	return s;
		////}

		////public static void DisplayAlert(this Page page, string message)
		////{
		////	page.ExecuteJavaScript("alert(\"" + JSEncodeString(message) + "\");");
		////}

		////public static void ExecuteJavaScript(this Page page, string javaScriptCode)
		////{
		////	if (!javaScriptCode.EndsWith(";"))
		////	{
		////		javaScriptCode += ";";
		////	}
		////	ScriptManager.RegisterStartupScript(page, page.GetType(), Guid.NewGuid().ToString(), javaScriptCode, true);
		////}

		////public static void SetWindowLocation(this Page page, string url, int waitMs = 0)
		////{
		////	page.ExecuteJavaScript("setTimeout(function(){window.location='" + url + "';}, " + waitMs + ");");
		////}

		#endregion

		#region DOM manipulation extensions

		////private static HtmlElement Parent(this HtmlElement control)
		////{
		////	return control.GetProperty("parentNode") as HtmlElement;
		////}

		////private static IEnumerable<HtmlElement> Children(this HtmlElement control)
		////{
		////	return ((ScriptObjectCollection)control.GetProperty("childNodes")).OfType<HtmlElement>();
		////}

		public static HtmlNodeType NodeType(this HtmlElement control)
		{
			return (HtmlNodeType)Convert.ToInt16(control.GetProperty("nodeType"));
		}

		public static bool IsElementNodeType(this HtmlElement control)
		{
			return control.NodeType() == HtmlNodeType.ELEMENT_NODE;
		}

		public static IEnumerable<HtmlElement> IsElementNodeType(this IEnumerable<HtmlElement> controls)
		{
			return controls.Where(o => o.IsElementNodeType()).OfType<HtmlElement>();
		}

		public static IEnumerable<HtmlElement> WhereNodeType(this IEnumerable<HtmlElement> controls, HtmlNodeType nodeType)
		{
			return controls.Where(o => o.NodeType() == nodeType).OfType<HtmlElement>();
		}

		/// <summary>
		/// returns descendants of TResult type
		/// </summary>
		/// <typeparam name="TResult"></typeparam>
		/// <param name="control"></param>
		/// <returns></returns>
		public static IEnumerable<HtmlElement> DescendantNodes(this HtmlElement control)
		{
			// select children than concat children's descendants
			foreach (HtmlElement ch in control.Children.OfType<HtmlElement>())
			{
				yield return ch;
				foreach (HtmlElement ch2 in ch.DescendantNodes())
				{
					yield return ch2;
				}
			}
		}

		public static IEnumerable<HtmlElement> AncestorNodes(this HtmlElement control)
		{
			while (control != null && (control = control.Parent) != null)
			{
				yield return control;
			}
		}

		public static IEnumerable<HtmlElement> DescendantNodes(this IEnumerable<HtmlElement> controls)
		{
			return controls.SelectMany(o => o.DescendantNodes());
		}

		public static IEnumerable<HtmlElement> AncestorNodes(this IEnumerable<HtmlElement> controls)
		{
			return controls.SelectMany(o => o.AncestorNodes());
		}

		public static IEnumerable<HtmlElement> ParentNodes(this IEnumerable<HtmlElement> controls)
		{
			return controls.Select(o => o.Parent).OfType<HtmlElement>();
		}

		public static IEnumerable<HtmlElement> ChildNodes(this IEnumerable<HtmlElement> controls)
		{
			return controls.SelectMany(o => o.ChildNodes());
		}

		public static IEnumerable<HtmlElement> ChildNodes(this HtmlElement control)
		{
			return control.Children.OfType<HtmlElement>();
		}

		public static HtmlElement RemoveChildren(this HtmlElement control)
		{
			control.Children.OfType<HtmlElement>().ForEach(ch =>
			{
				ch.Parent.RemoveChild(ch);
			});

			return control;
		}

		public static HtmlElement AddChildren(this HtmlElement control, params HtmlElement[] children)
		{
			// must create collection copy because controles are removed from children collection when added to different collection
			if (control != null && control.IsElementNodeType())
			{
				children.Where(o => o != null).OfType<HtmlElement>().ForEach(o => control.AppendChild(o));
			}
			return control;
		}

		////public static TSource AddChildren<TSource>(this TSource control, params ITemplate[] templates) where TSource : ScriptObject
		////{
		////	templates.Where(o=>o != null).ForEach(o=>o.InstantiateIn(control));
		////	return control;
		////}

		public static HtmlElement InsertChildren(this HtmlElement parent, params HtmlElement[] children)
		{
			// must create collection copy because controls are removed from children collection when added to different collection
			children.Where(o => o != null).Reverse().OfType<HtmlElement>().ForEach(o =>
			{
				if (parent != null)
				{
					HtmlElement refEl = parent.Children.OfType<HtmlElement>().FirstOrDefault();
					if (refEl == null)
					{
						parent.AppendChild(o);
					}
					else
					{
						parent.AppendChild(o, refEl);
					}
				}
			});
			return parent;
		}

		public static HtmlElement InsertBefore(this HtmlElement control, params HtmlElement[] items)
		{
			items.Reverse().OfType<HtmlElement>().ForEach(o =>
			{
				if (control != null && control.Parent != null)
				{
					control.Parent.AppendChild(o, control);
				}
			});

			return control;
		}

		////public static TSource InsertAfter<TSource>(this TSource control, params Control[] items) where TSource : Control
		////{
		////	int index = control.Parent.Controls.IndexOf(control);
		////	items.Reverse().ForEach(o => control.Parent.Controls.AddAt(index + 1, o));
		////	return control;
		////}

		#endregion

		#region next siblings

		public static IEnumerable<HtmlElement> NextSiblings(this HtmlElement node)
		{
			if (node.Parent != null)
			{
				foreach (HtmlElement s in node.Parent.Children.OfType<HtmlElement>().SkipWhile(o => o != node).Skip(1))
				{
					yield return s;
				}
			}
		}

		public static IEnumerable<HtmlElement> NextSiblings(this HtmlElement node, Func<HtmlElement, bool> predicate)
		{
			return node.NextSiblings().Where(predicate);
		}

		public static IEnumerable<HtmlElement> NextSiblings(this IEnumerable<HtmlElement> list)
		{
			return list.SelectMany(o => o.NextSiblings());
		}

		public static IEnumerable<HtmlElement> NextSiblings(this IEnumerable<HtmlElement> list, Func<HtmlElement, bool> predicate)
		{
			return list.NextSiblings().Where(predicate);
		}

		#endregion

		#region previous siblings

		public static IEnumerable<HtmlElement> PreviousSiblings(this HtmlElement node)
		{
			if (node.Parent != null)
			{
				HtmlElement el = node.Parent as HtmlElement;
				if (el != null)
				{
					foreach (HtmlElement s in el.Parent.Children.OfType<HtmlElement>().TakeWhile(o => o != node).Reverse())
					{
						// backward:
						// n0, n1, n2, n3, n4=current, n5, n5, will return nodes: n3, n2, n1, n0
						yield return s;
					}
				}
			}
		}

		public static IEnumerable<HtmlElement> PreviousSiblings(this HtmlElement node, Func<HtmlElement, bool> predicate)
		{
			return node.PreviousSiblings().Where(predicate);
		}

		public static IEnumerable<HtmlElement> PreviousSiblings(this IEnumerable<HtmlElement> list)
		{
			return list.SelectMany(o => o.PreviousSiblings());
		}

		public static IEnumerable<HtmlElement> PreviousSiblings(this IEnumerable<HtmlElement> list, Func<HtmlElement, bool> predicate)
		{
			return list.PreviousSiblings().Where(predicate);
		}

		#endregion

		#region Css Style extensions

		public static List<KeyValuePair<string, string>> ParseStyleValues(string style)
		{
			return new Regex("([^:;]+):([^:;]*)").Matches(style ?? "").Cast<Match>().Select(m => new KeyValuePair<string, string>(m.Groups[1].Value.Trim(), m.Groups[2].Value.Trim()))
				.Where(o => !string.IsNullOrEmpty(o.Key)).ToList();
		}
		/*
		/// <summary>
		/// Remove previous style values with the same names and append new vlues.
		/// </summary>
		/// <param name="sc"></param>
		/// <param name="style"></param>
		/// <returns></returns>
		public static CssStyleCollection ApplyValues(this CssStyleCollection sc, List<KeyValuePair<string, string>> style)
		{
			foreach (KeyValuePair<string, string> kv in style)
			{
				// we must remove preivous values, not just overwrite. By removing first, new values will be appended to the end. Css always use last stye value.
				// eg. in this scenario, if "background" is not removed first, background-color will always be blue because "background" is changed to green, but "background-color" overrides it
				//Panel ppp = new Panel();
				//ppp.Style.SetValues("background:red; background-color:blue;");
				//ppp.Style.SetValues("background:green;");

				sc.Remove(kv.Key); // remove it first so the new value will be appended to the end
				
				sc.Add(kv.Key, kv.Value);
			}
			return sc;
		}

		/// <summary>
		/// Remove previous style values with the same names and append new vlues.
		/// </summary>
		/// <param name="sc"></param>
		/// <param name="style"></param>
		/// <returns></returns>
		public static CssStyleCollection ApplyValues(this CssStyleCollection sc, string style)
		{
			return sc.ApplyValues(ParseStyleValues(style));
		}
		*/

		public static HtmlElement ApplyStyle(this HtmlElement control, string style)
		{
			if (!string.IsNullOrEmpty(style))
			{
				if (control != null && control.IsElementNodeType())
				{
					ScriptObject ss = control.GetProperty("style") as ScriptObject;
					if (ss != null)
					{
						object cssTxt = ss.GetProperty("cssText");
						ss.SetProperty("cssText", cssTxt += ";" + style);
					}
				}
			}
			return control;
		}

		public static IEnumerable<HtmlElement> ApplyStyle(this IEnumerable<HtmlElement> source, string style)
		{
			return source.ForEach(o => o.ApplyStyle(style));
		}

		#endregion

		#region Css class extensions

		public static HtmlElement AddClass(this HtmlElement control, string cssClass)
		{
			if (!string.IsNullOrEmpty(cssClass))
			{
				if (control != null && control.IsElementNodeType())
				{
					string cls = control.GetProperty("className") + "";
					//remove duplicates
					cls = cls.Replace(" " + cssClass + " ", " ").Trim();
					// setvalue
					control.SetProperty("className", (cls + " " + cssClass).Trim());
				}
			}
			return control;
		}

		public static IEnumerable<HtmlElement> AddClass(this IEnumerable<HtmlElement> source, string cssClass)
		{
			return source.ForEach(o => o.AddClass(cssClass));
		}

		public static HtmlElement RemoveClass(this HtmlElement control, string cssClass)
		{
			if (!string.IsNullOrEmpty(cssClass))
			{
				if (control != null && control.IsElementNodeType())
				{
					string cls = " " + control.GetProperty("className") + " ";
					control.SetProperty("className", (cls.Replace(" " + cssClass + " ", " ")).Trim());
				}
			}
			return control;
		}

		public static IEnumerable<HtmlElement> RemoveClass(this IEnumerable<HtmlElement> source, string cssClass)
		{
			return source.ForEach(o => o.RemoveClass(cssClass));
		}

		public static bool HasClass(this HtmlElement control, string cssClass)
		{
			if (!string.IsNullOrEmpty(cssClass))
			{
				if (control != null && control.IsElementNodeType())
				{
					string cls = " " + control.GetProperty("className") + " ";
					return cls.Contains(" " + cssClass + " ");
				}
			}
			return false;
		}

		public static HtmlElement ToggleClass(this HtmlElement control, string cssClass)
		{
			if (control.HasClass(cssClass))
			{
				return control.RemoveClass(cssClass);
			}
			else
			{
				return control.AddClass(cssClass);
			}
		}

		public static IEnumerable<HtmlElement> ToggleClass(this IEnumerable<HtmlElement> source, string cssClass)
		{
			return source.ForEach(o => o.ToggleClass(cssClass));
		}

		#endregion

		#region ListControl extensions

		////public static bool SetSelectedValue(this ListControl control, object value)
		////{
		////	control.SelectedIndex = -1; // select none or first, if value will not be found, this way we have first (default) item selected

		////	ListItem item = control.Items.FindByValue(value + "");
		////	if (item != null)
		////	{
		////		control.SelectedIndex = control.Items.IndexOf(item);
		////		return true;
		////	}
		////	else
		////	{
		////		return false;
		////	}
		////}

		////public static bool SetSelectedText(this ListControl control, string text)
		////{
		////	control.SelectedIndex = -1; // select none or first, if value will not be found, this way we have first (default) item selected

		////	ListItem item = control.Items.FindByText(text);
		////	if (item != null)
		////	{
		////		control.SelectedIndex = control.Items.IndexOf(item);
		////		return true;
		////	}
		////	else
		////	{
		////		return false;
		////	}
		////}

		////public static TResult GetSelectedValue<TResult>(this ListControl control)
		////{
		////	TResult o;
		////	IceTypeConverter.TryChangeType<TResult>(control.SelectedValue, out o);
		////	return o;
		////}

		#endregion

		#region TreeView extensions

		////public static IEnumerable<TreeNode> DescendantNodes(this TreeNode node)
		////{
		////	if (node != null && node.ChildNodes != null && node.ChildNodes.Count > 0)
		////	{
		////		foreach (TreeNode n in node.ChildNodes)
		////		{
		////			yield return n;

		////			foreach (TreeNode n1 in n.DescendantNodes())
		////			{
		////				yield return n1;
		////			}
		////		}
		////	}
		////}

		////public static IEnumerable<TreeNode> AncestorNodes(this TreeNode node)
		////{
		////	while ((node = node.Parent) != null)
		////	{
		////		yield return node;
		////	}
		////}

		////public static IEnumerable<TreeNode> DescendantNodes(this TreeView treeView)
		////{
		////	return treeView.Nodes.Cast<TreeNode>().Concat(treeView.Nodes.Cast<TreeNode>().SelectMany(o => o.DescendantNodes()));
		////}

		#endregion

		#region ice css selector

		class SelectorReader
		{
			int _currPosition = 0;
			string selector;

			public SelectorReader(string iceSelector)
			{
				this.selector = iceSelector;
			}

			/// <summary>
			/// Peek next char.
			/// </summary>
			/// <param name="forwardBy">default:0 (first available on stack)</param>
			/// <returns></returns>
			public char? Peek(int forwardBy = 0)
			{
				if (selector.Length > _currPosition + forwardBy)
				{
					return selector[_currPosition + forwardBy];
				}
				return null;
			}

			/// <summary>
			/// Pop first available value, than advance index + 1.
			/// </summary>
			/// <returns></returns>
			public char? Pop()
			{
				if (selector.Length > _currPosition)
				{
					return selector[_currPosition++];
				}
				return null;
			}

			/// <summary>
			/// Match from (first + origin) position, advance index by (match length + origin) and return match.
			/// </summary>
			/// <param name="regex"></param>
			/// <param name="origin"></param>
			/// <returns></returns>
			public Match PopMatch(Regex regex, int origin = 0)
			{
				Match mm = regex.Match(selector.Substring(_currPosition + origin));
				if (mm.Success)
				{
					_currPosition += mm.Value.Length + origin;
					return mm;
				}
				return null;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="source"></param>
		/// <param name="attrName">value from an element (current value)</param>
		/// <param name="attrSign"></param>
		/// <param name="attrVal">expected value</param>
		/// <returns></returns>
		private static IEnumerable<HtmlElement> CreateAttributeQuery(IEnumerable<HtmlElement> source, string attrName, string attrSign, string attrVal)
		{
			// should be used o.getAttribute(name) which is case non-senseitve for name parameter?

			var func = new Func<HtmlElement, bool>(o =>
			{
				// get property first, if has no value, then get attribute value
				string attr = (o.GetProperty(attrName) ?? "") + ""; // WARNING: this is not tested yet <<<<<<<<<<<<< WARNING <<<<<<<<<<<<<<
				if (string.IsNullOrEmpty(attr))
				{
					attr = o.GetAttribute(attrName);
				}

				string currValue = (attr == null ? "" : attr ?? ""); // current attribute value

				string m = attrSign;

				if (string.IsNullOrEmpty(m))
				{
					// the same as =*
					// no attr sign, statement [attrName] means has attribute value, doesn't matter which value is
					return !string.IsNullOrEmpty(currValue);
				}
				else if (m == "=" || m == "==")
				{
					return currValue == attrVal;
				}
				else if (m == "=*")
				{
					// has any value, but not empty
					return !string.IsNullOrEmpty(currValue);
				}
				else if (m == "~=") // E[foo~="warning"]	Matches any E element whose "foo" attribute value is a list of space-separated values, one of which is exactly equal to "warning"
				{
					return (" " + currValue + " ").Contains(" " + attrVal + " ");
				}
				else if (m == "*=") // contains value
				{
					return currValue.Contains(attrVal);
				}
				else if (m == "^=") // starts with
				{
					return currValue.StartsWith(attrVal);
				}
				else if (m == "$=") // ends with
				{
					return currValue.EndsWith(attrVal);
				}
				else // <, >, <=, >=, % etc
				{
					decimal curr1, val1;
					if (!(decimal.TryParse(currValue, out curr1) && decimal.TryParse(attrVal, out val1)))
					{
						return false;
					}

					switch (m)
					{
						case "==": return curr1 == val1;
						case "<": return curr1 < val1;
						case ">": return curr1 > val1;
						case ">=": return curr1 >= val1;
						case "<=": return curr1 <= val1;
					}

					throw new NotSupportedException(m);
				}
			});

			return source.Where(o => func(o));
		}

		private static IEnumerable<HtmlElement> CreateIdQuery(IEnumerable<HtmlElement> source, string id)
		{
			return source.Where(o => o.Id == id);
		}

		private static IEnumerable<HtmlElement> CreateClassQuery(IEnumerable<HtmlElement> source, string className)
		{
			return source.Where(o => (" " + o.CssClass + " ").Contains(" " + className + " "));
		}

		private static IEnumerable<HtmlElement> CreateTagQuery(IEnumerable<HtmlElement> source, string tag)
		{
			return source.Where(o => (o.TagName == tag));
		}

		private static IEnumerable<HtmlElement> CreateColonWordQuery(IEnumerable<HtmlElement> source, string colonWord, string colonValue)
		{
			int count = 1;
			if (!int.TryParse(colonValue, out count))
			{
				count = 1;
			}

			switch (colonWord)
			{
				case "first":
				case "take":
					return source.Take(count);
				case "last":
					return source.Reverse().Take(count).Reverse();
				case "skip":
					return source.Skip(count);
				case "is":
				case "not":
					// select elements from source where subquery in :is(subquery) is true or :not(subquery) is false, input to subquery is each element from source query
					// $('div//*:not(span)'), select div's descendants, except span, '*' can be removed
					// $(".firstquery...").Query(":not(a>span, .cls1>>div)") or .Query(":not(a>span), :not(.cls1>>div)"), // select all items from first query where item is not "a" with child "span" and item with class 'cls' has no 2nd child 'div'
					// $(".firstquery...").Query(":is(a>span)"), // select all items from first query where item is "a" with child "span"
					// $(".firstquery...").Query(":is(*>span)"), // * is optional: .Query(":is(>span)") select all items from first query where item has child "span"
					return source.Where(o => o.Query(colonValue).Any() == (colonWord == "is"));
				////case "has":
				////case "hasnot":
				////	// OBSOLETE: use :is or :not instead since "is" or "not" is much more sophisticated version than this crappy jQuery.has(...)
				////	// The expression $('div:has(p)') matches a <div> if a <p> exists anywhere among its descendants, not just as a direct child.
				////	bool doesHave = colonWord == "has";
				////	return source.Where(o => o.DescendantNodes().Query(colonValue).Any() == doesHave);
				case "reverse":
					return source.Reverse();
				case "ancestor":
					return source.AncestorNodes();
				case "descendant":
					return source.DescendantNodes();
				case "child":
					return source.ChildNodes();
				case "parent":
					return source.ParentNodes();
				case "next-sibling":
					return source.NextSiblings();
				case "previous-sibling":
					return source.PreviousSiblings();
				default:
					throw new NotSupportedException(colonWord + "(" + colonValue + ")");
			}
		}

		public static IEnumerable<HtmlElement> SelectFilteredSingle(this IEnumerable<HtmlElement> source, string singleCssSelector)
		{
			var origsrc = source;
			var newsrc = source;

			string iceSelector = singleCssSelector.Replace("\"", "").Replace("'", "").Trim(); // remove " and '

			iceSelector = new Regex("([ ]*)([\\/\\<\\>\\!\\~\\^\\$\\*\\=]+)([ ]*)").Replace(iceSelector, "$2"); // remove spaces arround special chars
			iceSelector = new Regex("[ ]+").Replace(iceSelector, " "); // replace multiple spaces with single space

			char? ch;
			Match mm;

			SelectorReader reader = new SelectorReader(iceSelector);

			while ((ch = reader.Pop()) != null)
			{
				if (ch == '#' && (mm = reader.PopMatch(new Regex("^([\\w\\-_]+)"))) != null)
				{
					// #id
					newsrc = CreateIdQuery(newsrc, mm.Groups[1].Value);
				}
				else if (ch == '.' && (mm = reader.PopMatch(new Regex("^([\\w\\-_]+)"))) != null)
				{
					// .class
					newsrc = CreateClassQuery(newsrc, mm.Groups[1].Value);
				}
				else if (ch == ' ')
				{
					// descendant
					newsrc = newsrc.DescendantNodes();
				}
				else if (ch == '/' && reader.Peek() == '/' && reader.Peek(1) != '/') // descendants, space or double //
				{
					// descendant
					reader.Pop(); // pop next / too
					newsrc = newsrc.DescendantNodes();
				}
				else if (ch == '*')
				{
					newsrc = newsrc.Select(o => o); // must create select if the only statement is "*", this way we return all, not empty collection
				}
				else if (ch == '^') // ancestors, ice specific
				{
					newsrc = newsrc.AncestorNodes();
				}
				else if (ch == '<') // parent, << will create 2 times: .ParentNodes().ParentNodes()
				{
					newsrc = newsrc.ParentNodes();
				}
				else if (ch == '>') // children, eg. >>> wil create 3 times .ChildNodes().ChildNodes().ChildNodes()
				{
					newsrc = newsrc.ChildNodes();
				}
				else if (ch == '/' && reader.Peek() != '/') // children, x-path first level children
				{
					newsrc = newsrc.ChildNodes();
				}
				else if (ch == ':' && (mm = reader.PopMatch(new Regex("^([\\w\\-_]+)([\\(]([^\\)]*)[\\)])*"))) != null)
				{
					// special attribute, eg. :take(10), :first, :first(5)
					newsrc = CreateColonWordQuery(newsrc, mm.Groups[1].Value, mm.Groups[3].Value);
				}
				else if (ch == '[' && (mm = reader.PopMatch(new Regex("^[ ]*([\\w\\-_]+)[ ]*([<>\\!\\=\\~\\^\\$\\*]*)[ ]*([\\w\\:\\;\\,\\.]*)[ ]*\\]"))) != null)
				{
					// '[attribute="value"]' // '[attribute == "value"]' // // '[attribute != "value"]' // '[attribute >= value]'
					newsrc = CreateAttributeQuery(newsrc, mm.Groups[1].Value, mm.Groups[2].Value, mm.Groups[3].Value);
				}
				else if ((mm = reader.PopMatch(new Regex("^([\\w\\-_]+)"), -1)) != null) // origin -1 to include current ch in match
				{
					// tag
					newsrc = CreateTagQuery(newsrc, mm.Groups[1].Value);
				}
				else
				{
					// unknown
					throw new Exception("Usupported iceSelector (" + (iceSelector) + ")");
				}
			}

			// if newsrc == origsrc, nothing was selected by iceSelector, return an empty eneumerable
			return newsrc == origsrc ? new List<HtmlElement>().AsEnumerable() : newsrc;
		}



		/// <summary>
		/// filter current items by iceSelector
		/// </summary>
		/// <param name="iceSelector">
		/// lambda or css iceSelector:
		/// <para>TAG name: html, body...</para>
		/// <para>ID: #idvalue...</para>
		/// <para>CLASS: .mydiv, a.mydiv...;</para>
		/// <para>ATTRIBUTES: [name] (contains attribute), [name=value], [name &gt; value], ==, =, &lt;, &gt;, !=, &lt;&gt;, [name*=value] (contains value), [name^=value] (starts with value), [name$=value] (ends with value)...</para>
		/// <para>TRAVERSING: space or // (descendants), / (children), ^ (ancestors), &gt;&gt;&gt; (n-th level children), &lt;&lt; (n-th level parent) :first(n) :last(n) :skip(n) :take(n) :reverse(), with or without (n), if there is no (n), default n == 1</para>
		/// <para>from x-path: :ancestor :descendant :child :parent :next-siblings :previous-siblings</para>
		/// </param>
		public static IEnumerable<HtmlElement> Query(this HtmlElement node, string iceSelector)
		{
			return new[] { node }.Query(iceSelector);
		}

		/// <summary>
		/// filter current items by iceSelector
		/// </summary>
		/// <param name="iceSelector">
		/// lambda or css iceSelector:
		/// <para>TAG name: html, body...</para>
		/// <para>ID: #idvalue...</para>
		/// <para>CLASS: .mydiv, a.mydiv...;</para>
		/// <para>ATTRIBUTES: [name] (contains attribute), [name=value], [name &gt; value], ==, =, &lt;, &gt;, !=, &lt;&gt;, [name*=value] (contains value), [name^=value] (starts with value), [name$=value] (ends with value)...</para>
		/// <para>TRAVERSING: space or // (descendants), / (children), ^ (ancestors), &gt;&gt;&gt; (n-th level children), &lt;&lt; (n-th level parent) :first(n) :last(n) :skip(n) :take(n) :reverse(), with or without (n), if there is no (n), default n == 1</para>
		/// <para>from x-path: :ancestor :descendant :child :parent :next-siblings :previous-siblings</para>
		/// </param>
		public static IEnumerable<HtmlElement> Query(this IEnumerable<HtmlElement> source, string iceSelector)
		{
			// translate iceSelector, split iceSelector by (,), but not inside (...)

			var iceSelectorsArr = new List<string>();
			var currPart = "";
			var digCount = 0;
			for (var n = 0; n < iceSelector.Length; n++)
			{
				var ch = iceSelector[n];
				if (ch == ')')
				{
					digCount--;
					currPart += ch;
				}
				else if (ch == '(')
				{
					digCount++;
					currPart += ch;
				}
				else if (ch == ',' && digCount == 0)
				{
					// split
					iceSelectorsArr.Add(currPart);
					currPart = "";
				}
				else
				{
					currPart += ch;
				}
			}

			if (currPart.Length > 0)
			{
				iceSelectorsArr.Add(currPart);
			}

			foreach (string sel in iceSelectorsArr.Select(o => o.Trim()).Where(o => !string.IsNullOrEmpty(o)))
			{
				foreach (HtmlElement node in source.SelectFilteredSingle(sel))
				{
					yield return node;
				}
			}
		}

		#endregion

		public static List<HtmlElement> CreateHtmlElements(string html)
		{
			// must create collection copy because controles are removed from children collection when added to different collection
			HtmlElement div = HtmlPage.Document.CreateElement("div");
			div.SetProperty("innerHTML", html);
			return div.ChildNodes().ToList();
		}

		/// <summary>
		/// Convert to GMT string for browser
		/// </summary>
		/// <param name="dt"></param>
		/// <returns></returns>
		public static string ToGMTString(this DateTime dt)
		{
			return dt.ToUniversalTime().ToString("ddd, dd MM yyyy HH:mm:ss") + " GMT";
		}

		public static List<Cookie> GetCookies(this HtmlDocument doc)
		{
			return HtmlPage.Document.Cookies.Split(';').Select(o => o.Trim()).Select(o =>
			{
				string[] aa = o.Split('=');
				return new Cookie(aa[0], aa[1]);
			}).ToList();
		}

		public static HtmlDocument SetCookie(this HtmlDocument doc, Cookie cookie)
		{
			StringBuilder sb = new StringBuilder();
			sb.Append(HttpUtility.UrlEncode(cookie.Name));
			sb.Append("=");
			sb.Append(HttpUtility.UrlEncode(cookie.Value));
			if (cookie.Expires.Year > 10)
			{
				sb.Append(";expires=" + cookie.Expires.ToGMTString());// Mon, 10 Mar 2014 11:47:49 GMT
			}
			if (!string.IsNullOrEmpty(cookie.Domain))
			{
				sb.Append(";domain=" + cookie.Domain);
			}
			if (!string.IsNullOrEmpty(cookie.Path))
			{
				sb.Append(";path=" + cookie.Path);
			}
			if (cookie.Secure)
			{
				sb.Append(";secure");
			}
			doc.Cookies = sb.ToString();
			return doc;
		}

		public static HtmlDocument LoadJSFile(this HtmlDocument doc, string scriptFile)
		{
			var s = doc.CreateElement("script");
			s.SetProperty("type", "text/javascript");
			s.SetProperty("src", scriptFile);
			doc.DocumentElement.AppendChild(s);
			return doc;
		}

		public static HtmlDocument LoadJS(this HtmlDocument doc, string javascriptCode)
		{
			var s = doc.CreateElement("script");
			s.SetProperty("type", "text/javascript");
			s.SetProperty("text", javascriptCode);
			s.SetProperty("textContent", javascriptCode); // ie9 or higher and gecko uses textContent, <=ie8 uses text
			doc.DocumentElement.AppendChild(s);
			return doc;
		}

		public static HtmlDocument LoadCSSFile(this HtmlDocument doc, string cssFile)
		{
			var s = doc.CreateElement("link");
			s.SetProperty("type", "text/css");
			s.SetProperty("rel", "stylesheet");
			s.SetProperty("href", cssFile);
			doc.DocumentElement.AppendChild(s);
			return doc;
		}

		public static HtmlDocument LoadCSS(this HtmlDocument doc, string cssScriptCode)
		{
			var s = doc.CreateElement("style");
			s.SetProperty("type", "text/css");
			s.SetProperty("textContent", cssScriptCode); // works with ie 9 or higher and gecko only, <=ie8 is not supported
			doc.DocumentElement.AppendChild(s);
			return doc;
		}

		public static void ExecuteJavaScript(this HtmlDocument doc, string javascriptCode)
		{
			doc.LoadJS(javascriptCode);
		}

		public static string JSEncodeString(string str)
		{
			string s = str.Replace("\r", "\\r").Replace("\n", "\\n")
				.Replace("'", "\\'")
				.Replace("\"", "\\\"");

			return s;
		}

		public static void DisplayAlert(this HtmlDocument doc, string message)
		{
			doc.ExecuteJavaScript("alert(\"" + JSEncodeString(message) + "\");");
		}

		public static void SetWindowLocation(this HtmlDocument doc, string url, int waitMs = 0)
		{
			doc.ExecuteJavaScript("setTimeout(function(){window.location='" + url + "';}, " + waitMs + ");");
		}


	}
}

#endif