﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Globalization;
using System.Linq;
using System.Net;
#if !SILVERLIGHT
using System.Web;
using System.Web.UI.WebControls;
using System.Collections;
using Ice.Converter;
#endif

namespace Ice.Utility
{
	public class IceTools
	{
		private static string GetJavaScriptTypeNameWorker(object obj)
		{
			Type ut;
			
			if (obj == null)
			{
				return "null";
			}
			else if(obj is String)
			{
				return (string)obj;
			}
			else if (obj is Type)
			{
				Type tt = (Type)obj;
				Type under = System.Nullable.GetUnderlyingType(tt);
				if (under != null)
				{
					return under.FullName + "?";
				}
				else
				{
					return (tt).FullName;
				}
			}
			else if ((ut = System.Nullable.GetUnderlyingType(obj.GetType())) != null)
			{
				return ut.FullName + "?";
			}
			else
			{
				return obj.GetType().FullName;
			}
		}


		/// <summary>
		/// Get Javascript qualified type name, replace + with .
		/// </summary>
		/// <param name="obj"></param>
		/// <param name="useShortNames">if true, replace System.String with string, System.Int32 with int, System.Boolean with bool</param>
		/// <returns></returns>
		public static string GetJavaScriptTypeName(object obj, bool useShortNames = false)
		{
			string ss = GetJavaScriptTypeNameWorker(obj)
				.Replace("+", ".");

				// can not replace System.String with string because registration of type will throw exception
			if (useShortNames)
			{
				ss = ss.Replace("System.String", "string")
				.Replace("System.Decimal", "decimal")
				.Replace("System.Int32", "int")
				.Replace("System.Boolean", "bool");
			}
			if(ss.Contains("[") || ss.Contains("]"))
			{
				return "[]";
			}
			return ss;
		}

		private static byte[] ReadDataFromStream(Stream stream)
		{
			MemoryStream ms = new MemoryStream();
			int read = -1;
			byte[] buffer = new byte[1024];
			do
			{
				read = stream.Read(buffer, 0, buffer.Length);
				ms.Write(buffer, 0, read);
			}
			while (read > 0);

			return ms.ToArray();
		}

		public static string GeneratePassword(int length)
		{
			Random rnd = new Random();
			//string table = "asdgdfdpoiuurwqt123465216097325FSGHYWoiyqwerohqwerFASGAMNBZXCV";
			string table = "123456789";
			string pass = "";
			while (pass.Length < length)
			{
				int index = rnd.Next(0, table.Length);
				pass += table[index];
			}
			return pass;
		}

		/// <summary>
		/// trim string, if string is null or empty, returns null, else return string
		/// </summary>
		/// <param name="str"></param>
		/// <returns></returns>
		public static string StringOrNullIfEmpty(string str)
		{
			if (string.IsNullOrEmpty(str))
			{
				return null;
			}

			str = str.Trim();

			if (string.IsNullOrEmpty(str))
			{
				return null;
			}
			else
			{
				return str;
			}
		}

		//static string _utfHeader = "" + (char)0xEF + (char)0xBB + (char)0xBF;
		// utf header char num value: 65279

		public static string RemoveUtfHeader(string str)
		{
			while(!string.IsNullOrEmpty(str) && str[0] > 6000)
			{
				str = str.Substring(1);
			}

			return str;
		}

		/// <summary>
		/// Shorten text to maxChars size including elipsis. If minChars >= 0, cut on word boundry, else cut string anywhere at exact maxChars size.
		/// </summary>
		/// <param name="str"></param>
		/// <param name="maxChars"></param>
		/// <param name="minChars"></param>
		/// <param name="elipsis"></param>
		/// <returns></returns>
		public static string ShortenText(string str, int maxChars, int? minChars = null, string elipsis = "...")
		{
			if (string.IsNullOrEmpty(str) || maxChars < 0 || str.Length <= maxChars)
			{
				return str;
			}

			int elipsisLength = !string.IsNullOrEmpty(elipsis) ? elipsis.Length : 0;
			if (maxChars <= elipsisLength)
			{
				throw new Exception("maxChars can't be less then elipsis length");
			}

			maxChars -= elipsisLength; // maxChars = chars + elipsis

			// if minChars is defined, cut on word, else just cut anywhere
			if (minChars == null)
			{
				return str.Remove(maxChars - 1) + elipsis;
			}

			for (int index = maxChars - 1; index >= minChars.GetValueOrDefault(); index--)
			{
				if (!char.IsLetterOrDigit(str, index))
				{
					// can cut here
					return str.Remove(index) + elipsis;
				}
			}

			// if comes here, index = 0, string was not cut, non letter or digit was not found, cut string at maxchars
			return str.Remove(maxChars) + elipsis;
		}

		public static string CapitalizeFirstLetter(string str)
		{
			if (string.IsNullOrEmpty(str))
			{
				return str;
			}

			return str.Substring(0, 1).ToUpper() + str.Substring(1);
		}

		public static string TrimNonWords(string str, bool trimStart, bool trimEnd)
		{
			if (string.IsNullOrEmpty(str))
			{
				return str;
			}

			if (trimStart && trimEnd)
			{
				return new Regex("(^[^\\w]+)|([^\\w]+$)").Replace(str, "");
			}

			if (trimStart)
			{
				return new Regex("(^[^\\w]+)").Replace(str, "");
			}

			if (trimEnd)
			{
				return new Regex("([^\\w]+$)").Replace(str, "");
			}

			throw new ArgumentOutOfRangeException("trimStart or trimEnd has to be true");
		}

		/// <summary>
		/// Replace multiple spaces with replacement
		/// </summary>
		/// <param name="str"></param>
		/// <param name="replacement"></param>
		/// <returns></returns>
		public static string ReplaceWhiteSpaces(string str, string replacement)
		{
			return new Regex("[\\s]+").Replace(str ?? "", replacement);
		}

		public static string ConvertToGooglMapDecimal(decimal? value)
		{
			if (value == null)
			{
				return "null";
			}
			return value.Value.ToString("N14", CultureInfo.InvariantCulture.NumberFormat);
		}

		public static decimal? ParseGoogleMapDecimal(string num)
		{
			if (string.IsNullOrEmpty(num))
			{
				return null;
			}
			decimal result;
			if (decimal.TryParse(num, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture, out result))
			{
				return result;
			}
			return null;
		}

		public static string FormatCroDateTime(DateTime? date, bool addDate, bool addTime)
		{
			if (date == null)
			{
				return null;
			}
			else if (addDate && addTime)
			{
				return date.GetValueOrDefault().ToString("dd.MM.yyyy. HH:mm:ss");
			}
			else if (addTime)
			{
				return date.GetValueOrDefault().ToString("HH:mm:ss");
			}
			else if (addDate)
			{
				return date.GetValueOrDefault().ToString("dd.MM.yyyy.");
			}
			else
			{
				return null;
			}
		}

		public static T GetPropertyValue<T>(object item, string propertyName)
		{
			object obj = item.GetType().GetProperty(propertyName).GetValue(item, null);
			return IceTypeConverter.ChangeType<T>(obj);
		}

		public static T GetFieldValue<T>(object item, string fieldName)
		{
			object obj = item.GetType().GetField(fieldName).GetValue(item);
			return IceTypeConverter.ChangeType<T>(obj);
		}


		/// <summary>
		/// returns first non-null argument
		/// </summary>
		/// <param name="args"></param>
		public static object Coalesce(params object[] args)
		{
			if (args != null)
			{
				foreach (object o in args)
				{
					if (o != null)
					{
						return o;
					}
				}
			}
			return null;
		}

		/// <summary>
		/// returns first non-null argument
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="args"></param>
		/// <returns></returns>
		public static T Coalesce<T>(params T[] args)
		{
			if (args != null)
			{
				foreach (T o in args)
				{
					if (o != null)
					{
						return o;
					}
				}
			}
			return default(T);
		}

		#region GetDomainFromHost methods

		#if !SILVERLIGHT

		public static string GetDomainFromHost()
		{
			return GetHttpWwwDomainFromHost(false, false);
		}

		/// <summary>
		/// kreira www.toolnix.com ili ako je subdomena, ne dodaje www, npr. nesto.toolnix.com
		/// </summary>
		/// <returns></returns>
		public static string GetWwwDomainFromHost()
		{
			return GetHttpWwwDomainFromHost(false, true);
		}

		/// <summary>
		/// kreira http://www.toolnix.com ili ako je subdomena, ne dodaje www, npr. http://nesto.toolnix.com
		/// </summary>
		/// <returns></returns>
		public static string GetHttpWwwDomainFromHost()
		{
			return GetHttpWwwDomainFromHost(true, true);
		}

		/// <summary>
		/// vraca se stvarna domena (host), sa http(s)://www. ispred, koristi se kao domena klijenta, ili city domena, ili mojkvart.hr
		/// </summary>
		/// <returns></returns>
		public static string GetHttpWwwDomainFromHost(bool addHttp, bool addWww)
		{
			string prefix = "http://";
			string port = null;

			if (HttpContext.Current.Request.Url.Port != 80 && HttpContext.Current.Request.Url.Port != 443)
			{
				port = ":" + HttpContext.Current.Request.Url.Port;
			}

			if (HttpContext.Current.Request.IsSecureConnection)
			{
				prefix = "https://";
				port = null;
			}

			string host = HttpContext.Current.Request.Url.Host.ToLower();
			int dots = host.Where(o => o == '.').Count();
			// www dodati ispred ako je npr toolnix.com, ali ako je nesto.toolnix.com, ne dodavati www
			// ako je localhost, ne dodavati www
			return (addHttp ? prefix : null) + (addWww && dots == 1 ? "www." : null) + host + port;
		}
		
		#endif

		#endregion

		#region IP to integer and integer to IP

		public static string IPAddressFromInteger(long ip)
		{
			return IPAddress.Parse(ip + "").ToString();
		}

		public static long IPAddressToInteger(string ip)
		{
			return IPAddress.Parse(ip).GetAddressBytes().Reverse<byte>().Select((val, index) => val * (long)Math.Pow(256, index)).Sum();
		}

		#endregion

		#region MSISDN & phone number methods

		/// <summary>
		/// <para>PhoneNumber format: 385911444555, 091 1444 555, 0911444555, 091/1222-333, 385912233343, +385913432324, 00385914326234</para> 
		/// <para>Returns MSISDN 3859..., or null if number is not valid</para>
		/// </summary>
		/// <param name="phoneNumber"></param>
		/// <returns></returns>
		public static string ConvertToMSISDN(string phoneNumber, string countryPrefix = "385")
		{
			if (string.IsNullOrEmpty(phoneNumber))
			{
				return null;
			}

			if (phoneNumber.StartsWith("+"))
			{
				// international
				// +44323456432234
				phoneNumber = "00" + phoneNumber.Substring(1);
			}

			string msisdn = new Regex("([^\\d]+)").Replace(phoneNumber, ""); 	// remove non-digits

			if (msisdn.StartsWith("00"))
			{
				// interanational 00385911234532
				msisdn = msisdn.Substring(2);
			}
			else if (msisdn.StartsWith("0"))
			{
				// national 09113423563
				msisdn = countryPrefix + msisdn.Substring(1);
			}
			else if (msisdn.Length > 10 && msisdn.Length < 20)
			{
				// intenacional number
				// 385911234234
				// 44323456432234
			}
			else
			{
				// invalid phone number
				return null;
			}

			if (string.IsNullOrEmpty(msisdn) || msisdn.Length < 10 || msisdn.Length > 20 || msisdn.StartsWith("000"))
			{
				return null;
			}
			else if (msisdn.Distinct(o => o).Count() < 3)
			{
				// moraju biti barem 3 razlicite znamenke upisane (3859...)
				// ako su sve znamenke iste, npr ako je upisano +00000
				return null;
			}

			return msisdn;
		}

		/// <summary>
		/// Returns 0911444555, MSISDN format: 385911444555, returns 0911444555
		/// </summary>
		/// <param name="msisdn"></param>
		/// <returns></returns>
		public static string ConvertToPhoneNum(string msisdn)
		{
			return "0" + msisdn.Substring(3);
		}

		/// <summary>
		/// returns 091/155-5545, or scrambled 091/XXX-5545
		/// </summary>
		/// <param name="msisdn"></param>
		/// <param name="scramble"></param>
		/// <returns></returns>
		public static string ConvertToPhoneNumWithSlash(string msisdn, bool scramble)
		{
			if (msisdn == null || msisdn.Length < 10)
			{
				return null;
			}
			string phone = ConvertToPhoneNum(msisdn);
			string p1 = phone.Remove(3);
			string p2 = phone.Substring(3, 3);
			string p3 = phone.Substring(6);
			if (scramble)
			{
				p2 = "XXX";
			}
			return p1 + "/" + p2 + "-" + p3;
		}

		public static string ConvertToPhoneNumWithSeparator(string msisdn, string separator1, string separator2)
		{
			if (msisdn == null || msisdn.Length < 10)
			{
				return null;
			}
			string phone = ConvertToPhoneNum(msisdn);
			Regex re = null;

			if (phone.StartsWith("01"))
			{
				re = new Regex("([\\d]{2})([\\d]{3})([\\d]+)");
			}
			else
			{
				re = new Regex("([\\d]{3})([\\d]{3})([\\d]+)");
			}
			return re.Replace(phone, "$1" + separator1 + "$2" + separator2 + "$3");
		}

		public static string FormatMsisdnWithPlus(string msisdn)
		{
			if (string.IsNullOrEmpty(msisdn))
			{
				return msisdn;
			}
			else if (msisdn.StartsWith("00"))
			{
				return "+" + msisdn.Substring(2);
			}
			else if (msisdn.StartsWith("+"))
			{
				return msisdn;
			}
			else if (msisdn.StartsWith("0"))
			{
				// npr 09112341234, ne znam country pa necemo dodavati country prefix
				return msisdn;
			}
			else
			{
				// pretpostavimo da je pravi msisdn, samo bez +
				return "+" + msisdn;
			}
		}

		#endregion

		#region html conversion methods

		public static string ConvertTextToHtml(string text)
		{
			if (string.IsNullOrEmpty(text))
			{
				return text;
			}

			//return text.TrimEnd('\r', '\n', ' ', '\t').Replace("\n", "<br/>").Replace("\r", "").Replace("  ", "&nbsp;&nbsp;");
			 // obavezno ostaviti <br/>\r\n jer neki e-mail programi uklanjaju html tagove pa da se ne spoje svi redovi

			return text.Replace("\r\n", "<br/>").Replace("\n", "<br/>"); // neki imaju samo \n
		}

		public static string RemoveHtmlTags(string htmlText, string replacement = "")
		{
			if (string.IsNullOrEmpty(htmlText))
			{
				return htmlText;
			}

			Regex re = new Regex("\\<[^\\>]+\\>");
			return re.Replace(htmlText, replacement);
		}

		/// <summary>
		/// replace &gt; and &lt; with &amp;gt; and &amp;lt;
		/// </summary>
		/// <param name="str"></param>
		/// <returns></returns>
		public static string HtmlEncodeSimple(string str)
		{
			if (string.IsNullOrEmpty(str))
			{
				return str;
			}

			return str.Replace("<", "&lt;").Replace(">", "&gt;");

		}

		/// <summary>
		/// replace &amp;gt; and &amp;lt; with &gt; and &lt;
		/// </summary>
		/// <param name="str"></param>
		/// <returns></returns>
		public static string HtmlDecodeSimple(string str)
		{
			if (string.IsNullOrEmpty(str))
			{
				return str;
			}

			return str.Replace("&lt;", "<").Replace("&gt;", ">");

		}

		/// <summary>
		/// convert text to html with simple html encode
		/// </summary>
		/// <param name="txt"></param>
		/// <returns></returns>
		public static string ConvertToHtmlWithHtmlEncodeSimple(string txt)
		{
			if (string.IsNullOrEmpty(txt))
			{
				return txt;
			}

			return ConvertTextToHtml(HtmlEncodeSimple(txt));

		}

		#endregion

		#region HR chars replacer

		public static string ReplaceHRChars(string inputText)
		{
			if (string.IsNullOrEmpty(inputText))
			{
				return inputText;
			}

			return inputText
						.Replace("č", "c")
						.Replace("ć", "c")
						.Replace("ž", "z")
						.Replace("š", "s")
						.Replace("đ", "d")
						.Replace("Č", "C")
						.Replace("Ć", "C")
						.Replace("Ž", "Z")
						.Replace("Š", "S")
						.Replace("Đ", "D");
		}

		public static string ReplaceHRCharsForWeb(string inputText)
		{
			if (string.IsNullOrEmpty(inputText))
			{
				return inputText;
			}
			return inputText
				.Replace("č", "&#269;")
				.Replace("ć", "&#263;")
				.Replace("ž", "&#382;")
				.Replace("š", "&#353;")
				.Replace("đ", "&#273;")
				.Replace("Č", "&#268;")
				.Replace("Ć", "&#262;")
				.Replace("Ž", "&#381;")
				.Replace("Š", "&#352;")
				.Replace("Đ", "&#272;");
		}

		#endregion

		#region url encode and clean

		private static readonly string mUrlAcceptedChars = new Func<string>(() =>
		{
			string s = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
			return s + s.ToLower() + "1234567890";
		}).Invoke();

		/// <summary>
		/// Replace HR slova, sve non-url prihvatljivo replace with -
		/// </summary>
		/// <param name="str"></param>
		/// <returns></returns>
		public static string GetUrlEncodedAndCleaned(string inputText, bool lowercase = false)
		{
			if (string.IsNullOrEmpty(inputText))
			{
				return inputText;
			}

			if (lowercase)
			{
				inputText = inputText.ToLower();
			}

			bool lastIsMinus = false;

			StringBuilder sb = new StringBuilder();
			foreach (char c in ReplaceHRChars(inputText).ToCharArray())
			{
				if (mUrlAcceptedChars.IndexOf(c) >= 0)
				{
					sb.Append(c);
					lastIsMinus = false;
				}
				else if (!lastIsMinus)
				{
					// replace with -
					sb.Append('-');
					lastIsMinus = true;
				}
			}
			return sb.ToString().Trim(' ', '-', '_');
		}

		#endregion

		#region DropDown & List control select and get selected value

		#if !SILVERLIGHT

		/// <summary>
		/// Return true if value was found and selected
		/// </summary>
		/// <param name="ddl"></param>
		/// <param name="value"></param>
		/// <returns></returns>
		public static bool SelectDropDownValue(ListControl ddl, object value)
		{
			ddl.SelectedIndex = -1; // if value is not found, let's default value be selected

			ListItem item = ddl.Items.FindByValue(value + "");
			if (item != null)
			{
				ddl.SelectedIndex = ddl.Items.IndexOf(item);
				return true;
			}
			else
			{
				return false;
			}
		}

		public static bool SetListControlSelectedValue(ListControl listControl, object value)
		{
			return SelectDropDownValue(listControl, value);
		}

		public static T GetListControlSelectedValue<T>(ListControl listControl)
		{
			T o;
			IceTypeConverter.TryChangeType<T>(listControl.SelectedValue, out o);
			return o;
		}

		#endif

		#endregion



	}
}
