﻿#if !SILVERLIGHT

using System;
using System.Web;
using System.Text;

namespace Ice.Web
{
	public class IcePathUtility
	{
		/// <summary>
		/// Returns server only without app path based on current request (http or https), example: http://myweb.com/
		/// </summary>
		public static string ServerHttpRoot
		{
			get
			{
				// value not cached, http <-> https switching wouldn't work if cached
				return (HttpContext.Current.Request.IsSecureConnection ? "https://" : "http://")
					+ HttpContext.Current.Request.ServerVariables["HTTP_HOST"] // includes port number
					;
			}
		}

		/// <summary>
		/// Returns application http root based on current request (http or https), example: http://myweb.com/shop/
		/// </summary>
		public static string AppHttpRoot
		{
			get
			{
				// value not cached, http & https switching wouldnt work if cached

				string path = ServerHttpRoot.TrimEnd('/') + HttpRuntime.AppDomainAppVirtualPath;
				if (!path.EndsWith("/"))
				{
					return path + "/";
				}
				return path;
			}
		}

		/// <summary>
		/// fix .net ApplicationPath to always return value with / at the end: /shop/ or just /
		/// </summary>
		public static string AppVirtualPath
		{
			get
			{
				string path = HttpRuntime.AppDomainAppVirtualPath; // ovo radi i iz drugog threada, non-request threada // /shop, or /
				if (!path.EndsWith("/"))
				{
					return path + "/";
				}
				return path;
			}
		}

	}
}
#endif
