﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using Ice.Utility;
#if !SILVERLIGHT
using System.Web;
#else
using System.Windows.Browser;
#endif

namespace Ice.Web
{
	/// <summary>
	/// Key case non-sensitive
	/// </summary>
	[DebuggerDisplay("{PathAndQueryString}")]
	public class QueryStringHelper
	{
		protected Dictionary<string, KeyValuePair<string, object>> _dic = new Dictionary<string, KeyValuePair<string, object>>();

		public QueryStringHelper()
		{

		}

		public QueryStringHelper(string url)
		{
			this.ParseImpl(url);
		}

		public virtual string BaseURL {get;set;}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="key"></param>
		/// <param name="value">if null, removes key and value from collection, bool value can be null, true, false</param>
		/// <returns></returns>
		public QueryStringHelper SetValue(string key, object value)
		{
			if (value == null)
			{
				this._dic.Remove(key);
			}
			else
			{
				if (value.GetType() == typeof(bool))
				{
					value = (bool)value == true ? "1" : "0";
				}

				this._dic[key.ToLower()] = new KeyValuePair<string, object>(key, value);
			}
			return this;
		}

		public QueryStringHelper RemoveValue(string key)
		{
			this._dic.Remove(key.ToLower());
			return this;
		}

		public T GetValue<T>(string key)
		{
			KeyValuePair<string, object> kv;
			if (this._dic.TryGetValue(key.ToLower(), out kv))
			{
				return IceTypeConverter.ChangeType<T>(kv.Value);
			}
			return default(T);
		}

		private void ParseParameters(string qstr)
		{
			qstr = qstr.Trim('?', '&', '\\', '/', ' ');

			foreach (string kv in qstr.Split('?', '&', '#'))
			{
				if (!string.IsNullOrEmpty(kv))
				{
					string[] kvarr = kv.Split('=');
					string key = null;
					string value = null;
					if (kvarr.Length > 0 && !string.IsNullOrEmpty(kvarr[0]))
					{
						key = HttpUtility.UrlDecode(kvarr[0]);
						if (kvarr.Length > 1)
						{
							value = HttpUtility.UrlDecode(kvarr[1]);
						}
					}

					if (!string.IsNullOrEmpty(key))
					{
						// keep lower-cased key
						this.SetValue(key, value);
					}
				}
			}
		}

		protected void ParseImpl(string rawUrl)
		{
			if(string.IsNullOrEmpty(rawUrl))
			{
				return;
			}
			else if (rawUrl.IndexOf("?") >= 0)
			{
				// base url with query string
				this.BaseURL = rawUrl.Substring(0, rawUrl.IndexOf("?"));
				this.ParseParameters(rawUrl.Substring(rawUrl.IndexOf("?")));
			}
			else if (rawUrl.IndexOf("=") >= 0 || rawUrl.IndexOf("&") >= 0)
			{
				// just query string values, there is no base url
				this.ParseParameters(rawUrl);
			}
			else
			{
				// base url only
				this.BaseURL = rawUrl;
			}
		}

		////private static QueryStringHelper Parse(string rawUrl)
		////{
		////	QueryStringHelper qq = new QueryStringHelper();
		////	qq.ParseImpl(rawUrl);
		////	return qq;
		////}

		private string QueryString
		{
			get { return _dic.Values.Select(o => HttpUtility.UrlEncode(o.Key) + "=" + HttpUtility.UrlEncode(o.Value + "")).ToStringJoined("&"); }
		}

		private string PathAndQueryString
		{
			get
			{
				string qstr = this.QueryString;
				bool addQuestion = !string.IsNullOrEmpty(this.BaseURL) && !string.IsNullOrEmpty(qstr);
				return this.BaseURL + (addQuestion ? "?" : null) + qstr;
			}
		}

		/// <summary>
		/// Return full url: full path and query string
		/// </summary>
		/// <returns></returns>
		public string GetUrl()
		{
			return this.PathAndQueryString;
		}
	}
}
