﻿#if !SILVERLIGHT

using System;
using System.Collections.Generic;
using System.Web;
using System.Text;
using System.IO;

namespace Ice.Web
{
	public class RequestDumper
	{
		private StringBuilder sb = new StringBuilder();

		private string GetInputTextDump(HttpContext context)
		{
			// Extract the XML from the request.
			Stream RequestStream = context.Request.InputStream;
			StreamReader RequestStreamReader = new StreamReader(RequestStream);
			string inputTxt = RequestStreamReader.ReadToEnd();
			RequestStream.Close();
			return inputTxt;
		}

		private void AppendLine(string txt, int newLines = 1)
		{
			if (!string.IsNullOrEmpty(txt))
			{
				sb.Append(txt);
			}
			AppendNewLine(newLines);
		}

		private void AppendNewLine(int lines = 1)
		{
			for (int n = 0; n < lines; n++)
			{
				sb.Append("\r\n");
			}
		}

		private void AppendTitle(string title)
		{
			AppendLine("========== " + title + " ==========");
		}

		private void CreateRequestDump(HttpContext context)
		{
			AppendTitle("QUERY STRING");
			foreach (string key in context.Request.QueryString.AllKeys)
			{
				AppendLine(key + ": " + context.Request.QueryString[key]);
			}

			AppendNewLine(3);
			AppendTitle("POST VARIABLES");
			foreach (string key in context.Request.Form.AllKeys)
			{
				AppendLine(key + ": " + context.Request.Form[key]);
			}

			AppendNewLine(3);
			AppendTitle("HEADER VARIABLES");
			foreach (string key in context.Request.Headers.AllKeys)
			{
				AppendLine(key + ": " + context.Request.Headers[key]);
			}

			AppendNewLine(3);
			AppendTitle("SERVER VARIABLES");
			foreach (string key in context.Request.ServerVariables.AllKeys)
			{
				AppendLine(key + ": " + context.Request.ServerVariables[key]);
			}

			AppendNewLine(3);
			AppendTitle("Request.Url");
			AppendLine("AbsolutePath: " + context.Request.Url.AbsolutePath);
			AppendLine("AbsoluteUri: " + context.Request.Url.AbsoluteUri);
			AppendLine("Authority: " + context.Request.Url.Authority);
			AppendLine("DnsSafeHost: " + context.Request.Url.DnsSafeHost);
			AppendLine("Fragment: " + context.Request.Url.Fragment);
			AppendLine("Host: " + context.Request.Url.Host);
			AppendLine("HostNameType: " + context.Request.Url.HostNameType);
			AppendLine("LocalPath: " + context.Request.Url.LocalPath);
			AppendLine("OriginalString: " + context.Request.Url.OriginalString);
			AppendLine("PathAndQuery: " + context.Request.Url.PathAndQuery);
			AppendLine("Port: " + context.Request.Url.Port);
			AppendLine("Query: " + context.Request.Url.Query);
			AppendLine("Scheme: " + context.Request.Url.Scheme);
			AppendLine("Segments: " + context.Request.Url.Segments);
			AppendLine("UserEscaped: " + context.Request.Url.UserEscaped);
			AppendLine("UserInfo: " + context.Request.Url.UserInfo);

			AppendNewLine(3);
			AppendTitle("Request");
			AppendLine("ApplicationPath: " + context.Request.ApplicationPath);
			AppendLine("AppRelativeCurrentExecutionFilePath: " + context.Request.AppRelativeCurrentExecutionFilePath);
			AppendLine("ContentLength: " + context.Request.ContentLength);
			AppendLine("ContentType: " + context.Request.ContentType);
			AppendLine("CurrentExecutionFilePath: " + context.Request.CurrentExecutionFilePath);
			AppendLine("FilePath: " + context.Request.FilePath);
			AppendLine("IsAuthenticated: " + context.Request.IsAuthenticated);
			AppendLine("IsLocal: " + context.Request.IsLocal);
			AppendLine("IsSecureConnection: " + context.Request.IsSecureConnection);
			AppendLine("Path: " + context.Request.Path);
			AppendLine("PathInfo: " + context.Request.PathInfo);
			AppendLine("PhysicalApplicationPath: " + context.Request.PhysicalApplicationPath);
			AppendLine("PhysicalPath: " + context.Request.PhysicalPath);
			AppendLine("QueryString: " + context.Request.QueryString);
			AppendLine("RawUrl: " + context.Request.RawUrl);
			AppendLine("RequestType: " + context.Request.RequestType);
			AppendLine("TotalBytes: " + context.Request.TotalBytes);
			AppendLine("Url: " + context.Request.Url);
			AppendLine("UrlReferrer: " + context.Request.UrlReferrer);
			AppendLine("UserAgent: " + context.Request.UserAgent);
			AppendLine("UserHostAddress: " + context.Request.UserHostAddress);
			AppendLine("UserHostName: " + context.Request.UserHostName);

			AppendNewLine(3);
			AppendTitle("HttpRuntime");
			try
			{
				AppendLine("AppDomainAppId: " + HttpRuntime.AppDomainAppId);

				AppendLine("AppDomainAppPath: " + HttpRuntime.AppDomainAppPath);
				AppendLine("AppDomainAppVirtualPath: " + HttpRuntime.AppDomainAppVirtualPath);
				AppendLine("AppDomainId: " + HttpRuntime.AppDomainId);
				AppendLine("AspClientScriptPhysicalPath: " + HttpRuntime.AspClientScriptPhysicalPath);
				AppendLine("AspClientScriptVirtualPath: " + HttpRuntime.AspClientScriptVirtualPath);
				AppendLine("AspInstallDirectory: " + HttpRuntime.AspInstallDirectory);
				AppendLine("BinDirectory: " + HttpRuntime.BinDirectory);
				AppendLine("ClrInstallDirectory: " + HttpRuntime.ClrInstallDirectory);
				AppendLine("CodegenDir: " + HttpRuntime.CodegenDir);
				AppendLine("IsOnUNCShare: " + HttpRuntime.IsOnUNCShare);
				AppendLine("MachineConfigurationDirectory: " + HttpRuntime.MachineConfigurationDirectory);
				AppendLine("UsingIntegratedPipeline: " + HttpRuntime.UsingIntegratedPipeline);
			}
			catch (Exception ex)
			{
				AppendLine(ex + "");
			}

			AppendNewLine(3);
			AppendTitle("Input stream dump");
			AppendLine(GetInputTextDump(context));

			AppendNewLine(3);
		}

		private string GetDumpLog()
		{
			return sb.ToString();
		}

		public static string GetRequestDump(HttpContext context)
		{
			RequestDumper dumper = new RequestDumper();
			dumper.CreateRequestDump(HttpContext.Current);
			return dumper.GetDumpLog();
		}

		public static string WriteDumpToAppData(string fileName = "Request")
		{
			RequestDumper dumper = new RequestDumper();
			dumper.CreateRequestDump(HttpContext.Current);
			string dump = dumper.GetDumpLog();
			try
			{
				System.IO.File.WriteAllText(HttpContext.Current.Request.PhysicalApplicationPath + "App_Data\\" + fileName + "_dump_" + DateTime.UtcNow.Ticks + ".txt", dump);
			}
			catch (Exception ex)
			{
				dumper.AppendTitle("WriteDumpToAppData");
				dumper.AppendLine(ex + "");
			}
			return dumper.GetDumpLog();
		}

	}
}
#endif
