﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace System.Linq
{
	public class IceYielder<T>
	{
		IEnumerable<T> _source;
		IEnumerator<T> _enumerator;
		object _readLock = new object();

		public int Index { get; private set; }
		public bool Finished { get; private set; }

		public IceYielder(IEnumerable<T> source)
		{
			this._source = source;
			this.Reset();
		}

		public void Reset()
		{
			lock (_readLock)
			{
				this.Index = -1;
				this.Finished = false;
				this._enumerator = null;
			}
		}

		/// <summary>
		/// Yield next item. Method is thread safe. Returns true if item yielded.
		/// </summary>
		/// <param name="item"></param>
		/// <returns></returns>
		public bool YieldNext(out T item)
		{
			lock (_readLock)
			{
				if (this._enumerator == null)
				{
					this._enumerator = this._source.GetEnumerator();
				}

				if (this._enumerator.MoveNext())
				{
					this.Index++;
					item = this._enumerator.Current;
					return true;
				}
				else
				{
					this.Finished = true;
					item = default(T);
					return false;
				}
			}
		}

		/// <summary>
		/// Yield next item. Method is thread safe. Execute action passing item as parameter. Returns true if item yielded.
		/// </summary>
		/// <param name="action"></param>
		/// <returns></returns>
		public bool YieldNext(Action<T> action)
		{
			T item;
			if (this.YieldNext(out item))
			{
				action.Invoke(item);
				return true;
			}
			return false;
		}
	}

}
