﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;

namespace Ice.Web
{
	public enum IceAjaxVersionEnum
	{
		/// <summary>
		/// <para>Ajax arguments are send as array of objects. ["John", 55]</para>
		/// <para>Response is object itself</para>
		/// </summary>
		Array,
		/// <summary>
		/// <para>.NET 2.0 compatible. Ajax arguments are strong named, example: {"name":"John", "age":55}</para>
		/// <para>Respose is object itself</para>
		/// </summary>
		StrongNames,
		/// <summary>
		/// <para>.NET 3.5 compatible. Ajax arguments are strong named, example: {"name":"John", "age":55}</para>
		/// <para>Respose object is placed into another's object d property: {d:object}</para>
		/// </summary>
		StrongNames_NET35
	}

	internal class IceWebSettings
	{
#if DEBUG
		internal static readonly bool IsDebugDefined = true;
#else
		internal static readonly bool IsDebugDefined = false;
#endif

		private static bool mWriteIceWebElmahLog = System.Configuration.ConfigurationManager.AppSettings["WriteIceWebElmahLog"] == "true";
		public static bool WriteIceWebElmahLog { get { return mWriteIceWebElmahLog; } }

		private static int mIceWebAjaxTimeoutMS = 90000; // default, miliseconds
		public static int IceWebAjaxTimeoutMS { get { return mIceWebAjaxTimeoutMS; } }

		static IceWebSettings()
		{
			int tmpint;
			if (int.TryParse(System.Configuration.ConfigurationManager.AppSettings["IceWebAjaxTimeoutSeconds"] + "", out tmpint))
			{
				mIceWebAjaxTimeoutMS = tmpint * 1000;
			}
		}

		/* web.config keys:
	<add key="IceAjaxVersion" value="StrongNames"/>
*/

		private static IceAjaxVersionEnum? mGAjaxVersion = null;

		/// <summary>
		/// <para>Specify in web.config, if not specified default is StrongNames</para>
		/// <para>add key="IceAjaxVersion" value="Array"
		/// <para>possible values:</para>
		/// <para>Array</para>
		/// <para>StrongNames</para>
		/// <para>StrongNames_NET35</para>
		/// </summary>
		public static IceAjaxVersionEnum AjaxVersion
		{
			get
			{
				if (mGAjaxVersion == null)
				{
					try
					{
#if SILVERLIGHT
						mGAjaxVersion = IceAjaxVersion.StrongNames;
#else
						mGAjaxVersion = (IceAjaxVersionEnum)Enum.Parse(typeof(IceAjaxVersionEnum), System.Configuration.ConfigurationManager.AppSettings["IceAjaxVersion"] + "", true);
#endif
					}
					catch
					{
						mGAjaxVersion = IceAjaxVersionEnum.StrongNames;
					}

				}
				return (IceAjaxVersionEnum)mGAjaxVersion;
			}
			set
			{
				mGAjaxVersion = value;
			}
		}

	}
}
