﻿/// <reference path="Ice.js" />

if (typeof (Ice) == "undefined") { Ice = {}; }

Ice.ArrayList = function (arr)
{
	///<summary>Constructor. JS Array wrapper</summary>
	/// <param name="arr" type="Array|String">initial data, array or null</param>
	//this._array(arr); // create array as source

	this._array = [];
	
	if(arr)
	{
		this.AddRange(arr);
	}
};


Ice.ArrayList.prototype = new (function ()
{
	this._array = null;

	this._typeName = "Ice.ArrayList";

	this.Add = function (value)
	{
		this._array.push(value);
		return this;
	};
	
	this.AddRange = function (arr)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="arr" type="Array|Ice.ArrayList|Ice.Enumerable"></param>
		
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			this._array.push(arr[0]);
			return this;
		}
		//#endif

		if (arr._typeName == this._typeName)
		{
			arr = this._array;
		}
		else if (arr.ToArray)
		{
			// support for Ice.Enumerable
			arr = arr.ToArray();
		}


		if (arr.push && arr.slice)
		{
			// must test for push because string has slice too
			this._array = this._array.concat(arr.slice(0)); // .slice(startIndex,endIndex) Selects a part of an array, and returns the new array
		}
		else if (arr.length >= 0)
		{
			// string or array or DOM array

			var useCharAt = typeof (arr) == "string"; // chars from string has to be read using charAt, array notation with [index] won't work in all browsers and doctypes

			for (var n = 0; n < arr.length; n++) // must use for(...) if adding dom array which can't be added using concat(arr)
			{
				this._array.push(useCharAt ? arr.charAt(n) : arr[n]);
			}

		}
		else
		{
			// unknown source, source must be an array or enumerable
			throw "source must be an array or enumerable, source: " + arr, "Ice.ListArray invalid source";
			return;
		}

		return this;
	};
	
	this.Count = function ()
	{
		return this._array.length;
	};

	this.Any = function ()
	{
		return this._array.length > 0;
	};

	this.Insert = function (index, value)
	{
		this._array.splice(index, 0, value);
		return this;
	};

	this.ElementAt = function (index)
	{
		return this._array[index];
	};

	this.Remove = function (value)
	{
		return this.RemoveAt(this.IndexOf(value));
	};

	this.RemoveAt = function (index)
	{
		return this.RemoveRange(index, 1);
	};

	this.RemoveRange = function (index, count)
	{
		this._array.splice(index, count || 1); // in-place splice
		return this;
	};

	this.Reverse = function ()
	{
		this._array.reverse(); // in place reverse
		return this;
	};

	this.IndexOf = function (value)
	{
		if (this._array.indexOf)
		{
			// ie8 doesn't support array.indexOf()
			return this._array.indexOf(value);
		}

		// if it is dom array or ie8 or ie7
		// var useCharAt = this.source.charAt; // this is slow as hell in firefox
		var useCharAt = typeof (this._array) == "string"; // invoke at init time, it is slow as hell if array.charAt invoked in foreach every time
		for (var n = 0; n < this._array.length; n++)
		{
			if ((useCharAt ? this._array.charAt[n] : this._array[n]) == value)
			{
				return n;
			}
		}
		return -1;
	};

	this.Contains = function (value)
	{
		return this.IndexOf(value) >= 0;
	};

	this.Clear = function ()
	{
		this._array = [];
		return this;
	};
	
	this.ForEach = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc" type="Function">function(item, index){....}</param>
		
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			callbackFunc(this._array[0]);
			return this;
		}
		//#endif

		for (var n = 0; n < this._array.length; n++)
		{
			if (callbackFunc(this._array[n], n))
			{
				break;
			}
		}
		return this;
	};

	this.ToArray = function ()
	{
		/// <summary>
		/// Returns new sliced JS array
		/// </summary>

		// .slice(start, end) returns the selected elements in an array, as a new array object
		// start 	Required. An integer that specifies where to start the selection (The first element has an index of 0). Use negative numbers to select from the end of an array
		// end: 	Optional. An integer that specifies where to end the selection. If omitted, all elements from the start position and to the end of the array will be selected. Use negative numbers to select from the end of an array	
		return this._array.slice(0);
	};

	this.ToList = function ()
	{
		return new Ice.ArrayList(this._array);
	};

	this.ToStringJoined = function (separator)
	{
		return this._array.join(separator || "");
	};

	this.Skip = function (skip)
	{
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this;
		}
		//#endif

		var list = new Ice.ArrayList();
		this.ForEach(function (item, index)
		{
			if(index >= skip)
			{
				list.Add(item);
			}
		});
		return list;
	};

	this.Take = function (take)
	{
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this;
		}
		//#endif

		var list = new Ice.ArrayList();
		this.ForEach(function (item, index)
		{
			if (index < take)
			{
				list.push(item);
			}
		});
		return list;
	};

	this.Where = function (predicate)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="predicate"  type="Function">function(item, index){....}</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			predicate(this._array[0]);
			return this;
		}
		//#endif

		var a1 = new Ice.ArrayList();
		this.ForEach(function (item, index)
		{
			if (predicate(item, index))
			{
				a1.Add(item);
			}
		});
		return a1;
	};

	this.Select = function (selector)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="selector"  type="Function">function(item, index){....}</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return new Ice.ArrayList([selector(this._array[0])]);
		}
		//#endif

		var a1 = new Ice.ArrayList();
		this.ForEach(function (item, index)
		{
			a1.Add(selector(item, index));
		});
		return a1;
	};

	this.FirstOrDefault = function (defaultValue)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="defaultValue" optional="true"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this._array[0];
		}
		//#endif

		return this._array.length > 0 ? this._array[0] : defaultValue;
	};

	this.LastOrDefault = function (defaultValue)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="defaultValue" optional="true"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this._array[0];
		}
		//#endif

		return this._array.length > 0 ? this._array[this._array.length - 1] : defaultValue;
	};
	

})();


