﻿
Ice.Convert = new (function ()
{
	var __ice = this;

	__ice.ToNumberWithCulture = function (strOrNum, defaultValue, decimalSeparator)
	{
		/// <summary>
		/// Parse as float number using current culture, or decimalSeparator. parse strOrNum and return number, if can't be parsed, return defaultValue
		/// </summary>
		/// <param name="strOrNum" type="String|Object|Number"></param>
		/// <param name="defaultValue" type="Number">default value to be returned if strOrNum is not number</param>
		/// <param name="decimalSeparator" type="String" optional="true">if not set, will use current culture</param>

		decimalSeparator = decimalSeparator || ".";

		if (arguments.length < 3)
		{
			decimalSeparator = Ice.Globalization.CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator;
		}

		strOrNum = (strOrNum + "").ReplaceAll(decimalSeparator == "." ? "," : ".", "").ReplaceAll(decimalSeparator, "."); // convert to US culture since parseFloat uses US culture always

		var parsed = parseFloat(strOrNum);
		if (!isNaN(parsed) && isFinite(parsed) && parsed + "" == strOrNum + "")
		{
			return parsed;
		}
		else
		{
			return defaultValue;
		}
	};

	__ice.ToNumberOrDefault = function (strOrNum, defaultValue)
	{
		/// <summary>
		/// Parse as float number using US culture. parse strOrNum and return number, if can't be parsed, return defaultValue
		/// </summary>
		/// <param name="strOrNum" type="String|Object|Number"></param>
		/// <param name="defaultValue" type="Number">default value to be returned if strOrNum is not number</param>

		var parsed = parseFloat(strOrNum);
		if (!isNaN(parsed) && isFinite(parsed) && parsed + "" == strOrNum + "")
		{
			return parsed;
		}
		else
		{
			return defaultValue;
		}
	};

	__ice.ToInt = function (strOrNum, defaultValue)
	{
		/// <summary>
		/// Parse as float number using US culture.
		/// </summary>
		/// <param name="strOrNum"></param>
		/// <param name="defaultValue"></param>
		/// <returns type=""></returns>
		var num = __ice.ToNumberOrDefault(strOrNum, defaultValue);
		if (!isNaN(num) && isFinite(num))
		{
			return num > 0 ? Math.floor(num) : Math.ceil(num);
		}
	};

	__ice.ToDecimal = __ice.ToNumberOrDefault;

	return __ice;

})();
