﻿//******** NO EXTERNAL DEPENDENCIES ********

if (typeof (Ice) == "undefined") { Ice = {}; }

Ice.CookieUtility = new (function ()
{
	var __util = this;

	__util.SetCookieValue = function (cookieName, cookieValue, expiresAfterSECONDS, domain, path, secure)
	{
		///<summary>Set cookie value</summary>
		///<param name="cookieName" type="String">Cookie name</param>
		///<param name="cookieValue" type="String">Cookie value</param>
		///<param name="expiresAfterSECONDS" type="Number">Namber of seconds to keep cookie alive (optional, can be null), if value not set or null, creates session cookie</param>
		///<param name="domain" type="String">Domain (optioal) with dot before: .domain.com or .www.domain.com</param>
		///<param name="path" type="String">Path, default / (optional)</param> 
		///<param name="secure" type="Bool">Is secure (optional)</param>

		// Set-Cookie:cookieName=cookieValue; domain=.pnew.hr; expires=Wed, 26-Oct-2112 07:54:37 GMT; path=/
		// 
		
		var expires = expiresAfterSECONDS ? new Date(Date.now() + (1000 * expiresAfterSECONDS)) : null; // getTime() return ticks in ms, new Date(ticks in ms)

		var cc = escape(cookieName+"") + '=' + escape(cookieValue + "") // convert cookieValue to string
			+ (expires ? '; expires=' + expires.toGMTString() : '')
			+ (domain ? ('; domain=' + domain) : '') // if not set, browser uses current host, if domain is different than current host or not subset of current host, cookie is not set
			+ ('; path=' + (path ? path : '/')) // default /
			+ (secure ? '; secure' : '');

		document.cookie = cc;
	};

	__util.GetCookieValue = function (cookieName)
	{
		///<summary>Get cookie value</summary>
		///<param name="cookieName" type="String">Cookie to get</param>
		///<returns type="String">String value</returns>

		// document.cookie has "name=value; name1=value1" only, last value doesn't have ;
		var cookieValue = '';
		var posName = document.cookie.indexOf(escape(cookieName) + '=');
		if (posName >= 0)
		{
			var posValue = posName + (escape(cookieName) + '=').length;
			var endPos = document.cookie.indexOf(';', posValue);
			if (endPos >= 0)
			{
				cookieValue = unescape(document.cookie.substring(posValue, endPos));
			}
			else
			{
				// there is no ; so it is last cookie
				cookieValue = unescape(document.cookie.substring(posValue));
			}
		}
		return (cookieValue);
	};

	__util.GetCookies = function ()
	{
		/// <summary>
		/// Get all cookies as array of Name-Values: [{Name:c1, Value:v1}, {Name:c2, Value:v2},...]
		/// </summary>

		var arr = document.cookie.split(';');
		var col = [];
		for(var n = 0; n < arr.length; n++)
		{
			var nv = arr[n].Trim();
			var a = nv.split('=');
			col.push({ Name: unescape(a[0]), Value: unescape(a[1]) });
		}
		return col;
	};

	__util.DeleteCookie = function (cookieName)
	{
		///<summary>Clear cookie</summary>
		///<param name="cookieName" type="String">Cookie to delete</param>
		///<returns type="void"></returns>
		var expiresAfterSECONDS = (-1) * 1000 * 60 * 60 * 24; // -1000 days from now
		__util.SetCookieValue(cookieName, "", expiresAfterSECONDS);

	};

	return __util;

})();

