﻿
if (typeof (Ice) == "undefined") { Ice = {}; }

Ice.DateTime = function (date)
{
	/// <summary>
	/// 
	/// </summary>
	/// <param name="date" type="Date">if not set, will use new Date(0) which is 1.1.1970.</param>
	var date = date || new Date(0);
	if (Ice.IsDebugDefined)
	{
		this._isoDateStr = date.ToStringFormated("yyyy-MM-dd HH:mm:ss.fff K");
	}
	this._dateItem = date;
	this.Kind = date.Kind;
};

//#region Ice.DateTime.dateFormat

Ice.DateTime.dateFormat = new (function ()
{
	var __df = this;

	////var token = /d{1,4}|M{1,4}|yy(?:yy)?|([HhmsTtFf])\1?|[LloKSZ]|"[^"]*"|'[^']*'/g,
	////timezone = /\b(?:[PMCEA][SDP]T|(?:Pacific|Mountain|Central|Eastern|Atlantic) (?:Standard|Daylight|Prevailing) Time|(?:GMT|UTC)(?:[-+]\d{4})?)\b/g,
	////timezoneClip = /[^-+\dA-Z]/g,

	// fixed regex for AjaxMin and Packer
	var token = new RegExp("d{1,4}|M{1,4}|yy(?:yy)?|[H]{1,2}|[h]{1,2}|[m]{1,2}|[s]{1,2}|[f]{1,3}|[t]{1,2}|[LloKSZT]|\"[^\"]*\"|'[^']*", "g"),
		timezone = new RegExp("\b(?:[PMCEA][SDP]T|(?:Pacific|Mountain|Central|Eastern|Atlantic) (?:Standard|Daylight|Prevailing) Time|(?:GMT|UTC)(?:[-+]\d{4})?)\b", "g"),
		timezoneClip = new RegExp("[^-+\dA-Z]", "g"),
		pad = function (val, len)
		{
			val = String(val);
			len = len || 2;
			while (val.length < len) val = "0" + val;
			return val;
		};

	// Regexes and supporting functions are cached through closure
	__df.Format = function (date, mask, utc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="date" type="Date"></param>
		/// <param name="mask" type="String|Ice.DateTime.dateFormat.masks"></param>
		/// <param name="utc" type="Boolean"></param>

		mask = String(__df.masks[mask] || mask || __df.masks.isoDateTime);

		// Allow setting the utc argument via the mask
		if (mask.slice(0, 4) == "UTC:")
		{
			mask = mask.slice(4);
			utc = true;
		}

		var get = utc ? "getUTC" : "get",
			datum = date[get + "Date"](),
			dan = date[get + "Day"](),
			month = date[get + "Month"](),
			year = date[get + "FullYear"](),
			hours = date[get + "Hours"](),
			min = date[get + "Minutes"](),
			s = date[get + "Seconds"](),
			L = date[get + "Milliseconds"](),
			L = pad(Math.floor(L) % 1000, 3), // create 3 chars, eg if L was 12 ticks, convert it  to "012"
			o = utc ? 0 : date.getTimezoneOffset(),
			flags = {
				dddd: __df.i18n.dayNamesLong[dan],
				ddd: __df.i18n.dayNames[dan],
				dd: pad(datum),
				d: datum,
				MMMM: __df.i18n.monthNamesLong[month],
				MMM: __df.i18n.monthNames[month],
				MM: pad(month + 1),
				M: month + 1,
				yy: String(year).slice(2),
				yyyy: year,
				h: hours % 12 || 12,
				hh: pad(hours % 12 || 12),
				H: hours,
				HH: pad(hours),
				m: min,
				mm: pad(min),
				s: s,
				ss: pad(s),
				f: String(L).substr(0, 1),
				ff: String(L).substr(0, 2),
				fff: L,
				t: hours < 12 ? "A" : "P",
				tt: hours < 12 ? "AM" : "PM",
				K: (o > 0 ? "-" : "+") + pad(Math.floor(Math.abs(o) / 60)) + ":" + pad(Math.abs(o) % 60),
				Z: utc ? "UTC" : (String(date).match(timezone) || [""]).pop().replace(timezoneClip, ""),
				//o: (o > 0 ? "-" : "+") + pad(Math.floor(Math.abs(o) / 60) * 100 + Math.abs(o) % 60, 4),
				S: ["th", "st", "nd", "rd"][datum % 10 > 3 ? 0 : (datum % 100 - datum % 10 != 10) * datum % 10]
			};

		return mask.replace(token, function ($0)
		{
			var dd = $0 in flags ? flags[$0] : $0.slice(1, $0.length - 1);
			////Ice.Diagnostic.WriteLine($0 + ": " + dd);
			return dd;
		});
	};

	function Get(valueArr, formatArr, posibleArr)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="valueArr" type="Array"></param>
		/// <param name="formatArr" type="Array"></param>
		/// <param name="posibleArr" type="Array"></param>
		
		for(var n = 0; n < formatArr.length; n++)
		{
			if (posibleArr.indexOf(formatArr[n]+"") >= 0)
			{
				return valueArr[n];
			}
		}
		return null;
	}

	var reg1 = new RegExp("([\\d]+)-([\\d]+)-([\\d]+)[\\s]*([\\d]*)[:]*([\\d]*)[:]*([\\d]*)[\\.]*([\\d]*)");
	var reg2 = new RegExp("([\\d]+).([\\d]+).([\\d]+)[\\.]*[\\s]*([\\d]*)[:]*([\\d]*)[:]*([\\d]*)[\\.]*([\\d]*)");

	__df.Parse = function (str, format)
	{
		if (format)
		{
			var f1 = format.split(new RegExp("[ \\.\\:\\,]"));
			var f2 = str.split(new RegExp("[^\\d]"));

			return Ice.DateTime.Create(
				Get(f2, f1, ["yy", "yyyy"]),
				Get(f2, f1, ["M", "MM"]),
				Get(f2, f1, ["d", "dd"]),
				Get(f2, f1, ["H", "HH"]),
				Get(f2, f1, ["m", "mm"]),
				Get(f2, f1, ["s", "ss"]),
				Get(f2, f1, ["f", "ff", "fff"])
			);
		}
		else
		{
			var m = str.match(reg1);
			if (m)
			{
				return Ice.DateTime.Create(
					parseInt(m[1], 10), // day
					parseInt(m[2], 10), // month
					parseInt(m[3], 10), // year
					parseInt(m[4], 10), // hour
					parseInt(m[5], 10), // minutes
					parseInt(m[6], 10), // seconds
					parseInt(m[7], 10) // ms
				);
			}
			var m = str.match(reg2);
			if (m)
			{
				return Ice.DateTime.Create(
					parseInt(m[3], 10), // year
					parseInt(m[2], 10), // month
					parseInt(m[1], 10), // day
					parseInt(m[4], 10), // hours
					parseInt(m[5], 10), // minutes
					parseInt(m[6], 10), // seconds
					parseInt(m[7], 10) // ms
				);
			}
		}
	};


	// Some common format strings
	__df.masks = {
		"default": "ddd MMM dd yyyy HH:mm:ss",
		shortDate: "M/d/yy",
		mediumDate: "MMM d, yyyy",
		longDate: "MMMM d, yyyy",
		fullDate: "dddd, MMMM d, yyyy",
		shortTime: "h:mm tt",
		mediumTime: "h:mm:ss tt",
		longTime: "h:mm:ss tt K",
		isoDate: "yyyy-MM-dd",
		isoTime: "HH:mm:ss",
		isoDateTime: "yyyy-MM-dd HH:mm:ss.fff",
		isoUtcDateTime: "UTC:yyyy-MM-dd HH:mm:ss.fff"
	};

	// Internationalization strings
	__df.i18n = {
		dayNames: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
		dayNamesLong: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
		monthNames: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
		monthNamesLong: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
		firstDayOfWeek: 0
	};

	return __df;

})();

//#endregion Ice.DateTime.dateFormat

Ice.DateTime.prototype = new (function ()
{
	this._typeName = "Ice.DateTime";

	this._dateItem = new Date();

	this.ToStringFormated = function (format) { return Ice.DateTime.dateFormat.Format(this._dateItem, format); };

	this.toString = function () { return this.ToStringFormated("yyyy-MM-dd HH:mm:ss.fff K"); }; // for convertion to string on this + ""

	this.ToLocalOffset = function ()
	{
		/// <summary>
		/// Apply browser's timezone offset to show correct date and time in browser which was received from server.
		/// </summary>

		return new Ice.DateTime(this._dateItem.ToLocalOffset());
	};

	this.ToServerOffset = function ()
	{
		/// <summary>
		/// Remove browser's timezone offset to get correct date and time which has to be sent to server.
		/// </summary>

		return new Ice.DateTime(this._dateItem.ToServerOffset());
	};

	this.DayOfWeek = function ()
	{
		/// <summary>
		/// JS and .NET has the same DayOfWeek, Sunday==0, enumeration Ice.DateTime.DayOfWeek
		/// </summary>
		
		return this._dateItem.getDay();
	}; 

	this.DatePart = function ()
	{
		/// <summary>
		/// Returns date component only and time 00:00:00
		/// </summary>
		var p = this.ToParts();
		return Ice.DateTime.Create(p.Year, p.Month, p.Day);
	};

	this.TimePart = function ()
	{
		/// <summary>
		/// Returns TimeSpan, a time interval that represents the fraction of the day that has elapsed since midnight. 
		/// </summary>
		var p = this.ToParts();
		return Ice.TimeSpan.Create(0, p.Hour, p.Minute, p.Second, p.Milisecond);
	};

	this.ToParts = function ()
	{
		/// <summary>
		/// Returns date-time parts
		/// </summary>

		// JS months: 0-11

		return {
			Year: this._dateItem.getFullYear(),
			Month: this._dateItem.getMonth() + 1, 
			Day: this._dateItem.getDate(),
			Hour: this._dateItem.getHours(),
			Minute: this._dateItem.getMinutes(),
			Second: this._dateItem.getSeconds(),
			Milisecond: this._dateItem.getMilliseconds(),
			DayOfWeek: this._dateItem.getDay(),
			Ticks: this._dateItem.getTime(),
			DateString: this._dateItem + ""
		};
	};

	this.AddHours = function (hours) { return new Ice.DateTime(new Date(this._dateItem.getTime() + hours * 60 * 60 * 1000)); };
	this.AddMinutes = function (minutes) { return new Ice.DateTime(new Date(this._dateItem.getTime() + minutes * 60 * 1000)); };
	this.AddSeconds = function (seconds) { return new Ice.DateTime(new Date(this._dateItem.getTime() + seconds * 1000)); };
	this.AddMiliseconds = function (misliseconds) { return new Ice.DateTime(new Date(this._dateItem.getTime() + misliseconds)); };
	this.AddDays = function (days) { return new Ice.DateTime(new Date(this._dateItem.getTime() + days * 24 * 60 * 60 * 1000)); };

	this.AddMonths = function (months)
	{
		var currMonths = this._dateItem.getFullYear() * 12 + this._dateItem.getMonth() + months;
		var year = currMonths / 12;
		var month = currMonths % 12;
		var date = new Date(this._dateItem.getTime());
		date.setMonth(month);
		date.setFullYear(year);
		return new Ice.DateTime(date);
	};

	this.AddYears = function (years)
	{
		var year = this._dateItem.getFullYear() + years;
		var date = new Date(this._dateItem.getTime());
		date.setFullYear(year);
		return new Ice.DateTime(date);
	};

})();

/// <reference path="template:DATE_TIME_LOCALIZATION" />

Ice.DateTime.Now = function ()
{
	return new Ice.DateTime(new Date());
};

Ice.DateTime.Create = function (year, month, day, hours, minutes, seconds, miliseconds)
{
	/// <summary>
	/// 
	/// </summary>
	/// <param name="year">eg. 2014</param>
	/// <param name="month">1-12</param>
	/// <param name="day">1-31</param>
	/// <param name="hours">0-23</param>
	/// <param name="minutes">0-59</param>
	/// <param name="seconds">0-59</param>
	/// <param name="miliseconds">0-999</param>

	// Date accepts month [0-11]
	return new Ice.DateTime(new Date(year || 1700, (month || 1) - 1, day || 1, hours || 0, minutes || 0, seconds || 0, miliseconds || 0));
};

Ice.DateTime.Parse = function (str, format)
{
	/// <summary>
	/// <para>automated parsing from any of:</para>
	/// <para>dd.MM.yyyy. HH:mm:ss.fff</para>
	/// <para>yyyy-MM-dd HH:mm:ss.fff</para>
	/// </summary>
	/// <param name="str"></param>
	/// <param name="format" type="String">.NET like date format string</param>
	
	return Ice.DateTime.dateFormat.Parse(str, format);
};

Ice.DateTime.DayOfWeek =
{
	// 0
	Sunday: 0,
	// 1
	Monday: 1,
	// 2
	Tuesday: 2,
	// 3
	Wednesday: 3,
	// 4
	Thursday: 4,
	// 5
	Friday: 5,
	// 6
	Saturday: 6
};

// used in IceDatePicker:
Date.prototype.ToStringFormated = function (format)
{
	/// <summary>
	/// format DateTime to string, default is yyyy-MM-dd HH:mm:ss.fff
	/// </summary>
	/// <param name="format" optional="true">.NET date time string format</param>
	
	return Ice.DateTime.dateFormat.Format(this, format);
};

Date.prototype.ToIceDateTime = function ()
{
	return new Ice.DateTime(this);
};
