﻿
Ice.Diagnostic.Dump = function (object, recusionLimit, encodeHtml)
{
	///<summary>Dump object content into json like formated string</summary>
	///<param name="object" type="Object|Mixed">Object to add</param>
	///<param name="recusionLimit" type="Number">Default is 100</param>
	///<param name="encodeHtml" type="Boolean">true to add br instead of new line</param>

	var indent = 4;
	var level = 0;
	var recursionLimit = arguments.length < 3 ? 100 : recusionLimit; // if invoked from Ice.Enumerable.ToConsole() or ToPrettyJson(), recursion has to be set greater than 1

	var arrItems = [];
	var recursion = 0;

	function HtmlEncode(value)
	{
		value += "";
		var re = new RegExp("(<)(.*?)(>)");
		while (value.match(re) != null)
		{
			value = value.replace(re, "&lt;$2&gt;");
		}
		return value;
	}

	function CopyStr(str, copies)
	{
		var txt = "";
		for (var nn = 0; nn < copies; nn++)
		{
			txt += str;
		}
		return txt;
	}

	function IsObject(obj)
	{
		try
		{
			for (var p in obj)
			{
				return true;
			}
		}
		catch (e)
		{
		}
		return false;
	}

	function IsArray(obj)
	{
		return obj && obj.push && obj.pop;
	}

	function IsString(obj)
	{
		return typeof (obj) == "string";
	}

	var nl = encodeHtml ? "<br/>" : "\r\n";

	function IsNewLineLast()
	{
		return arrItems.length == 0 || ((arrItems[arrItems.length - 1] + "") == nl);
	}

	function NewLine()
	{
		if (!IsNewLineLast())
		{
			arrItems.push(nl);
		}
	}

	function AddIndent()
	{
		arrItems.push(CopyStr(encodeHtml ? "&nbsp;" : " ", indent * level));
	}

	function AddValue(o)
	{
		if (IsNewLineLast())
		{
			AddIndent();
		}
		arrItems.push(o + "");
	}

	function DumpObject(tmpObj)
	{
		if (tmpObj && IsObject(tmpObj))
		{
			NewLine();
			AddValue("{");
			NewLine();
			level++;

			var n = 0;

			for (var prop in tmpObj)
			{
				if (n++ > 0)
				{
					arrItems.push(",");
				}
				NewLine();
				AddValue(prop + ": ");

				try
				{
					Dump(tmpObj[prop]);
				}
				catch (e)
				{
					Dump("*** error ***");
				}
			}

			level--;
			NewLine();
			AddValue("}");
		}
		else
		{
			AddValue(tmpObj);
		}
	}

	function DumpArray(array)
	{
		NewLine();
		AddValue("[");
		level++;

		for (var n = 0; n < array.length; n++)
		{
			if (n > 0)
			{
				arrItems.push(",");
			}
			NewLine();
			Dump(array[n]);
		}

		NewLine();
		level--;
		AddValue("]");
	}

	function Dump(o)
	{
		recursion++;

		if (o == null)
		{
			AddValue("null");
		}
		else if (typeof (o) == "undefined")
		{
			AddValue("undefined");
		}
		else if (typeof (o) == "function")
		{
			AddValue(o + "");
		}
		else if (IsString(o))
		{
			AddValue("\"" + (encodeHtml ? HtmlEncode(o) : o) + "\"");
		}
		else if (IsArray(o))
		{
			if (recursion <= recursionLimit)
			{
				DumpArray(o);
			}
			else
			{
				AddValue("[...]");
			}
		}
		else if (IsObject(o))
		{
			if (recursion <= recursionLimit)
			{
				DumpObject(o);
			}
			else
			{
				AddValue("{...}");
			}
		}
		else
		{
			// number, boolean
			AddValue(o);
		}

		recursion--;
	}

	Dump(object);

	return arrItems.join("");

};



