﻿/// <reference path="Ice.Diagnostic.js" />

Ice.Math = new (function ()
{
	var __ice = this;

	__ice.Rnd = function (minVal, maxVal)
	{
		/// <summary>
		/// Returns random float value.
		/// </summary>
		/// <param name="minVal" optional="true">inclusive min value, default: 0</param>
		/// <param name="maxVal" optional="true">exclusive max value, default: 1</param>
		minVal = minVal || 0;
		maxVal = maxVal || 1;
		if (minVal > maxVal)
		{
			Ice.Diagnostic.Exception("Invalid range in Ice.Math.Rnd(...)");
		}

		return Math.random() * (maxVal - minVal) + minVal;
	};

	__ice.RndInt = function (minVal, maxVal)
	{
		/// <summary>
		/// Returns random int value.
		/// </summary>
		/// <param name="minVal" optional="true">inclusive min value, default: 0</param>
		/// <param name="maxVal" optional="true">exclusive max value, default: 1</param>
		return Math.floor(__ice.Rnd(minVal, maxVal));
	};

	__ice.NewGuid = function ()
	{
		/// <summary>
		/// Generate rfc4122 version 4 compliant new GUID as function of current time and random number.
		/// </summary>
		
		var d = new Date().getTime();
		var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(new RegExp("[xy]", "g"), function (c)
		{
			var r = (d + Math.random() * 16) % 16 | 0;
			d = Math.floor(d / 16);
			return (c == 'x' ? r : (r & 0x7 | 0x8)).toString(16);
		});
		return uuid;
	};

	return __ice;

})();
