﻿/// <reference path="Ice.js" />
/// <reference path="Ice.Diagnostic.js" />
/// <reference path="Ice.Utility.js" />
/// 
////// example:
//////new Ice.Net.WebRequest("/TestFiles/tests/Anketa.xml").OnComplete(function (data)
//////{
//////	var dd = 10;
//////	var items = Ice.DomLinq.FromXml(data);
//////	var gdsf = items.ToArray();
//////	var items2 = items.DescendantNodes().ToArray();
//////}).Get();

// **************************************************************************
// cross domain request is implemented by "origin" header, IE8 & IE9 uses XDomainRequest
// **************************************************************************

Ice.RegisterNs("Ice.Net.WebRequest");

Ice.Net.WebRequest = function (baseUrl)
{
	///<summary>constructor. Create new Ice.Net.WebRequest instance</summary>
	// create new configuration
	this._config(baseUrl);
};

Ice.Net.WebRequest.GetOriginUrl = function ()
{
	return window.location.protocol + "//" + window.location.host;
};

Ice.Net.WebRequest.IsCrossdomain = function (url)
{
	if (url)
	{
		var httpRoot = Ice.Net.WebRequest.GetOriginUrl(); // host includes port (this is origin for crossdomain requests)
		return url.indexOf("http") == 0 && url.indexOf(httpRoot) != 0; // url is absolute and don't starts with httpRoot (origin)
	}
	return false;
};

Ice.Net.WebRequest.GetXMLHttpRequest = function (url)
{
	/// <summary>
	/// Static method to get XMLHTTP object
	/// </summary>
	/// <param name="url" optional="true">if url is provided, and host is different than current host, will use XDomainRequest on IE8 and IE9 </param>
	/// <returns type=""></returns>
	
	if (window.XMLHttpRequest)
	{
		// Mozilla, Safari, ...
		// XMLHttpRequest can be used for crossdomain request too
		return new XMLHttpRequest();
	}
	else if (Ice.Net.WebRequest.IsCrossdomain(url) && window.XDomainRequest)
	{
		// ie8, ie9 ? // custom headers can not be set in XDomainRequest
		return new XDomainRequest();
	}
	else if (window.ActiveXObject) // IE < version 8
	{
		var axO = ['Microsoft.XMLHTTP', 'Msxml2.XMLHTTP', 'Msxml2.XMLHTTP.6.0', 'Msxml2.XMLHTTP.4.0', 'Msxml2.XMLHTTP.3.0'];
		for (var i = 0; i < axO.length; i++)
		{
			try { return new ActiveXObject(axO[i]); } catch (e) { }
		}
	}

	////alert("Your browser doesn't support AJAX. Please use the latest version of Internet Explorer, Firefox, Chrome, Safari, Opera etc.");
	return null;
};

Ice.Net.WebRequest.Globals =
{
	// global method, function(respObj){...}
	OnComplete: null,
	// global method, function(errorMsg){...}
	OnError: null,
	// global method, function(isLoading){...}
	OnLoading: null,
	// global method, logging method function(msg, this){...};
	OnLog: null,
	escapeLevel: 1, // this is set by IceScriptManager property, use 2 or more to prevent from IIS invalid query string chars validation

	httpMethod: "GET",
	timeoutMs: 90000, // modify with value from C#: IceWebSettings.IceWebAjaxTimeoutMS
	logItemLength: -1, // max msg text length per an log item
	url: null,
	deserializeFromJSON: true, // default true, deserialize responseText if it has JSON header
	serializeToJSON: false, // if true, serialize data before sending
	async: true,
	data: null // key-value pairs, plain text, or js object to be sent
};

Ice.Net.WebRequest.Reference = new (function ()
{
	var __this = this;

	var uniqueCount = 0;
	var requestsCache = [];

	var globalInstance = {};

	__this.ShowLoading = function (isLoading)
	{
		/// <summary>
		/// Create dummy item. At the end, remove it. This may be used to show spinner while multiple ajax request are made in between.
		/// </summary>
		/// <param name="isLoading" type="boolean"></param>

		if(isLoading)
		{
			__this.SaveReference(globalInstance);
		}
		else
		{
			__this.ReleaseReference(globalInstance);
		}
	};

	__this.ReleaseReference = function (ajaxInstance)
	{
		// array.splice(index,howmany,item1,.....,itemX)
		// The splice() method adds/removes items to/from an array, and returns the removed item(s).
		////console.log("ReleaseReference: " + requestsCache.length + ", " + !!Ice.Net.WebRequest.Globals.OnLoading);
		if (requestsCache.length ==1 && Ice.Net.WebRequest.Globals.OnLoading)
		{
			// invoke global OnLoading
			////console.log("hide loading: " + requestsCache.length);
			Ice.Net.WebRequest.Globals.OnLoading.call(this, false);
		}

		var index = requestsCache.indexOf(ajaxInstance);
		if (index >= 0)
		{
			// Note: This method changes the original array.
			requestsCache.splice(index, 1);
		}
		////console.log("after splice: " + requestsCache.length);

		// novo:end response
		Ice.Page._onIceAjaxEndRequest.Invoke();
	};

	__this.SaveReference = function (ajaxInstance)
	{
		/// <summary>
		/// create new reference, save to cache, return referecne object
		/// </summary>
		////console.log("SaveReference: " + requestsCache.length + ", " + !!Ice.Net.WebRequest.Globals.OnLoading);
		// novo:end response
		Ice.Page._onIceAjaxBeginRequest.Invoke();

		if (requestsCache.length == 0 && Ice.Net.WebRequest.Globals.OnLoading)
		{
			// invoke global OnLoading
			////console.log("show loading: " + requestsCache.length);
			Ice.Net.WebRequest.Globals.OnLoading.call(this, true);
		}

		requestsCache.push(ajaxInstance);
		////console.log("after push: " + requestsCache.length);

		var rr = ajaxInstance._reference = {
			ReqCount: ++uniqueCount,
			StartTicks: new Date().getTime(),
			WebRequest: ajaxInstance
		};
	};


});

Ice.Net.WebRequest.prototype = new (function ()
{
	this._config = function (baseUrl)
	{
		this._config = Ice.Extend({}, Ice.Net.WebRequest.Globals); // create new copy every time

		// don't use global OnLoading, it is invoked from Ice.Net.WebRequest.Reference
		this._config.OnLoading = null;

		this._config.url = baseUrl;

		this._xmlhttp = null;

		this._resp =
			{
				timeoutId: null,
				isHandled: false
			};
	};

	this.typeName = "Ice.Net.WebRequest";

	function Bind(method, context)
	{
		return method.BindContext(context);
	};

	function ReleaseReference()
	{
		Ice.Net.WebRequest.Reference.ReleaseReference(this);
	}

	function SaveReference()
	{
		Ice.Net.WebRequest.Reference.SaveReference(this);
	}

	function ReadyStateCallbackFunc()
	{
		if (this._config.OnLog)
		{
			WriteLog.call(this, "readyState: " + this._xmlhttp.readyState);
		}

		if (this._xmlhttp.readyState == 4) // complete
		{
			if (this._resp.isHandled)
			{
				return;
			}
			this._resp.isHandled = true;

			this._xmlhttp.onload = function () { }; // XDomainRequest
			this._xmlhttp.onreadystatechange = function () { };

			// response received
			if (this._resp.timeoutId != null)
			{
				window.clearTimeout(this._resp.timeoutId);
				this._resp.timeoutId = null;
			}

			// don't use second thread, this way we may use async = false for request
			// eg. if window reload is requested in onComplete, it would this ajax session to be aborted and throw status 0 exception, this way we let the ajax readystate callback to finish
			Loading.call(this, false); // user defined callback function, hide waiting image
			ResponseReceivedHandler.call(this);

			// must be invoked at the end because there is _onIceAjaxEndRequest delegate
			ReleaseReference.call(this);
		}
	}

	function TimeoutCallbackFunc()
	{
		if (this._resp.isHandled)
		{
			return;
		}
		this._resp.isHandled = true;

		if (this._xmlhttp)
		{
			// if crossdomain request, xmlhttp is not used, it is undefined
			this._xmlhttp.onload = function () { }; // XDomainRequest
			this._xmlhttp.onreadystatechange = function () { };
			try
			{
				this._xmlhttp.abort(); // this triggers new invocation to ReadyStateCallbackFunc()
			}
			catch(e)
			{}
		}

		// don't use second thread, this way we may use async = false for request
		Loading.call(this, false); // user defined callback function, hide waiting image
		// not ok

		var msg = "Ajax request timeout, server didn't respond in " + (this._config.timeoutMs / 1000) + " seconds.";

		SendError.call(this, msg);
	}

	function SendError(msg, description)
	{
		// first release reference, than invoke error handler where exception is thrown
		ReleaseReference.call(this);

		if (!description)
		{
			try
			{
				description = "\r\nurl: " + this._config.url;
				if (this._xmlhttp)
				{
					description += "\r\nstatus: " + this._xmlhttp.status + "\r\nresponse: " + (this._xmlhttp ? this._xmlhttp.responseText : "");
				}

				description += "\r\nsent: " + (typeof (this._config.data) == "string" ? this._config.data : Ice.JSON.Serialize(this._config.data));
			} catch (e) { }
		}

		WriteLog.call(this, msg);

		if (this._config.OnError)
		{
			this._config.OnError.call(this, msg + (description ? (", " + description) : ""));
		}
		else
		{
			Ice.Diagnostic.Exception(msg, description);
		}

		if (this._config.OnError && Ice.IsDebugDefined)
		{
			// let's show error alert always in debug mode, even if there is OnError defined
			Ice.Diagnostic.Exception(msg, description);
		}

	}

	function Substring(str, maxLength)
	{
		if (maxLength >= 0 && str && str.length > maxLength)
		{
			return str.substring(0, maxLength) + "...";
		}
		return str;
	}

	function WriteLog(msg)
	{
		if (this._config.OnLog)
		{
			msg = Substring(msg, this._config.logItemLength);
			sender._config.OnLog.call(this, msg);
		}
	}

	function ResponseReceivedHandler()
	{
		if (this._config.OnLog)
		{
			WriteLog.call(this, "response headers: " + (this._xmlhttp.getAllResponseHeaders ? this._xmlhttp.getAllResponseHeaders() : null));
			WriteLog.call(this, "reponse status: " + this._xmlhttp.status + ", " + this._xmlhttp.statusText);
			WriteLog.call(this, "responseText: " + this._xmlhttp.responseText);
		}

		if (this._xmlhttp.status == 0)
		{
			// browser disconnected or aborted request, do nothing
			return;
		}

		var isErrorStatus = (this._xmlhttp.status < 200 || this._xmlhttp.status >= 300);
		var contentType = this._xmlhttp.contentType || (this._xmlhttp.getResponseHeader ? this._xmlhttp.getResponseHeader("Content-type") : ""); // XDomainRequest doesn't support getResponseHeader
		var isJson = (contentType || "").indexOf("application/json") >= 0;

		var respTxt;
		var respJsObj;

		try
		{
			respTxt = this._xmlhttp.responseText;
			if (isJson)
			{
				respJsObj = Ice.JSON.Deserialize(respTxt);
			}
		}
		catch (ex)
		{
			var errorText = "Ajax response error: " + ex;
			SendError.call(this, errorText);
			return;
		}

		if (isErrorStatus)
		{
			var errorText = "Ajax response error in ResponseReceivedHandler";
			SendError.call(this, errorText);
			return;
		}
		else if (isJson)
		{
			HandleRepJsObj.call(this, respJsObj);
		}
		else
		{
			// pure text response (not json)
			if (this._config.OnComplete)
			{
				this._config.OnComplete.call(this, respTxt);
			}
		}
	}

	function HandleRepJsObj (respJsObj)
	{
		if (respJsObj && respJsObj.Type == "IceWebServiceResponse")
		{
			var isOnCompleteInvoked = false;

			if ("ExceptionMessage" in respJsObj)
			{
				SendError.call(this, respJsObj.ExceptionMessage);
			}
			else if ("ResponseObject" in respJsObj && this._config.OnComplete)
			{
				this._config.OnComplete.call(this, respJsObj.ResponseObject);
				isOnCompleteInvoked = true;
			}

			// just like MS ajax, execute js at the end
			if ("JavaScript" in respJsObj)
			{
				////eval("(" + respJsObj.JavaScript + ")"); // doesn't work if there is html inside
				// when respJsObj is deserialized, respJsObj.JavaScript contains real javascript, not string any more, so load it as javascript, not using eval
				Ice.ScriptLoader.LoadJS(respJsObj.JavaScript);
			}

			if(!isOnCompleteInvoked && this._config.OnComplete)
			{
				// if IceResponse.Current.UseAjaxUpdate used, web service method returns void, so invoke OnComplete on end to handle user code after the html is inserted using UseAjaxUpdate
				this._config.OnComplete.call(this);
				isOnCompleteInvoked = true;
			}

		}
		else
		{
			// not IceWebServiceResponse, but it is json object
			if (this._config.OnComplete)
			{
				this._config.OnComplete.call(this, respJsObj);
			}
		}
	};

	function Loading(boolShow)
	{
		if (this._config.OnLoading)
		{
			this._config.OnLoading.call(this, boolShow);
		}
	}

	function StartRequest()
	{
		if (this._xmlhttp != null)
		{
			Ice.Diagnostic.Exception("Request alreay made from this instance.\r\nYou have to create new instance");
			return this;
		}

		SaveReference.call(this);

		this._resp.timeoutId = window.setTimeout(Bind(TimeoutCallbackFunc, this), this._config.timeoutMs); //<<< callback func

		Loading.call(this, true);

		var ajaxUrl = this._config.url;
		var dataStr = this._config.data; // it can be already string

		/* window.location:
		Location {replace: function, assign: function, ancestorOrigins: DOMStringList, origin: "http://localhost:1059", hash: ""…}
					ancestorOrigins: DOMStringList
					assign: function () { [native code] }
					hash: ""
					host: "localhost:1059"
					hostname: "localhost"
					href: "http://localhost:1059/public/login"
					origin: "http://localhost:1059"
					pathname: "/public/login"
					port: "1059"
			protocol: "http:"
		*/


		if (this._config.serializeToJSON)
		{
			dataStr = Ice.JSON.Serialize(this._config.data);
		}
		else if (this._config.data && this._config.data.push && this._config.data.pop)
		{
			var arr = [];
			for (var n = 0; n < dataStr.length; n++) // array of Name-Value objects: [{Name:name1, Value: value1},{Name:name2, Value: value2},{Name:name3, Value: value3}]
			{
				var nv = dataStr[n];
				arr.push(encodeURIComponent(nv.Name + "") + "=" + encodeURIComponent(nv.Value + ""));
			}
			dataStr = arr.join("&");
		}
		else
		{
			// dataStr is string (JSON) or undefined or null or false ???
		}

		if (this._config.httpMethod == "GET" && dataStr)
		{
			ajaxUrl = ajaxUrl + (ajaxUrl.indexOf("?") >= 0 ? "&" : "?") + dataStr;
			dataStr = null; // set null to prevent sending using POST
		}

		//****************************************************************************************************
		// move ajax version parameter to the end because some browsers shorten url, so jsonarg is shorten and invalid, 
		// this way, if url is shorten, version will be removed, so the response from server will be command to reload page
		var m1 = ajaxUrl.match(new RegExp("[\\&]icevr[\\=][\\d]+")); // if not found, m1 is null
		if (m1 && (m1 = m1[0])) // index [0] is all, there is no matching groups
		{
			ajaxUrl = ajaxUrl.replace(m1, "") + m1;
		}
		////console.log("ajaxUrl: " + ajaxUrl);
		//****************************************************************************************************

		if (this._config.OnLog)
		{
			WriteLog.call(this, "request to url: " + ajaxUrl);
			WriteLog.call(this, "sending " + this._config.httpMethod + ": " + (dataStr || ""));
		}

		this._xmlhttp = Ice.Net.WebRequest.GetXMLHttpRequest(ajaxUrl);

		if (this._xmlhttp != null)
		{
			if (this._config.async)
			{
				// asynchronous
				// hack for IE8, IE9 crossdomain:
				if (window.XDomainRequest && this._xmlhttp instanceof window.XDomainRequest)
				{
					// onload is invoked once
					// XDomainRequest will add "origin" header by itself
					this._xmlhttp.onload = Bind(ReadyStateCallbackFunc, this); //<<< callback func
					this._xmlhttp.readyState = 4;
				}
				else
				{
					// all other browsers
					this._xmlhttp.onreadystatechange = Bind(ReadyStateCallbackFunc, this); //<<< callback func
				}
			}
			else
			{
				// synchronous
				this._xmlhttp.onreadystatechange = function () { };
			}

			// IE throws invalidstate error:
			////this._xmlhttp.timeout = this._config.timeoutMs + 10000; // let's set system timeout 10 seconds more to make it trigger later than my custom timeout handler

			try
			{
				//open request
				this._xmlhttp.open(this._config.httpMethod, ajaxUrl, this._config.async); //GET | POST | HEAD, URL, ASYNC request = true

				if (this._xmlhttp.setRequestHeader)
				{
					// IE8, IE9 XDomainRequest doesn't support setRequestHeader and it ads Origin header by itself

					// not all browsers support: window.location.origin, so create it as window.location.protocol + "//" + window.location.host
					// always send origin for crossdomain requests to work
					// Origin header is set by the browser, only on crossdomain request. If tryint to set it manually,  browser reports error: Refused to set unsafe header "Origin"
					//	this._xmlhttp.setRequestHeader("Origin", Ice.Net.WebRequest.GetOriginUrl());

					if (!Ice.Net.WebRequest.IsCrossdomain(ajaxUrl))
					{
						// custom header can't be set on crossdomain request
						this._xmlhttp.setRequestHeader("x-ice-ajax", "ice-ajax");
					}

					if (this._config.serializeToJSON)
					{
						this._xmlhttp.setRequestHeader('Content-Type', 'application/json');
					}
					else if (this._config.httpMethod == "POST")
					{
						this._xmlhttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
						////this._xmlhttp.setRequestHeader('Content-Type', 'multipart/form-data');										
					}
				}

				//send request
				this._xmlhttp.send(dataStr);

				if (!this._config.async)
				{
					// synchronous
					ReadyStateCallbackFunc.call(this);
				}

			}
			catch (e)
			{
				var msg = ("description: " + e.description + "\n" + "message: " + e.message + "\n" + "name: " + e.name + "\n" + "number: " + e.number);
				SendError.call(this, msg);
			}
		}

		return this;
	};

	this.Timeout = function (timeoutMs)
	{
		/// <summary>
		/// Set timeout
		/// </summary>
		/// <param name="timeoutMs"></param>
		this._config.timeoutMs = timeoutMs;
		return this;
	};

	this.Post = function (isSync)
	{
		/// <summary>
		/// Starts POST request.
		/// </summary>
		/// <param name="isSync" optional="true">default: false. If true, send synchronus request.</param>
		this._config.async = !isSync;
		this._config.httpMethod = "POST";
		StartRequest.call(this);
		return this;
	};

	this.Get = function (isSync)
	{
		/// <summary>
		/// Starts GET request.
		/// </summary>
		/// <param name="isSync" optional="true">default: false, If true, send synchronus request.</param>
		this._config.async = !isSync;
		this._config.httpMethod = "GET";
		StartRequest.call(this);
		return this;
	};

	this.Abort = function ()
	{
		///<summary>Abort request</summary>
		this._resp.isHandled = true;

		if (this._resp.timeoutId != null)
		{
			window.clearTimeout(this._resp.timeoutId);
			this._resp.timeoutId = null;
		}
		if (this._xmlhttp)
		{
			this._xmlhttp.onload = function () { }; // XDomainRequest
			this._xmlhttp.onreadystatechange = function () { };
			this._xmlhttp.abort();
		}
		Loading.call(this, false);
		return this;
	};

	this.OnComplete = function (func)
	{
		/// <summary>
		/// callback func on complete
		/// </summary>
		/// <param name="func">function(data){ }, this is this instance</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { func.call(this, {}); }
		//#endif

		this._config.OnComplete = func;
		return this;
	};

	this.OnError = function (func)
	{
		/// <summary>
		/// callback func on error
		/// </summary>
		/// <param name="func">function(data){ }, this is this instance</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { func.call(this, {}); }
		//#endif

		this._config.OnError = func;
		return this;
	};

	this.OnLoading = function (func)
	{
		/// <summary>
		/// callback on loading start or end
		/// </summary>
		/// <param name="func">function(isLoading){}, this is this instance</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { func.call(this, {}); }
		//#endif

		this._config.OnLoading = func;
		return this;
	};

	this.OnLog = function (func, maxLen)
	{
		/// <summary>
		/// write log
		/// </summary>
		/// <param name="func">function(data){ }, this is this instance</param>
		/// <param name="maxLen" type="int">max text length</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { func.call(this, {}); }
		//#endif

		this._config.OnLog = func;
		if (maxLen)
		{
			this._config.logItemLength = maxLen;
		}
		return this;
	};

	this.AddParameter = function (pkey, pvalue)
	{
		/// <summary>
		/// add key=value pair to be sent
		/// </summary>
		/// <param name="name"></param>
		/// <param name="value"></param>
		this._config.data = this._config.data || [];
		this._config.data.push({ Name: pkey, Value: pvalue });
		return this;
	};

	this.AddJsObject = function (jsObject)
	{
		/// <summary>
		/// JS object to be sent. Will serialize into JSON before sending. Received data will be deserialized and sent to OnComplete(data).
		/// </summary>
		/// <param name="jsObject"></param>

		// should use POST method, but  json can be send using GET method
		this._config.data = jsObject;
		this._config.serializeToJSON = true;
		this._config.httpMethod = "POST";
		return this;
	};

})();

Ice.Net.WebService = function (url, defaultOnComplete, defaultOnError, defaultOnLoading, defaultTimeoutMs)
{
	///<summary>constructor. Create new Ice.Net.WebService instance</summary>
	/// <param name="defaultOnComplete" type="Function" optional="true">function(object data){...}</param>
	/// <param name="defaultOnError" type="Function" optional="true">function(string errorMsg){...}</param>
	/// <param name="defaultOnLoading" type="Function" optional="true">function(bool isLoading){....}</param>
	/// <param name="defaultTimeoutMs" type="Int" optional="true">function(bool isLoading){....}</param>

	this.serviceUrl = url;
	this.OnComplete = defaultOnComplete;
	this.OnError = defaultOnError;
	this.OnLoading = defaultOnLoading;
	this.TimeoutMs = defaultTimeoutMs;
};

Ice.Net.WebService.prototype = new (function ()
{
	this.typeName = "Ice.Net.WebService";

	this.serviceUrl = null;
	this.strongParams = true;
	this.OnComplete = null;
	this.OnError = null;
	this.OnLoading = null;
	//// this.OnLoading = function (isLoading, ajaxObj) { if (document.documentElement) { document.documentElement.style.cursor = isLoading ? "progress" : "default"; } };
	this.TimeoutMs = null;

	function G(args)
	{
		/// <summary>
		/// return count of parameters which are not function (first n parameters which are not function)
		/// </summary>
		/// <param name="args"></param>
		
		for (var n = 0; n < args.length; n++) { if (typeof (args[n]) == "function") { return n; } }
		return args.length; // no callback method, all items are parameters
	}

	function ErrorIfFunction(val, parName)
	{
		if (typeof (val) == "function") { Ice.Diagnostic.Exception("Invalid parameter! '" + parName + "' can't be function!"); return true; }
	}

	function ErrorIfNotFunction(val, parName)
	{
		if (typeof (val) != "function") { Ice.Diagnostic.Exception("Invalid parameter! '" + parName + "' should be function!"); return true; }
	}

	function GetSargs(args, parNames, strongParams)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="args" type="Array"></param>
		/// <param name="parNames" type="Array"></param>
		/// <param name="strongParams" type="Boolean"></param>
		
		if (strongParams && !parNames) { Ice.Diagnostic.Exception("Invalid service definition! StrongParams is set, but missing parameters names!"); return; }
		var parametersCount = parNames ? parNames.length : G(args);
		var o = strongParams ? {} : [];
		for (var n = 0; n < parametersCount; n++)
		{
			var val = args[n];
			if (typeof (val) == "undefined")
			{
				val = null; // can not send undefined to service
			}

			if (strongParams)
			{
				if(ErrorIfFunction(val, parNames[n]))
				{
					return;
				}
				o[parNames[n]] = val;
			}
			else
			{
				o.push(val);
			}
		}
		// service method parameters: [argVal0, argVal1, argVal2...] or strongParams: {arg0: argVal0, arg1: argVal1, ...}
		return o;
	}

	// __ice_ss__
	// return service.FF({method:'" + o.MethodName + "', timeout:" + o.RequestTimeout + ", sync:" + o.Synchronous + ", sstate:" + o.SessionState + "} , arguments" + o.ParametersNames + @");

	this.FF = function (settings, args, parNames)
	{
		///<summary>Asynchronous ajax request</summary>

		/*
		"/customer/admin/publisher-edit?fads=432#fdsa.hsd=rewr"
		location
		{
			[functions]: ,
			hash: "#fdsa.hsd=rewr",
			host: "www.ima.test",
			hostname: "www.ima.test",
			href: "http://www.ima.test/customer/admin/publisher-edit?fads=432#fdsa.hsd=rewr",
			origin: "http://www.ima.test",
			pathname: "/customer/admin/publisher-edit",
			port: "",
			protocol: "http:",
			search: "?fads=432"
		}
		location.pathname + location.search
		"/customer/admin/publisher-edit?fads=432"
		*/

		var url = (this.serviceUrl || "")
			.replace("__ice_web_method_name__", settings.method)
			.replace("__ice_ss__", settings.sstate ? "1" : "0")
			.replace("__ice_current_url__", Ice.Utility.IceBase64.ToBase64String(window.location.pathname + window.location.search)); // path + query, important: exclude # has
		var jsObject = GetSargs(args, parNames, this.strongParams); // get service method parameters: [argVal0, argVal1, argVal2...] or strongParams: {arg0: argVal0, arg1: argVal1, ...}

		var onComplete, onError, onLoading, timeoutMs;
		var parametersCount = parNames ? parNames.length : G(args);
		if (args && args.length > parametersCount)
		{
			ErrorIfNotFunction(onComplete = args[parametersCount], "onComplete");
		}
		if (args && args.length > parametersCount+1)
		{
			ErrorIfNotFunction(onError = args[parametersCount+1], "onError");
		}
		if (args && args.length > parametersCount+2)
		{
			ErrorIfNotFunction(onLoading = args[parametersCount+2], "onLoading");
		}
		if (args && args.length > parametersCount+2)
		{
			timeoutMs = args[parametersCount+3];
		}

		var ajax = new Ice.Net.WebRequest(); // new ajax request instance
		ajax._config.async = !settings.sync;
		ajax._config.OnComplete = onComplete || this.OnComplete || ajax._config.OnComplete;
		ajax._config.OnError = onError || this.OnError || ajax._config.OnError;
		ajax._config.OnLoading = onLoading, this.OnLoading || ajax._config.OnLoading;
		ajax._config.url = url; // use queryString for method. pathInfo doesn't work with dots in name
		ajax._config.serializeToJSON = false; // already serialized
		ajax._config.deserializeFromJSON = true;
		ajax._config.timeoutMs = settings.timeout || timeoutMs || this.TimeoutMs || ajax._config.timeoutMs;
		////ajax._config.OnLog = function (msg, ajaxObj) { Ice.Console.WriteLine(msg); };
		var json = Ice.JSON.Serialize(jsObject);

		// use base64 instead of escaping, this way we may send anything, including <>?%^&$# etc.
		var enc = Ice.Utility.IceBase64.ToBase64String(json);
		if (enc && url && (url.length + enc.length) > 1000)
		{
			ajax._config.data = "jsonarg=" + enc;
			return ajax.Post();
		}
		else
		{
			ajax._config.data = "jsonarg=" + enc;
			return ajax.Get();
		}
	};

})();

/////*******************************************************/
////// default instance used for web methods
////Ice.Net.WebService.DefaultInstance = new Ice.Net.WebService();
/////*******************************************************/

/// <reference path="template:NET_GLOBAL_TIMEOUT" />
