﻿//*******************************************************************
// NO EXTERNAL DEPENDENCIES
//*******************************************************************

if (typeof (Ice) == "undefined") { Ice = {}; }

Ice.QueryString = function (url)
{
	/// <summary>
	/// Constructor. Create new instance from url if provided.
	/// </summary>
	/// <param name="url" optional="true">if provided, parse url</param>

	this._items = [];

	if(url)
	{
		this.ParseImpl(url);
	}
};

Ice.QueryString.prototype = new (function ()
{
	this._typeName = "Ice.QueryString";

	this._items = null;
	this._baseUrl = null;

	this.ToQueryString = function (baseUrl)
	{
		///<summary>create urlEncoded query string</summary>

		var arr = new Array();
		baseUrl = baseUrl || this._baseUrl;

		if (baseUrl)
		{
			arr.push(baseUrl);
		}

		for (var n = 0; n < this._items.length; n++)
		{
			if (n == 0)
			{
				arr.push("?");
			}

			var item = this._items[n];
			var key = item.key;
			var val = item.value;
			if (n > 0)
			{
				arr.push("&");
			}
			if (typeof (val) == "array")
			{
				val = val.join(",");
			}
			arr.push(encodeURIComponent(key));
			arr.push("=");
			arr.push(encodeURIComponent(val));
		}
		return arr.join("");
	};

	// overload
	this.toString = this.ToQueryString;

	this.Clear = function ()
	{
		///<summary>remove all items</summary>
		this._items = [];
	};

	this.Set = function (name, value)
	{
		///<summary>case non-sensitive name search and add or update if exists</summary>

		if (name != null)
		{
			name += "";
			value += "";

			var item = this.Get(name);
			if (!item)
			{
				item = { key: "", value: "", lowerKey: "" };
				this._items.push(item);
			}
			// update values
			item.key = name;
			item.value = value;
			item.lowerKey = name.toLowerCase();
		}
		return this;
	};

	this.Remove = function (name)
	{
		///<summary>case non-sensitive name search and remove</summary>
		if (name != null)
		{
			name = name.toLowerCase();
			var newArr = new Array();
			for (var n = 0; n < this._items.length; n++)
			{
				if (this._items[n].lowerKey != name)
				{
					newArr.push();
				}
			}
			this._items = newArr;
		}
		return this;
	};

	this.Get = function (name)
	{
		///<summary>case non-sensitve name search</summary>
		if (name != null)
		{
			name = name.toLowerCase();
			for (var n = 0; n < this._items.length; n++)
			{
				var obj = this._items[n];
				if (obj.lowerKey == name)
				{
					return obj.value;
				}
			}
		}
		return null;
	};

	this.ParseImpl = function(url)
	{
		var qq = this;

		var queryString = arguments.length > 0 ? (url + "") : null;

		if (queryString != null)
		{
			var index = queryString.indexOf("?");
			if (index > 0)
			{
				qq._baseUrl = queryString.substr(0, index);
				queryString = queryString.substr(index + 1) || "";
			}
			else
			{
				qq._baseUrl = queryString;
				queryString = "";
			}
			
			var pairsArr = queryString.split(new RegExp("[\\&\\#\\?]", "ig"));
			for (var n = 0; n < pairsArr.length; n++)
			{
				var pair = pairsArr[n].split('=');
				var name = decodeURIComponent(pair[0]);
				var value = decodeURIComponent(pair[1]);
				qq.Set(name, value);
			}
		}
		return qq;
	};


})();

Ice.QueryString.Parse = function (url)
{
	///<summary>Convert query string to name-value collection</summary>
	///<param name="url" type="String">url string with name=value pairs</param>
	return new Ice.QueryString(url);
};

