﻿/// <reference path="Ice.MulticastDelegate.js" />

Ice.Utility.FullScreen = new (function ()
{
	// NOTE: Fullscreen requests to be invoked from event handler's thread, otherwise permision is denied.

	var _this = { /**/ };

	_this.RequestFullScreen = function (divObj)
	{
		var requestFunc = divObj.requestFullscreen
			|| divObj.msRequestFullscreen
			|| divObj.mozRequestFullScreen
			|| divObj.webkitRequestFullscreen
			|| divObj.webkitRequestFullScreen /*safari*/
			|| divObj.webkitEnterFullScreen
			|| divObj.webkitEnterFullscreen;

		//Use the specification method before using prefixed versions
		if (requestFunc)
		{
			requestFunc.apply(divObj);
		}
	};
	
	var cancelFunc = document.cancelFullscreen
		|| document.msExitFullscreen
		|| document.msCancelFullscreen
		|| document.mozCancelFullScreen
		|| document.webkitExitFullscreen
		|| document.webkitExitFullScreen /*iPhone*/
		|| document.webkitCancelFullscreen
		|| document.webkitCancelFullScreen /*safari*/;

	_this.CancelFullScreen = function ()
	{
		//Use the specification method before using prefixed versions
		// IE has msExitFullscreen
		if (cancelFunc)
		{
			cancelFunc.apply(document);
		}
	};

	_this.GetFullScreenElement = function ()
	{
		//  get full screen element across several browsers
		return document.fullscreenElement
			|| document.msFullscreenElement
			|| document.mozFullScreenElement
			|| document.webkitFullscreenElement
			|| document.webkitCurrentFullScreenElement /*safari*/
			|| document.webkitDisplayingFullscreen; // warning: this should be true/false only, not the element which is in fullscreen
	};

	_this.IsInFullscreen = function ()
	{
		return !! _this.GetFullScreenElement();
	};

	function IsIframeFullscreenEnabled()
	{
		/// <summary>
		/// Test if iframe has allowFullscreen attribute
		/// </summary>
		
		return !!(document.fullscreenEnabled
			|| document.msFullscreenEnabled
			|| document.mozFullScreenEnabled
			|| document.webkitFullscreenEnabled
			|| "webkitIsFullScreen" in document /*for safari*/);
	};

	_this.IsSupported = function ()
	{
		if (window.parent)
		{
			return !!(cancelFunc && IsIframeFullscreenEnabled());
		}
		else
		{
			return !!(cancelFunc);
		}
	};

	var chg = ["MSFullscreenChange", "webkitfullscreenchange", "mozfullscreenchange", "fullscreenchange"];
	var err = ["MSFullscreenError", "webkitfullscreenerror", "mozfullscreenerror", "fullscreenerror"];
	var _evChk = false;
	var _evErr = false;

	// MulticastDelegate to execute on fullscreenchange
	// .Add(function (sender, isFullscreen){....})
	_this.OnFullscreenChanged = new Ice.MulticastDelegate().OnAdd(function (mc)
	{
		// add events to document on first add
		if (!_evChk)
		{
			_evChk = true;
			$$ice(document).On(chg, function (sender, ev)
			{
				// fullscreen detection _this.IsInFullscreen() doesn't work ok
				_this.OnFullscreenChanged.Invoke(sender, ev);
			});
		}
	});

	_this.OnFullscreenError = new Ice.MulticastDelegate().OnAdd(function (mc)
	{
		if(!_evErr)
		{
			_evErr = true;
			$$ice(document).On(err, function (sender, ev)
			{
				_this.OnFullscreenError.Invoke(sender, ev);
			});
		}
	});



	return _this;
})();

