﻿
//#region Function extensions

if (!Function.prototype.BindContext)
{
	Function.prototype.BindContext = function (context)
	{
		/// <summary>
		/// Bind current function with context.
		/// </summary>
		/// <param name="context"></param>
		var method = this;
		var cx = context || this || {};

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return method; } // keep intellisense, return original method with arguments names
		//#endif

		return function () { return method.apply(cx, arguments); };
	};
}

//#endregion

//#region Array extensions

if (!Array.prototype.indexOf)
{
	// for < IE9, used in Ice.Enumerable.Distinct()

	Array.prototype.indexOf = function (item)
	{
		for (var n = 0; n < this.length; n++)
		{
			if (this[n] === item)
			{
				return n;
			}
		}
		return -1;
	};
}

if (!Array.prototype.AsEnumerable)
{
	Array.prototype.AsEnumerable = function ()
	{
		/// <summary>
		/// Creates new Ice.List instance from current array.
		/// </summary>

		return new Ice.List(this);
	};
}

//#endregion

//#region Date extensions

if (!Date.prototype.ToLocalOffset && !Date.prototype.ToServerOffset && !Date.prototype.Kind)
{
	Date.prototype.Kind = "Unspecified";

	Date.prototype.ToLocalOffset = function ()
	{
		/// <summary>
		/// Apply browser's timezone offset to show correct date and time in browser which was received from server.
		/// </summary>

		// we receive from server time ticks with time zone offset removed

		// getTimezoneOffset() returns minutes
		if (this.Kind != "Local")
		{
			var dt = new Date(this.getTime() + this.getTimezoneOffset() * 60 * 1000);
			dt.Kind = "Local";
			return dt;
		}
		return this;
	};

	Date.prototype.ToServerOffset = function ()
	{
		/// <summary>
		/// Remove browser's timezone offset to get correct date and time which has to be sent to server. Convert to time with 0 offset.
		/// </summary>

		// JS Date() always apply local time zone offset and shows time with local offset applyed
		// we want to send to the server exact time ticks with time zone offset removed
		// eg. "2014-11-15 11:23:56.470 +01:00".ToServerOffset() will give "2014-11-15 12:23:56.470 +01:00" which means: "2014-11-15 11:23:56.470 +00:00" and we send to server time with 0 offset

		if (this.Kind != "Server")
		{
			var dt = new Date(this.getTime() - this.getTimezoneOffset() * 60 * 1000);
			dt.Kind = "Server";
			return dt;
		}
		return this;
	};
}

//#endregion


