﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;
using System.Web;
using System.Web.Services;
using System.Text.RegularExpressions;
using Ice.Web.UI;
using System.Linq;

namespace Ice.Web.Script.Services
{
	public class CachedScript
	{
		public string ControlPathKey { get; set; }
		public string ScriptPathKey { get; set; }
		public string ScriptContent{get;set;}
		public int ScriptLength { get; set; }
		public string ETag{get;set;}
		public string SourceDirectory{get;set;}
		public bool IsCSS{get;set;}
		public IceScriptManager Manager { get; set; }
		public string Timestamp { get; set; }
		public string Url { get; set; }
		public int Index { get; set; }

		/// <summary>
		/// DateTime bez milisekundi jer browser ne salje milisekunde, mora biti Local time because after parsing header value, we get local time too
		/// </summary>
		public DateTime LastModified { get; set; }

		private static object mDicLock = new object();

		private static Dictionary<string, List<CachedScript>> mCachedByControlPathKeyDictionary = new Dictionary<string, List<CachedScript>>();
		private static Dictionary<string, CachedScript> mCachedByScriptPathKeyDictionary = new Dictionary<string, CachedScript>();

		public static CachedScript GetByScriptPathKey(string cacheKey)
		{
			CachedScript cc = null;
			mCachedByScriptPathKeyDictionary.TryGetValue(cacheKey, out cc);
			return cc;
		}

		public static List<CachedScript> GetByControlPathKey(string cacheKey)
		{
			List<CachedScript> cc = null;
			mCachedByControlPathKeyDictionary.TryGetValue(cacheKey, out cc);
			return cc;
		}

		public static void AddToCache(List<CachedScript> cc)
		{
			lock (mDicLock)
			{
				mCachedByControlPathKeyDictionary.Add(cc.First().ControlPathKey, cc); // let's throw exception if the same key exist

				foreach (CachedScript ss in cc)
				{
					mCachedByScriptPathKeyDictionary.Add(ss.ScriptPathKey, ss); // let's throw exception if the same key exist
				}
			}
		}

	}
}
