﻿namespace Ice.Web.Script.Services
{
	using System;
	using System.Reflection;
	using System.Web;
	using System.Web.Services;
	using Serialization;
	using Ice.Serialization.Json.Core.Serializer;

	class WebServiceData
	{
		public Type WebServiceType {get;set;}

		internal WebServiceData(Type serviceType)
		{
			this.WebServiceType = serviceType;
		}

		internal WebServiceMethodData GetMethodData(string methodName)
		{
			BindingFlags bi = BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static;
			MethodInfo mi = this.WebServiceType.GetMethod(methodName, bi);
			if (mi == null)
			{
				throw new Exception(methodName + ", unknown method!");
			}
			WebServiceMethodData data = GetMethodData(mi);
			if (data == null)
			{
				throw new Exception(methodName + ", not web service method!");
			}
			return data;
		}

		private WebServiceMethodData GetMethodData(MethodInfo method)
		{
			// must test if method has IceWebMethodAttribute to prevent hackers from calling public methods inside type which are not IceWebMethods
			object[] customAttributes = method.GetCustomAttributes(typeof(IceWebMethodAttribute), true);
			if (customAttributes.Length > 0)
			{
				return new WebServiceMethodData(this, method);
			}
			return null;
		}

		private IceJavaScriptSerializer _serializer = null;

		public IceJavaScriptSerializer Serializer
		{
			get 
			{
				if (this._serializer == null)
				{
					this._serializer = new IceJavaScriptSerializer();
				}
				return this._serializer;
			}
		}

	}


}