﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;
using System.Web;
using System.Web.Services;
using System.Text.RegularExpressions;
using Ice.Web.UI;
using System.Linq;
using Ice.Web.Extensions;

namespace Ice.Web.Script.Services
{
	class WsjsClientProxyHandler : IHttpHandler
	{
		static object mLock = new object();

		public void ProcessRequest(HttpContext context)
		{
			DoProcess(context);
		}

		private void SendResourceData(HttpContext context, string resourceFile)
		{
			byte [] data = Ice.Utility.EmbeddedHelper.GetFileBytes(resourceFile);
			string mime = "image/" + Path.GetExtension(resourceFile).Substring(1);
			context.Response.Cache.SetExpires(DateTime.UtcNow.AddDays(30)); // query string has timestamp, so don't worry about cache invalidation
			context.Response.Cache.SetCacheability(HttpCacheability.Public);
			context.Response.AppendHeader("content-type", mime);
			context.Response.OutputStream.Write(data, 0, data.Length);
			context.Response.OutputStream.Flush();
		}

		private void DoProcess(HttpContext context)
		{
			context.Response.Clear();

			IceScriptManager.DecodedKey key = IceScriptManager.DecodeScriptPathKey(context);
			CachedScript script;

			if (key.ResourceFile != null)
			{
				try
				{
					SendResourceData(context, key.ResourceFile);
				}
				catch
				{
					context.Response.StatusCode = (int)System.Net.HttpStatusCode.NotFound; // 404 not found
				}
			}
			else if ((script = CachedScript.GetByScriptPathKey(key.ScriptPathKey)) == null)
			{
				////System.Threading.Thread.Sleep(5000); // to prevent to many requests! // ovo bi moglo usporiti cijeli iis jer iis ima ograniceni broj requesta koje procesira
				//script = new CachedScipt() { ScriptContent = ""};
				// empty! don't auto-refresh page with (endless loop) "window.location = window.location;" }; 
				// reload page to create cached scripts on the server and to create script tag with new cachedkey

				context.Response.Cache.SetExpires(DateTime.UtcNow.AddDays(-10));
				context.Response.Cache.SetCacheability(HttpCacheability.NoCache);
				context.Response.Cache.SetNoStore();

				context.Response.StatusCode = (int)System.Net.HttpStatusCode.NotFound; // <<< return 404 not found, user has to push F5 to reload complete web page
			}
			else // script != null
			{
				string receiveETag = context.Request.Headers["If-None-Match"];

				if (!string.IsNullOrEmpty(receiveETag) && receiveETag == script.ETag)
				{
					context.Response.StatusCode = (int)System.Net.HttpStatusCode.NotModified;
					return;
				}

				/*
				// ovo mi ne treba jer se gleda sadraj skripti i kreira e-tag i timestamp ovisno o sadraju, a ne vise o vremenu starta applikacije
				DateTime lastModified;
				string lastModifiedString = context.Request.Headers["If-Modified-Since"]; // datetime, bez milisekundi, browser ne salje milisekunde
				if (!string.IsNullOrEmpty(lastModifiedString) && DateTime.TryParse(lastModifiedString, out lastModified)) // after parsing, it is Local time
				{
					if (lastModified == script.LastModified)
					{
						context.Response.StatusCode = (int)System.Net.HttpStatusCode.NotModified;
						return;
					}
				}
				*/

				if (key.Timestamp != script.Timestamp)
				{
					// ako browser ima stari file u cacheu (stari timestamp)
					context.Response.RedirectLocation = script.Url;
					context.Response.StatusCode = (int)System.Net.HttpStatusCode.Redirect; // 302
					context.Response.Cache.SetExpires(DateTime.UtcNow.AddDays(-10));
					context.Response.Cache.SetCacheability(HttpCacheability.NoCache);
					context.Response.Cache.SetNoStore();
					////return; // don't return here, send new content for browsers which doesn't read 301 redirect
				}

				context.Response.Cache.VaryByParams["*"] = true; // mora bit * da bi kao na serveru cache-key koristio full url with query string
				context.Response.Cache.SetOmitVaryStar(true); // ovim se kaze da ne zelim slati * klijentovom browseru, ako ne saljem *, prvo cita iz klijentovog cachea, ako saljem * klijentu, ignorira se njegov cache i svaki put ide request na server

				context.Response.Cache.SetExpires(DateTime.UtcNow.AddDays(30));
				context.Response.Cache.SetLastModified(script.LastModified); // last modified se salje na server kao "If-Modified-Since" header key, uvijek se salje klijentu, neovisno o vrsti cachinga
				context.Response.Cache.SetCacheability(HttpCacheability.Public); // mora biti Public ili ServerAndPrivate ukljucen inace nece poslati eTag klijentu
				context.Response.Cache.SetETag(script.ETag); // eTag se salje na server kao "If-None-Match", kad user lupi F5, vraca se response 304 not modified ako je etag poklapa				

				if (script.IsCSS)
				{
					context.Response.ContentType = "text/css";
				}
				else
				{
					context.Response.ContentType = "application/x-javascript";
				}

				//enable compression
				if (script.ScriptLength > 500) // only if larger than 500 bytes
				{
					Ice.Web.IceHttpCompressionFilter.AddCompressionFilter(context);
				}

				context.Response.Write(script.ScriptContent);
				//context.Response.Flush(); // requires full trust security
				//context.Response.End(); // throws Thread exception!
			}
		}

		public bool IsReusable
		{
			get
			{
				return false;
			}
		}



	}
}
