﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ice.Web.UI;
using System.IO;
using System.Web.UI;
using Ice.Web.UI.Css;

namespace Ice.Web.Resources.Themes
{
	/// <summary>
	/// Modified base theme with styling system buttons, inputs, selects and textareas.
	/// </summary>
	class Aero : ThemeBase
	{
		public override IceThemeEnum Theme { get { return IceThemeEnum.Aero; } }
		public override string ButtonBackgroundImage { get { return IceScriptManager.CreateUrl(".Aero.Images.btnback.png"); } }
		public override string ButtonBackgroundColor { get { return "#E2E2E2"; } }
		public override string ButtonBorderColor { get { return "gray"; } }
		public override string ButtonTextColor { get { return "black"; } }
		public override string FrameColor { get { return "#F0F0F0"; } }
		public override string FrameBorderColor { get { return "gainsboro"; } }
		public override string SplitBarHoverColor { get { return "#E2E2E2"; } }
		public override string WindowFrameColor { get { return "#1A6F96"; } } // deepBlue

		protected override void WriteThemeCss(IceStyleSheet styleSheet)
		{
			base.WriteThemeCss(styleSheet);

			// use opacity:0 to hide input, don't use display:none becuase serialization wouldn't work

			styleSheet.AddCss(@"
input[type=checkbox],
input[type=radio]
{
	position:absolute;
	overflow:hidden;
	width:1px;
	height:1px;
	filter:alpha(opacity=0);
	opacity:0;
}
		
input[type=checkbox]+label:before
{
	background-image:url(" + CheckboxBackgroundImage + @");
	background-position:-1px -1px;
	background-repeat:no-repeat;
}
		
input[type=radio]+label:before
{
	background-image:url(" + RadioboxBackgroundImage + @");
	background-position:-1px 2px;
	background-repeat:no-repeat;	
}
		
input[type=checkbox]+label, 
input[type=radio]+label
{
	padding-left:25px;
	/*line-height:24px;*/
	position:relative;
	cursor:pointer;
}
		
input[type=checkbox]+label:before, 
input[type=radio]+label:before
{
	content:"" "";
	display:block;
	float:left;
	height:24px;
	width:24px;
	padding:0;
	position:absolute;
	left:0;
	top:-3px;
}
		
input[type=checkbox]+label:hover:before
{
	background-position:-1px -201px;
}
		
input[type=radio]+label:hover:before
{
	background-position:-1px -98px;
}
		
input[type=checkbox]:checked+label:before
{
	background-position:-1px -375px;
}
		
input[type=radio]:checked+label:before
{
	background-position:-1px -198px;
}
		
input[type=checkbox]:checked+label:hover:before
{
	background-position:-1px -575px;
}
		
input[type=radio]:checked+label:hover:before
{
	background-position:-1px -298px;
}
			
.iceInputStyle,
input[type=text], 
input[type=password], 
textarea, 
select
{
	border:2px solid #E5E5E5;
	border-radius:2px;
	padding:3px;
	box-shadow:1px 1px 6px #EEEEEE inset;
	background-color:white;
}

select
{
	padding:2px;
}
		
.iceInputStyle:hover, 
input[type=text]:hover, 
input[type=password]:hover, 
textarea:hover, 
select:hover
{
	box-shadow:0.5px 0.5px 6px #E0E0E0;
}
		
.iceInputStyle:focus, 
input[type=text]:focus, 
input[type=password]:focus, 
textarea:focus, 
select:focus
{
	box-shadow:0.5px 0.5px 6px #E0E0E0;
	background-color:white;
}
		
.error
{
	border-color:Red !important;
}

");

			styleSheet.AddCss(@"
textarea
{
	overflow:auto;
}

.iceButton, 
input[type=submit], 
input[type=button], 
button
{
	background:url(" + this.ButtonBackgroundImage + @") repeat-x scroll center center " + this.ButtonBackgroundColor + @";
	color:" + this.ButtonTextColor + @";
	border:solid 1px " + this.ButtonBorderColor + @";
	border-radius:3px;
}

.iceButton:hover, 
input[type=submit]:hover, 
input[type=button]:hover, 
button:hover
{
	box-shadow:0 -2px 2px 0 " + this.ButtonHoverColor + @" inset;
	cursor:pointer;
}

.iceTitleBar
{
	background:url(" + this.ButtonBackgroundImage + @") repeat-x scroll center center " + this.ButtonBackgroundColor + @";
	color:" + this.ButtonTextColor + @";	
}

");
		}


	}
}
