﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Ice.Web.UI.Css
{
	public class IceMediaRule : IIceCssItem
	{
		protected string _mediaType { get; set; }

		private List<IIceCssItem> _items = new List<IIceCssItem>();

		/// <summary>
		/// mediaType example: screen and (max-width: 500px)
		/// </summary>
		/// <param name="mediaType">all, aural, braille, embossed, handheld, print, projection, screen, tty, tv</param>
		public IceMediaRule(string mediaType)
		{
			this._mediaType = mediaType;
		}

		protected IceMediaRule()
		{

		}

		public IceMediaRule Add(params IIceCssItem[] cls)
		{
			cls.ForEach(o => this._items.Add(o));
			return this;
		}

		/// <summary>
		/// Deep clone everything.
		/// </summary>
		/// <returns></returns>
		public IceMediaRule Clone()
		{
			return new IceMediaRule(this._mediaType) { _items = this._items.Select(o => o.Clone()).ToList() };
		}


		IIceCssItem IIceCssItem.Clone()
		{
			return this.Clone();
		}

		public string ToCssString()
		{
			return "@media " + this._mediaType + @"
{
" + this._items.Select(o => o.ToCssString()).Where(o=>!string.IsNullOrEmpty(o)).ToStringJoined("\r\n\r\n").Split('\n').Select(o=>"\t" + o).ToStringJoined("\n")  + @"
}";
		}

		public virtual IceMediaRule AddToStyleSheet(IceStyleSheet sheet)
		{
			sheet.Add(this);
			return this;
		}

		public class MediaScreenSettings
		{
			public int Width { get; set; }
			public int Height { get; set; }
		}

		public IEnumerable<IIceCssItem> Items()
		{
			return _items.AsEnumerable();
		}

		public IceMediaRule NewCssClass(string className, Action<IceCssClass> c)
		{
			IceCssClass cls = new IceCssClass(className);
			this.Add(cls);
			c(cls);
			return this;
		}

		public IceMediaRule NewPlainText(string text)
		{
			this.Add(new IcePlainCssText(text));
			return this;
		}

	}


}
