﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using Ice.Web.Extensions;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using Ice.Web.Script.Services;

namespace Ice.Web.UI.Direct
{
	public class IceDirect
	{
		List<string> _jsItems = new List<string>();

		public string CurrentJs { get { return _jsItems.ToStringJoined(); } }

		private IceDirect Append(string js)
		{
			_jsItems.Add(js);
			return this;
		}

		private IceDirect()
		{

		}

		internal static IceDirect FromSelector(string iceSelector)
		{
			return new IceDirect().Append(string.Format("$$ice('{0}')", iceSelector));
		}

		/// <summary>
		/// Create subquery.
		/// </summary>
		/// <param name="querySelector"></param>
		/// <returns></returns>
		private IceDirect Query(string iceSelector)
		{
			if(_jsItems.Any())
			{
				return Append(string.Format(".Query('{0}')", iceSelector));
			}
			else
			{
				return FromSelector(iceSelector);
			}
		}

		public IceDirect Sleep(int sleepMs, Action<IceDirect> onExpired)
		{
			IceDirect callback = new IceDirect().Append("a");
			onExpired(callback);
			string func = "function(a){" + callback.CurrentJs + ";}";
			return this.Append(string.Format(".Sleep({0}, {1})", sleepMs, func));
		}

		public IceDirect SetAttribute(string name, string value)
		{
			return this.Append(string.Format(".SetAttribute('{0}', '{1}')", name, Ice.Web.Extensions.ControlExtensions.JSEncodeString(value)));
		}

		public IceDirect RemoveAttribute(string name)
		{
			return this.Append(string.Format(".RemoveAttribute('{0}')", name));
		}

		public IceDirect SetProperty(string name, string value)
		{
			return this.Append(string.Format(".ForEach(function(o){2}o['{0}']='{1}';{3})", name, Ice.Web.Extensions.ControlExtensions.JSEncodeString(value), "{", "}"));
		}

		public IceDirect SetInnerHtml(string innerHtml)
		{
			return this.Append(string.Format(".SetInnerHtml('{0}')", Ice.Web.Extensions.ControlExtensions.JSEncodeString(innerHtml)));
		}

		public IceDirect ApplyStyle(string style)
		{
			return this.Append(string.Format(".ApplyStyle('{0}')", style));
		}

		public IceDirect ApplyStyle(string name, string value)
		{
			return this.Append(string.Format(".ApplyStyle('{0}', '{1}')", name, value));
		}

		public IceDirect AddClass(string name)
		{
			return this.Append(string.Format(".AddClass('{0}')", name));
		}

		public IceDirect RemoveClass(string name)
		{
			return this.Append(string.Format(".RemoveClass('{0}')", name));
		}

		public IceDirect ToggleClass(string name)
		{
			return this.Append(string.Format(".ToggleClass('{0}')", name));
		}

		public IceDirect SetClass(string name)
		{
			return this.Append(string.Format(".SetClass('{0}')", name));
		}

		public IceDirect SetEnabled(bool isEnabled)
		{
			if (isEnabled)
			{
				// disabled has to be set or removed from/to current element and all descendants???
				////return this.Append(string.Format(".Query(*, *//*).RemoveAttribute('{0}')", "disabled"));
				return this.Append(string.Format(".RemoveAttribute('{0}')", "disabled"));
			}
			else
			{
				////return this.Append(string.Format(".Query(*, *//*).SetAttribute('{0}', '{1}')", "disabled", "disabled"));
				return this.Append(string.Format(".SetAttribute('{0}', '{1}')", "disabled", "disabled"));
			}
		}

		public IceDirect SetText(string text)
		{
			return this.SetInnerHtml(text);
		}

		public IceDirect SetValue(string value)
		{
			return this.SetProperty("value", value);
		}

		public IceDirect SetChecked(bool isChecked)
		{
			if (isChecked)
			{
				return this.RemoveAttribute("checked");
			}
			else
			{
				return this.SetAttribute("checked", "checked");
			}
		}

	}

	public static class IceDirectExtensions
	{
		/// <summary>
		/// Use iceSelctor to select control(s).
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="control"></param>
		/// <param name="iceSelector"></param>
		/// <param name="action"></param>
		/// <returns></returns>
		public static T UseIceDirect<T>(this T page, string iceSelector, Action<IceDirect> action) where T : Page
		{
			IceDirect direct = IceDirect.FromSelector(iceSelector);
			action(direct);

			page.ExecuteJavaScript(direct.CurrentJs);

			return page;
		}

		/// <summary>
		/// Use current control.
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="control"></param>
		/// <param name="action"></param>
		/// <returns></returns>
		public static T UseIceDirect<T>(this T control, Action<IceDirect> action) where T : Control
		{
			control.Page.UseIceDirect("#" + control.ClientID, action);
			return control;
		}

		/// <summary>
		/// Use current control collection.
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="controls"></param>
		/// <param name="action"></param>
		/// <returns></returns>
		public static IEnumerable<T> UseIceDirect<T>(this IEnumerable<T> controls, Action<IceDirect> action) where T:Control
		{
			controls = controls.ToList();

			string selector = controls.Select(o=> "#" + o.ClientID).ToStringJoined(", ");
			Control cc = controls.FirstOrDefault();
			if (cc != null)
			{
				cc.Page.UseIceDirect(selector, action);
			}
			return controls;
		}

	}


}
