﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using Ice.Web.Extensions;
using Ice.Web.UI;

//*****************************************************************
// works with IceScriptManager and with MS ScriptManager
//*****************************************************************

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(true)]
	public class IceColorPicker : PlaceHolder
	{
		public enum ModeEnum
		{
			/// <summary>
			/// Default
			/// </summary>
			Continuous = 0,
			Predefined = 1
		}

		public string TargetControlID { get; set; }

		public string TargetControlClientID { get; set; }

		public event EventHandler ColorSelected;

		////public string SelectedColor { get { return _hdnSelectedColor.Value; } set { _hdnSelectedColor.Value = value; } }

		/// <summary>
		/// JavaScript function to execute when color is changed.
		/// </summary>
		public string OnColorChangedHandler { get; set; }

		Button _btnColorSelected = new Button().ApplyStyle("position:absolute;display:none;");
		
		HiddenField _hdnSelectedColor = new HiddenField();
		HiddenField _hdnPickerMode = new HiddenField() { Value = "0" };

		public ModeEnum Mode
		{
			get { return (ModeEnum) Convert.ToInt32(_hdnPickerMode.Value); }
			set { _hdnPickerMode.Value = ((int)value).ToString(); }
		}

		private void CreateControl()
		{
			string targetId = this.TargetControlClientID;
			if (string.IsNullOrEmpty(targetId))
			{
				// allow any control to be target, this way we may use <input...> controls as target
				try
				{
					TextBox control = this.NamingContainer.DescendantControls().OfType<TextBox>().Where(o => o.ID == this.TargetControlID).FirstOrDefault();
					targetId = control.ClientID;
				}
				catch
				{
					throw new Exception("TargetControlID is not found");
				}
			}

			if (!IceScriptManager.IsIceAjaxRequest())
			{
				this.AddChildren
				(
					_hdnPickerMode,
					_hdnSelectedColor,
					_btnColorSelected.Set(bb => bb.Click += (sender, ev) =>
					{
						if (this.ColorSelected != null)
						{
							this.ColorSelected.Invoke(this, ev);
						}
					})
				);
			}

			if (this.Mode == ModeEnum.Continuous)
			{
				string str = new List<string>(){
				"#" + targetId,
				IceScriptManager.CreateUrl(".IceColorPicker.Resources.arrow.gif"),
				IceScriptManager.CreateUrl(".IceColorPicker.Resources.cross.gif"),
				IceScriptManager.CreateUrl(".IceColorPicker.Resources.hs.png"),
				IceScriptManager.CreateUrl(".IceColorPicker.Resources.hv.png"),
				"(" + (this.OnColorChangedHandler ?? "function(){}") + ")"
			}.Select(o => "'" + o + "'").ToStringJoined(", ");

				// initialize once
				this.Page.ExecuteJavaScript("Ice.Web.UI.WebControls.IceColorPicker.Setup(" + str + ");");
			}
			else if(this.Mode == ModeEnum.Predefined)
			{
				string str = new List<string>(){
					"#" + targetId,
					IceScriptManager.CreateUrl(".IceColorPicker.PredefPicker.colormap.gif"),
					"(" + (this.OnColorChangedHandler ?? "function(){}") + ")"
				}.Select(o => "'" + o + "'").ToStringJoined(", ");

				// initialize once
				this.Page.ExecuteJavaScript("Ice.Web.UI.WebControls.IceColorPickerPredef.Setup(" + str + ");");
			}
		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			// engine script has to be created always, even if control is not currently visible, later it may become visible
			if (this.Page != null && !this.Page.IsInAsyncPostBack())
			{
				// eg. if ToolTip is in ascx control which is loaded using IceAjax, we don't have reference to page where IceScriptManager is

				if (this.Mode == ModeEnum.Continuous)
				{
					IceScriptManager.GetCurrent(this).RegisterScriptFile(this, ".IceColorPicker.Resources.jscolor.js");
				}
				else if (this.Mode == ModeEnum.Predefined)
				{
					IceScriptManager.GetCurrent(this).RegisterScriptFile(this, ".IceColorPicker.PredefPicker.PredefPicker.js");
				}
			}
		}

		protected override void Render(HtmlTextWriter writer)
		{
			CreateControl();

			base.Render(writer);
		}
	}
}
