﻿using System;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;
using Ice.Web.Extensions;

//*****************************************************************
// works with IceScriptManager and with MS ScriptManager
//*****************************************************************

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(true)]
	public class IceDatePicker:PlaceHolder
	{
		public enum AlignEnum
		{
			/// <summary>
			/// Default
			/// </summary>
			Left,

			Right
		}

		public AlignEnum Align
		{
			get { return (AlignEnum)(this.ViewState["_iceDateAlign"] as int?).GetValueOrDefault(); }
			set { this.ViewState["_iceDateAlign"] = (int) value; }
		}

		public string TargetControlID { get; set; }

		public string TargetControlClientID { get; set; }

		/// <summary>
		/// .net date format string
		/// </summary>
		public string Format
		{
			get { return this.ViewState["_iceDateFormat"] as string; }
			set { this.ViewState["_iceDateFormat"] = value;}
		}

		private void CreateControl()
		{
			string targetId = this.TargetControlClientID;

			if (string.IsNullOrEmpty(targetId))
			{
				// allow any control to be target, this way we may use <input...> controls as target
				try
				{
					Control control = this.NamingContainer.DescendantControls().OfType<Control>().Where(o => o.ID == this.TargetControlID).FirstOrDefault();
					targetId = control.ClientID;
				}
				catch
				{
					throw new Exception("TargetControlID is not found");
				}
			}

			// initialize once
			this.Page.ExecuteJavaScript("Ice.Web.UI.WebControls.IceDatePickerManager.Setup('#" + targetId + "', '" + this.Format + "', '" + (this.Align.ToString().ToLower()) + "');");
		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			// engine script has to be created always, even if control is not currently visible, later it may become visible
			// script has to be loaded always, even if panel is not currently visible. It may be set visible later in UpdatePanel
			if (this.Page != null && !this.Page.IsInAsyncPostBack())
			{
				// js has to be initialized even if control is not visible and later may be set visible using UpdatePanel
				IceScriptManager manager = IceScriptManager.GetCurrent(this).RegisterScriptFile(this, ".IceDatePicker.IceDatePicker.css", ".IceDatePicker.IceDatePicker.js");
			}
		}

		protected override void Render(HtmlTextWriter writer)
		{
			CreateControl();

			base.Render(writer);
		}

	}
}
