﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.IO;
using System.Web.UI.WebControls;
using Ice.Web.Extensions;
using Ice.Web.Resources;

//*****************************************************************
// ASP.NET VERSION, CAN'T BE USED WITH IceScriptManager
//*****************************************************************

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(true)]
	public class IceFileUpload : IcePanel
	{
		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate ButtonTemplate { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate ProgressTemplate { get; set; }

		public string ToolTip
		{
			get { return this.Attributes["title"]; }
			set { this.Attributes["title"] = value; }
		}

		public event EventHandler UploadStart;
		public event EventHandler UploadComplete;

		FileUpload fileUpload = new FileUpload();
		Button btnUploadStart = new Button().ApplyStyle("position:absolute;display:none;");
		Button btnUploadComplete = new Button().ApplyStyle("position:absolute;display:none;");
		WebControl pnlProgress;
		WebControl pnlFileupload;

		public new bool Enabled
		{
			get { return base.Enabled; }
			set { base.Enabled = fileUpload.Enabled = value; } // fileUpload must be disabled too, else, in firefox click works even if pnlContainer is disabled
		}

		protected void btnUploadStart_Click(object sender, EventArgs e)
		{
			if (this.UploadStart == null)
			{
				throw new NotSupportedException("Missing OnUploadStart event callback");
			}
			this.UploadStart.Invoke(fileUpload, e);
		}

		protected void btnUploadComplete_Click(object sender, EventArgs e)
		{
			this.UploadComplete.Invoke(fileUpload, e);
		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			if (this.Page != null && !this.Page.IsInAsyncPostBack())
			{
				IceScriptManager.GetCurrent(this).RegisterScriptFile(this, ".IceFileUpload.IceFileUpload.js");
			}

			this.ApplyStyle("position:relative;display:inline-block;")
			.AddChildren
			(
				pnlFileupload = new WebControl(HtmlTextWriterTag.Span).ApplyStyle("position:relative;overflow:hidden;display:inline-block;")
				.AddChildren
				(
					this.ButtonTemplate
				)
				.AddChildren
				(
					btnUploadStart.Set(o => o.Click += new EventHandler(btnUploadStart_Click)),
					btnUploadComplete.Set(o => o.Click += new EventHandler(btnUploadComplete_Click)),
					// fileUpload is very large, and invisible, ButtonTemplate is placed under it
					fileUpload.ApplyStyle("position:absolute;opacity:0;filter:alpha(opacity=0);top:-10px;right:-10px;font-size:500px;cursor:pointer;")
				),
				pnlProgress = new WebControl(HtmlTextWriterTag.Span).ApplyStyle("position:relative;overflow:hidden;display:none;").AddChildren
				(
					this.ProgressTemplate
				)
			);

			var updatePanel = this.AncestorControls().OfType <UpdatePanel>().FirstOrDefault();
			if (updatePanel == null)
			{
				throw new NotSupportedException("IceFileUpload must be palced into UpdatePanel");
			}

			// controls in update panel must have clientid set
			updatePanel.Triggers.Add(new AsyncPostBackTrigger() { ControlID = btnUploadComplete.ID = btnUploadComplete.ClientID });
			updatePanel.Triggers.Add(new PostBackTrigger() { ControlID = btnUploadStart.ID = btnUploadStart.ClientID });
			
		}

		protected override void Render(HtmlTextWriter writer)
		{
			string async = this.UploadComplete != null ? "true" : "false"; // if have UploadComplete event, upload in iframe, than make another request to refresh images

			this.Page.ExecuteJavaScript("Ice.Web.UI.WebControls.IceFileUpload.Setup('" + fileUpload.ClientID + "', '" + btnUploadStart.ClientID + "', '" + btnUploadComplete.ClientID + "', '" + pnlProgress.ClientID + "', '" + pnlFileupload.ClientID + "', " + async + ")");

			base.Render(writer);
		}


	}
}
