﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using Ice.Web.Extensions;
using System.Web.UI.WebControls;
using Ice.Web.Resources;

namespace Ice.Web.UI.WebControls
{
	public class IceLabel : System.Web.UI.WebControls.Label
	{
		private string _tagName = "table";

		protected override string TagName
		{
			get { return _tagName; }
		}

		protected override HtmlTextWriterTag TagKey
		{
			get { return HtmlTextWriterTag.Table; }
		}

		public enum IconPositionEnum
		{
			/// <summary>
			/// Default
			/// </summary>
			Left = 0,
			Top = 1,
			Right = 2,
			Bottom = 3
		}

		public IconEnum Icon { get { return (IconEnum)(this.ViewState["_icon"] as int?).GetValueOrDefault(); } set { this.ViewState["_icon"] = (int)value; } }
		public IconPositionEnum IconPosition { get { return (IconPositionEnum)(this.ViewState["_ip"] as int?).GetValueOrDefault(); } set { this.ViewState["_ip"] = (int)value; } }
		public string IconStyle { get { return this.ViewState["_is"] as string; } set { this.ViewState["_is"] = value; } }

		public override void RenderControl(HtmlTextWriter writer)
		{
			if (!this.Visible)
			{
				return;
			}

			if (this.Icon != IconEnum.None)
			{
				Image img = new Image() { ImageUrl = IceScriptManager.CreateUrl(this.Icon) }
					.ApplyStyle("border:0;padding:0;display:block;")
					.ApplyStyle("margin:" + (this.IconPosition == IconPositionEnum.Left ? "0px 3px 0 0" : IconPosition == IconPositionEnum.Right ? "0px 0 0 3px" : "0")); // margin-top:3px is fix, all browsers require it to be verticaly centered

				if (!string.IsNullOrEmpty(this.Text))
				{
					this.Controls.Add(new LiteralControl(this.Text));
				}

				var controls = this.Controls.Cast<Control>().ToArray(); // create copy
				this.Controls.Clear();

				switch (this.IconPosition)
				{
					case IconPositionEnum.Left:
						this.AddChildren(new HtmlTableRow().AddChildren(new HtmlTableCell().AddChildren(img), new HtmlTableCell().AddChildren(controls)));
						break;
					case IconPositionEnum.Right:
						this.AddChildren(new HtmlTableRow().AddChildren(new HtmlTableCell().AddChildren(controls), new HtmlTableCell().AddChildren(img)));
						break;
					case IconPositionEnum.Top:
						this.AddChildren(new HtmlTableRow().AddChildren(new HtmlTableCell().AddChildren(img)), new HtmlTableRow().AddChildren(new HtmlTableCell().AddChildren(controls)));
						break;
					case IconPositionEnum.Bottom:
						this.AddChildren(new HtmlTableRow().AddChildren(new HtmlTableCell().AddChildren(controls)), new HtmlTableRow().AddChildren(new HtmlTableCell().AddChildren(img)));
						break;
				}
			}
			else
			{
				this.AddChildren(new HtmlTableRow().AddChildren(new HtmlTableCell().AddChildren(new LiteralControl(this.Text))));
			}

			if(!string.IsNullOrEmpty(this.AssociatedControlID))
			{
				throw new NotSupportedException("AssociatedControlID can not be set in IceLabel");
			}
			
			IAttributeAccessor tt = (IAttributeAccessor)this;
			tt.SetAttribute("cellpadding", "0");
			tt.SetAttribute("cellspacing", "0");

			base.RenderControl(writer);

		}
	}

}
