﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using Ice.Web.Script.Services;
using System.Web.UI.WebControls;
using Ice.Web.Extensions;
using System.Text.RegularExpressions;
using System.Web.UI.HtmlControls;

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(true)]
	public class IceTextWatermark : PlaceHolder
	{
		public string TargetControlID { get; set; }
		public string WatermarkCssClass { get; set; }
		public string ErrorCssClass { get; set; }
		public bool IsError { get; set; }
		public string EmptyText { get; set; }

		/// <summary>
		/// Visible mask in text box. eg. __/__/___ __:__ , if there is no Format defined, underscores are replaced with digits
		/// </summary>
		public string Mask { get; set; }

		/// <summary>
		/// Formating mask. Not visible to user. eg. dd.dd.dddd.
		/// d: digit, a: alpha (letter), w: word (letter or digit),
		/// &lt; fill from right to left, default is left to right
		/// &lt;ddd.ddd.ddd,dd kn, currency and chars are filled from right to left (&lt;)
		/// </summary>
		public string Format { get; set; }

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			if(this.Page != null && !this.Page.IsInAsyncPostBack())
			{
				// js has to be initialized even if control is not visible and later may be set visible using UpdatePanel
				IceScriptManager.GetCurrent(this).RegisterScriptFile(this, ".IceTextWatermark.IceTextWatermark.js");
			}
		}

		protected override void OnLoad(EventArgs e)
		{

			if (string.IsNullOrEmpty(this.TargetControlID))
			{
				throw new Exception("TargetControlID is not set");
			}

			if (this.Page.IsPostBack)
			{
				FixTextControlValue();
			}

			base.OnLoad(e);
		}

		protected override void OnPreRender(EventArgs e)
		{
			SetupControl();

			base.OnPreRender(e);
		}


		private void SetupControl()
		{
			// search TargetControlID in NamingContainer container only
			Control control = this.NamingContainer.DescendantControls().Where(o => o.ID == this.TargetControlID).FirstOrDefault();
			if (control == null)
			{
				throw new Exception("Control " + this.TargetControlID + " not found!");
			}
			if (!(control is TextBox || control is DropDownList))
			{
				throw new Exception("Control " + this.TargetControlID + " has to be TextBox or DropDownList!");
			}

			if (!control.Visible)
			{
				return;
			}

			if (this.IsError && string.IsNullOrEmpty(this.ErrorCssClass))
			{
				throw new Exception("ErrorCssClass can not be empty, TargetControlID " + this.TargetControlID);
			}

			// IceTextWatermark(id, wmClass, emptyTxt, errorClass)
			this.Page.ExecuteJavaScript(string.Format("Ice.Web.UI.WebControls.IceTextWatermark.Setup(\"{0}\", \"{1}\", \"{2}\", \"{3}\", {4}, \"{5}\", \"{6}\");",
				"#" + control.ClientID,
				this.WatermarkCssClass ?? "",
				this.EmptyText ?? "",
				this.IsError ? (this.ErrorCssClass ?? "") : "",
				this.IsError ? "true" : "false",
				this.Mask,
				this.Format
				));
		}

		private void FixTextControlValue()
		{
			// search TargetControlID in parent container only
			TextBox control = this.NamingContainer.DescendantControls().OfType<TextBox>().Where(o => o.ID == this.TargetControlID).FirstOrDefault();
			if (control != null)
			{
				if (!string.IsNullOrEmpty(this.EmptyText))
				{
					if (this.EmptyText == control.Text)
					{
						control.Text = "";
					}
				}
				else if (control.ToolTip == control.Text)
				{
					control.Text = "";
				}
			}
		}

		public static void MarkError(Control control, bool isError)
		{
			// search TargetControlID in parent container only
			IceTextWatermark wm = control.NamingContainer.DescendantControls().OfType<IceTextWatermark>().Where(o => o.TargetControlID == control.ID).FirstOrDefault();
			if (wm == null)
			{
				throw new Exception("IceTextWatermark for TargetControlID " + control.ID + " not found!");
			}

			wm.IsError = isError;
		}
	}
}
