﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Web.UI;
using System.Linq;
using Ice.Web.Extensions;
using System.Web.UI.WebControls;

//*****************************************************************
// works with IceScriptManager and with MS ScriptManager
//*****************************************************************

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(true)]
	[PersistChildren(true)]
	public class IceToolTip : PlaceHolder
	{
		public enum PositionEnum
		{
			/// <summary>
			/// default
			/// </summary>
			TopCenter = 0,
			TopLeft,
			TopRight,

			BottomLeft,
			BottomCenter,
			BottomRight,

			LeftTop,
			LeftMiddle,
			LeftBottom,

			RightTop,
			RightMiddle,
			RightBottom
		}

		//use any of Content... and Anchor... properties to set anchor content and content content

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate AnchorTemplate { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate ContentTemplate { get; set; }

		public Panel AnchorPanel { get; set; }

		public Panel ContentPanel { get; set; }

		public string AnchorHtml { get; set; }

		public string ContentHtml { get; set; }



		public PositionEnum Position { get; set; }

		[DefaultValue(typeof(Color), "")]
		[TypeConverter(typeof(WebColorConverter))]
		public virtual Color BackColor { get; set; }

		[DefaultValue(typeof(Color), "")]
		[TypeConverter(typeof(WebColorConverter))]
		public virtual Color ForeColor { get; set; }

		[DefaultValue(typeof(Color), "")]
		[TypeConverter(typeof(WebColorConverter))]
		public virtual Color BorderColor { get; set; }

		public virtual Unit BorderWidth { get; set; }

		public Unit TipWidth { get; set; }
		public Unit TipMaxHeight { get; set; }

		private Panel _anchor = new Panel() { CssClass = "iceTipAnchor" };
		private Panel _tip;
		private Panel _content = new Panel();

		private void CreateControl()
		{
			/*
			<div class="iceTipAnchor">
				<span>?</span>
				<div class="iceTip pos-bottom-center">
					<div class="iceTipContent">
						<div>ovo je neki sadrzaj tooltipa</div>
						<div>to je cijeli tip</div>
					</div>
				</div>
			</div>
			*/

			string pos = this.Position.ToString();
			string sec = pos.Skip(1).SkipWhile(o => char.IsLower(o)).ToStringJoined();
			string first = pos.TakeFirst(pos.Length - sec.Length);

			string position = ("pos-" + first + "-" + sec).ToLower();

			this.AddChildren(_anchor.AddChildren(
				this.AnchorTemplate).AddChildren(this.AnchorPanel).AddChildren(this.AnchorHtml == null ? null : new LiteralControl(this.AnchorHtml)).AddChildren(
				_tip = new Panel() { CssClass = "iceTip " + position }.AddChildren(
				_content.AddClass("iceTipContent").AddChildren(this.ContentTemplate).AddChildren(this.ContentPanel).AddChildren(this.ContentHtml == null ? null : new LiteralControl(this.ContentHtml))
				)
			));

			if (!this.TipWidth.IsEmpty) { _content.ApplyStyle("width:" + this.TipWidth.Value + "px"); }
			if (!this.TipMaxHeight.IsEmpty) { _content.ApplyStyle("max-height:" + this.TipMaxHeight.Value + "px"); }

			if (!this.BackColor.IsEmpty) { _tip.BackColor = this.BackColor; }
			if (!this.ForeColor.IsEmpty) { _tip.ForeColor = this.ForeColor; }
			if (!this.BorderColor.IsEmpty) { _tip.BorderColor = this.BorderColor; }
			if (!this.BorderWidth.IsEmpty) { _tip.BorderWidth = this.BorderWidth; }

			// register to Setup control in both cases: if request is MS Ajax, or IceAjax
			this.Page.ExecuteJavaScript("Ice.Web.UI.WebControls.ToolTip.Setup('#" + _anchor.ClientID + "');");
		}

		//**********************************************************************************************
		// script has to be loaded always, even if panel is not currently visible. It may be set visible later in UpdatePanel
		// control has to be created on Render(...). This way we may use IceAjax or MS Ajax in UpdatePanel and it will work fine
		//**********************************************************************************************

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			// engine script has to be created always, even if control is not currently visible, later it may become visible
			// script has to be loaded always, even if panel is not currently visible. It may be set visible later in UpdatePanel
			if (this.Page != null && !this.Page.IsInAsyncPostBack())
			{
				// eg. if ToolTip is in ascx control which is loaded using IceAjax, we don't have referenct to page where IceScriptManager is

				IceScriptManager.GetCurrent(this.Page)
						.RegisterScriptFile(this, ".IceToolTip.IceToolTip.css")
						.RegisterScriptFile(this, ".IceToolTip.IceToolTip.js");
			}
		}

		protected override void Render(HtmlTextWriter writer)
		{
			////CreateScripts();

			CreateControl();

			base.Render(writer);
		}

	}
}