﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using Ice.Web.Script.Services;
using System.Web.UI.WebControls;
using Ice.Web.Extensions;
using System.Text.RegularExpressions;
using System.Web.UI.HtmlControls;
using System.ComponentModel;
using System.Drawing;
using Ice.Colorspace;

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(true)]
	[PersistChildren(true)]
	public partial class IceVerticalSplit : IcePanel
	{
		public class VSContentStyle
		{
			public Unit MinWidth { get; set; }
			public Unit MaxWidth { get; set; }
			/// <summary>
			/// initial width
			/// </summary>
			public Unit Width { get; set; }
			public string CssClass { get; set; }
			public Unit BorderWidth { get; set; }

			[DefaultValue(typeof(Color), "")]
			[TypeConverter(typeof(WebColorConverter))]
			public virtual Color BorderColor { get; set; }

			public string Style { get; set; }
		}

		public class VertSplitStyle
		{
			/// <summary>
			/// split bar width
			/// </summary>
			public Unit Width { get; set; }
			/// <summary>
			/// if true, panels are not resizable
			/// </summary>
			public bool Static { get; set; }

			[DefaultValue(typeof(Color), "")]
			[TypeConverter(typeof(WebColorConverter))]
			public virtual Color HoverColor { get; set; }
		}

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate LeftContent { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate RightContent { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate SplitContent { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public VertSplitStyle SplitStyle { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public VSContentStyle LeftStyle { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public VSContentStyle RightStyle { get; set; }


		public Unit FrameWidth { get; set; }

		[DefaultValue(typeof(Color), "")]
		[TypeConverter(typeof(WebColorConverter))]
		public virtual Color FrameColor { get; set; }


		Panel leftContainer = new Panel().ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;overflow:hidden;");
		Panel splitBarContainer = new Panel().ApplyStyle("top:0; bottom:0;position:absolute;left:0;right:auto;background-color:#D9E5F3;overflow:hidden;");
		Panel rightContainer = new Panel().ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;overflow:hidden;");

		Panel leftInnerContainer = new Panel().ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;overflow:auto;");
		Panel rightInnerContainer = new Panel().ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;overflow:auto;");

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			this.AddChildren
			(
				leftContainer.AddChildren(leftInnerContainer.AddChildren(this.LeftContent)),
				splitBarContainer.AddChildren(this.SplitContent),
				rightContainer.AddChildren(rightInnerContainer.AddChildren(this.RightContent))
			);

			if (this.Page != null && !this.Page.IsInAsyncPostBack())
			{
				// script has to be loaded always, even if panel is not currently visible. It may be set visible later in UpdatePanel
				IceScriptManager.GetCurrent(this.Page).RegisterScriptFile(this, ".IceVerticalSplit.IceVerticalSplit.js");
			}
		}

		private void SetupControls()
		{
			IceScriptManager manager = IceScriptManager.GetCurrent(this.Page);

			VertSplitStyle splitStyle = this.SplitStyle ?? new VertSplitStyle();
			VSContentStyle leftStyle = this.LeftStyle ?? new VSContentStyle();
			VSContentStyle rightStyle = this.RightStyle ?? new VSContentStyle();

			leftInnerContainer.BorderColor = leftStyle.BorderColor.IsEmpty ? System.Drawing.ColorTranslator.FromHtml(manager.CurrentTheme.FrameBorderColor) : leftStyle.BorderColor;
			leftInnerContainer.BorderWidth = leftStyle.BorderWidth;
			leftInnerContainer.ApplyStyle(leftStyle.Style);
			leftInnerContainer.AddClass(leftStyle.CssClass);

			rightInnerContainer.BorderColor = rightStyle.BorderColor.IsEmpty ? System.Drawing.ColorTranslator.FromHtml(manager.CurrentTheme.FrameBorderColor) : rightStyle.BorderColor;
			rightInnerContainer.BorderWidth = rightStyle.BorderWidth;
			rightInnerContainer.ApplyStyle(rightStyle.Style);
			rightInnerContainer.AddClass(rightStyle.CssClass);

			string frameColor = this.FrameColor.IsEmpty ? manager.CurrentTheme.FrameColor : this.FrameColor.ToHtml();
			// set from theme
			splitBarContainer.Style["background-color"] = frameColor;
			splitBarContainer.Width = splitStyle.Width;

			this.ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;");
			this.ApplyStyle("border:solid " + (this.FrameWidth.IsEmpty ? "0" : this.FrameWidth.ToString()) + " " + (frameColor));

			if (leftStyle.Width.IsEmpty)
			{
				if (rightStyle.Width.IsEmpty)
				{
					throw new Exception("ContainerWidth has to be defined for LeftPanel or RightPanel");
				}
				else
				{
					// right width is defined
					splitBarContainer.AddClass("toRight");
					splitBarContainer.Style["right"] = rightStyle.Width.ToString();
					splitBarContainer.Style["left"] = "auto";

					rightContainer.Style["width"] = rightStyle.Width.ToString();
					rightContainer.Style["left"] = "auto";
					leftContainer.Style["right"] = (splitStyle.Width.Value + rightStyle.Width.Value) + "px";
				}
			}
			else
			{
				if (!rightStyle.Width.IsEmpty)
				{
					throw new Exception("ContainerWidth can't be defined for both panels");
				}
				else
				{
					// left width is defined
					splitBarContainer.AddClass("toLeft");
					splitBarContainer.Style["left"] =leftStyle.Width.ToString();
					splitBarContainer.Style["right"] = "auto";

					leftContainer.Style["width"] = leftStyle.Width.ToString();
					leftContainer.Style["right"] = "auto";
					rightContainer.Style["left"] = (splitStyle.Width.Value + leftStyle.Width.Value) + "px";
				}
			}

			if (!splitStyle.Static)
			{
				this.Page.ExecuteJavaScript(string.Format("Ice.Web.UI.WebControls.IceVerticalSplit.Setup('{0}', '{1}', '{2}', {3}, {4}, {5}, {6})",
					splitBarContainer.ClientID,
					frameColor,
					splitStyle.HoverColor.IsEmpty ? manager.CurrentTheme.SplitBarHoverColor : splitStyle.HoverColor.ToHtml(),
					(int)leftStyle.MinWidth.Value,
					(int)(leftStyle.MaxWidth.IsEmpty ? int.MaxValue : leftStyle.MaxWidth.Value),
					(int)rightStyle.MinWidth.Value,
					(int)(rightStyle.MaxWidth.IsEmpty ? int.MaxValue : rightStyle.MaxWidth.Value)
					));
			}

		}

		protected override void OnPreRender(EventArgs e)
		{
			SetupControls();

			base.OnPreRender(e);

		}


	}
}