﻿/// <reference path="../../ice-fix-ext-before.js" />
/// <reference path="../../Ext_21/source/core/Ext.js" />
/// <reference path="../../Ext_21/source/adapter/ext-base.js" />
/// <reference path="../../Ext_21/ext-all-debug.js" />
/// <reference path="../../ice-fix-ext-after.js" />
/// <reference path="../../IceExtExtensions.js" />
/// <reference path="/Scripts/IceFormSerialization.js" />
/// <reference path="DbAdminNamespace.js" />

Ice.DbAdmin.DatabaseSchemaTables = {};
Ice.DbAdmin.CachedDialogWindow = {};

Ice.DbAdmin.ShowFormAddNew = function(config){

	//alert("ShowFormAddNew");
	/*
	config is the same as GridWithPaging plus some extra properties:
		config.tableName;
		config.hideSchemaInfo=true;
		config.animateTarget = sender;
		config.onSaved = AfterSaveMethod;
		config.record = record;
		config.forceInsert = forceInsert;
		config.iconCls = iconCls;
		config.multiline = false; // use TextArea editor instead of TextField
		config.enableFormEditing = true; // if hidden in grid, but it may be editable in form
		config.beforeInsert(data); // call before insert, egg. apply categoryID
		config.formEditable = true; // allow entering value into combo box
	*/
	
	var data = config.record ? (config.record.data || {}) : {};
	
	
	// use cached window
	var win = Ice.DbAdmin.CachedDialogWindow[config.tableName + config.iconCls + config.forceInsert + config.formNameBase];
	if(win != null)
	{
		win.onFormSaved = config.onSaved;
		win.currentRecord = config.record;
		PopulateWithValues(data, win.formItems);
		win.show(config.animateTarget);		
		return win;
	}
	
	function GetDetails(column)
	{
		var details = column.TypeName;
		column.AllowDBNull ? null : (details+= ", <span style='color:red;'>NotNull</span>");
		column.MaxLength > 0 ? (details += ", Max=" + column.MaxLength) : null;
		column.PK ? (details += ", <b>PK</b>") : null;
		return details;
	}
	
	function GetName(name, isRed)
	{
		if(isRed)
		{
			return "<span style='color:red;'>" + name + "</span>" ;
		}
		else
		{
			return name;
		}
	}
	
	function GetColumnName(schemaColumn)
	{
		return GetName(schemaColumn.ColumnName, !schemaColumn.AllowDBNull);
	}
	
	function GetField(column)
	{

////		public string ColumnName;
////		public bool AllowDBNull;
////		public bool AutoIncrement;
////		public string TypeName;
////		public int MaxLength;
////		public bool ReadOnly;
////		public bool Unique;
////		public bool PK;

		// apply custom configuration
		var custom = customColumns[column.ColumnName] || {};
		delete(customColumns[column.ColumnName]); // once used, remove it
		
		var editor = null;

		if(custom.formEditor)
		{
			editor = new custom.formEditor();
			editor.fieldLabel = GetName(custom.header, custom.formRequired);
		}

		var tn = column.TypeName;
		if(editor == null)
		{
			switch(column.TypeName)
			{
				case "DateTime":
					editor = new Ext.form.DateTimeField({width:200});
					break;
				case "Boolean":
				case "Byte": // for MySQL
				case "SByte":
					editor = new Ext.form.Checkbox();
					break;
				case "String":
					if(custom.multiline || config.multiline)
					{
						editor = new Ext.form.TextArea({width:200, height: custom.height || 50});
					}
					else
					{
						editor = new Ext.form.TextField({width:200});
					}
					break;
				default:
					if(tn.indexOf("Int") >= 0)// || tn.indexOf("Byte") >= 0) // Int, UInt, Byte, SByte
					{
						editor = new Ext.form.NumberField({width:100, allowDecimals: false});
					}
					else if(tn.indexOf("Double") || tn.indexOf("Single") || tn.indexOf("Decimal"))
					{
						editor = new Ext.form.NumberField({width:100, allowDecimals: true, decimalPrecision:8});
					}
					else
					{
						editor = new Ext.form.TextField({width:200}); // unknown type
					}
					break;
			}
		}
		
		if(!editor.fieldLabel)
		{
			editor.fieldLabel = GetColumnName(column);
			if( ! config.hideSchemaInfo)
			{
				editor.fieldLabel += " (" + GetDetails(column) + ")";
			}
		}
		
		if(column.ReadOnly || column.AutoIncrement || custom.disableFormEditing)
		{
			editor.disable();
		}
		else if( ! column.AllowDBNull)
		{
			editor.required = true;
		}
		
		editor.name = column.ColumnName;
		
		if(custom.formEditable)
		{
			editor.editable = true; // allow entering value into combo box
		}
		if(custom.disableFormEditing)
		{
			editor.disable();
		}
		if(custom.hidden && ! custom.enableFormEditing)
		{
			editor = null;
		}
		
		return editor;
	}

	var customColumns = Ext.apply({}, config.columns);
	
	function GetFormItems(schema)
	{
		var itemsAll = [];
		for(var n = 0; n < schema.length; n++)
		{
			var editor = GetField(schema[n]);
			if(editor != null)
			{
				itemsAll.push(editor);
			}
		}
		// add custom fields
		for(var prop in customColumns)
		{
			var custom = customColumns[prop];
			if(custom.formEditor)
			{
				var formEditorInstance = new custom.formEditor();
				formEditorInstance.fieldLabel = custom.header;
				formEditorInstance.name = custom.dataIndex;
				itemsAll.push(formEditorInstance);
			}
		}
		return itemsAll;
	}	
	
	function GetObject(rec)
	{
		var data = {};
		for(var n = 0; n < win.formItems.length; n++)
		{
			var formItem = win.formItems[n];
			var value = formItem.getValue();
			
			if(formItem.required && (value == null || (value+"").length == 0)) // NotNull fields must be assigned
			{
				return null;
			}
			
			if(!config.forceInsert && rec.set)
			{
				rec.set(formItem.name, value);
				data = rec.data;
			}
			else
			{
				data[formItem.name] = value;
			}
		}
		return data;
	}
	
	function PopulateWithValues(dataObj, formItems)
	{
		for(var n = 0; n < formItems.length; n++)
		{
			var formItem = formItems[n];
			if(formItem.setValue)
			{
				formItem.setValue(dataObj[formItem.name]);
			}
		}
	}
	
	var widthReduction = 0;// config.hideSchemaInfo ? 60 : 0;
	
	function GetFormPanel(formItems, data)
	{	
		formPanel = new Ext.FormPanel({
			labelWidth: 200 - widthReduction,
		   // frame:true,
			hideBorders: true,
			border:false,
			autoScroll: true,
			bodyStyle: "background-color:#ECECEC; padding:5px;",
			//defaults: {width: 230},
			items: formItems,
			defaultType: 'textfield',
			layout:"form"	
		});
		
		PopulateWithValues(data, formItems);
		
		return formPanel;
	}

	var win = new Ext.Window({

		title: config.tableName + (config.forceInsert ? " - Add New Record" : " - Update Record" ),
		minimizable:false,
		modal:true,
		maximizable:false,
		resizable:false,
		closable:true,
		maximized:false,
		iconCls: config.iconCls,
		border:true,
		width: 450 - widthReduction,
		height: 300,
		shim:false,
		closeAction:'hide',
		animCollapse:true,
		autoScroll: true,
		layout: 'fit',
		buttons: 
		[
			{
				text: config.forceInsert ? "Add New" : "Update",
				iconCls:"icon_table_save",
				handler: function()
				{
					var obj = GetObject(win.currentRecord);
					if(obj == null)
					{
						alert("NotNull values must be assigned!")
					}
					else
					{	
						if(config.forceInsert)
						{
						
							GService.InsertRows(config.tableName, [obj], function(newRows){
								if(newRows && newRows.length > 0)
								{
									win.hide();
									win.onFormSaved(newRows);
								}						
							});
						}
						else
						{
							GService.UpdateRows(config.tableName, [obj], function(newRows){
								if(newRows && newRows.length > 0)
								{
									win.hide();
									win.onFormSaved(newRows);
								}						
							});
						}
					
					}
				}
			},
			{
				text: "Close",
				iconCls:"icon_cancel",
				handler: function()
				{
					win.hide();
				}
			}
		]
	});

	//win.on("hide", win.close);
	
	win.currentRecord = config.record;
	win.onFormSaved = config.onSaved;
	
	Ice.DbAdmin.CachedDialogWindow[config.tableName + config.iconCls + config.forceInsert + config.formNameBase] = win;
	
	function ShowWindow(schema)
	{
		win.formItems = GetFormItems(schema);
		win.add(GetFormPanel(win.formItems, data));
		win.render(document.body);
		
		// resize window
		//var formPanel = win.items.items[0].getForm();
		var height = win.items.items[0].getForm().el.dom.offsetHeight + 90;
		if((height + 80) > document.body.offsetHeight)
		{
			height = document.body.offsetHeight - 80;
		}
		
		win.setHeight(height);
		
		win.show(config.animateTarget);
	}
	
	var sch = Ice.DbAdmin.DatabaseSchemaTables[config.tableName];

	if( sch == null)
	{
		// get schema from database
		GService.GetSchemaColumns(config.tableName, function(schema){
			Ice.DbAdmin.DatabaseSchemaTables[config.tableName] = schema;			
			ShowWindow(schema);
		});
	}
	else
	{
		ShowWindow(sch);
	}	
	
	return win;
	
};

	