﻿/// <reference path="../language/en.js" />

Ice.ProductImagesGrid = function (productID)
{
	var __this = this;

	function OnImageDoubleClick(imageData, options)
	{
		options.record.set("ImageLocation", imageData.relativeHttpPath);
	}

	function CellClicked(grid, rowIndex, columnIndex, e)
	{
		if (columnIndex != 0)
		{
			return;
		}
		// open gallery
		var gallery = QoDesk.getModule("ImageGalleryEditor");
		gallery.OnImageDoubleClick =
		{
			minimize: true, // minimize gallery after image double click
			handler: OnImageDoubleClick,
			options:
			{
				record: grid.getStore().getAt(rowIndex)
			}
		};
		gallery.createWindow();

	}

	function OnDataReady(data)
	{
		__this.store.removeAll();
		var m = Ice.Ext.GetMetaData(data, __this.store);
		__this.store.loadData(m);
	}

	function ReloadImages()
	{
		__this.store.removeAll();
		GService.GetProductImageCollection(productID, OnDataReady);
	}

	function AddNewImage()
	{
		GService.NewProductImage(productID, ReloadImages);
	}

	function DeleteImage(a, b, c, d, e)
	{
		var selected = __this.selModel.getSelected();
		if (selected && selected.data)
		{
			GService.DeleteProductImage(selected.data, ReloadImages);
		}
	}

	function SaveImages(reloadData, prodId)
	{
		var arr = [];
		var rec = __this.store.getRange();
		for (var n = 0; n < rec.length; n++)
		{
			if (rec[n].dirty)
			{
				arr.push(rec[n].data);
			}
		}
		prodId = prodId ? prodId : productID;
		GService.SaveProductImageCollection(arr, prodId, reloadData != false ? ReloadImages : null);
	}
	this.SaveImages = SaveImages;

	var imgIndex = 0;

	function SetImage(imgLocation)
	{
		imgLocation += "";

		if (imgLocation.length > 3)
		{
			var spanId = "span_loading_" + imgIndex;
			imgIndex++;

			var img = new Image();
			img.onload = function ()
			{
				img.title = img.width + " x " + img.height;

				img = Ice.ProductImagesGrid.ImageLoaded2(img);
				function ReplaceSpan()
				{
					var ss = document.getElementById(spanId);
					if (ss != null)
					{
						ss.parentNode.replaceChild(img, ss);
					}
					else
					{
						setTimeout(ReplaceSpan, 100);
					}
				}
				ReplaceSpan();
			};
			img.src = GService.AppRoot + imgLocation;
			img.style.border = "solid 1px #c3c3c3";
			img.style.cursor = "pointer";

			return "<div title='select new image' style='padding:10px 0 10px 2px; width:50px; background-color:#F5F5F5; cursor:pointer;' id='" + spanId + "'>loading...</div>";
		}
		else
		{
			return "<div title='select new image' style='padding:10px 0 10px 2px; width:50px; background-color:#F5F5F5; cursor:pointer;'>select...</div>";
		}
	}

	Ice.ProductImagesGrid.superclass.constructor.call(this, {
		width: 225,
		height: 200,
		border: false,
		viewConfig: { forceFit: true },
		sm: new Ext.grid.RowSelectionModel({ singleSelect: true }),
		store: new Ext.data.Store({
			reader: new Ext.data.JsonReader(
			{
				root: "",
				fields:
				[
					{ name: "ProductImageID", type: "int" }
				]
			})
		}),
		cm: new Ext.grid.ColumnModel([
		//	new Ext.grid.RowNumberer(),
				{
				header: CartLanguage.Image,
				width: 50,
				dataIndex: "ImageLocation",
				renderer: SetImage
			},
				{
					header: CartLanguage.Location,
					sortable: true,
					dataIndex: 'ImageLocation',
					editor: new Ext.form.TextField({ allowBlank: false })
				}
			]),

		tbar:
		[
			{
				//text:'Add',
				tooltip: CartLanguage.AddNew,
				iconCls: 'demo-grid-add tabs',
				handler: AddNewImage
			},
			'-',
			{
				//text:'Delete',
				tooltip: CartLanguage.DeleteSelected,
				iconCls: 'demo-grid-delete',
				handler: DeleteImage,
				disabled: false
			},
			"-",
			{
				//text:"Save Changes",
				tooltip: CartLanguage.SaveChanges,
				iconCls: "demo-grid-saveall",
				handler: function () { SaveImages(true, productID); }
			},
			"-",
			{
				//text: "Reload",
				tooltip: CartLanguage.ReloadFromDatabase,
				iconCls: "demo-grid-refresh",
				handler: ReloadImages
			}
		]
	});

	this.on("render", ReloadImages);
	this.on("cellclick", CellClicked);

};

Ice.ProductImagesGrid = Ext.extend(Ice.ProductImagesGrid, Ext.grid.EditorGridPanel);

Ice.ProductImagesGrid.ImageLoaded2 = function(senderImage)
{
	function ResizeImage(image, options)
	{
		var maxHeight = options.height;
		var maxWidth = options.width;
		
		if(image.width <= maxWidth && image.height <= maxWidth)
		{
			return;
		}
		var ratio = image.width / image.height;
		var thumbRatio = maxWidth / maxHeight;
		if (ratio > thumbRatio)
		{
			// too width
			image.width = maxWidth;
			image.height = image.width / ratio;
		}
		else
		{
			// to height
			image.height = maxHeight;
			image.width = image.height * ratio;
		}
	}
	
	ResizeImage(senderImage, {width:50, height:50});
	return senderImage;
	
};