﻿/// <reference path="../../ice-fix-ext-before.js" />
/// <reference path="../../Ext_21/source/core/Ext.js" />
/// <reference path="../../Ext_21/source/adapter/ext-base.js" />
/// <reference path="../../Ext_21/ext-all-debug.js" />
/// <reference path="../../ice-fix-ext-after.js" />
/// <reference path="DbAdminNamespace.js" />
/// <reference path="GridWithPaging.js" />
/// <reference path="TabCloseMenu.js" />
/// <reference path="CategoriesTree.js" />

QoDesk.GetUsersTab = function(roleData){

	//alert("GetCategoryTab");
	
	var roleName = roleData ? roleData.Name : "all users";
	var roleId = roleData ? roleData.RoleID : "";
	
	function RemoveFromRole(tableName, arrayData, onDone)
	{	
		var userIds = [];
		for(var n = 0; n < arrayData.length; n++)
		{
			userIds.push(arrayData[n].UserID);
		}
		GService.UsersInRoles([roleId], userIds, false, onDone);
	}
	
	var usersGrid = Ice.DbAdmin.GetGridWithPaging({
		tableName:"User", 
		baseParams: {RoleID:roleId},
		sortInfo: {field:'UserID', direction:'DESC'},
		delayLoad:500,
		disableInlineEditing:true,
		hideSchemaInfo:true,
		methodRemove: RemoveFromRole,
		showRemove: roleId > 0,
		menuItemsBefore:
		[
			{
				text: "Address",
				iconCls:"icon_overlays",
				handler: function(sender, e){
					var record = null;
					var selections = usersGrid.getSelectionModel().getSelections(); // use selected data as template for new record
					if(selections && selections.length > 0)
					{
						record = selections[0];
						new QoDesk.UserEdit(record);
					}
				}
			},
			"-"
		]
	});
	
	var tab = new Ext.Panel({
		title: roleName,
		border:false,
		closable: roleId > 0,
		iconCls:"icon_application_view_columns",
		layout:"fit",
		items:
		[
			{
				layout:"fit",
				items: usersGrid,
				border:false
			}
		]
	});
	
	tab.RoleID = roleId;
	
	var count = 0;
	
	tab.on("show", function(){
		if(count ++ > 0) // don't reload on first time show
		{
			usersGrid.Reload();
		}
	});
	
	return tab;
};




QoDesk.GetUsersMainForm = function(){

	//alert("GetCategoriesMainForm");
	
	function NodeClicked(node, e)
	{
		// find tab by RoleID
		for(var n = 0; n < tabs.items.items.length; n++)
		{
			var tab = tabs.items.items[n];
			if(tab.RoleID == node.nodeId)
			{
				tabs.setActiveTab(tab);
				return;
			}
		}
		// create new tab
		if(node.nodeData)
		{
			var tab = QoDesk.GetUsersTab(node.nodeData); 
			tabs.add(tab);
			tabs.show(tab);
			setTimeout(function(){tabs.setActiveTab(tab);}, 10);
		}
	}
	
	// right pane tabs
	var tabs = new Ext.TabPanel({
		resizeTabs:true,
		autoWidth:false,
		tabWidth:150,
		minTabWidth:150,
		enableTabScroll:true,
		activeTab:0,
		border:false,
		split:true,
		items:[QoDesk.GetUsersTab(null)],
		plugins: new Ext.ux.TabCloseMenu()
	});
	
	// roles tab
	var rolesTabs = new Ext.TabPanel({
		resizeTabs:true,
		autoWidth:false,
		tabWidth:150,
		minTabWidth:150,
		enableTabScroll:true,
		activeTab:0,
		border:false,
		items:
		[
			{
				title:"Roles",
				layout:"fit",
				items:  QoDesk.GetRolesTree(NodeClicked)
			}
		]
	});
	
	// main form
	var panel = new Ext.Panel({
		layout:"border",
		border:false,		
		items:
		[
			{
				region:"west",
				split:true,
				width:250,
				border:false,
				items: rolesTabs,
				layout:"fit"
				
			},
			{
				region:"center",
				layout:"fit",
				items: tabs
			}
		]
	});

	return panel;
           
};


QoDesk.UsersMain = function(config){
	QoDesk.UsersMain.superclass.constructor.apply(this, arguments);
};

QoDesk.UsersMain = Ext.extend(QoDesk.UsersMain, Ext.app.Module, {

	name: "Users",
	windowId: "UsersInRolesWin",
	moduleType: 'demo',
    moduleId: 'UsersInRolesEditorGrid',
	autoload: true,
	iconCls : "users-icon-small",
	shortcutIconCls : "users-icon-large" ,

    init : function(){
        this.launcher = {
            handler : this.createWindow,
            iconCls: this.iconCls,
            scope: this,
            shortcutIconCls: this.shortcutIconCls,
            text: this.name,
            tooltip: "<b>" + this.name + "</b>"
        }
    },

    createWindow : function(){
        var desktop = this.app.getDesktop();
        var win = desktop.getWindow(this.windowId);
		
		if(win)
		{
			win.show();
		}
		else
        {
			win = desktop.createWindow({
		        id: this.windowId,
                title: this.name,
                width:document.body.offsetWidth > 1200 ? 1200 - 40 : document.body.offsetWidth - 40,
                height: document.body.offsetHeight > 800 ? 800 - 40 : document.body.offsetHeight - 40,
                //closeAction:'hide',
                iconCls: this.iconCls,
                shim:false,
                animCollapse:false,
                constrainHeader:true,
				layout: 'fit',
                taskbuttonTooltip: "<b>" + this.name + "</b>"
			});
			
			win.show();
			
			setTimeout(function(){
				 win.add(QoDesk.GetUsersMainForm());
				 win.render();
			}, 500);

        }
      
    }
 });
 
